#!/usr/bin/python

# suppress deprecation warnings for modules, that we cannot fix
# at the moment we support python 2 >= 2.3 (RHEL4)
import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)
import sets     # TODO use standard type set (python >= 2.6)
warnings.resetwarnings()

from poaupdater import uUtil, uConfig, uSysDB, uLogging, uDialog, uAction, uHCL, uTasks, openapi, pbaapi

from migration.objects import *

from migration.config import PrerequisiteError, MigrationError, getTargetServicePlanInfo

uLogging.do_time = True
billingPeriodType = {0: 'Hour(s)', 1: 'Day(s)', 2: 'Month(s)', 3: 'Year(s)'}

RESOURCE_STATE_NO_MAPPED_PAIR = "-------"


def printUsage():
    print 'Usage: %s [--batch] --subscription-id <subscription-id> --target-id <target-node-id> --config <path-to-config> migrate' % sys.argv[0]
    print ' to migrate subscription <subscription-id> from source host to Web Cluster/Server NG <target node id> using service template mapping from <path-to-config> file.'
    for opt in __long_options:
        print "    --%s - %s" % (opt[0].rstrip('='), opt[1])


def doInit(options):

    def __useOption(key, attr, default=None):
        if key in options:
            value = options[key]
            if value == "":
                value = True
            setattr(config.config, attr, value)
        else:
            setattr(config.config, attr, default)

    config.config = uConfig.Config(
        verbose='--verbose' in options,
        batch='--batch' in options,
        skipwarnings='--skip-warnings' in options,
        skip_resource_usage_check='--skip-resource-usage-check' in options,
        skip_incomplete_tasks='--skip-incomplete-tasks' in options,
        log_file_rotation='--skip:log-file-rotation' not in opts
    )
    config.config.log_file = '/var/log/migrate.log'

    __useOption('--source-id', 'sourceid')
    __useOption('--target-id', 'targetid')
    __useOption('--config', 'configfile', "migrate.cfg")
    __useOption('--subscription-id', 'subscriptionid')
    __useOption('--aspect', 'aspect')
    __useOption('--checkpoint-file', 'checkpointfile')
    __useOption('--replace-abs-paths-policy', 'replaceabspathspolicy')
    __useOption('--skip-replace-aps-path', 'skipreplaceapspath')
    __useOption('--skip-backups', 'skipbackups')
    __useOption('--skip:shutdown-host', 'skip_shutdownhost', False)
    __useOption('--skip:check-prerequisites', 'skip_checkprerequisites', False)
    __useOption('--skip:migration', 'skip_migration', False)
    __useOption('--skip:kill-active-sessions', 'skip_killactivesessions', False)
    __useOption('--enable', 'enable', False)
    __useOption('--subscription-ids', 'subscriptionids')
    __useOption('--service-template-id', 'servicetemplateid')
    __useOption('--service-plan-id', 'serviceplanid')
    __useOption('--subscription-period-id', 'subscriptionperiodid')
    __useOption('--webspace-id', 'webspaceid')
    __useOption('--cluster-id', 'clusterid')
    __useOption('--host-id', 'hostid')
    __useOption('--service-name', 'servicename')
    __useOption('--command', 'command')
    __useOption('--demo-mode', 'demomode', False)
    __useOption('--traceback', 'traceback')
    __useOption('--with-pba', 'withpba')
    __useOption('--results-file', 'resultsfile')

    uLogging.init(config.config)
    uSysDB.init(config.config)
    openapi.initFromEnv(config.config)

    pba_integrated = False

    try:
        pbaapi.initFromEnv()
        if not pbaapi._settings.username:
            pbaapi._settings.username = 'admin'

            con = uSysDB.connect()
            cur = con.cursor()
            cur.execute("SELECT value FROM bmbridge_settings WHERE name = 'bm.common.password'")
            rows = cur.fetchall()
            decrypted_password = rows[0][0]
            pbaapi._settings.password = uCrypt.decryptData(decrypted_password)

        pba_integrated = True
    except Exception, e:
        msg = str(e)
        if "bmbridge_settings" in msg:
            # OK, POA is not integrated with PBA
            pass
        else:
            raise

    if pba_integrated and not config.config.withpba:
        message = "This POA installation is integrated with PBA. Launch the script with '--with-pba' argument to migrate websites."
        raise Exception(message)

    # avoid user interaction in batch mode
    if config.config.batch:
        uDialog.askYesNo = mockUser
        uDialog.askVariants = mockAskVariants
        uAction.retriable = mockRetriable


def mockUser(question):
    # consider 'yes' answer to every question in batch mode
    uLogging.info(question)
    uLogging.info("Yes")
    return True


def mockAskVariants(question, variants=None):
    # consider 'all' answer to every question with variants in batch mode
    uLogging.info(question)
    uLogging.info("A")
    return "A"


def mockRetriable(fun, raise_on_ignore=False, allow_console=None):
    # just run it without retries
    def fn(*args, **kwargs):
        try:
            return fun(*args, **kwargs)
        except Exception, e:
            uLogging.err("Retriable raised error in batch mode: %s", e)
            raise

    return fn


class Migration:

    def initCheckpoint(self, fromHost, toHost, subs, migrationConfig):
        self.checkpoint = None

        def equalSubLists(subs1, subs2):
            # convert SubscriptionSets into lists if necessary
            if type(subs1) != 'list':
                subs1 = subs1.getSubs()
                subs2 = subs2.getSubs()

            subs1 = [sub.subID for sub in subs1]
            subs2 = [sub.subID for sub in subs2]

            diff = sets.Set(subs1) - sets.Set(subs2)

            return 0 == len(diff)

        # load checkpoint
        checkpoint = None
        try:
            if os.path.exists(config.config.checkpointfile):
                checkpoint = Checkpoint.load(config.config.checkpointfile)
        except Exception, e:
            answer = uDialog.askYesNo(
                "Failed to load checkpoint file %s: %s. Do you want to continue?" % (config.config.checkpointfile, e))
            if not answer:
                raise MigrationError("Aborted")

        if checkpoint:
            # if we start migration from checkpoint AFTER_COMMIT, there won't be any
            # subs, because they were already bound to target
            if ((not fromHost and not checkpoint.hostID) or (fromHost and checkpoint.hostID == fromHost.hostID)) and checkpoint.clusterID == toHost.hostID and ((checkpoint.point > CHECKPOINT_BEFORE_COMMIT and (not fromHost or (type(subs) == 'list' and 0 == len(subs)) or (isinstance(subs, SubscriptionSet) and 0 == len(subs.getSubs())))) or (checkpoint.point <= CHECKPOINT_BEFORE_COMMIT and equalSubLists(checkpoint.subs, subs))):
                answer = uDialog.askYesNo("There is a file %s that contains information about the objects to migrate and the migration progress. It is likely that this file was created the last time you ran the migration tool. Do you want to use this file? Type Y if you want to use it, or type N to create a new file." %
                    config.config.checkpointfile
                )
                if answer:
                    self.checkpoint = checkpoint
                else:
                    if not subs or not subs.getSubs():
                        raise PrerequisiteError(
                            "%s does not provide hosting services for any subscriptions. There is nothing to migrate." %
                            fromHost
                        )
            else:
                if checkpoint.hostID:
                    details = "subscriptions %s from host %s to web cluster/server %s" % (
                        ", ".join([str(sub.subID) for sub in checkpoint.subs.getSubs()]), checkpoint.hostID, checkpoint.clusterID)
                else:
                    details = "subscriptions %s to web cluster/server %s" % (
                        ", ".join([str(sub.subID) for sub in checkpoint.subs.getSubs()]), checkpoint.clusterID)

                answer = uDialog.askYesNo("Checkpoint file '%s' is valid, but contains data for another migration (%s). Proceeding migration will overwrite it, do you want to continue?" % (
                    config.config.checkpointfile, details))
                if not answer:
                    raise MigrationError("Aborted")

        if not self.checkpoint:
            self.checkpoint = Checkpoint(fromHost, toHost, subs, migrationConfig)
            Checkpoint.save(CHECKPOINT_INITIAL, self.checkpoint)
            return fromHost, toHost, subs, migrationConfig
        else:
            if self.checkpoint.hostID:
                return Host(self.checkpoint.hostID), WebCluster(self.checkpoint.clusterID), self.checkpoint.subs, self.checkpoint.config
            else:
                return None, WebCluster(self.checkpoint.clusterID), self.checkpoint.subs, self.checkpoint.config

    def checkPrerequisites(self, toHost, subs, migrationConfig):
        # Check prerequisites
        uLogging.debug("Checkpoint: %s", self.checkpoint.__dict__)
        uLogging.info("Checking prerequisites for migration of subscriptions to %s", toHost)
        checkList = []
        checkList.extend(self.subscriptionsPrecheck(subs, migrationConfig))
        if not config.config.skip_incomplete_tasks:
            checkList.extend(self.tasksPrecheck())
        checkList.extend(self.sourceHostsPrecheck(subs))
        checkList.extend(self.connectionsPrecheck(toHost, subs))
        checkList.extend(self.configPrecheck(subs, migrationConfig))
        checkList.extend(self.targetHostPrecheck(toHost, subs, migrationConfig))
        if not config.config.skipbackups:
            checkList.extend(self.diskspaceBackupPrecheck(subs))

        return checkList

    def subscriptionsPrecheck(self, subs, migrationConfig):
        # Checks:
        # 1) whether there are subscriptions to be migrated.
        # 2) whether it is possible to change the subscriptions' plans from Linux Shared Hosting to WebHosting Linux
        uLogging.info("Checks whether there are subscriptions to be migrated and whether it is possible to change the subscriptions' plans from Linux Shared Hosting to WebHosting Linux.")
        if self.checkpoint.point >= CHECKPOINT_AFTER_COMMIT:
            return []

        if not subs.getSubs():
            raise PrerequisiteError("There are no subscriptions on the source server. There is nothing to migrate.")

        if config.config.withpba:
            for sub in subs.getSubs():
                if not sub.isSwitchPlanAllowed():
                    raise PrerequisiteError(
                        "Unable to switch the plan for the subscription %s. Make sure that there are no incomplete orders in billing and the subscription is active." %
                        sub.id
                    )

                servicePlanID, subscriptionPeriodID = sub.getServicePlanInfo()
                targetServicePlanID, _ = getTargetServicePlanInfo(servicePlanID, subscriptionPeriodID, migrationConfig)
                allowedPlanIds = [x[0] for x in sub.getAllowedPlans()]
                if targetServicePlanID not in allowedPlanIds:
                    raise PrerequisiteError(
                        "Unable to upgrade the subscription %s from the plan %s to the plan %s. Switch on the option to upgrade to this plan in billing settings." %
                        (sub.id, servicePlanID, targetServicePlanID)
                    )

        for sub in subs.getSubs():
            apps = sub.getUnmigrateableSaasApplications()
            if not apps:
                continue

            raise PrerequisiteError(
                "The following SaaS applications will be deployed on the provider's domains: %s. Service templates of such subscriptions cannot be changed during migration."
                % ", ".join([x[2] for x in apps])
            )

        return [
            "whether there are subscriptions to be migrated",
            "whether it is possible to change the subscriptions' plans from Linux Shared Hosting to WebHosting Linux",
            "a list of SaaS applications installed on a provider's domain.",
        ]

    def sourceHostsPrecheck(self, subs):
        # Checks whether all the services required by subscriptions are running on the source servers.
        uLogging.info("Checks whether the required services are running on the server.")
        if self.checkpoint.point >= CHECKPOINT_AFTER_COMMIT:
            return []

        fromHosts = subs.getFromHosts()
        for fromHost in fromHosts:
            fromHost.checkServicePrerequisites()

        return [
            "whether all the services required by subscriptions are running on the source servers",
        ]

    def connectionsPrecheck(self, toHost, subs):
        # Checks whether it is possible to establish SSH connection without a password.
        uLogging.info("Checks the connection between the source and the target servers.")
        if self.checkpoint.point >= CHECKPOINT_AFTER_COMMIT:
            return []

        fromHosts = subs.getFromHosts()
        for fromHost in fromHosts:
            self.checkSSHAccess(fromHost, toHost)

        return [
            "whether it is possible to establish SSH connection without a password",
        ]

    def tasksPrecheck(self):
        # Checks whether there are incompleted tasks that must be completed before starting migration.
        uLogging.info("Checks whether there are incompleted tasks that must be completed before starting migration.")
        if self.checkpoint.point >= CHECKPOINT_AFTER_COMMIT:
            return []

        self.checkTasks()
        return [
            "whether there are incompleted tasks that must be completed before starting migration",
        ]

    def configPrecheck(self, subs, migrationConfig):
        # Checks:
        # 1) whether the proper mapping of service plans is specified in the configuration file.
        # 2) whether the mapping of resources is specified.
        # 3) whether there are enough resources on the target servers.
        uLogging.info("Checks whether the proper mapping of service plans is specified in the configuration file; whether the resources of the source plan can be mapped to the resources of the target plan and whether there are enough resources on the target servers.")
        if self.checkpoint.point >= CHECKPOINT_AFTER_COMMIT:
            return []

        self.checkConfig(migrationConfig, subs.getSubs())
        return [
            "whether the proper mapping of service plans is specified in the configuration file",
            "whether the resources of the source plan can be mapped to the resources of the target plan",
            "whether there are enough resources on the target servers",
        ]

    def targetHostPrecheck(self, toHost, subs, migrationConfig):
        # Checks:
        # 1) if subscriptions from a host can be provisioned on a web cluster or server.
        # 2) target service templates.
        # 3) if there is at least one website with FTP access.
        # 4) whether there is at least one parked domain.
        # 5) if destination host can provision such subscriptions.
        uLogging.info("Checks if subscriptions from a host can be provisioned on a web cluster or server; if there is at least one website with FTP access; and if there is at least one parked domain.")
        if self.checkpoint.point >= CHECKPOINT_AFTER_COMMIT:
            return []

        toHost.checkConfigPrerequisites(subs, migrationConfig)
        return [
            "if subscriptions from a host can be provisioned on a web cluster or server",
            "if there is at least one website with FTP access",
            "if there is at least one parked domain",
        ]

    def diskspaceBackupPrecheck(self, subs):
        # Check:
        # 1) if there is enough available disk space at backup node to unpack backup and save original copy of backup
        uLogging.info("Check if there is enough disk space at backup nodes")
        webspaceIDs = [webspace.id for sub in subs.getSubs() for webspace in sub.webspaces]

        cur = uSysDB.connect().cursor()
        cur.execute(
            """
            SELECT bg.host_id, bg.store_dir, sum(bb.size)
            FROM backups_groups bg
            JOIN backups_backups bb ON bg.group_id=bb.group_id
            JOIN domain_services ds on ds.ds_id = bb.object_id
            WHERE ds.hd_id in %s
            GROUP BY bg.host_id, bg.store_dir
            """, (tuple(webspaceIDs),)
        )
        for hostID, storeDir, size in cur.fetchall():
            host = HostFactory.createHost(hostID)
            if host.is_win:
                continue

            disk_space = host.get_free_disk_space(storeDir)
            # we require disk space with margin 10% to avoid unsufficient disk space
            # if other process will use disk together with migrator
            size = int(size) * 1.1
            if disk_space <= size:
                raise PrerequisiteError(
                    "The amount of free disk space on the node #%s, in the %s directory, is %iKB. Required: %iKB. Free up some disk space and try again" %
                    (hostID, storeDir, disk_space, size)
                )

        return ["if there is enough disk space at backup nodes"]

    def migrateSubscriptions(self, subs, toHost, migrationConfig):
        if subs.hasProviderSubscription():
            raise MigrationError("Provider's subscription #1 cannot be migrated.")

        fromHost, toHost, subs, migrationConfig = self.initCheckpoint(None, toHost, subs, migrationConfig)

        self.checkPrerequisites(toHost, subs, migrationConfig)

        uLogging.info("Migrating subscriptions [%s] to %s", ", ".join(
            [str(sub.subID) for sub in subs.getSubs()]), toHost)

        if self.checkpoint.point < CHECKPOINT_BEFORE_COMMIT:
            # must unprovide services before locking subscription, because POA will
            # fail to unprovide locked service (at least anonymous FTP)
            for sub in subs.getSubs():
                beautyRetriable(sub.unprovideUnsupportedServices)()

            # lock access to webspaces
            self.lock(subs.getSubs(), True)

            # migrate content
            subs.syncContent(toHost)

            Checkpoint.save(CHECKPOINT_BEFORE_COMMIT, self.checkpoint)

        if config.config.demomode:
            pause()

        # migrate configuration data in DB
        # do not modify DB, if it was already modified
        if self.checkpoint.point < CHECKPOINT_AFTER_COMMIT:
            # !!> sites become unavailable

            uLogging.info("Sites become unavailable")

            # stop activity on host

            # TODO how to delete crontabs for selected subscriptions?
            # TODO taskCleanupDS
            # fromHost.deleteCrontabs()

            # TODO is it bad, that Apache still be serving websites?
            # beautyRetriable(fromHost.serviceCtl)("stop", "pemhttpd")

            beautyRetriable(subs.switchToSharedIPType)()

            con = uSysDB.connect()
            cur = con.cursor()

            self.migrateSubscriptionsDBData(con, toHost, subs, False)

            if config.config.demomode:
                pause()

            Checkpoint.save(CHECKPOINT_AFTER_COMMIT, self.checkpoint)
            con.commit()

        if self.checkpoint.point < CHECKPOINT_AFTER_RESTORE_IP_TYPES:
            if config.config.demomode:
                pause()

            self.syncIPResolver()
            # unlock access to webspaces

            # need to leave migrating state before syncState()
            # because OpenAPI method setMigratingState doesn't propagate state changes
            # to cluster configuration databases
            self.lock(subs.getSubs(), False)

            # sync cluster
            beautyRetriable(toHost.syncState)(subs.getSubs())
            beautyRetriable(toHost.setUpIPs)()

            uLogging.info("Migrated sites are becoming available on %s", toHost)
            if config.config.demomode:
                pause()

            # !!> sites become available on new host

            # switch / renew IP types, so that POA builds proper references between objects (ips and hostings)
            # it's critical to do this before change of service template, because it might want to find IPs using references
            subs.restoreWebspaceIPTypes()
            Checkpoint.save(CHECKPOINT_AFTER_RESTORE_IP_TYPES, self.checkpoint)

        # change service templates
        self.changeServiceTemplates(subs.getSubs(), migrationConfig, False)

        # perform second step of content synchronization
        subs.syncContent(toHost)
        subs.finalizeContent(toHost)
        self.updateLimits(subs.getSubs())

        con = uSysDB.connect()
        cur = con.cursor()

        # after service template change POA resets properties of webspace
        # (cgi, ssi, ssl..) so we restore them to previous values
        subs.restoreWebspaceProperties(cur)

        # clean up db
        subs.cleanUpDB(con, cur, toHost)

        con.commit()

        beautyRetriable(toHost.syncState)(subs.getSubs())

        # crontabs need to be dump after sync, because this operation requires access to users on host
        subs.dumpCrontabs()

        # update new logparser after previous one is definitely stopped
        toHost.updateLogParser()

        # update logrotate
        toHost.dumpLogrotate()

        # create local PEAR installations
        # subs.createLocalPear()
        subs.syncHtAccess()

        if not config.config.skipreplaceapspath:
            subs.checkContentForAbsPaths(toHost)

        if not config.config.skipbackups:
            subs.migrateBackups(toHost)

        if config.config.demomode:
            pause()

    def revertSubscriptions(self, checkpoint):
        self.checkpoint = checkpoint

        if checkpoint.point < CHECKPOINT_INITIAL or CHECKPOINT_AFTER_RESTORE_IP_TYPES < checkpoint.point:
            raise MigrationError("Invalid checkpoint: %s" % checkpoint.point)

        # TODO we should remove content from cluster

        # construct objects from checkpoint data
        subs = checkpoint.subs
        cluster = WebCluster(checkpoint.clusterID)

        uLogging.info("Reverting subscriptions %s from %s", subs.getSubs(), cluster)

        revertConfig = config.RevertConfig(subs.getSubs(), checkpoint.config)

        self.checkConfig(revertConfig, subs.getSubs())

        # lock subscriptions
        self.lockAndKill(cluster, subs.getSubs(), True)

        if config.config.demomode:
            pause()

        if self.checkpoint.point >= CHECKPOINT_AFTER_COMMIT:
            beautyRetriable(subs.switchToSharedIPType)()

            con = uSysDB.connect()
            # revert database changes
            self.migrateSubscriptionsDBData(con, cluster, subs, True)

            Checkpoint.save(CHECKPOINT_BEFORE_COMMIT, checkpoint)
            con.commit()

            if config.config.demomode:
                pause()

        self.syncIPResolver()
        # sync cluster
        cluster.syncState(subs.getSubs())
        cluster.setUpIPs()

        if config.config.demomode:
            pause()

        # remove crontabs of host from cluster
#		try:
#			cluster.deleteCrontabs(host.getServiceID("crontabs"))
#		except MigrationError, e:
#			uLogging.info("Cannot remove crontabs from %s: %s" % (cluster, e))

        # restore crontabs
        subs.dumpCrontabs()

        # ensure, that logparser configuration is up to date on both hosts
        cluster.updateLogParser()
        subs.updateLogParser()

        # update logrotate
        cluster.dumpLogrotate()

        # unlock subscriptions
        self.lock(subs.getSubs(), False)

        # must be done after unlock, it simply doesn't restore authentication configs if done otherwise
        # and before change subscription, because it might want to find IPs by
        # references (which are built using ip_type_renew)
        subs.restoreWebspaceIPTypes()

        # change service templates to previous ones
        self.changeServiceTemplates(subs.getSubs(), revertConfig, True)

        # after service template change POA resets properties of webspace
        # (cgi, ssi, ssl..) so we restore them to previous values
        con = uSysDB.connect()
        cur = con.cursor()
        subs.restoreWebspaceProperties(cur)
        con.commit()

    def migrateHost(self, fromHost, toHost, migrationConfig):
        if not config.config.skip_checkprerequisites:
            fromHost.checkHostPrerequisites(toHost)

        subs = fromHost.getSubscriptionSet()

        if subs.hasProviderSubscription():
            raise MigrationError(
                "%s provides hosting for webspaces from provider's subscription #1. They cannot be migrated." % fromHost)

        fromHost, toHost, subs, migrationConfig = self.initCheckpoint(fromHost, toHost, subs, migrationConfig)

        self.checkPrerequisites(toHost, subs, migrationConfig)

        if config.config.skip_migration:
            return

        if config.config.demomode:
            pause()

        uLogging.info("Migrating subscriptions [%s] from %s to %s", ", ".join(
            [str(sub.subID) for sub in subs.getSubs()]), fromHost, toHost)

        if self.checkpoint.point < CHECKPOINT_BEFORE_COMMIT:
            # must unprovide services before locking subscription, because POA will
            # fail to unprovide locked service (at least anonymous FTP)
            for sub in subs.getSubs():
                beautyRetriable(sub.unprovideUnsupportedServices)()

            # lock access to webspaces
            self.lockAndKill(fromHost, subs.getSubs(), True)

            # migrate content
            beautyRetriable(subs.syncContent)(toHost)

            Checkpoint.save(CHECKPOINT_BEFORE_COMMIT, self.checkpoint)

            if config.config.demomode:
                pause()

        # migrate configuration data in DB
        # do not modify DB, if it was already modified
        if self.checkpoint.point < CHECKPOINT_AFTER_COMMIT:
            # !!> sites become unavailable

            uLogging.info("Sites become unavailable on %s", fromHost)

            # stop activity on host
            fromHost.deleteCrontabs()
            beautyRetriable(fromHost.serviceCtl)("stop", "pemhttpd")

            con = uSysDB.connect()
            self.migrateDBData(con, fromHost, toHost, subs, False)

            if config.config.demomode:
                pause()

            # perform second step of content synchronization
            beautyRetriable(subs.syncContent)(toHost)
            beautyRetriable(subs.finalizeContent)(toHost)

            Checkpoint.save(CHECKPOINT_AFTER_COMMIT, self.checkpoint)
            con.commit()

            if config.config.demomode:
                pause()

        if self.checkpoint.point < CHECKPOINT_HOST_SHUTDOWN:
            if not config.config.skip_shutdownhost:
                beautyRetriable(fromHost.shutdown)()

            Checkpoint.save(CHECKPOINT_HOST_SHUTDOWN, self.checkpoint)

            if config.config.demomode:
                pause()

        self.syncIPResolver()
        # unlock access to webspaces

        # need to leave migrating state before syncState()
        # because OpenAPI method setMigratingState doesn't propagate state changes
        # to cluster configuration databases
        self.lockAndKill(toHost, subs.getSubs(), False)

        # sync cluster

        beautyRetriable(toHost.syncState)(subs.getSubs())
        beautyRetriable(toHost.setUpIPs)()

        uLogging.info("Migrated sites become available on %s", toHost)
        if config.config.demomode:
            pause()

        # !!> sites become available on new host

        # change service templates
        self.changeServiceTemplates(subs.getSubs(), migrationConfig, False)
        self.updateLimits(subs.getSubs())

        # after service template change POA resets properties of webspace
        # (cgi, ssi, ssl..) so we restore them to previous values
        con = uSysDB.connect()
        cur = con.cursor()
        subs.restoreWebspaceProperties(cur)
        con.commit()

        beautyRetriable(toHost.syncState)(subs.getSubs())

        # crontabs need to be dump after sync, because this operation requires access to users on host
        subs.dumpCrontabs()

        # update new logparser after previous one is definitely stopped
        toHost.updateLogParser()

        # update logrotate
        toHost.dumpLogrotate()

        # subs.createLocalPear()
        subs.syncHtAccess()
        subs.checkContentForAbsPaths(toHost)

        if config.config.demomode:
            pause()

    def revertHost(self, checkpoint):
        self.checkpoint = checkpoint

        if checkpoint.point < CHECKPOINT_INITIAL or CHECKPOINT_HOST_SHUTDOWN < checkpoint.point:
            raise MigrationError("Invalid checkpoint: %s" % checkpoint.point)

        # TODO we should remove content from cluster

        # construct objects from checkpoint data
        subs = checkpoint.subs
        host = Host(checkpoint.hostID)
        cluster = WebCluster(checkpoint.clusterID)

        uLogging.info("Reverting subscriptions %s from %s to %s", subs.getSubs(), cluster, host)

        revertConfig = config.RevertConfig(subs.getSubs(), checkpoint.config)

        self.checkConfig(revertConfig, subs.getSubs())

        # lock subscriptions
        self.lockAndKill(cluster, subs.getSubs(), True)

        if config.config.demomode:
            pause()

        if self.checkpoint.point >= CHECKPOINT_AFTER_COMMIT:
            con = uSysDB.connect()
            # revert database changes
            self.migrateDBData(con, host, cluster, subs, True)

            Checkpoint.save(CHECKPOINT_BEFORE_COMMIT, checkpoint)
            con.commit()

            if config.config.demomode:
                pause()

        self.syncIPResolver()
        # sync cluster
        beautyRetriable(cluster.syncState)(subs.getSubs())
        beautyRetriable(cluster.setUpIPs)()

        if config.config.demomode:
            pause()

        # ensure that previous host is up
        uLogging.info("Checking uptime of %s. You need to start it if it was shut down.", host)
        beautyRetriable(host.uptime)()

        # ensure that apache is running
        host.serviceCtl("start", "pemhttpd")

        # remove crontabs of host from cluster
        try:
            cluster.deleteCrontabs(host.getServiceID("crontabs"))
        except MigrationError, e:
            uLogging.info("Cannot remove crontabs from %s: %s", cluster, e)
        # restore crontabs
        subs.dumpCrontabs()

        # ensure, that logparser configuration is up to date on both hosts
        cluster.updateLogParser()
        host.updateLogParser()

        # update logrotate
        cluster.dumpLogrotate()

        # change service templates to previous ones
        self.changeServiceTemplates(subs.getSubs(), revertConfig, True)

        # after service template change POA resets properties of webspace
        # (cgi, ssi, ssl..) so we restore them to previous values
        con = uSysDB.connect()
        cur = con.cursor()
        subs.restoreWebspaceProperties(cur)
        con.commit()

        # unlock subscriptions
        self.lockAndKill(host, subs.getSubs(), False)

    def checkTasks(self):
        if countChangeServiceTemplateTasks() != 0:
            raise PrerequisiteError(
                "Before proceeding with migration, wait until the tasks 'Dump config image for domain service*' and 'Restart the Apache service*' are carried out."
            )

    def checkSSHAccess(self, fromHost, toHost):
        try:
            return fromHost.cmd(
                "/usr/bin/ssh",
                args=[
                    "ssh", "-o", "StrictHostKeyChecking=no", "-o", "CheckHostIP=no",
                    "-o", "UserKnownHostsFile=/dev/null", "root@%s" % toHost.getBacknetIP(), "uptime"
                ]
            )
        except uUtil.ExecFailed, e:
            uLogging.debug(
                "Could not establish SSH connection without a password from %s to the director of %s: %s",
                fromHost, toHost, e
            )
            raise PrerequisiteError(
                "Could not establish SSH connection without a password from %s to the director of %s." %
                (fromHost, toHost)
            )

    def checkConfig(self, migrationConfig, subs):
        if not config.config.withpba:
            self.checkMapping(migrationConfig.items(), "service template", ServiceTemplate, subs)
        else:
            self.checkMapping(migrationConfig.items(), "service plan", ServicePlan, subs)

    def checkMapping(self, mapping, objectName, ObjectClass, subs):
        mappingObjects = {}
        for key, value in mapping:
            # check, that source service templates exist
            if isinstance(key, tuple):
                source = ObjectClass(key[0])
            else:
                source = ObjectClass(key)

            if not source.exists():
                raise PrerequisiteError("%s doesn't exist." % str(source).capitalize())

            # check, that destination service templates exist and are active
            if isinstance(value, tuple):
                destination = ObjectClass(value[0])
            else:
                destination = ObjectClass(value)

            if not destination.exists() or not destination.isActive():
                raise PrerequisiteError("%s doesn't exist or is not active." % str(destination).capitalize())

            source.checkCompatibility(destination)

            if source in mappingObjects.keys():
                if mappingObjects[source].id != destination.id:
                    raise PrerequisiteError(
                        "The mapping for the %s '%s' is configured incorrectly in the migration configuration file. It points at two different target %s: '%s' and '%s'. Remove the unnecessary mapping entry." %
                        (objectName, source.id, objectName, destination.id, mappingObjects[source].id)
                    )
            else:
                mappingObjects[source] = destination

        uLogging.info("%s mapping:", objectName)
        for source, destination in mappingObjects.items():
            # TODO print destination subscription period here too
            uLogging.info("%s(%s) -> %s(%s)", source.name, source.id, destination.name, destination.id)

            if ServiceTemplate == ObjectClass:
                resource_mapping = self.buildMappingResourceTable(source, destination)
                self.showResourceMappingTable(source, destination, resource_mapping)
                self.checkResourceMapping(subs, resource_mapping)
                self.checkResourceUsage(subs, resource_mapping, objectName, source, destination)
            elif ServicePlan == ObjectClass:
                api = pbaapi.PBAAPI()
                mapping = api.GetResourceMapping(source.serviceTemplate.id, 0)
                mapping = [(m.FromID, m.ToID) for m in mapping]
                resource_mapping = self.buildMappingResourceTable(
                    source.serviceTemplate, destination.serviceTemplate, mapping)
                self.showResourceMappingTable(source.serviceTemplate, destination.serviceTemplate, resource_mapping)
                self.checkResourceMapping(subs, resource_mapping)
                self.checkResourceUsage(subs, resource_mapping, objectName, source, destination)
            else:
                uLogging.warn("Invalid object type: %s", type(source))

        uLogging.warn(
            "Any existing 'MS FrontPage', 'FTP Anonymous access', 'AWStats Web Statistics', 'Urchin Web Statistics Service' and legacy WebApps will be unprovisioned during migration")

        answer = uDialog.askYesNo(
            "Are you sure this mapping is suitable and source/destination %ss are compatible?" % objectName)
        if not answer:
            raise PrerequisiteError("Aborted.")

    def checkResourceMapping(self, subs, resource_mapping):
        apps_classes = ["siteapps", "rc.saas.application"]
        apps_without_mapping = []

        rc_map = {}

        for src_rt_id, src_rt_name, rt_class_id, rt_class_name, dst_rt_name, dst_rt_id, state in resource_mapping:
            # collect wrongly mapped APS app resources
            if rt_class_name in apps_classes and "+" != state:
                apps_without_mapping.append((src_rt_id, src_rt_name, rt_class_id))

            resource_map = rc_map.get(rt_class_id, [])
            resource_map.append((src_rt_id, dst_rt_id))
            rc_map[rt_class_id] = resource_map

        if apps_without_mapping:
            uLogging.info(
                "Following resource types have classes of APS applications and don't have matching resource types in destination service template:")
            for res in apps_without_mapping:
                uLogging.info("%s", res)

            answer = uDialog.askYesNo(
                "Proceeding with migration will cause unprovisioning of installed applications, customer data loss and possible errors during migration. Do you want to continue?")
            if not answer:
                raise PrerequisiteError("Aborted")

        for sub in subs:
            for rt_class_id, rt_map in rc_map.iteritems():
                mapped_resources = filter(
                    lambda rt: rt[0] != RESOURCE_STATE_NO_MAPPED_PAIR and rt[1] != RESOURCE_STATE_NO_MAPPED_PAIR,
                    rt_map
                )
                dst_to_src_map = {}
                for rt in mapped_resources:
                    rts = dst_to_src_map.get(rt[1], [])
                    rts.append(rt[0])
                    dst_to_src_map[rt[1]] = rts

                for src_rts in dst_to_src_map.itervalues():
                    if len(src_rts) > 1:
                        rts_with_usage = []
                        for rt_id, usage, restype_name, providable in sub.getResourcesTypes(src_rts):
                            if usage > 0:
                                rts_with_usage.append((rt_id, restype_name, rt_class_id))

                        if len(rts_with_usage) > 1:
                            message = "The following resources from the %s must be unprovided or merged prior to migration:" % sub
                            for rt in rts_with_usage:
                                message += "\n%s" % str(rt)

                            raise PrerequisiteError(message)

                unmapped_resources = map(lambda rt: rt[0],
                                         filter(lambda rt: rt[1] == RESOURCE_STATE_NO_MAPPED_PAIR, rt_map))
                if not unmapped_resources:
                    continue

                mapped_resources = map(lambda rt: rt[0],
                                       filter(lambda rt: rt[1] != RESOURCE_STATE_NO_MAPPED_PAIR, rt_map))
                if not mapped_resources:
                    uLogging.info(
                        "The following resource types are not mapped to destination service template and will be unprovided from %s during migration:", sub)
                    for rt_id, usage, restype_name, providable in sub.getResourcesTypes(unmapped_resources):
                        rt = (rt_id, restype_name, rt_class_id)
                        uLogging.info(rt)

                    answer = uDialog.askYesNo("Do you want to continue?")
                    if not answer:
                        raise PrerequisiteError("Aborted")
                else:
                    # partially mapped
                    for rt_id, usage, restype_name, providable in sub.getResourcesTypes(unmapped_resources):
                        if usage > 0:
                            rt = (rt_id, restype_name, rt_class_id)
                            message = "The following resources from the %s must be unprovided or merged prior to migration:" % sub
                            message += "\n%s" % str(rt)
                            for rt_id, usage, restype_name, providable in sub.getResourcesTypes(mapped_resources):
                                rt = (rt_id, restype_name, rt_class_id)
                                message += "\n%s" % str(rt)

                            raise PrerequisiteError(message)

    def checkResourceUsage(self, subs, resource_mapping, objectName, source, destination):
        con = uSysDB.connect()
        cur = con.cursor()
        rt_id_mapping = {}
        if config.config.withpba:
            source_st = source.serviceTemplate
            destination_st = destination.serviceTemplate
        else:
            source_st = source
            destination_st = destination

        for src_rt_id, src_rt_name, rt_class_id, rt_class_name, dst_rt_name, dst_rt_id, state in resource_mapping:
            if isNumber(src_rt_id):
                rt_id_mapping[src_rt_id] = dst_rt_id

        for sub in subs:
            if sub.getServiceTemplateID() != source_st.id:
                continue

            for rt_id, usage, restype_name, providable in sub.getResourcesTypes(rt_id_mapping.keys()):
                dst_rt_id = rt_id_mapping[rt_id]
                if dst_rt_id == RESOURCE_STATE_NO_MAPPED_PAIR:
                    raise PrerequisiteError(
                        "The subscription '%s' has used '%s' of the resource '%s', although the %s '%s' does not provide a similar resource." % (
                            sub.id, usage, rt_id, objectName, destination.id
                        )
                    )

                dst_limit = destination_st.getResourceLimitById(cur, dst_rt_id)
                if dst_limit != -1 and dst_limit < usage and not config.config.skip_resource_usage_check:
                    raise PrerequisiteError(
                        "The subscription '%s' has used '%s' of the resource '%s', although the %s '%s' provides only '%s'. To ignore this issue, use the option '--skip-resource-usage-check'." % (
                            sub.id, usage, rt_id, objectName, destination.id, dst_limit
                        )
                    )

    def buildMappingResourceTable(self, first, second, userDefinedMapping=[]):
        resources1 = first.getResources()
        resources2 = second.getResources()

        used = []
        resource_map_table = []

        for res1 in resources1:
            res1_rt_id, res1_rt_name, res1_rc_id, res1_rc_name = res1
            mapped = None

            # try to find mapping with same id and class
            for res2 in resources2:
                res2_rt_id, res2_rt_name, res2_rc_id, res2_rc_name = res2
                if res1_rt_id == res2_rt_id:
                    if res1_rc_id == res2_rc_id:
                        mapped, state = res2, "+"
                    else:
                        mapped, state = res2, "?"

            # try to find mapping with using user mapping
            if not mapped:
                for res2 in [res for res in resources2 if res not in used]:
                    res2_rt_id, res2_rt_name, res2_rc_id, res2_rc_name = res2
                    for mapping in userDefinedMapping:
                        if res1_rt_id == mapping[0] and res2_rt_id == mapping[1]:
                            if res1_rc_id == res2_rc_id:
                                mapped, state = res2, "+m"
                            else:
                                mapped, state = res2, "?m"

            # match with first available from same class
            if not mapped:
                for res2 in [res for res in resources2 if res not in used]:
                    res2_rt_id, res2_rt_name, res2_rc_id, res2_rc_name = res2
                    if res1_rc_id == res2_rc_id:
                        mapped, state = res2, "!"

            if not mapped:
                mapped, state = [RESOURCE_STATE_NO_MAPPED_PAIR, "", ""], " "
            else:
                used.append(mapped)

            resource_map_table.append((res1_rt_id, res1_rt_name, res1_rc_id, res1_rc_name, mapped[1], mapped[0], state))

        # show left resources from destination template
        for res2 in [res for res in resources2 if res not in used]:
            mapped, state = [RESOURCE_STATE_NO_MAPPED_PAIR, "", ""], " "
            resource_map_table.append((mapped[0], mapped[1], res2[2], res2[3], res2[1], res2[0], state))

        return resource_map_table

    def showResourceMappingTable(self, first, second, resource_mapping):
        # shows table of resource mapping between first and second service templates
        # mapping - can be user defined resource mapping (dict from {first resource}->{second resource})
        # id1, name1, class, name2, id2, mapping_state

        uLogging.info("Resource mapping between %s and %s", first, second)
        uLogging.info("State: ")
        uLogging.info("+  - resources from the same class with same IDs")
        uLogging.info("+m - resources from the same class with user-defined resource mapping in PBA")
        if config.config.withpba:
            uLogging.info(
                "!  - resources from the same class with different IDs (possible PBA duplication after migration) ")
        else:
            uLogging.info("!  - resources from the same class with different IDs")
        uLogging.info("?  - resources from different classes with same ID")
        uLogging.info("?m - resources from different classes with user-defined resource mapping in PBA")
        uLogging.info("   - resources from different classes")

        rowFormat = "%s\t%30s\t%s\t%30s\t%s\t%s"
        uLogging.info(rowFormat, "ID 1", "Name 1", "Class", "Name 2", "ID 2", "State")

        for row in resource_mapping:
            src_rt_id, src_rt_name, src_rt_rc_id, rc_name, dst_rt_name, dst_rt_id, state = row
            uLogging.info(rowFormat, src_rt_id, src_rt_name, src_rt_rc_id, dst_rt_name, dst_rt_id, state)

    def updateLimits(self, subs):
        uLogging.info("Update website limits of subscriptions, that were not upgraded to new service templates")
        api = openapi.OpenAPI()

        def func(batch):
            request = api.beginRequest()
            scheduled = []
            for sub in batch:
                # for each subscription, that was not upgraded during migration to another service template
                # upgrade it to current service template (so that new activation parameters get propagated)
                if config.config.withpba:
                    doIt = sub.initialServicePlanID == sub.getServicePlanID()
                else:
                    doIt = sub.initialServiceTemplateID == sub.getServiceTemplateID()

                if doIt:
                    beautyRetriable(sub.updateLimits)(api)
                    scheduled.append(sub)

            api.commit()
            if scheduled:
                openapi.waitRequestComplete(
                    request, "Updating website limits (Check running or unprocessed tasks in Task Manager)", True
                )

        batchExecute(subs, func)

    def changeServiceTemplates(self, subs, cfg, revert=False):
        if not config.config.withpba:
            uLogging.info("Upgrading subscriptions to service templates")

            api = openapi.OpenAPI()

            index = 0
            step = 100
            while index < len(subs):
                part = subs[index:index + step]
                uLogging.info("Batch %s\%s" % (index / step + 1, len(subs) / step + 1))

                scheduled = []
                for sub in part:
                    if sub.getServiceTemplateID() in cfg.values():
                        uLogging.info(
                            "Subscription %s is already upgraded to destination service template %s", sub.subID, sub.getServiceTemplateID())
                        continue

                    if not revert:
                        dst = cfg.templateMapping[sub.getServiceTemplateID()]
                    else:
                        dst = cfg.mapping[(sub.subID, sub.getServiceTemplateID())]

                    beautyRetriable(sub.changeServiceTemplate)(dst, api)
                    scheduled.append(sub)

                index += step

        else:
            uLogging.info("Switching subscriptions service plans")

            subscriptionOrders = []
            for sub in subs:
                servicePlanID, subscriptionPeriodID = sub.getServicePlanInfo()
                if servicePlanID in cfg.values():
                    uLogging.info("Subscription %s is already upgraded to destination service template %s",
                                  sub.subID, sub.getServiceTemplateID())
                    continue

                if not revert:
                    dst = getTargetServicePlanInfo(servicePlanID, subscriptionPeriodID, cfg)
                else:
                    dst = cfg.mapping[(sub.subID, servicePlanID)]

                order_details = beautyRetriable(sub.changeServicePlan)(*dst)
                subscriptionOrders.append((sub, order_details.OrderID))

            if subscriptionOrders:
                if config.config.batch:
                    for sub, orderId in subscriptionOrders:
                        beautyRetriable(sub.waitOrderCompleted)(*(orderId, ))
                else:
                    question = """Upgrade orders to switch subscription service plans were created successfully. You should check and open them in PBA UI, migration can be continued after all orders will be completed. Do you want to continue?"""
                    answer = uDialog.askYesNo(question)
                    if not answer:
                        raise MigrationError("Aborted")

            self.checkSubscriptionsPlansAndTemplates(subs, cfg)
            waitChangeServiceTemplate()

    def checkSubscriptionsPlansAndTemplates(self, subs, cfg):
        def checkSubscriptionTemplate(sub):
            serviceTemplateID = sub.getServiceTemplateID()
            if not sub.getServiceTemplateID() in cfg.values():
                raise MigrationError("Current service template %s of %s is not one of destinations" %
                                     (serviceTemplateID, sub))

        def checkSubscriptionPlan(sub):
            servicePlanID = sub.getServicePlanID()
            if not servicePlanID in cfg.values():
                raise MigrationError("Current service plan %s of %s is not one of destinations" % (servicePlanID, sub))

            poaServiceTemplateID = sub.getServiceTemplateID()
            pbaServiceTemplateID = ServicePlan(servicePlanID).serviceTemplate.id
            if poaServiceTemplateID != pbaServiceTemplateID:
                raise MigrationError("%s service templates doesn't match in POA(%s) and PBA(%s)" %
                                     (str(sub).capitalize(), poaServiceTemplateID, pbaServiceTemplateID))

        if not config.config.withpba:
            for sub in subs:
                beautyRetriable(checkSubscriptionTemplate)(sub)
        else:
            for sub in subs:
                beautyRetriable(checkSubscriptionPlan)(sub)

    def lock(self, subs, lock):
        return self.lockAndKill(None, subs, lock)

    def lockAndKill(self, host, subs, lock):
        op = "Setting"
        if not lock:
            op = "Resetting"

        if host:
            uLogging.info("%s migrating state for subscriptions on source %s", op, host)
        else:
            uLogging.info("%s migrating state for subscriptions %s", op, subs)

        for sub in subs:
            sub.setMigratingState(lock)

        if lock and host and not config.config.skip_killactivesessions:
            # kill all active user sessions
            beautyRetriable(host.killSSHSessions)()
            beautyRetriable(host.killFTPSessions)()

    def migrateSubscriptionsDBData(self, con, toHost, subs, revert):
        cur = con.cursor()

        subs.migrateDBData(con, cur, toHost, revert)

        cur.close()

    def migrateDBData(self, con, fromHost, toHost, subs, revert):
        cur = con.cursor()

        if revert and self.checkpoint.mainProftpdVhostID:
            dstFTPService = fromHost.getServiceID("Proftpd", cur)
            # restore 'main' proftpd vhost
            cur.execute("UPDATE proftpd_vhosts SET service_id = %s, h2e = 'n' WHERE vhost_id = %s",
                        dstFTPService, self.checkpoint.mainProftpdVhostID)

        if not revert:
            fromIP = fromHost.getBacknetIP()
            toIP = fromHost.getIPs(CONFIGURED_IP_TYPE_SHARED)[0]
            type = 'p'

            subs.migrateDBData(con, cur, toHost, revert)
        else:
            subs.migrateDBData(con, cur, toHost, revert)

            fromIP = fromHost.getIPs(CONFIGURED_IP_TYPE_SHARED)[0]
            toIP = fromHost.getBacknetIP()
            type = 'i'

        if not revert:
            dstFTPService = toHost.getServiceID("Proftpd", cur)
            srcFTPService = fromHost.getServiceID("Proftpd", cur)

            try:
                mainProftpdVhostID, vhostIP = fromHost.getMainProftpdVhostID()

                # replace Proftpd 'main' vhost with 'h2e main shared' vhost
                cur.execute(
                    "UPDATE proftpd_vhosts SET service_id = %s, h2e = 'y' WHERE service_id = %s AND vhost_type = 1", dstFTPService, srcFTPService)

                self.checkpoint.mainProftpdVhostID = mainProftpdVhostID
            except Exception, e:
                uLogging.warn("Failed to find main ProFTPD vhost on source host: %s", e)
                self.checkpoint.mainProftpdVhostID = 0

        # modify IP resolver for SSL proxy
        cur.execute("UPDATE ipresolver_hosts SET comm_ip = %s, type = %s, status = 'i' WHERE comm_ip = %s",
                    normalizeIP(toIP), type, normalizeIP(fromIP))

        cur.close()

    def syncIPResolver(self):
        con = uSysDB.connect()
        cur = con.cursor()

        cur.execute("SELECT count(*) FROM service_classes WHERE name = 'InternalIPResolver'")
        if cur.fetchone():
            ref = uTasks.SCRef("SDK::IPResolver")
            task = uTasks.Task(ref, "Syncing Internal IP Resolver Database with external data source", "taskSync")
            task.schedule()

            beautyRetriable(waitTasksCompleteRef)(ref)


def main():
    # treat last argument as command
    command = sys.argv[-1]

    if 'sync-cluster' == command:
        cluster = WebCluster(config.config.clusterid)
        cluster.syncState([])
        cluster.setUpIPs()
    elif 'set-migrating-state' == command:
        uLogging.info("Set migrating state of subscription %s to %s",
                      config.config.subscriptionid, config.config.enable)
        sub = Subscription(config.config.subscriptionid)
        sub.setMigratingState(config.config.enable)
    elif 'uptime' == command:
        host = Host(config.config.hostid)
        uLogging.info(host.uptime())
    elif 'migrate-one' == command:
        pass
    elif 'shutdown' == command:
        host = Host(config.config.hostid)
        host.shutdown()
    elif 'add-log' == command:
        host = Host(config.config.hostid)
        con = uSysDB.connect()

        con.insertRecord("logfiles", service_id=host.getServiceID("logparser"), dir_path=config.config.log.dirpath, mask_type="x",
                         file_mask=config.config.log.filemask, fmt="%h %l %u %t \"%r\" %s %b", native_format_preamble="", ascn_key="u", todo="n", todo_status="n", case_sens="y")

        con.commit()
    elif 'change-service-template' == command:
        api = openapi.OpenAPI()
        sub = Subscription(config.config.subscriptionid)
        sub.changeServiceTemplate(config.config.servicetemplateid, api)
    elif 'unprovide-unsupported-services' == command:
        sub = Subscription(config.config.subscriptionid)
        sub.unprovideUnsupportedServices()
    elif 'change-service-plan' == command:
        sub = Subscription(config.config.subscriptionid)
        sub.changeServicePlan(config.config.serviceplanid, config.config.subscriptionperiodid)
    elif 'change-service-templates' == command:
        subids = config.config.subscriptionids.split(",")
        subs = [Subscription(int(subid)) for subid in subids]

        migrationConfig = config.MigrationConfig()
        migrationConfig.read(config.config.configfile)

        migration = Migration()
        migration.checkConfig(migrationConfig, subs)
        migration.changeServiceTemplates(subs, migrationConfig)
    elif 'create-local-pear' == command:
        webspace = Webspace(config.config.webspaceid)
        webspace.createLocalPear()
    elif 'sync-htaccess' == command:
        webspace = Webspace(config.config.webspaceid)
        webspace.syncHtAccess()
    elif 'dump-crontabs' == command:
        webspace = Webspace(config.config.webspaceid)
        webspace.dumpCrontabs()
    elif 'replace-abs-paths' == command:
        webspace = Webspace(config.config.webspaceid)
        cluster = WebCluster(config.config.clusterid)
        webspace.checkContentForAbsPaths(cluster)
    elif 'check-abs-paths' == command:
        if not config.config.checkpointfile or not os.path.exists(config.config.checkpointfile):
            raise Exception("Checkpoint file was not specified or points to invalid location. Use --checkpoint-file.")

        checkpoint = Checkpoint.load(config.config.checkpointfile)
        cluster = WebCluster(checkpoint.clusterID)
        subs = checkpoint.subs
        uLogging.info(
            "Checking for absolute paths on %s in content of webspaces of subscriptions %s", cluster, subs.getSubs())
        pause()
        for sub in subs.getSubs():
            sub.checkContentForAbsPaths(cluster)
    elif 'finalize-content' == command:
        if not config.config.checkpointfile or not os.path.exists(config.config.checkpointfile):
            raise Exception("Checkpoint file was not specified or points to invalid location. Use --checkpoint-file.")
        checkpoint = Checkpoint.load(config.config.checkpointfile)
        cluster = WebCluster(checkpoint.clusterID)
        subs = checkpoint.subs
        uLogging.info(
            "Checking for absolute paths on %s in content of webspaces of subscriptions %s", cluster, subs.getSubs())
        pause()
        subs.finalizeContent(cluster)
    elif 'print-host-info' == command:
        host = Host(config.config.hostid)

        uLogging.info("Host id:        %s", host.hostID)
        uLogging.info("Plesk agent id: %s", host.getAgentID())
        uLogging.info("Backnet IP:     %s", host.getBacknetIP())
        uLogging.info("Host type:      %s", host.getType())
        if host.isVE():
            uLogging.info("Offline management: %s", host.getVEAttribute("offline_management"))
        uLogging.info("Backups:        %s", host.hasBackups())
    elif 'subscription-info' == command:
        if config.config.withpba:
            for subId in config.config.subscriptionid.split(','):
                sub = Subscription(subId)
                subscriptionInfo = sub.getBillingSubscriptionInfo()
                uLogging.info('=' * 60)
                uLogging.info("Subscription ID: %s", subscriptionInfo[0])
                uLogging.info("Subscription name: %s", subscriptionInfo[1])
                uLogging.info("Billing service plan ID: %s", subscriptionInfo[3])
                uLogging.info("Billing service plan name: %s", subscriptionInfo[4])
                if subscriptionInfo[17] in billingPeriodType.keys():
                    periodType = billingPeriodType[subscriptionInfo[17]]
                else:
                    periodType = subscriptionInfo[17]

                uLogging.info("Subscription period: %s %s", subscriptionInfo[16], periodType)
                apsApplications = sub.getApsApplications()
                if apsApplications:
                    uLogging.info("Subscription applications: %s", ', '.join([x[0] for x in apsApplications]))
                else:
                    uLogging.info("Subscription has no applications")

                externalIps = sub.getExternalIps()
                if externalIps:
                    uLogging.info("Subscription external ips: %s", ', '.join([x[0] for x in externalIps]))
                else:
                    uLogging.info("Subscription has no external ips")
    elif 'print-service-template-info' == command:
        template = ServiceTemplate(config.config.servicetemplateid)

        uLogging.info("Service template: %s", template)
        uLogging.info("Resources: ")
        for resource in template.getResources():
            uLogging.info(resource)
    elif 'service-ctl' == command:
        host = Host(config.config.hostid)
        host.serviceCtl(config.config.command, config.config.servicename)
    elif 'ping-pba' == command:
        api = pbaapi.PBAAPI()

        methods = api.system.listMethods()
        uLogging.info("Available methods: %s", methods)
        for method in methods:
            if "system.multicall" == method:
                # for some reason this method is reported in list
                # but methodHelp doesn't know about it
                continue

#			is not available in PBA
#			uLogging.info(api.system.methodSignature(method))
            uLogging.info("%s: %s", method, api.system.methodHelp(method))
    elif 'check' == command:
        ids = config.config.subscriptionid.split(',')
        subs = SubscriptionSet.createFromListOfSubscriptionIDs(ids)
        if not config.config.checkpointfile:
            config.config.checkpointfile = defaultWritablePath("migrate-sub%s.cpt" % subs.getSubs()[0].id)

        migrationConfig = config.MigrationConfig()
        migrationConfig.read(config.config.configfile)
        toHost = WebCluster(config.config.targetid)
        migration = Migration()
        _, toHost, subs, migrationConfig = migration.initCheckpoint(None, toHost, subs, migrationConfig)

        if config.config.aspect == 'subscriptions':
            checkList = migration.subscriptionsPrecheck(subs, migrationConfig)
        elif config.config.aspect == 'tasks':
            checkList = migration.tasksPrecheck()
        elif config.config.aspect == 'source-hosts':
            checkList = migration.sourceHostsPrecheck(subs)
        elif config.config.aspect == 'connections':
            checkList = migration.connectionsPrecheck(toHost, subs)
        elif config.config.aspect == 'config':
            checkList = migration.configPrecheck(subs, migrationConfig)
        elif config.config.aspect == 'target-host':
            checkList = migration.targetHostPrecheck(toHost, subs, migrationConfig)
        elif config.config.aspect == 'backup-diskspace':
            checkList = migration.diskspaceBackupPrecheck(subs)
        else:
            checkList = migration.checkPrerequisites(toHost, subs, migrationConfig)

        uLogging.info("All checks completed successfully")
        beginString = "- "
        okString = " \033[92m[OK]\033[0m"
        if len(checkList) > 0:
            maxStringLength = max([len(x) for x in checkList])
            for checkItem in checkList:
                uLogging.info(beginString + checkItem + ' ' * (maxStringLength - len(checkItem)) + okString)

    elif 'revert' == command:
        question = """
Reversion migration of sites to Linux Shared Hosting NG.
Any changes that were made to site content on web cluster/server will be lost after revert.

Do you want to start reverting migration of sites to Linux Shared Hosting NG using checkpoint file '%s'?
""" % (config.config.checkpointfile)
        answer = uDialog.askYesNo(question)
        if answer:
            checkpoint = Checkpoint.load(config.config.checkpointfile)

            uLogging.debug("Checkpoint: %s", checkpoint)

            migration = Migration()
            if checkpoint.hostID:
                migration.revertHost(checkpoint)
            else:
                migration.revertSubscriptions(checkpoint)

            uLogging.info("Reversion of migration completed successfully.")
    elif 'sync-content' == command:
        cluster = WebCluster(config.config.targetid)
        ids = config.config.subscriptionid.split(',')
        subs = SubscriptionSet.createFromListOfSubscriptionIDs(ids)
        uLogging.info("Syncing content for webspaces of subscriptions %s to %s", subs.getSubs(), cluster)
        subs.syncContent(cluster)
    elif 'migrate' == command:
        # we can migrate host or subscriptions
        if not (config.config.sourceid and config.config.targetid and config.config.configfile) and \
           not (config.config.subscriptionid and config.config.targetid and config.config.configfile):
            printUsage()
            return

        subs, srcHost, cluster = None, None, None
        if config.config.subscriptionid:
            ids = config.config.subscriptionid.split(',')
            subs = SubscriptionSet.createFromListOfSubscriptionIDs(ids)
        else:
            srcHost = Host(config.config.sourceid)

        cluster = WebCluster(config.config.targetid)

        # set up default checkpoint
        if not config.config.checkpointfile:
            if subs:
                config.config.checkpointfile = defaultWritablePath("migrate-sub%s.cpt" % subs.getSubs()[0].id)
            else:
                config.config.checkpointfile = defaultWritablePath("migrate-host%s.cpt" % srcHost.hostID)

        warning = """
Migration of sites from standard Linux Shared Hosting to Linux Shared Hosting NG.
Warning: This operation is potentially dangerous. It's STRONGLY recommended to make backup copy of POA database.
"""
        if subs:
            question = warning + """
Do you want to start migrating all sites from subscriptions %s to %s using config file '%s'?
""" % (", ".join([str(sub.id) for sub in subs.getSubs()]), cluster, config.config.configfile)
        else:
            question = warning + """
To perform content synchronization `from` host must have ssh access without password to `to` web cluster director host/web server.
Before migration make sure, that source and destination service templates are compatible (e.g. destination service templates contain all needed resource types)

Do you want to start migrating all sites from %s to %s using config file '%s'?
""" % (srcHost, cluster, config.config.configfile)

        answer = uDialog.askYesNo(question)
        if answer:
            try:
                migrationConfig = config.MigrationConfig()
                migrationConfig.read(config.config.configfile)

                migration = Migration()
                if subs:
                    migration.migrateSubscriptions(subs, cluster, migrationConfig)
                else:
                    migration.migrateHost(srcHost, cluster, migrationConfig)

                if subs:
                    uLogging.info("Migration of all sites from subscriptions %s to %s completed successfully.", ", ".join(
                        [str(sub.id) for sub in subs.getSubs()]), cluster)
                else:
                    uLogging.info("Migration of all sites from %s to %s completed successfully. %s was shut down, you should manually unregister it from POA management UI.", srcHost, cluster, str(
                        srcHost).capitalize())

            except PrerequisiteError:
                # don't need to show in case of prerequisite error
                raise
            except:
                uLogging.info("Migration of subscriptions %s was failed." % ', '.join([str(sub.id) for sub in subs.getSubs()]))
                raise
    else:
        print """You did not specify any commands or the command is not supported. Specify one of supported commands:
subscription-info - print information about subscriptions.
    Usage: ./migrate.py --subscription-id <subscription-ids> --with-pba subscription-info
check - run pre-checks for subscriptions.
    Usage: ./migrate.py --target-id <host-id> --subscription-id <subscription-ids> --config <path-to-config> --with-pba [--batch] check
migrate - start the migration of subscriptions.
    Usage: ./migrate.py --target-id <host-id> --subscription-id <subscription-ids> --config <path-to-config> --with-pba [--batch] migrate
sync-content - copy the contents of subscriptions.
    Usage: ./migrate.py --target-id <host-id> --subscription-id' <subscription-ids> --with-pba sync-content"""


def saveMigrationResultToFile(opts, exit_code, message=''):
    if opts and '--results-file' in opts:
        filename = opts['--results-file']
    else:
        return

    import json
    data = {
        'exit_code': exit_code,
        'message': message,
    }
    f = open(filename, 'w')
    try:
        json.dump(data, f)
    finally:
        f.close()


__long_options = [
    ('help', 'show this help page and exit (all other arguments are ignored)'),
    ('batch', 'avoid user interaction in the batch mode'),
    ('verbose', 'if specified, will output a detailed trace to a log file'),
    ('config=', 'path to a custom configuration file'),
    ('source-id=', 'ID of the source host'),
    ('target-id=', 'ID of the target host'),
    ('subscription-id=', 'ID of the subscription to migrate'),
    ('skip:log-file-rotation', 'do not recreate a log file if it exists'),
    ('checkpoint-file=', 'checkpoint file'),
    ('replace-abs-paths-policy', 'replace old absolute paths in subscription content with new paths'),
    ('skip-replace-aps-path', 'skip to replace old absolute paths in subscription content with new paths'),
    ('skip-backups', 'skip the migration of backups of webspaces'),
    ('skip:shutdown-host', 'do not shut down the source server after migration'),
    ('skip:check-prerequisites', 'do not check the settings of the source host during migration'),
    ('skip:migration', 'do not migrate subscriptions'),
    ('skip:kill-active-sessions', 'do not close SSH and FTP sessions after the completion of migration'),
    ('enable', 'new subscription status. It is used for the set-migrating-state command.'),
    ('subscription-ids', 'a list of subscription IDs. It is used for the change-service-templates command.'),
    ('service-template-id', 'service template id. It is used for the commands change-service-template and print-service-template-info'),
    ('service-plan-id', 'service plan id. It is used for the change-service-plan command.'),
    ('subscription-period-id', 'subscription period id. It is used for the change-service-plan command.'),
    ('webspace-id', 'webspace ID'),
    ('cluster-id', 'cluster ID'),
    ('host-id', 'host ID'),
    ('aspect=', 'Run a series of checks that check if the system is ready for migration'),
    ('service-name', 'service name. It is used for the service-ctl command.'),
    ('command', 'new service status. It is used for the service-ctl command.'),
    ('demo-mode', 'the migration tool will wait for a user action in each step'),
    ('traceback', 'print out the traceback if a source host is configured inappropriately'),
    ('with-pba', 'OA is integrated with PBA'),
    ('results-file=', 'Path to migration report'),
    ('skip-warnings', 'Skip minor issues and continue the migration process'),
    ('skip-resource-usage-check', 'Do not check whether the plan that is used for migration provides enough resources.'),
    ('skip-incomplete-tasks', 'Do not check incomplete tasks before migration.'),
]  # list of available long options


if __name__ == '__main__':
    reload(sys)
    sys.setdefaultencoding('utf-8')

    import getopt
    try:
        opts, args = getopt.getopt(sys.argv[1:], '', dict(__long_options).keys())
        opts = dict(opts)
    except getopt.GetoptError, err:
        print str(err)
        printUsage()
        sys.exit(2)

    if opts and '--help' in opts:
        printUsage()
        sys.exit(0)

    try:
        doInit(opts)
        main()
        saveMigrationResultToFile(opts, 0)
    except PrerequisiteError, e:
        uLogging.err("Unsatisfied prerequisite: %s", e)
        saveMigrationResultToFile(opts, 1, "%s" % e)
        if config.config is not None and config.config.traceback:
            raise
    except Exception, e:
        uLogging.err("%s", e)
        saveMigrationResultToFile(opts, 1, "%s" % e)
        raise
