#!/usr/bin/python

from poaupdater import uTasks, uSysDB, uLogging


class Service:
    APACHENG = 'ApacheNG'
    PROFTPDNG = 'ProftpdNG'
    LOGROTATE = 'logrotate'
    LOGPARSER = 'logparser'
    CRONTABS = 'crontabs'


class SSHAccess:
    @staticmethod
    def fetch_params(cur, webspace):
        cur.execute("SELECT login, password, state FROM apache_ssh_access WHERE homedir_id = %s", webspace.webspace_id)
        rv = cur.fetchone()
        if not rv:
            return "", "", ""

        return rv[0], rv[1], rv[2]

    @staticmethod
    def create_task(name, method, webspace, host):
        pobjref = uTasks.PObjRef(
            uTasks.SCRef("apache"), "getDomainService", webspace.get_domain_service_id(Service.APACHENG)
        )

        task = uTasks.Task(pobjref, name, method)
        task.setParam("service_id", host.get_service_id(Service.APACHENG))
        task.setParam("webspace_id", str(webspace.webspace_id))

        return pobjref, task

    def enable(self, webspace, host):
        con = uSysDB.connect()
        cur = con.cursor()

        login, password, state = self.fetch_params(cur, webspace)
        if not login:
            return

        if 'e' == state:
            uLogging.debug("SSH access for webspace %s is already enabled", webspace.webspace_id)
            return

        cur.execute(
            "UPDATE apache_ssh_access SET state = 'e', ops_scheduled = ops_scheduled + 1 WHERE homedir_id = %s",
            webspace.webspace_id)
        cur.close()
        con.commit()

        pobjref, task = self.create_task(
            "Enable SSH access for webspace %s" %
            webspace.webspace_id, "taskEnableSSHAccess", webspace, host
        )
        task.setParam("login", login)
        task.setParam("password", password)
        task.schedule()

    def disable(self, webspace, host):
        con = uSysDB.connect()
        cur = con.cursor()

        login, password, state = self.fetch_params(cur, webspace)
        if not login or "d" == state:
            return

        cur.execute(
            "UPDATE apache_ssh_access SET state = 'd', ops_scheduled = ops_scheduled + 1 WHERE homedir_id = %s",
            webspace.webspace_id)
        cur.close()
        con.commit()

        pobjref, task = self.create_task(
            "Disable SSH access for webspace %s" % webspace.webspace_id, "taskDisableSSHAccess", webspace, host
        )
        task.setParam("lock", "1")
        task.setParam("login", login)
        task_id = task.schedule()
        uTasks.waitTasks(con, [task_id], 30, 2)


def listmerge(s):
    return reduce(lambda d, el: d.extend(el) or d, s, [])
