# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core import _T
from migrator.core.actions.base.common_action import CommonAction
from migrator.core.data.model.migration_project import MigrationMode
from migrator.core.data.model.subscription import has_any_of_service_types, get_services
from migrator.settings import \
    SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, SourceHostingClass, SOURCE_SERVICE_DNS_NS, SOURCE_PRODUCT_OBAS, \
    SOURCE_PRODUCT_PLESK, MIGRATION_TYPE_FULL, SKIP_DNS_ZONE_FORWARDING


class RestartSourceNameservers(CommonAction):
    def run(self, global_context):
        zone_manager = global_context.source_product.get_dns_zone_manager()
        source_host_dns_forwarding_impl = {}
        for subscription in global_context.iter_all_subscriptions():
            if not self._need_to_restart_ns_for_subscription(global_context, subscription):
                continue

            for service in get_services(subscription.source_info, SOURCE_SERVICE_DNS_NS):
                if service.source_host_external_id in source_host_dns_forwarding_impl:
                    continue

                source_host_dns_forwarding_impl[service.source_host_external_id] = zone_manager.get_dns_forwarding_impl(
                    service
                )

        for dns_forwarding_impl in source_host_dns_forwarding_impl.values():
            dns_forwarding_impl.restart_nameserver()

    @staticmethod
    def _need_to_restart_ns_for_subscription(global_context, subscription):
        return has_any_of_service_types(subscription.source_info, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.DNS_NS]) and \
               not global_context.execution_wrapper.is_failed_subscription(subscription.source_info.external_id)

    def filter_action(self, global_context):
        """
        :type global_context: migrator.core.global_context.GlobalMigrationContext
        """
        source_product_type = global_context.source_type
        return global_context.migration_mode == MigrationMode.PRODUCTION and \
               ((source_product_type == SOURCE_PRODUCT_PLESK and global_context.migration_type == MIGRATION_TYPE_FULL) or
               (source_product_type == SOURCE_PRODUCT_OBAS)) and \
               not SKIP_DNS_ZONE_FORWARDING

    def get_description(self):
        return _T("RESTART_SOURCE_NAME_SERVERS")

    def get_failure_message(self, global_context):
        """
        :type global_context: migrator.core.global_context.GlobalMigrationContext
        """
        return _T("FAILED_TO_RESTART_SOURCE_NAME_SERVERS")
