# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.cli.commands.declaration_base import MigratorOptionDeclaration, MigratorArgumentDeclaration
from migrator.core.language import _T
from migrator.settings import HOSTING_SERVICE_TYPES


class AsyncOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--async'

    @property
    def help(self):
        return _T("OPTION_ASYNC_HELP")

    @property
    def is_flag(self):
        return True


class QuietOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--quiet'

    @property
    def help(self):
        return _T("OPTION_QUIET_HELP")

    @property
    def is_flag(self):
        return True


class SkipProfilingOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--skip-profiling'

    @property
    def help(self):
        return _T("OPTION_SKIP_PROFILING_HELP")

    @property
    def is_flag(self):
        return True


class SkipServiceCheckOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--skip-services-checks'

    @property
    def help(self):
        return _T("DO_NOT_CHECK_SERVICES_FOR_ANY")

    @property
    def is_flag(self):
        return True


class ReloadSourceDataOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--reload-source-data'

    @property
    def help(self):
        return _T("FORCE_FETCHING_DATA_FROM_SOURCE_PANELS")

    @property
    def is_flag(self):
        return True


class UseCachedOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--use-cached'

    @property
    def help(self):
        return _T("DO_NOT_USE_RELOAD_SOURCE_DATA")

    @property
    def is_flag(self):
        return True


class MigrationListFileOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--migration-list-file'

    @property
    def help(self):
        return _T("MIGRATION_LIST_FILENAME_DEFAULT_VALUE_IS")


class IpMappingFileOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--ip-mapping-file'

    @property
    def help(self):
        return _T("IP_MAPPING_FILENAME_FILE_CONTAINS_SOURCE")


class SkipLicenseChecksOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--skip-license-checks'

    @property
    def help(self):
        return _T("DO_NOT_CHECK_ANY_LICENSES_ON_TARGET")

    @property
    def is_flag(self):
        return True


class SkipCapabilityChecksOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--skip-capability-checks'

    @property
    def help(self):
        return _T("SKIP_CAPABILITY_CHECKS_OPTION")

    @property
    def is_flag(self):
        return True


class SkipDnsForwardingTestOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--skip-dns-forwarding-test'

    @property
    def help(self):
        return _T("DO_NOT_CHECK_THAT_DNS_QUERIES")

    @property
    def is_flag(self):
        return True


class IgnoreFetchSourceErrorsOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--ignore-fetch-source-errors'

    @property
    def help(self):
        return _T("IGNORE_ERRORS_FETCHSOURCE_STAGE_HSPHERE_MIGRATION")

    @property
    def is_flag(self):
        return True


class CommandOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--command'

    @property
    def help(self):
        return _T("SUSPEND_ACCOUNTS_IN_HSPHERE_EVEN_IF")

    @property
    def is_flag(self):
        return True


class RunPostMigrationChecks(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--run-post-migration-checks'

    @property
    def help(self):
        return _T("RUN_POST_MIGRATION_CHECKS")

    @property
    def is_flag(self):
        return True


class ServiceTypeOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--service-type'

    @property
    def type(self):
        return HOSTING_SERVICE_TYPES.keys()


class AccountIdsOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--account-ids'


class HostIdsOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--host-ids'


class SubscriptionIdsOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--subscription-ids'


class PlanIdsOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--plan-ids'


class OverwriteOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--overwrite'

    @property
    def help(self):
        return _T("OVERWRITE_FILE_IF_IT_ALREADY_EXISTS")

    @property
    def is_flag(self):
        return True


class ScriptArgument(MigratorArgumentDeclaration):
    @property
    def name(self):
        return 'script'


class ConfigOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--config'

    @property
    def help(self):
        return _T("CONFIG_FILE_OPTION")

    @property
    def default(self):
        return 'config.ini'

    @property
    def required(self):
        return True


class MigrationProjectApsIdOption(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--project-aps-id'

    @property
    def help(self):
        return _T("MIGRATION_PROJECT_APS_ID_OPTION")

    @property
    def required(self):
        return True


class FilterSettingsFile(MigratorOptionDeclaration):
    @property
    def name(self):
        return '--filter-settings-file'

    @property
    def required(self):
        return False


import_options = [
    MigrationListFileOption,
    IpMappingFileOption,
    ReloadSourceDataOption,
    IgnoreFetchSourceErrorsOption
]
