# (C)2017 Ingram Micro Inc. all Rights Reserved
import json
import logging

from migrator.core import MigrationError
from migrator.core.checks.base import CheckAbstract
from migrator.core.cli.base import CommandResponse
from migrator.core.cli.commands.implementation_base import MigratorCommandImplementation, handle_exceptions
from migrator.core.cli.migrator_factory import create_migrator
from migrator.core.language import _T
from migrator.odin_automation.platform.checks.manager import CheckManager

logger = logging.getLogger(__name__)


class MigratorObjectCommandImplementation(MigratorCommandImplementation):
    @handle_exceptions
    def run(self, options):
        try:
            migrator = self._create_and_configure_migrator_object(options)
            result = self.main(migrator, options)
        except MigrationError as e:
            logger.debug(_T('LOG_EXCEPTION'), exc_info=True)
            result = {
                'exit_code': 1,
                'error_message': str(e)
            }
        except Exception as e:
            logger.debug(_T('LOG_EXCEPTION'), exc_info=True)
            raise e

        print json.dumps(result, indent=4)

    def _create_and_configure_migrator_object(self, options):
        migrator = create_migrator(options)

        if hasattr(migrator, 'set_options'):
            migrator.set_options(options)
        if not options.skip_profiling and hasattr(migrator, 'configure_profiler'):
            migrator.configure_profiler()

        return migrator

    def main(self, migrator, options):
        raise NotImplementedError()


class ActionRunnerCommandImplementation(MigratorObjectCommandImplementation):
    def main(self, migrator, options):
        migrator.action_runner.run_entry_point(self.entry_point_name)

        if migrator.is_migration_successful():
            result = {'exit_code': 0}
        else:
            result = {'exit_code': 1}

        return result

    @property
    def entry_point_name(self):
        raise NotImplementedError()


class CheckCommandImplementation(MigratorObjectCommandImplementation):
    def __init__(self, check):
        self.check = check

    @handle_exceptions
    def run(self, options):
        self._create_and_configure_migrator_object(options)
        check_manager = CheckManager()
        check_response = check_manager.run_check(self.check)
        if check_response['status'] == CheckAbstract.STATUS_ERROR:
            exit_code = 1
        elif check_response['status'] == CheckAbstract.STATUS_WARNING:
            exit_code = 2
        else:
            exit_code = 0

        print json.dumps(
            CommandResponse(
                exit_code=exit_code,
                message=check_response['message'],
                details={'status': check_response['status']}
            )
        )