# (C)2017 Ingram Micro Inc. all Rights Reserved
import json
import logging

from migrator.core import MigrationError
from migrator.core.cli.commands.implementation.action_runner import MigratorObjectCommandImplementation
from migrator.core.cli.commands.implementation_base import handle_exceptions
from migrator.core.language import _T
from migrator.core.product.lib.agent import is_agent_service
from migrator.core.utils.networking.run_command import WinexeExecuteError

logger = logging.getLogger(__name__)


class InstallAgentImplementation(MigratorObjectCommandImplementation):
    @handle_exceptions
    def run(self, options):
        result = {'exit_code': 0}
        migrator = self._create_and_configure_migrator_object(options)
        try:
            agent = migrator.global_context.source_product.get_agent_instance()
            agent.install()
            if is_agent_service(agent):
                agent.start()

            logger.debug(_T("MIGRATION_AGENT_INSTALLED") % agent.agent_dir)
        except WinexeExecuteError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            result = {
                'exit_code': 1,
                'error_message': ' '.join([_T("MIGRATION_MANAGER_COULD_NOT_INSTALL").format(host=e.host), e.message])
            }
        except MigrationError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            result = {'exit_code': 1, 'error_message': e.message}
        except Exception as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            raise e

        print json.dumps(result)
