# (C)2017 Ingram Micro Inc. all Rights Reserved
import json
import logging
import psycopg2
import psycopg2.extras

from migrator.core.cli.commands.implementation.action_runner import ActionRunnerCommandImplementation
from migrator.core.cli.commands.implementation_base import handle_exceptions
from migrator.core.language import _T
from migrator.odin_automation.platform.lib.db import get_platform_db_connection

logger = logging.getLogger(__name__)


class ListIpPoolsImplementation(ActionRunnerCommandImplementation):
    @handle_exceptions
    def run(self, options):
        try:
            connection = get_platform_db_connection(cursor_factory=psycopg2.extras.RealDictCursor)
            cursor = connection.cursor()
            cursor.execute('''SELECT pool_id AS id, pool_name AS name, ip_from, ip_to, netmask FROM ip_pools''')
            result = {
                'exit_code': 0,
                'ip_pools': cursor.fetchall()
            }
        except (psycopg2.OperationalError, psycopg2.ProgrammingError) as e:
            result = {
                'exit_code': 1,
                'ip_pools': None,
                "error_message": "DB Error: " + str(e)
            }
        except Exception as e:
            logger.debug(_T('LOG_EXCEPTION'), exc_info=True)
            raise e

        print json.dumps(result, indent=4)
