# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import ParallelExecutionError
from migrator.core.cli.commands.implementation.action_runner import MigratorObjectCommandImplementation
from migrator.core.utils.filetypes.config_utils import get_source_type_section
from migrator.core.utils.concurrency.file_lock import FileLock, FileLockException
from migrator.odin_automation.platform.lib.data.loader import get_loader
from migrator.settings import TargetServiceType

logger = logging.getLogger(__name__)


class RefreshDataImplementation(MigratorObjectCommandImplementation):
    """Update existing source system information in the database"""
    def main(self, migrator, options):
        try:
            with self._get_export_data_lock(migrator.global_context):
                migrator.agent.install()
                loader = get_loader(
                    TargetServiceType.PLATFORM, get_source_type_section(migrator.global_context.config).get('aps-id')
                )
                account_ids = options.account_ids
                if account_ids is not None:
                    account_ids = set(account_ids.split(','))

                host_ids = options.host_ids
                if host_ids is not None:
                    host_ids = set(host_ids.split(','))

                loader.refresh(account_ids=account_ids, host_ids=host_ids)
            return {'exit_code': 0}
        except FileLockException:
            raise ParallelExecutionError()

    def _get_export_data_lock(self, global_context):
        return FileLock(global_context.session_files.get_path_to_export_data_lock())
