# (C)2017 Ingram Micro Inc. all Rights Reserved
import codecs
import json
import logging
import os

from migrator.core import LshNgMigratorError
from migrator.core.cli.commands.implementation.action_runner import MigratorObjectCommandImplementation
from migrator.core.cli.commands.implementation_base import handle_exceptions
from migrator.core.cli.migrator_factory import create_migrator
from migrator.core.data.model.migration_project import MigrationMode
from migrator.core.data.model.subscription import SubscriptionMigrationStatus
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils.networking.run_command import LocalUnixRunner
from migrator.core.utils.registry import Registry
from migrator.settings import LSH_NG_MIGRATOR_BASE_DIR

logger = logging.getLogger(__name__)


class RevertSubscriptionImplementation(MigratorObjectCommandImplementation):
    @handle_exceptions
    def run(self, options):
        create_migrator(options)
        global_context = Registry.get_instance().get_context()
        panel = global_context.target_panel_obj

        subscription = PlatformReaderDB().get_subscription(options.subscription_id)
        try:
            self._update_subscription_status(subscription, SubscriptionMigrationStatus.IN_PROGRESS)
            self._update_subscription_production_status(subscription, False)

            migrator_path = os.path.join(LSH_NG_MIGRATOR_BASE_DIR, 'migrate.py')
            checkpoint_file = os.path.join(LSH_NG_MIGRATOR_BASE_DIR, 'migrate-sub%s.cpt' % subscription.external_id)
            results_file = global_context.migrator_server.get_session_file_path('result_file')
            is_billing_module_installed = panel.is_billing_module_installed()
            cmd = '{migrator_path} --checkpoint-file {checkpoint_file} --results-file {results_file} --batch'
            if is_billing_module_installed:
                cmd += ' --with-pba'

            cmd += ' revert'
            local_runner = LocalUnixRunner()
            local_runner.sh_unchecked(cmd.format(
                migrator_path=migrator_path,
                checkpoint_file=checkpoint_file,
                results_file=results_file,
            ))

            with codecs.open(results_file, 'r', 'utf-8') as f:
                data = json.load(f)
                if data['exit_code'] != 0:
                    raise LshNgMigratorError(data['message'])

            self._update_subscription_status(subscription, SubscriptionMigrationStatus.NEW_SUBSCRIPTION)
        except LshNgMigratorError as e:
            self._update_subscription_status(subscription, SubscriptionMigrationStatus.FINISHED_ERRORS, e.message)
        except Exception:
            logger.debug(_T('LOG_EXCEPTION'), exc_info=True)
            self._update_subscription_status(
                subscription,
                SubscriptionMigrationStatus.FINISHED_ERRORS,
                [_T('UNABLE_MIGRATE_LSH_SUBSCRIPTION').format(subscription_name=subscription.name)]
            )

    def _update_subscription_status(self, subscription, migration_status, message=None):
            subscription.migration_status = migration_status
            if message is not None:
                messages = [{'message': message, 'type': 'error'}]
            else:
                messages = []

            subscription.message = json.dumps(messages)
            PlatformReaderDB().update_subscription_status(subscription)

    def _update_subscription_production_status(self, subscription, in_production):
            subscription.in_production = in_production
            PlatformReaderDB().update_subscription_production_status(subscription)
            subscription.migration_mode = MigrationMode.PRODUCTION if in_production else MigrationMode.TEST_MIGRATION
            PlatformReaderDB().update_subscription_status(subscription)
