# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from ConfigParser import NoOptionError
from ConfigParser import NoSectionError

from migrator.core import APIError, MigrationError, MigrationNoContextError, MigrationConfigurationFileError
from migrator.core.cli.base import _get_context_msg
from migrator.core.language import _T

logger = logging.getLogger(__name__)


class MigratorCommandImplementation(object):
    def run(self, options):
        raise NotImplementedError()


def handle_exceptions(func):
    def new_func(self, options):
        try:
            func(self, options)
        except APIError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            logger.error(e.how_to_reproduce)
            return 1
        except MigrationError as e:
            context = getattr(e, 'context', '')
            logger.debug(_T("EXCEPTION_CONTEXT").format(context=context), exc_info=True)
            errmsg = unicode(e)
            if errmsg.endswith('.'):
                errmsg = errmsg[:-1]

            logger.error(u'%s%s.', errmsg, _get_context_msg(context))
            return 1
        except KeyboardInterrupt:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            logger.info(_T("MIGRATION_STOPPED_BY_USER_REQUEST"))
            return 1
        except MigrationNoContextError as e:
            logger.error(u"%s", e)
            return 1
        except UnicodeError as e:
            context = getattr(e, 'context', '')
            context_msg = _get_context_msg(context)
            logger.debug(_T("LOG_UNICODE_EXCEPTION"), exc_info=True)
            logger.debug(_T("ERROR_OCCURRED_WHILE_WORKING_FOLLOWING_STRING"), e.object)
            logger.error(_T("INTERNAL_MIGRATOR_ERROR_CAUSED_BY_UNICODE") % e + context_msg)
            return 1
        except NoOptionError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            logger.error(
                _T("EXCEPTION_FAILED_TO_READ_CONFIG_ABSENT_OPTION"), e.option, e.section, options.config
            )
            return 1
        except NoSectionError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            logger.error(_T("FAILED_READ_CONFIGURATION_FILE_THERE_IS"), e.section, options.config)
            return 1
        except MigrationConfigurationFileError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            logger.error(_T("S_PLEASE_FIX_MIGRATION_TOOL_CONFIGURATION"), e, options.config)
            return 1

        return 0

    return new_func
