# (C)2017 Ingram Micro Inc. all Rights Reserved
import os

from migrator.core.utils.lang.loader import load_class, get_root_module_dir


def list_target_panels(product_name):
    return list(_iter_target_panels(product_name))


def list_source_panels():
    """
    :return: list[] Example: ['plesk', 'oa.wsh', 'oa.lsh', 'oa.ng', 'obas']
    """
    return list(_iter_source_panels())


def load_target_panel_class(target_panel_type, module_name, class_name, optional=False):
    """
    Load target panel class 'class_name' from 'target_panel_type' module's sub module 'module_name'.
    'module_name' can point to sub module with any level of deepness.

    Example:
        'main'
        'takeover.workflow'
        'some.more.deeper.module'

    :type target_panel_type: str | unicode
    :type module_name: str | unicode
    :type class_name: str | unicode
    :type optional: bool
    :rtype: type | None
    """
    # first, try to load the class from migrator.products.<panel>.target
    target_class = load_class(
        'migrator.products.%s.target.%s' % (target_panel_type, module_name), class_name, optional=True)

    if target_class is None:
        # if class loading from migrator.product has failed, then fallback to
        # the default product returned by get_product_name
        product_name = get_product_name()
        target_class = load_class(
            'migrator.%s.services.%s.%s' % (product_name, target_panel_type, module_name), class_name, optional)

    return target_class


def load_source_panel_class(source_panel_type, module_name, class_name, optional=False):
    """
    Load source panel class 'class_name' from 'source_panel_type' module's sub module 'module_name'.

    'module_name' can point to sub module with any level of deepness.
    Examples:
        'main'
        'takeover.workflow'
        'some.more.deeper.module'

    :type source_panel_type: str | unicode Examples: 'oa.wsh', 'plesk', 'obas'
    :type module_name: str | unicode Examples: 'main', 'workflow.transfer', 'workflow'
    :type class_name: str | unicode Examples: 'Migrator', 'WorkflowExtension'
    :type optional: bool
    :rtype: type | None
    """
    return load_class('migrator.products.%s.%s' % (source_panel_type, module_name), class_name, optional)


def _iter_target_panels(product_name):
    module_base_dir = os.path.join(get_root_module_dir(), product_name, 'services')
    for module_dir in os.listdir(module_base_dir):
        module_absolute_path = os.path.join(module_base_dir, module_dir)
        workflow_file_path = os.path.join(module_absolute_path, 'workflow', 'transfer.py')
        if os.path.isdir(module_absolute_path) and os.path.exists(workflow_file_path):
            yield module_dir


def _iter_source_panels():
    source_panel_dir = os.path.join(get_root_module_dir(), 'products')
    if not os.path.exists(source_panel_dir):
        return

    for module_dir in os.listdir(source_panel_dir):
        source_module_dir = os.path.join(source_panel_dir, module_dir)
        if os.path.exists(os.path.join(source_module_dir, 'workflow.py')):
            yield module_dir
        else:
            if not os.path.isdir(source_module_dir):
                continue

            for sub_module_dir in os.listdir(source_module_dir):
                if os.path.exists(os.path.join(source_module_dir, sub_module_dir, 'workflow.py')):
                    yield '%s.%s' % (module_dir, sub_module_dir)


def get_product_name():
    """Return the product name for the target panel."""
    return 'odin_automation'
