# (C)2017 Ingram Micro Inc. all Rights Reserved

from contextlib import contextmanager

from migrator.core.language import _T
from migrator.core.session import session_dir
from migrator.core.session.config import get_local_session_dir, get_common_local_session_dir
from migrator.core.utils.networking.run_command import LocalUnixRunner


class MigratorServer(object):
    """Class that presents server where the migration tools are running"""
    def __init__(self, config):
        if config is not None:
            session_dir_path = get_local_session_dir()
            self._session_dir = session_dir.UnixSessionDir(self.runner, session_dir_path)
            common_dir_path = get_common_local_session_dir()
            self.common_dir = session_dir.UnixSessionDir(self.runner, common_dir_path)

    def get_session_file_path(self, filename):
        """Return full path to a temporary file on the server by
        short file name. Function considers session directory settings
        and ensures that session directory exists on the server"""
        return self._session_dir.get_file_path(filename)

    def session_dir(self):
        """Return path to session directory - directory where
        we should store all temporary files"""
        return self._session_dir.get_session_dir_path()

    @contextmanager
    def runner(self):
        """Get runner object to execute commands on the server"""
        yield LocalUnixRunner()

    def description(self):
        return _T("MIGRATOR_SERVER_DESCRIPTION")

