# (C)2017 Ingram Micro Inc. all Rights Reserved
class WriterBase(object):
    """Saves information from data models hierarchy to data storage"""

    def write(self, model):
        """

        :param model: migrator.core.utils.lang.entity.Entity
        """
        raise NotImplementedError()


class ReaderBase(object):
    """Reads information from data storage to data models"""

    def read(self, model_entity_class, id):
        """

        :return migrator.core.utils.lang.entity.Entity
        """
        raise NotImplementedError()

    def read_all(self, model_entity_class):
        """

        :return [migrator.core.utils.lang.entity.Entity]
        """
        raise NotImplementedError()


class PagedReader(object):
    """Provides paging implementation for Reader classes"""
    DEFAULT_PAGE_SIZE = 200

    def __init__(self):
        self.page_size = self.DEFAULT_PAGE_SIZE

    def _paginate(self, object_list):
        """Split a list into a series of batches."""
        batch_start_indices = xrange(0, len(object_list), self.page_size)
        for index in batch_start_indices:
            yield object_list[index:index + self.page_size]

    def _get_page_size(self):
        return self._page_size

    def _set_page_size(self, page_size):
        self._page_size = page_size

    page_size = property(_get_page_size, _set_page_size)