# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.utils import parse_bool
from migrator.core.utils.lang.entity import Entity


class MailHostingModel(Entity):
    data_prefix = 'mail_hosting_shm'

    def __init__(self, subscriptions=None):
        """

        :type subscriptions: list [SubscriptionModel]
        """
        self.subscriptions = subscriptions or []


class SubscriptionModel(Entity):
    data_prefix = 'subscription'

    def __init__(self, external_id, webspaces=None):
        """

        :type external_id: str
        :type webspaces: list [WebspaceModel]
        """
        self.external_id = external_id
        self.webspaces = webspaces or []


class WebspaceModel(Entity):
    data_prefix = 'webspace'

    def __init__(self, external_id, sites=None):
        """

        :type external_id: str
        :type sites: list [SiteModel]
        """
        self.external_id = external_id
        self.sites = sites or []


class SiteModel(Entity):
    data_prefix = 'site'

    def __init__(self, external_id, name, mailboxes=None):
        """

        :type external_id: str
        :type name: str
        :type mailboxes: list [MailboxModel]
        """
        self.external_id = external_id
        self.name = name
        self.mailboxes = mailboxes or []


class MailboxModel(Entity):
    data_prefix = 'mailbox'

    def __init__(
        self, external_id, name, password, is_password_generated, is_access_to_customer_panel, disk_quota,
        antivirus, alias=None, autoresponder=None, forwarding=None, spam_filter=None
    ):
        """

        :type external_id: str
        :type name: str
        :type password: str
        :type is_password_generated: bool
        :type is_access_to_customer_panel: bool
        :type disk_quota: int
        :type antivirus: str
        :type alias: AliasModel
        :type autoresponder: AutoresponderModel
        :type forwarding: ForwardingModel
        :type spam_filter: SpamFilterModel

        """
        self.external_id = external_id
        self.name = name
        self.password = password
        self.is_password_generated = parse_bool(is_password_generated)
        self.is_access_to_customer_panel = parse_bool(is_access_to_customer_panel)
        self.disk_quota = int(disk_quota)
        self.alias = alias
        self.autoresponder = autoresponder
        self.antivirus = antivirus
        self.forwarding = forwarding
        self.spam_filter = spam_filter


class AutoresponderModel(Entity):
    data_prefix = 'autoresponder'

    def __init__(self, subject=None, message=None, forward_to=None, end_date=None, enabled=False):
        """

        :type subject: str
        :type message: str
        :type forward_to: str
        :type end_date: str
        """
        self.subject = subject
        self.message = message
        self.forward_to = forward_to
        self.end_date = end_date
        self.enabled = parse_bool(enabled)


class ForwardingModel(Entity):
    data_prefix = 'forwarding'

    def __init__(self, addresses=None):
        """

        :type addresses: list[AddressModel]
        """
        self.addresses = addresses


class AliasModel(Entity):
    data_prefix = 'alias'

    def __init__(self, addresses=None):
        """

        :type addresses: list[AddressModel]
        """
        self.addresses = addresses


class SpamFilterModel(Entity):
    data_prefix = 'spam_filter'

    def __init__(self, sensitivity, rewrite_header, action, blacklist=None, whitelist=None):
        """

        :type sensitivity: float or None
        :type rewrite_header: str or None
        :type action: str or None
        :type blacklist: list[AddressModel]
        :type whitelist: list[AddressModel]
        """
        self.sensitivity = sensitivity
        self.rewrite_header = rewrite_header
        self.action = action
        self.blacklist = blacklist or []
        self.whitelist = whitelist or []


class AddressModel(Entity):
    data_prefix = 'address'

    def __init__(self, address=None):
        """

        :type address: string
        """
        self.address = address
