# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.data.format.billing import BillingReader
from migrator.core.data.model.billing import *
from migrator.core.data.model.account import AddressModel, PhoneModel
from migrator.core.data.source.xml import CommonStreamWriterXml, CommonReaderXml
from migrator.settings import OA_MIGRATOR_BILLING_NAMESPACE, OA_MIGRATOR_BILLING_XSD

logger = logging.getLogger(__name__)

DATA_TYPE = 'billing'


class BillingWriterXml(CommonStreamWriterXml):
    def __init__(self, filename):
        super(BillingWriterXml, self).__init__(filename)

        self._data_type = DATA_TYPE
        self._xml_namespace = OA_MIGRATOR_BILLING_NAMESPACE
        self._schema_location = self.SCHEMA_LOCATION_TEMPLATE.format(
            xml_namespace=OA_MIGRATOR_BILLING_NAMESPACE, xsd_file=OA_MIGRATOR_BILLING_XSD
        )


class BillingReaderXml(CommonReaderXml, BillingReader):
    def _init_reader(self):
        self._class_model_list = [
            AddressModel, PhoneModel, SourceBillingSystemModel, AccountBillingModel, ResellerBillingModel,
            BankAccountModel, CreditCardModel, CreditCardInfoModel, SubscriptionBillingModel
        ]
        self._array_tags = ['resellers', 'credit_cards', 'accounts', 'bank_accounts', 'subscriptions']
        self._additional_data_prefixes = {
            'fax': PhoneModel
        }
        self._choice_tags = []
        self._data_type = DATA_TYPE
        self._xml_namespace = OA_MIGRATOR_BILLING_NAMESPACE
        self._xsd_path = OA_MIGRATOR_BILLING_XSD

    def get_subscription(self, subscription_id):
        return self.read_model([(SubscriptionBillingModel, 'external_id', subscription_id)])

    def get_reseller(self, reseller_id):
        return self.read_model([(ResellerBillingModel, 'external_id', reseller_id)])

    def get_account(self, account_id):
        return self.read_model([(AccountBillingModel, 'external_id', account_id)])
