# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.data.format.mail_hosting_shm import ShmMailHostingReader
from migrator.core.data.model.mail_hosting_shm import \
    SubscriptionModel, WebspaceModel, SiteModel, MailboxModel, MailHostingModel, AutoresponderModel, ForwardingModel, \
    AliasModel, SpamFilterModel
from migrator.core.data.source.xml import CommonReaderXml, CommonWriterXml
from migrator.settings import OA_MIGRATOR_MAIL_HOSTING_SHM_NAMESPACE, OA_MIGRATOR_MAIL_HOSTING_SHM_XSD

DATA_TYPE = 'WebHosting Plesk mail hosting'


class ShmMailHostingWriterXml(CommonWriterXml):
    def __init__(self, filename):
        super(ShmMailHostingWriterXml, self).__init__(filename)

        self._data_type = DATA_TYPE
        self._xml_namespace = OA_MIGRATOR_MAIL_HOSTING_SHM_NAMESPACE
        self._schema_location = self.SCHEMA_LOCATION_TEMPLATE.format(
            xml_namespace=OA_MIGRATOR_MAIL_HOSTING_SHM_NAMESPACE, xsd_file=OA_MIGRATOR_MAIL_HOSTING_SHM_XSD
        )
        self._primitive_array_element_tag = {
            'addresses': 'address',
            'blacklist': 'address',
            'whitelist': 'address',
        }


class ShmMailHostingReaderXml(CommonReaderXml, ShmMailHostingReader):
    def _init_reader(self):
        self._class_model_list = [
            SubscriptionModel, WebspaceModel, SiteModel, MailboxModel, MailHostingModel, AutoresponderModel,
            ForwardingModel, AliasModel, SpamFilterModel
        ]
        self._array_tags = ['subscriptions', 'webspaces', 'sites', 'mailboxes']
        self._primitive_arrays = ['addresses', 'blacklist', 'whitelist']
        self._additional_data_prefixes = {}
        self._choice_tags = []
        self._data_type = DATA_TYPE
        self._xml_namespace = OA_MIGRATOR_MAIL_HOSTING_SHM_NAMESPACE
        self._xsd_path = OA_MIGRATOR_MAIL_HOSTING_SHM_XSD

    def get_subscription(self, subscription_id):
        return self.read_model((SubscriptionModel, 'external_id', subscription_id))

    def get_webspace(self, subscription_id, webspace_id):
        return self.read_model(
            [
                (SubscriptionModel, 'external_id', subscription_id),
                (WebspaceModel, 'external_id', webspace_id),
            ]
        )

    def get_webspaces_list(self, subscription_id):
        subscription = self.get_subscription(subscription_id)
        return [] if subscription is None else subscription.webspaces

    def get_site(self, subscription_id, webspace_id, site_id):
        return self.read_model(
            [
                (SubscriptionModel, 'external_id', subscription_id),
                (WebspaceModel, 'external_id', webspace_id),
                (SiteModel, 'external_id', site_id)
            ]
        )

    def get_sites_list(self, subscription_id, webspace_id):
        webspace = self.get_webspace(subscription_id, webspace_id)
        return [] if webspace is None else webspace.sites
