# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.data.format.web_hosting_shm import ShmWebHostingReader
from migrator.core.data.model.web_hosting_shm import *
from migrator.core.data.source.xml import CommonWriterXml, CommonReaderXml
from migrator.settings import OA_MIGRATOR_WEB_HOSTING_SHM_NAMESPACE, OA_MIGRATOR_WEB_HOSTING_SHM_XSD

logger = logging.getLogger(__name__)


DATA_TYPE = 'WebHosting Plesk web hosting'


class ShmWebHostingWriterXml(CommonWriterXml):
    def __init__(self, filename):
        super(ShmWebHostingWriterXml, self).__init__(filename)

        self._data_type = DATA_TYPE
        self._xml_namespace = OA_MIGRATOR_WEB_HOSTING_SHM_NAMESPACE
        self._schema_location = self.SCHEMA_LOCATION_TEMPLATE.format(
            xml_namespace=OA_MIGRATOR_WEB_HOSTING_SHM_NAMESPACE, xsd_file=OA_MIGRATOR_WEB_HOSTING_SHM_XSD
        )
        self._choice_tags = ['hosting']
        self._primitive_array_element_tag = {
            'alt_hosts': 'alt_host',
        }


class ShmWebHostingReaderXml(CommonReaderXml, ShmWebHostingReader):
    def _init_reader(self):
        self._class_model_list = [
            SubscriptionModel, WebspaceModel, SystemUser, IpAddressModel, SiteModel, PhysicalHostingModel,
            DatabaseModel, ForwardingStdModel, AliasModel, WebuserModel, FtpuserModel, ProtectedDirectoryModel,
            PropertyModel, HostingPropertiesModel, ScriptingModel, PHPSettingsModel, DatabaseServerModel,
            DatabaseUserModel, ProtectedDirectoryUserModel, NoHostingModel, SslCertificateModel, ApplicationPoolModel,
            ApplicationModel, VirtualDirectoryModel
        ]
        self._array_tags = [
            'webspaces', 'ip_addresses', 'sites', 'databases', 'protected_directories', 'properties', 'database_users',
            'database_servers', 'ftp_users', 'web_users', 'pd_users', 'applications', 'virtual_directories'
        ]
        self._additional_data_prefixes = {}
        self._primitive_arrays = ['alt_hosts']
        self._choice_tags = ['hosting']
        self._data_type = DATA_TYPE
        self._xml_namespace = OA_MIGRATOR_WEB_HOSTING_SHM_NAMESPACE
        self._xsd_path = OA_MIGRATOR_WEB_HOSTING_SHM_XSD

    def get_subscription(self, subscription_id):
        return self.read_model([(SubscriptionModel, 'external_id', subscription_id)])

    def get_webspaces_list(self, subscription_id):
        subscription = self.get_subscription(subscription_id)
        return [] if subscription is None else subscription.webspaces

    def get_webspace(self, subscription_id, webspace_id):
        return self.read_model(
            [
                (SubscriptionModel, 'external_id', subscription_id),
                (WebspaceModel, 'external_id', webspace_id),
            ]
        )

    def get_database_server(self, database_server_id):
        return self.read_model([(DatabaseServerModel, 'id', database_server_id)])

    def get_sites_list(self, subscription_id, webspace_id):
        webspace = self.get_webspace(subscription_id, webspace_id)
        return [] if webspace is None else webspace.sites

    def get_site(self, subscription_id, webspace_id, site_id):
        return self.read_model(
            [
                (SubscriptionModel, 'external_id', subscription_id),
                (WebspaceModel, 'external_id', webspace_id),
                (SiteModel, 'external_id', site_id)
            ]
        )

