# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.utils.lang.entity import Entity


class DatabaseConnectionInfo(Entity):
    """Connection information to particular database - host, port, login, password, database name"""

    def __init__(self, host, port, login, password, db_name):
        """
        :type host: basestring
        :type port: basestring
        :type login: basestring
        :type password: basestring
        :type db_name: basestring
        """
        self._host = host
        self._port = port
        self._login = login
        self._password = password
        self._db_name = db_name

    @property
    def host(self):
        """
        :rtype: basestring
        """
        return self._host

    @property
    def port(self):
        """
        :rtype: basestring
        """
        return self._port

    @property
    def login(self):
        """
        :rtype: basestring
        """
        return self._login

    @property
    def password(self):
        """
        :rtype: basestring
        """
        return self._password

    @property
    def db_name(self):
        """
        :rtype: basestring
        """
        return self._db_name


def get_db_connection_info(server, db_name):
    """Return an object that defines database connection.

    :type server: parallels.core.connections.database_server.DatabaseServer
    :type db_name: basestring
    :rtype: parallels.core.utils.database_utils.DatabaseConnectionInfo
    """
    return DatabaseConnectionInfo(
        host=server.host(), port=server.port(), login=server.user(),
        password=server.password(), db_name=db_name
    )


