# (C)2017 Ingram Micro Inc. all Rights Reserved
import os
import sys

from argparse import ArgumentParser

from lib.core.ctl import get_agent_ctl_class
from lib.core.config import AGENT_CONFIG_FILE, AgentConfig

LIBDIR = os.path.realpath(os.path.dirname(__file__))


def get_argument_parser():
    parser = ArgumentParser()

    parser.add_argument(
        'command', choices=['start', 'stop', 'restart', 'status'], help='Send command to migration agent daemon'
    )
    parser.add_argument('-f', '--force', action='store_true', help='Force start, stop or restart operation')

    return parser


def main(argv):
    parser = get_argument_parser()
    if not argv:
        parser.print_help()
        return 2

    msg = ''
    exit_code = 0

    config = AgentConfig(os.path.join(LIBDIR, 'conf', AGENT_CONFIG_FILE), os.path.realpath(LIBDIR))
    agent = get_agent_ctl_class()(config)

    args = parser.parse_args(argv)
    if args.command == 'start':
        exit_code, msg = agent.start(args.force)
    elif args.command == 'stop':
        exit_code, msg = agent.stop(args.force)
    elif args.command == 'status':
        exit_code, msg = agent.status()
    elif args.command == 'restart':
        return restart(agent, args.force)

    print(msg)

    return exit_code


def restart(agent, force=False):
    exit_code, msg = agent.stop(force=force)
    print(msg)

    if exit_code:
        return exit_code

    exit_code, msg = agent.start(force=force)
    print(msg)

    return exit_code


if __name__ == '__main__':
    try:
        sys.exit(main(sys.argv[1:]))
    except Exception as e:
        sys.stderr.write('ERROR: {0}'.format(e))
        sys.stderr.flush()
        sys.exit(1)
