# (C)2017 Ingram Micro Inc. all Rights Reserved
import os
import ssl

from SocketServer import ThreadingMixIn
from jsonrpclib.SimpleJSONRPCServer import SimpleJSONRPCServer

from lib.core import utils
from lib.core.agent_api import AgentAPI
from lib.core.pidfile import TimeoutPIDLockFile, DEFAULT_PIDFILE_TIMEOUT


# TODO: HTTP Basic Auth support
class AgentServer(ThreadingMixIn, SimpleJSONRPCServer):
    def __init__(self, config):
        self._config = config

        port = int(self._config['port'])
        SimpleJSONRPCServer.__init__(self, ('', port), bind_and_activate=False)

        if int(self._config['use_ssl']):
            self.socket = ssl.wrap_socket(
                self.socket, server_side=True, cert_reqs=ssl.CERT_REQUIRED,
                keyfile=self._config['agent_key'], certfile=self._config['agent_cert'],
                ca_certs=self._config['client_cert']
            )

        self.server_bind()
        self.server_activate()


class Agent(object):
    def __init__(self, config):
        self._config = config

        self._pidfile_path = config['pid_file']
        timeout = config.get('pid_file_timeout', DEFAULT_PIDFILE_TIMEOUT)
        self._pidfile = TimeoutPIDLockFile(self._pidfile_path, timeout)

    def run(self):
        self._daemonize()

        with self._pidfile:
            server = self._get_server_instance()
            server.serve_forever()

    def _get_server_instance(self):
        api = AgentAPI()
        srv = AgentServer(self._config)
        srv.register_instance(api)

        return srv

    def _daemonize(self):
        raise NotImplementedError()


def get_agent_class():
    if utils.is_windows():
        from .platform.win32.agent_impl import AgentWindows
        return AgentWindows
    else:
        from .platform.linux.agent_impl import AgentLinux
        return AgentLinux


def redirect_stream(system_stream, target_stream):
    if target_stream is None:
        target_fd = os.open(os.devnull, os.O_RDWR)
    else:
        target_fd = target_stream.fileno()

    os.dup2(target_fd, system_stream.fileno())
