# (C)2017 Ingram Micro Inc. all Rights Reserved
from SimpleXMLRPCServer import resolve_dotted_attribute

from lib.api.common import CommonAPI
try:
    from lib.api.product import ProductAPI
    _product_api = ProductAPI()
except ImportError:
    _product_api = None

_common_api = CommonAPI()


class AgentAPI(object):
    """
    Dispatcher class to provide access to both common and product agent api.

    Common api is always available.
    Product api is accessible only if present.

    Example:
        import jsonrpclib

        agent = jsonrpclib.Server('http://<ip>:29443')
        res1 = agent.common.<method>(...)
        res2 = agent.product.<method>(...)
    """
    def _dispatch(self, method, params):
        func = None

        try:
            instance, _method = self._find_method(method)
            func = resolve_dotted_attribute(instance, _method, False)
        except AttributeError:
            pass

        if func is not None:
            return func(*params)
        else:
            raise Exception('method "%s" is not supported' % method)

    @staticmethod
    def _find_method(method):
        if '.' not in method:
            raise AttributeError()

        ns, _method = method.split('.', 1)
        if ns == 'common':
            return _common_api, _method
        elif ns == 'product' and _product_api:
            return _product_api, _method
        else:
            raise AttributeError()
