# (C)2017 Ingram Micro Inc. all Rights Reserved
from ConfigParser import RawConfigParser
from . import AgentConfigError

AGENT_CONFIG_FILE = 'agent.config'


class AgentConfig(dict):
    _required_options = {
        'agent_cmd', 'pid_file', 'log_file', 'port', 'use_ssl', 'agent_key', 'agent_cert', 'client_cert'
    }
    _optional_options = {'agent_dir', 'pid_file_timeout', 'password'}

    def __init__(self, config_file, default_agent_dir):
        super(AgentConfig, self).__init__()

        config = RawConfigParser()
        if not config.read(config_file):
            raise AgentConfigError('Failed to read agent configuration file {0}: no such file'.format(config_file))

        self.filename = config_file

        sections = config.sections()
        if 'agent' not in sections:
            raise AgentConfigError(
                "Invalid agent configuration file {0}: section 'agent' is missing".format(config_file)
            )

        options = set(config.options('agent'))
        missing_required_options = self._required_options - (self._required_options & options)
        if missing_required_options:
            raise AgentConfigError(
                "Invalid agent configuration file {0}: required option(s) missing: {1}".format(
                    config_file, ', '.join(missing_required_options)
                )
            )

        for k in self._required_options | self._optional_options:
            if config.has_option('agent', k):
                self[k] = config.get('agent', k)

        agent_dir = self.get('agent_dir', default_agent_dir)
        for k, v in self.iteritems():
            if v:
                self[k] = v.format(agent_dir=agent_dir)

        self['agent_dir'] = agent_dir
