# (C)2017 Ingram Micro Inc. all Rights Reserved
import sys
import logging
import subprocess as sp

from . import AgentExecError

logger = logging.getLogger(__name__)


def is_windows():
    return sys.platform == 'win32'


def run_cmd_raw(command, stdin=None, env=None):
    _stdin = sp.PIPE if stdin else None
    _shell = False if isinstance(command, (tuple, list)) else True
    _stdout = _stderr = sp.PIPE

    logger.debug('EXEC: %s', command)
    logger.debug('EXEC (stdin): %s', stdin and '\n%s' % stdin)

    p = sp.Popen(command, stdin=_stdin, stdout=_stdout, stderr=_stderr, env=env, shell=_shell)
    out, err = p.communicate(stdin)

    logger.debug('EXEC (stdout): %s', out and '\n%s' % out)
    logger.debug('EXEC (stderr): %s', err and '\n%s' % err)

    logger.debug('EXEC (exit): %s', p.returncode)

    return p.returncode, out, err


def run_cmd(command, stdin=None, env=None, valid_exit_codes=None):
    if valid_exit_codes is None:
        valid_codes = [0]
    else:
        valid_codes = valid_exit_codes

    exit_code, out, err = run_cmd_raw(command, stdin=stdin, env=env)
    if exit_code not in valid_codes:
        raise AgentExecError('Failed to execute command: %s' % command)

    return out, err


def run(command, stdin=None):
    return run_cmd(command, stdin=stdin)
