# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator import settings
from migrator.core.language import _T
from migrator.core.product.panel import UnknownPanelInfo
from migrator.core.utils.lang.loader import load_class
from migrator.settings import WINEXE_GET_SYSTEM_INFO_COMMAND_TIMEOUT


def get_service_status_linux(runner, service_name):
    """Get status of a Linux service
    Return tuple of:
    - status string , one of: 'Running', 'Stopped', 'Not installed'
    - exit code returned by the service checking command
    - status command stdout
    """
    exit_code, stdout, stderr = runner.sh_unchecked(u'service %s status' % service_name)
    stdout = stdout.lower()
    stderr = stderr.lower()

    initd_service_is_running = exit_code == 0
    initd_service_loaded = ('is stopped' in stdout or 'unrecognized service' not in stderr)
    systemd_service_loaded = 'loaded: loaded' in stdout

    if initd_service_is_running:
        status_string = _T("SERVICE_STATUS_RUNNING")
    elif initd_service_loaded or systemd_service_loaded:
        status_string = _T("SERVICE_STATUS_STOPPED")
    else:
        status_string = _T("SERVICE_STATUS_NOT_INSTALLED")

    return status_string, exit_code, stdout


def get_service_status_windows(runner, service_name):
    """Return Windows service status.

    Unlike Linux services, Windows service status command does not return exit code.
    We return '0' for started, '1' for stopped Windows service.
    """
    STARTED_EXIT_CODE, STOPPED_EXIT_CODE = 0, 1
    with runner.set_timeout(WINEXE_GET_SYSTEM_INFO_COMMAND_TIMEOUT):
        result = runner.sh(u'sc query %s' % service_name).lower()
    if 'running' in result and 'not running' not in result:
        return _T("SERVICE_STATUS_RUNNING"), STARTED_EXIT_CODE

    return _T("SERVICE_STATUS_STOPPED"), STOPPED_EXIT_CODE


def get_panel_info_instance(runner, config_section):
    """Detect source panel product and create the corresponding PanelInfo object"""

    for product_name in settings.SUPPORTED_SOURCE_PRODUCTS:
        panel_package_name = '%s.%s.%s' % (
            settings.SOURCE_PRODUCT_PACKAGE_PREFIX,
            product_name,
            settings.SOURCE_PRODUCT_PACKAGE_POSTFIX
        )
        panel_info_class = load_class(panel_package_name, 'PanelInfo', optional=True)

        if panel_info_class is not None and panel_info_class.is_installed(runner):
            # XXX should we try to detect all panels and warn user if there is more than one matches?
            return panel_info_class.factory(runner, config_section)

    return UnknownPanelInfo(None)
