# (C)2017 Ingram Micro Inc. all Rights Reserved
import ssl
import urllib
import logging

from jsonrpclib import jsonrpc, Server

from migrator.core import MigrationError
from migrator.core.language import _T

logger = logging.getLogger(__name__)


class SafeTransportWithCert(jsonrpc.SafeTransport):
    def __init__(self, cert, pkey):
        super(SafeTransportWithCert, self).__init__()

        self._cert_file = cert
        self._key_file = pkey

    def make_connection(self, host):
        host_with_cert = (host, {'key_file': self._key_file, 'cert_file': self._cert_file})
        return jsonrpc.SafeTransport.make_connection(self, host_with_cert)


class JsonRpcClient(Server):
    def __init__(self, uri, ssl_info=None):
        schema, _ = urllib.splittype(uri)
        if schema == 'https':
            transport = SafeTransportWithCert(ssl_info['cert_file'], ssl_info['key_file'])
            transport.context = ssl._create_unverified_context()
        else:
            transport = None

        logger.debug('Initializing JSON RPC Client for server at %s', uri)
        Server.__init__(self, uri, transport=transport)

    def run(self, cmd, args=None, stdin=None):
        cmd = cmd if not args else ([cmd] + args)
        exit_code, stdout, stderr = self.common.run_unchecked(cmd, stdin)
        if exit_code != 0:
            raise MigrationError(_T("INTERNAL_ERROR_OCCURRED_DURING_EXECUTION_COMMAND"))

        return stdout

    def sh_unchecked(self, cmd, args=None, stdin=None):
        cmd = cmd if not args else ([cmd] + args)
        return self.common.run_unchecked(cmd, stdin)
