# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator import settings
from migrator.core import MigrationError
from migrator.core.session.migration_progress import MigrationProgress
from migrator.core.utils.lang.loader import load_class


class GlobalMigrationContext(object):
    """
    :type target_panel_obj: parallels.core.target_panel_base.TargetPanelBase
    :type import_api: parallels.core.import_api.import_api.ImportAPI
    :type progress: parallels.core.utils.migration_progress.MigrationProgress
    :type pre_check_report: parallels.core.checking.Report
    :type session_files: parallels.core.session_files.CommonSessionFiles
    """
    def __init__(self):
        self.conn = None  # connections to source and target servers
        self.session_files = None  # utility class to construct paths to session files
        self.migrator_server = None  # instance of MigratorServer
        self.import_api = None
        self.execution_wrapper = None
        self.target_panel_type = None
        self.target_panel_obj = None
        self.source_servers = None
        self.migration_list_data = None
        self.ip_mapping = None
        self.iter_all_subscriptions = None
        self.options = None
        self.config = None
        self.target_existing_objects = None
        self.hosting_analyzer_enabled = True
        self.post_migration_checks_enabled = True
        self.ssh_key_pool = None
        self.rsync_pool = None
        self.get_rsync = None
        self._progress = None
        self._source_product_instance = None
        self.migration_project_aps_id = None
        self.migration_type = None
        self.migration_mode = None

        self.dns_forwarding_report = None
        self.pre_check_report = None
        self.post_migration_check_report = None
        self.test_services_report = None
        # List of dumps agents that were deployed during this migration tool's execution
        # Fill the list to uninstall agent once migration was finished
        self.deployed_dump_agents = []
        self.created_mailbox_without_restoring_password = []

        # XXX Migrator object, for legacy actions - do not use for new code
        # avoid usage for old code
        self.migrator = None

    @property
    def progress(self):
        if self._progress is None:
            self._progress = MigrationProgress()

        return self._progress

    @property
    def source_type(self):
        return self.config.get('GLOBAL', 'source-type').lower()

    @property
    def source_product(self):
        """Return an instance of source product; create one, if not already exists."""
        if self._source_product_instance is None:
            product_name = self.source_type
            if product_name not in settings.SUPPORTED_SOURCE_PRODUCTS:
                raise MigrationError('Specified source product is not supported: "%s"' % product_name)

            panel_package_name = '%s.%s.%s' % (
                settings.SOURCE_PRODUCT_PACKAGE_PREFIX,
                product_name,
                settings.SOURCE_PRODUCT_PACKAGE_POSTFIX
            )
            self._source_product_instance = load_class(panel_package_name, 'Product')

        return self._source_product_instance
