# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.language import _T

logger = logging.getLogger(__name__)


class ContentListReaderBase(object):
    def get_subscription_content(self, global_context, subscription):
        """

        :rtype: list [migrator.core.actions.content.web.copy_content_base.ContentUnit]
        :type subscription: migrator.odin_automation.platform.lib.migrated_subscription.MigratedSubscription
        """
        try:
            for content_migrator, content_list in self._get_content_list_and_migrator(global_context, subscription):
                items = []
                for contexted_content in content_list:
                    self._substitute_content_macros(
                        contexted_content.items, contexted_content.context_macro_definition
                    )
                    items.extend(contexted_content.items)

                yield [items, content_migrator]
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            raise Exception(
                _T("FAILED_TO_RETRIEVE_CONTENT_LIST").format(subscription_name=subscription.source_info.name)
            )

    def _get_content_list_and_migrator(self, global_context, subscription):
        """

        :type subscription: migrator.odin_automation.platform.lib.migrated_subscription.MigratedSubscription
        :rtype: list[ContextedContent]
        """
        raise NotImplementedError()

    def _substitute_content_macros(self, content, context_macro_definition):
        """

        :type content: list[migrator.core.data.model.web_content_shm.ContentItemModel]
        :type context_macro_definition: migrator.core.transfer.context_macro_definition.ContextMacroDefinition
        """
        for item in content:
            self._substitute_target_path_macros(item, context_macro_definition)
            self._substitute_permissions_macros(item, context_macro_definition)
            self._substitute_filters_macros(item, context_macro_definition)

    def _substitute_target_path_macros(self, item, context_macro_definition):
        """

        :type item: migrator.core.data.model.web_content_shm.ContentItemModel
        :type context_macro_definition: migrator.core.transfer.context_macro_definition.ContextMacroDefinition
        """
        item.target_path = context_macro_definition.substitute(item.target_path)

    def _substitute_permissions_macros(self, item, context_macro_definition):
        """

        :type item: migrator.core.data.model.web_content_shm.ContentItemModel
        :type context_macro_definition: migrator.core.transfer.context_macro_definition.ContextMacroDefinition
        """
        if item.permissions and item.permissions.owners:
            for owner in item.permissions.owners:
                if owner.user:
                    owner.user = context_macro_definition.substitute(owner.user)
                if owner.group:
                    owner.group = context_macro_definition.substitute(owner.group)

    def _substitute_filters_macros(self, item, context_macro_definition):
        """

        :type item: migrator.core.data.model.web_content_shm.ContentItemModel
        :type context_macro_definition: migrator.core.transfer.context_macro_definition.ContextMacroDefinition
        """
        item.include = self._get_substituted_filters(item.include, context_macro_definition)
        item.exclude = self._get_substituted_filters(item.exclude, context_macro_definition)

    def _get_substituted_filters(self, filters, context_macro_definition):
        """

        :type filters: [str]
        :type context_macro_definition: migrator.core.transfer.context_macro_definition.ContextMacroDefinition
        """
        if filters is not None:
            substituted_filters = []
            for path in filters:
                substituted_filters.append(context_macro_definition.substitute(path))

            return substituted_filters
        else:
            return None


class ContextedContent(object):
    def __init__(self, items, context_macro_definition):
        """

        :type items: list [migrator.core.data.model.web_content_shm.ContentItemModel]
        :type context_macro_definition: migrator.core.transfer.context_macro_definition.ContextMacroDefinition
        """
        self.items = items
        self.context_macro_definition = context_macro_definition
