# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import MigrationNoRepeatError
from migrator.core.language import _T
from migrator.core.transfer.content_migrator import get_content_migrator
from migrator.core.transfer.db.common import DatabaseContentTransfer, DatabaseTransferOptions
from migrator.core.transfer.db.mssql import MSSqlTransferConfiguration
from migrator.core.transfer.db.mysql import MysqlTransferConfiguration
from migrator.core.transfer.db.pgsql import PostgresqlTransferConfiguration
from migrator.core.utils.loggers.log import trace
from migrator.settings import DatabaseServerType

logger = logging.getLogger(__name__)


@trace
def copy_db_content(global_context, database_info):
    source = database_info.source_database_server
    target = database_info.target_database_server
    database_name = database_info.database_name
    target_database_name = database_info.target_database_name
    webspace = database_info.webspace

    content_migrator = get_content_migrator(global_context, None, source.utilities_server, database_info.plesk_target_server, None)
    transfer_options = DatabaseTransferOptions(content_migrator=content_migrator)

    logger.info(_T("LOG_COPY_DB_CONTENT").format(
        db_name=database_name, source=source.description(), target=target.description()
    ))

    if source.type() == DatabaseServerType.MYSQL:
        configuration = MysqlTransferConfiguration(database_name, target_database_name, webspace)
    elif source.type() == DatabaseServerType.MSSQL:
        configuration = MSSqlTransferConfiguration(database_name, target_database_name, webspace)
    elif source.type() == DatabaseServerType.POSTGRESQL:
        configuration = PostgresqlTransferConfiguration(database_name, target_database_name, webspace)
    else:
        raise MigrationNoRepeatError(_T("CANNOT_TRANSFER_DATABASE_OF_UNSUPPORTED_TYPE") % source.type())

    transfer_processor = DatabaseContentTransfer(configuration, transfer_options)
    transfer_processor.transfer(source, target, database_info.plesk_target_server)
