# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.utils.operating_system import windows_utils


class BaseWebPathConverter(object):
    """Class to convert abstract path descriptor to concrete absolute path"""

    def expand(self, path):
        """Convert abstract path descriptor to concrete absolute path

        :type path: parallels.core.utils.web_paths.WebHostingPath
        :rtype: str | unicode
        """
        raise NotImplementedError()


class WindowsRsyncPathConverter(BaseWebPathConverter):
    """Path converter, adding conversion from Windows native to cygwin rsync format

    So all paths like:
    c:\windows\system32
    will be converted to
    /cygdrive/c/windows/system32
    """

    def expand(self, path):
        """Convert abstract path descriptor to concrete absolute path, in rsync (cygwin) format

        :type path: string Example: 'C:\oam-session\V87YPn0x7y'
        :rtype: str|unicode Example: '/cygdrive/c/oam-session/V87YPn0x7y'
        """
        return windows_utils.convert_path_to_cygwin(path)
