# (C)2017 Ingram Micro Inc. all Rights Reserved
import os
import copy
import tempfile
import logging

from cryptography.x509.oid import NameOID

from migrator.core.utils.filetypes.ssl_certificate import SSLCertificate
from migrator.settings import OA_MIGRATOR_SSL_CERT, OA_MIGRATOR_SSL_PKEY

logger = logging.getLogger(__name__)

_dn = {
    NameOID.COUNTRY_NAME: u'US',
    NameOID.STATE_OR_PROVINCE_NAME: u'Odin',
    NameOID.LOCALITY_NAME: u'Odin',
    NameOID.ORGANIZATION_NAME: u'Odin',
    NameOID.ORGANIZATIONAL_UNIT_NAME: u'OAM',
    NameOID.COMMON_NAME: u'OAM Backend',
}


def get_oam_ssl_certificate():
    files = (OA_MIGRATOR_SSL_CERT, OA_MIGRATOR_SSL_PKEY)
    if not all([os.path.exists(filename) for filename in files]):
        cert = generate_ssl_certificate(_dn, OA_MIGRATOR_SSL_CERT, OA_MIGRATOR_SSL_PKEY)
    else:
        cert = SSLCertificate.load(OA_MIGRATOR_SSL_CERT, OA_MIGRATOR_SSL_PKEY)

    return cert


def generate_ssl_certificate(dn, cert_filename, pkey_filename):
    logger.debug('Generating new SSL certificate (%s, %s)', cert_filename, pkey_filename)

    cert = SSLCertificate.generate(dn)
    cert.save(cert_filename, pkey_filename)
    return cert


def generate_agent_ssl_certificate():
    agent_dn = copy.deepcopy(_dn)
    agent_dn[NameOID.COMMON_NAME] = u'OAM Agent'

    cert_filename = _new_tempfile()
    pkey_filename = _new_tempfile()

    return generate_ssl_certificate(agent_dn, cert_filename, pkey_filename)


def _new_tempfile():
    with tempfile.NamedTemporaryFile(delete=False) as tmp:
        return tmp.name
