# (C)2017 Ingram Micro Inc. all Rights Reserved
from __future__ import absolute_import
from lxml import etree
from copy import deepcopy
from xml.dom import minidom
from cStringIO import StringIO

from migrator.core.language import _T


def seq(*items):
    return [item for item in items if item is not None]


def elem(name, children=None, attrs=None):
    if attrs is None:
        attrs = {}
    n = etree.Element(name, attrs)
    if children is not None:
        for child in children:
            if child is not None:
                n.append(child)

    return n


def text_elem(name, value, attrs=None):
    if value is None:
        return None
    if attrs is None:
        attrs = {}
    el = etree.Element(name, attrs)
    el.text = unicode(value)
    return el


def xml_bool(value):
    return 'true' if value else 'false'


def bool_elem(name, value):
    return text_elem(name, xml_bool(value))


def create_xml_stream(xml, xml_declaration=True):
    stream = StringIO()
    xml.write(stream, xml_declaration=xml_declaration, encoding='utf-8')
    stream.seek(0)
    return stream


def replace_root_node_if_name_matches(source_xml, root_tag_name, new_root_node):
    if source_xml.tag == root_tag_name:
        target_xml = deepcopy(new_root_node)
        for child in source_xml:
            target_xml.append(deepcopy(child))
        return target_xml
    else:
        return deepcopy(source_xml)


def prettify_xml(xml_string):
    """Prettify XML with indentation

    :type xml_string: basestring
    :rtype: str | unicode
    """
    xml = minidom.parseString(xml_string)
    return xml.toprettyxml(indent='  ', encoding='utf-8')


def xml_to_string_pretty(tree):
    """Serialize XML into a string, with pretty indentation

    :type tree: xml.etree.ElementTree.ElementTree
    :rtype: str | unicode
    """
    return prettify_xml(
        etree.tostring(tree.getroot(), encoding='utf-8', method='xml')
    )


def reporter(msg):
    raise Exception(msg)

# This one is borrowed from
# https://bitbucket.org/formencode/official-formencode/src/3be5078c6030/formencode/doctest_xml_compare.py#cl-7

def compare_xml(x1, x2, reporter=reporter):
    if x1.tag != x2.tag:
        if reporter:
            reporter(_T("XML_TAGS_DO_NOT_MATCH") % (x1.tag, x2.tag))
        return False
    for name, value in x1.attrib.items():
        if x2.attrib.get(name) != value:
            if reporter:
                reporter(
                    _T("XML_ATTRIBUTES_DO_NOT_MATCH") % (
                        name, value, name, x2.attrib.get(name)
                    )
                )
            return False
    for name in x2.attrib:
        if name not in x1.attrib:
            if reporter:
                reporter(_T("XML_ATTRIBUTE_MISSING") % name)
            return False
    if not text_compare(x1.text, x2.text):
        if reporter:
            reporter(_T("XML_TEXT_DIFFERS") % (x1.text, x2.text))
        return False
    if not text_compare(x1.tail, x2.tail):
        if reporter:
            reporter(_T("XML_TAIL_DIFFERS") % (x1.tail, x2.tail))
        return False
    cl1 = x1.getchildren()
    cl2 = x2.getchildren()
    if len(cl1) != len(cl2):
        if reporter:
            reporter(
                _T("XML_NUMBER_OF_CHILD_DIFFERS") % (
                    x1.tag, len(cl1), len(cl2)
                )
            )
        return False
    i = 0
    for c1, c2 in zip(cl1, cl2):
        i += 1
        if not compare_xml(c1, c2, reporter=reporter):
            if reporter:
                reporter(_T("XML_CHILDREN_DO_NOT_MATCH") % (i, c1.tag))
            return False
    return True


def text_compare(t1, t2):
    if not t1 and not t2:
        return True
    if t1 == '*' or t2 == '*':
        return True
    return (t1 or '').strip() == (t2 or '').strip()

