# (C)2017 Ingram Micro Inc. all Rights Reserved
import threading
from contextlib import contextmanager

thread_local = threading.local()


@contextmanager
def log_context(name):
    init_thread()
    thread_local.current_context.append(unicode(name))
    thread_local.current_context_str = '/'.join(
        thread_local.current_context
    )
    try:
        yield
    except Exception as e:
        if not hasattr(e, 'context'):
            e.context = thread_local.current_context_str
        raise
    finally:
        thread_local.current_context.pop()
        thread_local.current_context_str = '/'.join(
            thread_local.current_context
        )


@contextmanager
def subscription_context(subscription_name):
    """Context manager inside which it is considered that we are working with specified subscription"""
    init_thread()
    if thread_local.current_subscription_name == subscription_name:
        yield
    else:
        thread_local.current_subscription_name = subscription_name
        try:
            yield
        finally:
            thread_local.current_subscription_name = None


def init_thread():
    current_context = getattr(thread_local, 'current_context', None)
    if current_context is None:
        thread_local.current_context = []
        thread_local.current_subscription_name = None

    current_context_str = getattr(thread_local, 'current_context_str', None)
    if current_context_str is None:
        thread_local.current_context_str = ''
