# (C)2017 Ingram Micro Inc. all Rights Reserved
import re
import socket

from migrator.core.utils import unique_list
from migrator.core.utils.networking.ip_utils import is_ipv4, is_ipv6


def resolve(address):
    """Resolve hostname into IP address.

    Return the first IP address, or None if we were not able to resolve.
    If address is already a valid IP address - return it as is.
    """
    addresses = resolve_all(address)
    if len(addresses) == 0:
        return None
    else:
        return addresses[0]


def resolve_all(address):
    """Resolve hostname into list of IP addresses.

    If address is already a valid IP address - return
    list that contains only that address.
    Otherwise resolve and return list of IP addresses
    (IPv6 and IPv4 all in one list).
    If we were not able to resolve, empty list is returned."""
    if is_ipv4(address) or is_ipv6(address):
        return [address]
    else:
        try:
            ip_addresses = unique_list([
                ainfo[4][0] for ainfo in socket.getaddrinfo(address, None)]
            )
            return ip_addresses
        except socket.gaierror:
            return []


def normalize_domain_name(domain_name):
    """Normalize domain name - convert to lowercase punycode

    :rtype basestring
    """
    return domain_name.encode('idna').lower()


def convert_to_national_domain_name(domain_name):
    return normalize_domain_name(domain_name).decode('idna')


def convert_to_national_subscription_name(subscription_name):
    try:
        return subscription_name.encode('idna').decode('idna')
    except UnicodeError:
        return subscription_name


def is_valid_domain_name(domain_name):
    """ Checks whether a given value is a valid domain name and returns a result. """
    domain_validate_pattern = re.compile(
        r'^(([a-zA-Z]{1})|([a-zA-Z]{1}[a-zA-Z]{1})|'
        r'([a-zA-Z]{1}[0-9]{1})|([0-9]{1}[a-zA-Z]{1})|'
        r'([a-zA-Z0-9][-_.a-zA-Z0-9]{0,61}[a-zA-Z0-9]))\.'
        r'([a-zA-Z]{2,13}|[a-zA-Z0-9-]{2,30}.[a-zA-Z]{2,3})$'
    )
    return domain_validate_pattern.match(normalize_domain_name(domain_name)) is not None
