# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging
import requests

from lxml import etree
from StringIO import StringIO
from contextlib import closing
from requests_toolbelt.adapters.source import SourceAddressAdapter

from migrator.core.language import _T

# disable urllib3 SSL SNI support
requests.packages.urllib3.util.ssl_.HAS_SNI = False
# enable RC4 SSL cipher to support Windows 2003
requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS += ':RC4'
# disable urllib3 SSL cert verification and missing SNI warnings
requests.packages.urllib3.disable_warnings(requests.packages.urllib3.exceptions.InsecureRequestWarning)
requests.packages.urllib3.disable_warnings(requests.packages.urllib3.exceptions.SNIMissingWarning)


class HttpClientError(Exception):
    def __init__(self, cause, content):
        self.cause = cause
        self.content = content

    def __str__(self):
        return repr(self)

    def __repr__(self):
        return 'HttpClientError(%s, %r)' % (self.cause, self.content)


class HttpXmlClient(object):
    def __init__(self, url, source_addr=None):
        self.logger = logging.getLogger(__name__)
        self.url = url
        self.source_addr = source_addr

    def send_raw(self, data):
        """Send raw string.

        Returns a file-like object. It is caller's responsibility to close it.
        """
        headers = self._get_headers()
        try:
            with requests.Session() as session:
                if self.source_addr:
                    session.mount(self.url, SourceAddressAdapter(self.source_addr))

                session.headers = headers
                # disable SSL cert verification
                session.verify = False

                response = session.post(self.url, data=data, stream=True)
                response.raise_for_status()

                return response
        except requests.exceptions.RequestException as e:
            content = e.response.content if e.response is not None else None
            raise HttpClientError(e, content)

    def send_xml(self, xml):
        """Serialize XML object and send an API request."""
        self.logger.debug(_T("DEBUG_API_REQUEST_TO"), self.url, xml)

        request = self._compose_xml(xml)
        response = self.send_raw(request)
        tree = self._parse_xml(response.raw)

        self.logger.debug(_T("DEBUG_API_RESPONSE_FROM"), self.url, tree)
        return tree

    @staticmethod
    def _compose_xml(xml):
        """Serialize ElementTree XML object to string."""
        with closing(StringIO()) as buf:
            xml.write(buf, encoding='utf-8', xml_declaration=True)
            return buf.getvalue()

    @staticmethod
    def _parse_xml(response):
        tree = etree.ElementTree()
        tree.parse(response)
        return tree

    def _get_headers(self):
        return {
            'Content-Type': 'text/xml',
            'Pragma': 'no-cache',
        }
