# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging
import socket
import time

from migrator.core.language import _T
from migrator.settings import NETWORK_PORT_CHECKING_TIMEOUT

logger = logging.getLogger(__name__)


COMMUNICATION_ERROR_CODE_WRONG_AUTHENTICATION = 1001

SOCKET_CHECK_MAX_ATTEMPTS = 3
SOCKET_CHECK_TIMEOUT = 1

PORTS_AVAILABILITY_STATUS_SUCCESS = 0
PORTS_AVAILABILITY_STATUS_ERROR = 1


def is_network_port_open(host, port, timeout=NETWORK_PORT_CHECKING_TIMEOUT):
    for attempt in range(0, SOCKET_CHECK_MAX_ATTEMPTS):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if timeout is not None:
            s.settimeout(timeout)

        try:
            logger.debug(_T("CHECK_HOST_PORT_TRYING_TO_CONNECT_DO_ATTEMPT").format(
                host_ip=host, host_port=str(port), attempt_number=attempt + 1, max_attempts=SOCKET_CHECK_MAX_ATTEMPTS
            ))
            s.connect((host, port))
            logger.debug(_T("CHECK_HOST_PORT_TRYING_TO_CONNECT_SUCCESS").format(host_ip=host, host_port=str(port)))
            return True
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            logger.debug(_T("CHECK_HOST_PORT_TRYING_TO_CONNECT_FAIL").format(host_ip=host, host_port=str(port)))
            if attempt == SOCKET_CHECK_MAX_ATTEMPTS - 1:
                return False

            time.sleep(SOCKET_CHECK_TIMEOUT)
        finally:
            s.close()

    return False


def get_unavailable_ports_for_host(ip, ports_to_check):
    """
    :param ip: string
    :param ports_to_check: list[int]
    :return: dict
    :raises NotImplemented
    """
    result = {'status': PORTS_AVAILABILITY_STATUS_SUCCESS, 'ip': ip, 'unavailable_ports': [], 'available_ports': []}
    for port_to_check in ports_to_check:
        if not is_network_port_open(ip, port_to_check):
            result['status'] = PORTS_AVAILABILITY_STATUS_ERROR
            result['unavailable_ports'].append(port_to_check)
        else:
            result['available_ports'].append(port_to_check)

    logger.debug(_T('PORTS_AVAILABILITY_RESULT_FOR_HOST').format(
        ip=ip,
        available_ports=', '.join(str(port) for port in result['available_ports']),
        unavailable_ports=', '.join(str(port) for port in result['unavailable_ports'])
    ))
    return result


def get_formatted_host_with_unavailable_ports(host_ip, unavailable_ports):
    return _T('HOST_WITH_UNAVAILABLE_PORTS').format(ip=host_ip, ports=', '.join(str(port) for port in unavailable_ports))
