# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

logger = logging.getLogger(__name__)


class PleskCLIRunnerBase(object):
    """Base class to run Plesk utilities"""

    def run(self, utility_name, arguments, env=None):
        raise NotImplementedError()


class PleskCLIRunnerCLI(PleskCLIRunnerBase):
    """Run Plesk utilities with runner provided by server object"""

    def __init__(self, plesk_server):
        self.plesk_server = plesk_server

    def run(self, utility_name, arguments, env=None):
        """
        :type utility_name: basestring
        :type arguments: list[basestring]
        """
        with self.plesk_server.runner() as runner:
            return runner.run(
                self.plesk_server.get_bin_util_path(utility_name),
                arguments,
                env=env
            )

    def run_unchecked(self, utility_name, arguments, env=None):
        with self.plesk_server.runner() as runner:
            return runner.run_unchecked(
                self.plesk_server.get_bin_util_path(utility_name),
                arguments,
                env=env
            )
