# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.language import _T
from migrator.core import MigrationError
from migrator.settings import SSH_DEFAULT_PORT

DEFAULT_RSYNC_PATH = '/usr/bin/rsync'


def detect_rsync_path(runner, node_info_string):
    if runner.file_exists(DEFAULT_RSYNC_PATH):
        return DEFAULT_RSYNC_PATH
    else:
        raise MigrationError(_T("UNABLE_TO_FIND_RSYNC_BINARY") % (node_info_string, DEFAULT_RSYNC_PATH))


def create_rsync_command(
    key_pathname, source_runner, source_user, source_ip,
    source_filepath, target_filepath, exclude=None,
    rsync_additional_args=None,
    source_rsync_bin=None,
    source_port=SSH_DEFAULT_PORT
):
    if exclude is None:
        exclude = []

    cmd = '/usr/bin/rsync'

    if rsync_additional_args is None:
        rsync_additional_args = []

    args = [
        "-e",
        "ssh -i {key} -p {port} -o StrictHostKeyChecking=no -o GSSAPIAuthentication=no".format(
            key=key_pathname,
            port=source_port
        ),
        "--archive"
    ] + rsync_additional_args + [
        u"{source_user}@{source_ip}:{source_filepath}".format(
            source_user=source_user,
            source_ip=source_ip,
            source_filepath=source_filepath
        ),
        u"%s" % (target_filepath,),
    ] + [u"--exclude=%s" % ex for ex in exclude]

    if source_rsync_bin is None:
        # detect rsync binary location on the source node
        source_rsync_bin = detect_rsync_path(
            source_runner, source_ip
        )
        if source_rsync_bin != DEFAULT_RSYNC_PATH: # non-default path
            args = ["--rsync-path=%s" % (source_rsync_bin,)] + args

    return cmd, args