# (C)2017 Ingram Micro Inc. all Rights Reserved
import ssl
import xmlrpclib
from httplib import HTTPSConnection

from migrator.core.language import _T
from migrator.core.utils.loggers import secure_logger


class LoggingTransport(object):

    def __init__(self, transport, logger, hidelist_request=None, hidelist_response=None):
        self.transport = transport
        self.logger = logger
        self.hidelist_request = hidelist_request
        self.hidelist_response = hidelist_response

    def request(self, host, handler, request_body, verbose=0):
        sane_host = self.logger.get_clean_xmlrpc_uri(host)
        self.logger.debug(
            _T('DEBUG_XML_RPC_REQUEST_TO'), sane_host,
            secure_logger.remove_sensitive_data_xml(request_body.decode('utf-8'), self.hidelist_request)
        )
        response = self.transport.request(host, handler, request_body, verbose)
        self.logger.debug(
            _T('DEBUG_XML_RESPONSE'), secure_logger.remove_sensitive_data_json(response[0], self.hidelist_response)
        )
        return response

    def close(self):
        self.transport.close()


class UnverifiedSafeTransport(xmlrpclib.SafeTransport):
    """Handles an HTTPS connection to an XML-RPC server, with no certificate verification

    Comparing to the original xmlrpclib.SafeTransport, this class allows to work
    with invalid SSL certificates on Python 2.7.10. On Python 2.7.9 and earlier, certificate
    validity check is disabled by default.
    """

    def make_connection(self, host):
        if hasattr(ssl, "_create_unverified_context"):
            # Special handling for Python >= 2.7.9, which got enabled SSL certificate verification by default.
            # Checking for attribute is required, because old versions of Python does not have such
            # internal method and don't know about contexts, and don't have such check by default.

            # As POA management node certificates are often not valid, we should not check them.
            # Just do HTTPS requests ignoring all certificate validity issues.
            unverified_context = ssl._create_unverified_context()

            if self._connection and host == self._connection[0]:
                return self._connection[1]

            chost, self._extra_headers, x509 = self.get_host_info(host)
            kwargs = x509 or {}
            kwargs['context'] = unverified_context
            self._connection = host, HTTPSConnection(chost, None, **kwargs)
            return self._connection[1]
        else:
            return xmlrpclib.SafeTransport.make_connection(self, host)
