# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

import paramiko

from migrator.core.language import _T
from migrator.core import MigrationNoContextError
from migrator.core.utils.networking.ssh import utils as ssh_utils

logger = logging.getLogger(__name__)


class ConnectionChecker(object):
    def check(self, check_list):
        """check_list contains items that should be checked here. Possible options are: ssh_auth, windows_auth."""
        for server, checks in check_list:
            server_options = server.node_settings
            if hasattr(server_options, "ssh_auth") and "ssh_auth" in checks:
                self.check_ssh(server.description(), server_options)
            if hasattr(server_options, "windows_auth") and "windows_auth" in checks:
                with server.runner() as runner:
                    runner.check(server.description())

    @staticmethod
    def check_ssh(server_name, server_options):
        logger.debug(_T("CHECK_SSH_CONNECTION_FOR_S_SERVER") % server_name)
        try:
            with ssh_utils.connect(server_options) as ssh_client:
                ssh_client.get_transport().open_session()
        except paramiko.AuthenticationException:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            raise MigrationNoContextError(
                _T("UNABLE_LOGIN_S_SSH_SERVICE_DUE") % (
                    server_name
                )
            )
        except (paramiko.SSHException, EnvironmentError) as err:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            raise MigrationNoContextError(_T("CHECKER_FAILED_TO_CONNECT_BY_SSH") % (server_name, err))
        except Exception as err:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            raise MigrationNoContextError(_T("CHECKER_FAILED_TO_CHECK_SSH_CONNECTION") % (server_name, err))
