# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.cli.panels import load_source_panel_class, load_target_panel_class
from migrator.core.workflow.shared_hosting_workflow import SharedHostingWorkflowExtension
from migrator.core.workflow.base_workflow import BaseWorkflow
from migrator.settings import MIGRATION_TYPE_TAKEOVER


def create_base_workflow():
    """Construct a default workflow object."""
    workflow = BaseWorkflow()
    shared_hosting_extension = SharedHostingWorkflowExtension()
    shared_hosting_extension.extend_workflow(workflow)
    return workflow


def create_workflow(source_panel_type, target_panel_type, migration_type):
    """Create a basic workflow, then apply source, target workflow extensions.

    :param source_panel_type: A string with source panel name
    :param target_panel_type: A string with target panel name
    :param migration_type: enum: [full|takeover]
    """
    workflow = create_base_workflow()

    workflow_package_name = 'workflow'
    module_name = '%s.transfer' % workflow_package_name
    if migration_type == MIGRATION_TYPE_TAKEOVER:
        module_name = '%s.takeover' % workflow_package_name

    # extend workflow with target-panel specific actions
    target_panel_workflow_constructor = load_target_panel_class(
        target_panel_type, module_name, 'WorkflowExtension'
    )
    target_workflow_extension = target_panel_workflow_constructor()
    target_workflow_extension.extend_workflow(workflow)

    # extend workflow with source-panel specific actions
    # 'takeover' migration type workflow is optional for source panel
    source_workflow_extension_constructor = load_source_panel_class(
        source_panel_type, module_name, 'WorkflowExtension', True)

    if source_workflow_extension_constructor is None:
        source_workflow_extension_constructor = load_source_panel_class(
            source_panel_type, 'workflow', 'WorkflowExtension'
        )

    source_workflow_extension = source_workflow_extension_constructor()
    source_workflow_extension.extend_workflow(workflow)

    return workflow
