# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from migrator.core.utils import generate_random_password
from migrator.odin_automation.billing.lib.api.bm import INT_NULL
from migrator.settings import OA_TARGET_OWNER_ID

PhoneNumber = namedtuple('PhoneNumber', (
    'country_code',  # Str(4)
    'area_code',  # Str(10)
    'number',  # Str(20)
    'extension',  # Str(10)
))

PersonInfo = namedtuple('PersonInfo', (
    'address1',  # Str(80)
    'address2',  # Str(80)
    'city',  # Str(40)
    'state',  # Str(80)
    'zip_code',  # Str(10)
    'country',  # Str(2)
    'first_name',  # Str(30)
    'middle_name',  # Str(30)
    'last_name',  # Str(30)
    'email',  # Str(100)
    'phone',  # instance of PhoneNumber
    'fax',  # instance of PhoneNumber
))

DomainSubscriptionUsers = namedtuple('DomainSubscriptionUsers', (
    'owner_id',  # Int customer account user ID. The user contact information is used as owner contact by registrar;
    'admin_id',  # Int customer account user ID. The user contact information is used as administrative contact by
    # registrar;
    'billing_id',  # Int customer account user ID. The user contact information is used as billing contact by registrar;
    'tech_id',  # Int customer account user ID. The user contact information is used as technical contact by registrar;
))


class Account(object):
    def __init__(self, api, get_api):
        self.api = api
        self.get_api = get_api

    def add_account(self, account_info, tax_reg_id):
        return self.api.BM.AccountAdd_API(
            OA_TARGET_OWNER_ID, 3, account_info.info.company_name, account_info.info.company_address.address1,
            account_info.info.company_address.address2, account_info.info.company_address.city,
            account_info.info.company_address.state, account_info.info.company_address.zipcode,
            account_info.info.company_address.country, '', account_info.info.admin_contact.first_name,
            account_info.info.admin_contact.middle_name, account_info.info.admin_contact.last_name,
            account_info.info.admin_contact.email, account_info.info.admin_contact.phone.country_code,
            account_info.info.admin_contact.phone.area_code, account_info.info.admin_contact.phone.phone_num,
            account_info.info.admin_contact.phone.ext_num, account_info.info.admin_contact.fax.country_code,
            account_info.info.admin_contact.fax.area_code, account_info.info.admin_contact.fax.phone_num,
            account_info.info.admin_contact.fax.ext_num, account_info.info.billing_contact.first_name,
            account_info.info.billing_contact.middle_name, account_info.info.billing_contact.last_name,
            account_info.info.billing_contact.email, account_info.info.billing_contact.phone.country_code,
            account_info.info.billing_contact.phone.area_code, account_info.info.billing_contact.phone.phone_num,
            account_info.info.billing_contact.phone.ext_num, account_info.info.billing_contact.fax.country_code,
            account_info.info.billing_contact.fax.area_code, account_info.info.billing_contact.fax.phone_num,
            account_info.info.billing_contact.fax.ext_num, account_info.info.tech_contact.first_name,
            account_info.info.tech_contact.middle_name, account_info.info.tech_contact.last_name,
            account_info.info.tech_contact.email, account_info.info.tech_contact.phone.country_code,
            account_info.info.tech_contact.phone.area_code, account_info.info.tech_contact.phone.phone_num,
            account_info.info.tech_contact.phone.ext_num, account_info.info.tech_contact.fax.country_code,
            account_info.info.tech_contact.fax.area_code, account_info.info.tech_contact.fax.phone_num,
            account_info.info.tech_contact.fax.ext_num, account_info.info.admin_contact.first_name,
            account_info.info.admin_contact.middle_name, account_info.info.admin_contact.last_name, '', '',
            account_info.info.admin_contact.email, account_info.info.admin_contact.phone.country_code,
            account_info.info.admin_contact.phone.area_code, account_info.info.admin_contact.phone.phone_num,
            account_info.info.admin_contact.phone.ext_num, account_info.info.admin_contact.fax.country_code,
            account_info.info.admin_contact.fax.area_code, account_info.info.admin_contact.fax.phone_num,
            account_info.info.admin_contact.fax.ext_num, 1, '', tax_reg_id, 0, 'BM'
        ).account_id

    def update_account(self, account_id, account_info):
        return self.api.BM.AccountUpdate_API(
            account_id, account_info.info.company_name, account_info.info.company_address.address1,
            account_info.info.company_address.address2, account_info.info.company_address.city,
            account_info.info.company_address.state, account_info.info.company_address.zipcode,
            account_info.info.company_address.country, '', account_info.info.admin_contact.first_name,
            account_info.info.admin_contact.middle_name, account_info.info.admin_contact.last_name,
            account_info.info.admin_contact.email, account_info.info.admin_contact.phone.country_code,
            account_info.info.admin_contact.phone.area_code, account_info.info.admin_contact.phone.phone_num,
            account_info.info.admin_contact.phone.ext_num, account_info.info.admin_contact.fax.country_code,
            account_info.info.admin_contact.fax.area_code, account_info.info.admin_contact.fax.phone_num,
            account_info.info.admin_contact.fax.ext_num, account_info.info.billing_contact.first_name,
            account_info.info.billing_contact.middle_name, account_info.info.billing_contact.last_name,
            account_info.info.billing_contact.email, account_info.info.billing_contact.phone.country_code,
            account_info.info.billing_contact.phone.area_code, account_info.info.billing_contact.phone.phone_num,
            account_info.info.billing_contact.phone.ext_num, account_info.info.billing_contact.fax.country_code,
            account_info.info.billing_contact.fax.area_code, account_info.info.billing_contact.fax.phone_num,
            account_info.info.billing_contact.fax.ext_num, account_info.info.tech_contact.first_name,
            account_info.info.tech_contact.middle_name, account_info.info.tech_contact.last_name,
            account_info.info.tech_contact.email, account_info.info.tech_contact.phone.country_code,
            account_info.info.tech_contact.phone.area_code, account_info.info.tech_contact.phone.phone_num,
            account_info.info.tech_contact.phone.ext_num, account_info.info.tech_contact.fax.country_code,
            account_info.info.tech_contact.fax.area_code, account_info.info.tech_contact.fax.phone_num,
            account_info.info.tech_contact.fax.ext_num, 0
        )

    def update_account_balance(self, account_id, account_balance):
        return self.api.BM.AccountBalanceUpdate_API(account_id, account_balance)

    def add_staff_member(self, account_id, staff_member_info):
        hidelist_request = [
            "/methodCall/params/param/value/struct/member/name[text()='Params']/../value/array/data/value[4]/string"
        ]

        password = staff_member_info.authentication.password
        if not password:
            password = generate_random_password()

        api = self.get_api(hidelist_request=hidelist_request)
        return api.BM.UserAdd_API(
            account_id, INT_NULL, staff_member_info.authentication.login,
            'XXX{password}'.format(password=password),
            staff_member_info.contact.first_name, staff_member_info.contact.middle_name,
            staff_member_info.contact.last_name, staff_member_info.contact.email, staff_member_info.address.address1,
            staff_member_info.address.address2, staff_member_info.address.city, staff_member_info.address.state,
            staff_member_info.address.zipcode, staff_member_info.address.country,
            staff_member_info.contact.phone.country_code, staff_member_info.contact.phone.area_code,
            staff_member_info.contact.phone.phone_num, staff_member_info.contact.phone.ext_num,
            staff_member_info.contact.fax.country_code, staff_member_info.contact.fax.area_code,
            staff_member_info.contact.fax.phone_num, staff_member_info.contact.fax.ext_num, 0
        )

    def update_staff_member(self, user_id, staff_member_info):
        return self.api.BM.UserUpdate_API(
            user_id, staff_member_info.authentication.login, staff_member_info.contact.first_name,
            staff_member_info.contact.middle_name, staff_member_info.contact.last_name, staff_member_info.contact.email,
            staff_member_info.address.address1, staff_member_info.address.address2, staff_member_info.address.city,
            staff_member_info.address.state, staff_member_info.address.zipcode, staff_member_info.address.country,
            staff_member_info.contact.phone.country_code, staff_member_info.contact.phone.area_code,
            staff_member_info.contact.phone.phone_num, staff_member_info.contact.phone.ext_num,
            staff_member_info.contact.fax.country_code, staff_member_info.contact.fax.area_code,
            staff_member_info.contact.fax.phone_num, staff_member_info.contact.fax.ext_num
        )

    def get_users_list_for_account(self, account_id):
        return self.api.BM.GetUsersListForAccount_API(account_id, 1)

    def get_user_details(self, user_id):
        return self.api.BM.UserDetailsGet_API(user_id)

    def init_users_from_registrar_api(self, domain_id):
        """
        return boolean:
        - True if user information was imported succesfully from domain registrars
        - False otherwise
        """
        return self.api.DOMAINGATE.InitUsersFromRegistrar_API(domain_id).result != 0
