# (C)2017 Ingram Micro Inc. all Rights Reserved
import base64
import logging
import urllib
import xmlrpclib
from collections import namedtuple

from migrator.core.language import _T
from migrator.core.utils.loggers.secure_logger import get_bleeping_logger
from migrator.core.utils.networking.xmlrpc import LoggingTransport, UnverifiedSafeTransport

INT_NULL = -2147483648
GetVersionResult = namedtuple('GetVersionResult', ('Version',))
AccountAddResult = namedtuple('AccountAddResult', ('account_id',))
GetOrderResult = namedtuple('GetOrderResult', (
    'id', 'number', 'vendor_account_id', 'customer_id', 'order_status_id', 'order_type_id', 'creation_time', 'total',
    'tax_total', 'discount_total', 'merch_total', 'description', 'expiration_date', 'currency_id',
))
SubscriptionAddResult = namedtuple('SubscriptionAddResult', ('SubscriptionID', ))
PlaceOrderAndAuthorizeResult = namedtuple('PlaceOrderAndAuthorizeResult', (
    'account_id', 'order_id', 'login', 'creation_time', 'total', 'tax_total', 'disc_total', 'merch_total',
    'description', 'order_number'
))
DomainSubscrWithIDAddResult = namedtuple('DomainSubscrWithIDAddResult', ('DomainID',))
GetFullDomainZoneListResult = namedtuple('GetFullDomainZoneListResult', (
    'ServiceTemplateID', 'DomainZoneID', 'Container', 'SelectedByDefault'
))
GetCustomerSubscriptionListResult = namedtuple('GetCustomerSubscriptionListResult', (
    'SubscriptionID', 'SubscriptionName', 'PlanID', 'PlanName', 'PlanPeriodID', 'Period', 'PeriodType', 'StartDate',
    'ExpirationDate', 'Status', 'ServStatus', 'ContainerName'
))
GetSubscriptionsListByOrderResult = namedtuple('GetSubscriptionsListByOrderResult', ('SubscriptionID',))
CreditCard2AccountAddResult = namedtuple('CreditCard2AccountAddResult', ('card_id', ))
PayToolTypeListForAccountGetResult = namedtuple('PayToolTypeListForAccountGetResult', (
    'pay_tool_id', 'pay_system', 'pay_system_name', 'pay_tool_type', 'issue_num_applicable', 'start_date_applicable',
    'cvv_required', 'bank_number_status', 'acc_holder_name_status', 'registered_place_status', 'is_3d_secure_active'
))
PayToolListForAccountResult = namedtuple('PayToolListForAccountResult', (
    'pay_tool_id', 'pay_tool_type', 'pay_system_id', 'pay_system', 'cut_number', 'exp_date', 'card_holder_name',
    'acc_holder_name', 'bank_number', 'account_number', 'use_for_auto_payments'
))
DomainSubscrAddResult = namedtuple('DomainSubscrAddResult', ('subscription_id',    'domain_id'))
GetUsersListForAccountResult = namedtuple('GetUsersListForAccountResult', ('user_id', 'login',    'full_name',))
UserDetailsGetResult = namedtuple('UserDetailsGetResult', (
    'user_id', 'login', 'account_id', 'first_name', 'middle_name',
    'last_name', 'email', 'address1', 'address2', 'city', 'state', 'zip_code', 'country',
    'phone_country_code', 'phone_area_code', 'phone_number', 'phone_extension',
    'fax_country_code',    'fax_area_code', 'fax_number', 'fax_extension',
    'status',
))
PlanPeriodListGetResult = namedtuple('PlanPeriodListGetResult',    (
    'plan_period_id', 'period', 'period_type', 'trial', 'setup_fee', 'subscription_fee', 'renewal_fee', 'transfer_fee',
    'non_refundable_amt', 'refund_period', 'enabled', 'number_of_periods', 'fee_text', 'sort_number', 'os_otfi',
    'deposit_fee', 'deposit_descr', 'refund_prorated_recurring_fee'
))
InitUsersFromRegistrarResult = namedtuple('InitUsersFromRegistrarResult', ('result', ))
UserAddResult = namedtuple('UserAddResult', ('user_id', ))
PlanDetailsGetResult = namedtuple('PlanDetailsGetResult', (
    'plan_id', 'name', 'plan_category_id', 'pcurrency', 'short_description', 'long_description', 'st_type', 'group_id',
    'is_parent_req', 'recurring_type', 'billing_period_type', 'billing_period', 'show_priority', 'default', 'is_otfi',
    'doc_id', 'account_id', 'st_id', 'price_period_type', 'is_published'
))
IsPlanDelegatedResult = namedtuple('IsPlanDelegatedResult', ('is_delegated'))
CertAddResult = namedtuple('CertAddResult', ('CertID',))
GetSubscriptionRenewOrderAmountResult = namedtuple('GetSubscriptionRenewOrderAmountResult', ('fee',))
GetSubscriptionBillOrderAmountResult = namedtuple('GetSubscriptionBillOrderAmountResult', ('fee',))
SubscriptionResourcesListGetResult = namedtuple('SubscriptionResourcesListGetResult', (
    'ResourceID', 'ResourceRateID', 'ResourceName', 'StoreDescription', 'StorePriceText', 'StoreSortOrder', 'Status',
    'ResourceCategory', 'IncludedAmount', 'AdditionalAmount', 'UsedAmount', 'OrderedAmount', 'Unit', 'MinUnits',
    'MaxUnits', 'Measurable', 'RelativeStatus', 'OrderNumber', 'SetupFee', 'RecurringFee', 'OveruseFee', 'Location',
    'IsOveruseFeeTiered', 'IsRecurringFeeTiered', 'IsSetupFeeTiered',
))

DomainInfoGetResult = namedtuple('DomainInfoGetResult', (
    'account_id', 'vendor_id', 'domain_id', 'full_domain_zone', 'domain_zone_id', 'status', 'external_dns',
    'registration_period', 'start_date', 'expiration_date', 'primary_name_server', 'secondary_name_server',
    'third_name_server', 'fourth_name_server', 'company_name', 'personal_fname', 'personal_mname', 'personal_lname',
    'personal_email', 'personal_phone', 'personal_fax', 'admin_fname', 'admin_mname', 'admin_lname', 'admin_email',
    'admin_phone', 'admin_fax', 'billing_fname', 'billing_mname', 'billing_lname', 'billing_email', 'billing_phone',
    'billing_fax', 'tech_fname', 'tech_mname', 'tech_lname', 'tech_email', 'tech_phone', 'tech_fax',
))
ADomainGetResult = namedtuple('ADomainGetResult', (
    'domain_id', 'check_result_attempts', 'domain_zone_id', 'full_domain_name', 'tech_domain_name',
    'last_operation_status', 'last_renewal_date', 'primary_name_server', 'secondary_name_server', 'third_name_server',
    'fourth_name_server', 'registered_to_account_id', 'owner_users_id', 'owner_handle_id', 'is_owner_contact_replaced',
    'admin_users_id', 'admin_handle_id', 'is_admin_contact_replaced', 'billing_users_id', 'billing_handle_id',
    'is_billing_contact_replaced', 'tech_users_id', 'tech_handle_id', 'is_tech_contact_replaced', 'registrar_id',
    'registration_date', 'status', 'subscription_id', 'time_to_check_result', 'update_date',
))
GetCertListResult = namedtuple('GetCertListResult', (
    'cert_id', 'name', 'plugin_name', 'product_name', 'subscription_id', 'status', 'expire_date',
))
CertSTGetResult = namedtuple('CertSTGetResult', (
    'service_template_id', 'plugin_id', 'cert_product_id'
))
CertProductGetResult = namedtuple('CertProductGetResult', (
    'cert_product_id', 'container_id', 'identity_code'
))
PlanGetResult = namedtuple('PlanGetResult', (
    'plan_id', 'name', 'service_template_id', 'plan_category', 'service_terms', 'short_description', 'long_description',
    'unique_group', 'published', 'attach_usage_statistics', 'account', 'notification_tmpl', 'customer_class',
    'recurring_type', 'billing_period_type', 'billing_period', 'billing_period_description', 'auto_renew_description',
    'auto_renew', 'show_priority', 'default', 'notif_schedule', 'v_is_otf_plan', 'v_h_image', 'v_show_hide_advanced',
))
PlanCategoryListGetResult = namedtuple('PlanCategoryListGetResult', ('id', 'name', 'description'))
PlanListAvailableGetResult = namedtuple('PlanListAvailableGetResult', (
    'id', 'name', 'short_description', 'long_description', 'category_id', 'st_type', 'is_published', 'group_id',
    'recurring_type', 'billing_period_type', 'billing_period', 'is_parent_req', 'show_priority',
    'default_plan_period_id'
))
GetOwnersPlansWithCategoryResult = namedtuple('GetOwnersPlansWithCategoryResult', (
    'id', 'name', 'category', 'short_description', 'published'  # PLAN_PUBLISHED_STATE: 0 - Yes, other - No
))
PlanGetServiceTemplateListResult = namedtuple('PlanGetServiceTemplateListResult', (
    'id', 'name', 'category', 'short_description', 'publication_status'  # PUBLICATION_PLAN_STATUS_FOR_TABLE: 1 - No, other - Yes
))

SubscriptionDetailsGetResult = namedtuple('SubscriptionDetailsGetResult', (
    'id', 'name', 'account_id', 'plan_id', 'plan_name', 'status', 'service_status'
))

SubscriptionDetailsGetExResult = namedtuple('SubscriptionDetailsGetExResult', (
    'id', 'name', 'account_id', 'plan_id', 'plan_name', 'status', 'service_status', 'start_date', 'expiration_date',
    'last_bill_date', 'next_bill_date', 'billing_period_type', 'billing_period', 'trial', 'shutdown_date',
    'terminate_date', 'period', 'period_type', 'freeze_prices', 'promo_code'
))

GetDomainZoneIDResult = namedtuple('GetDomainZoneIDResult', ('tld', ))

PEMNSSettingsGetResult = namedtuple('PEMNSSettingsGetResult', (
    'current_account_id', 'default_ns_st_id', 'own'
))


class BillingAPIError(Exception):
    def __init__(self, message, short_message, method_name, method_args, inner):
        """
        message - exception message
        method - name of the called method
        args - arguments passed to the method, as string
        inner - root exception, instance of xmlrpclib.Fault
        """
        super(BillingAPIError, self).__init__(message)
        self.method_name = method_name
        self.method_args = method_args
        self.short_message = short_message
        self.inner = inner


class BMAPILoggingTransport(LoggingTransport):
    CommonXPathes = ["/methodCall/params/param/value/struct/member/name[text()='Password']/../value/string"]

    def __init__(self, transport, logger, hidelist_request=None, hidelist_response=None):
        if hidelist_request:
            hidelist_request.extend(self.CommonXPathes)

        super(BMAPILoggingTransport, self).__init__(transport, logger, hidelist_request or self.CommonXPathes, hidelist_response)


class BMAPI(object):

    class Server(object):
        def __init__(self, name, api):
            self.name = name
            self.api = api

        def __getattr__(self, name):
            return BMAPI.Method(name, self)

    class Method(object):
        def __init__(self, name, server):
            self.name = name
            self.server = server

        def __call__(self, *args):
            return self.server.api(self.server.name, self.name, *args)

    logger = logging.getLogger(__name__)

    resultTypes = {
        # Legacy
        "GetVersion_API": GetVersionResult,
        # Import of Account/User
        "AccountAdd_API": AccountAddResult,
        # Import of Subscription
        "PlaceOrderAndAuthorize_API": PlaceOrderAndAuthorizeResult,
        "GetOrder_API": GetOrderResult,
        "SubscriptionAdd_API": SubscriptionAddResult,
        "AddPEMSubscription_API": SubscriptionAddResult,
        "DomainSubscrWithIDAdd_API": DomainSubscrWithIDAddResult,
        "GetFullDomainZoneList_API": GetFullDomainZoneListResult,
        "GetCustomerSubscriptionList_API": GetCustomerSubscriptionListResult,
        "GetSubscriptionsListByOrder_API": GetSubscriptionsListByOrderResult,
        "SubscriptionResourcesListGet_API": SubscriptionResourcesListGetResult,
        # Import of CreditCard
        "CreditCard2AccountAdd_API": CreditCard2AccountAddResult,
        "PayToolTypeListForAccountGet_API": PayToolTypeListForAccountGetResult,
        "PayToolListForAccountGet_API": PayToolListForAccountResult,
        "DomainSubscrAdd_API": DomainSubscrAddResult,
        "GetUsersListForAccount_API": GetUsersListForAccountResult,
        "InitUsersFromRegistrar_API": InitUsersFromRegistrarResult,
        "UserDetailsGet_API": UserDetailsGetResult,
        "UserAdd_API": UserAddResult,
        "PlanPeriodListGet_API": PlanPeriodListGetResult,
        "PlanDetailsGet_API": PlanDetailsGetResult,
        "IsPlanDelegated": IsPlanDelegatedResult,
        "CertAdd": CertAddResult,
        "GetSubscriptionRenewOrderAmount_API": GetSubscriptionRenewOrderAmountResult,
        "GetSubscriptionBillOrderAmount_API": GetSubscriptionBillOrderAmountResult,
        "DomainInfoGet_API": DomainInfoGetResult,
        "ADomainGet": ADomainGetResult,
        "GetCertList": GetCertListResult,
        "CertSTGet": CertSTGetResult,
        "CertProductGet": CertProductGetResult,
        "PlanGet": PlanGetResult,
        "PlanCategoryListGet_API": PlanCategoryListGetResult,
        "PlanListAvailableGet_API": PlanListAvailableGetResult,
        "GetOwnersPlansWithCategory": GetOwnersPlansWithCategoryResult,
        "PlanGetServiceTemplateList": PlanGetServiceTemplateListResult,
        "SubscriptionDetailsGet_API": SubscriptionDetailsGetResult,
        "SubscriptionDetailsGetEx_API": SubscriptionDetailsGetExResult,
        "GetDomainZoneID_API": GetDomainZoneIDResult,
        "PEMNSSettingsGet": PEMNSSettingsGetResult,
    }

    def __init__(self, url, user, password, hidelist_request=None, hidelist_response=None):
        proto, _ = urllib.splittype(url)

        if proto == 'https':
            transport = UnverifiedSafeTransport(use_datetime=0)
        elif proto == 'http':
            transport = xmlrpclib.Transport(use_datetime=0)
        else:
            assert False, _T("ONLY_HTTP_AND_HTTPS_PROTOCOLS_ARE_SUPPORTED")

        logger = get_bleeping_logger(self.logger)
        transport = BMAPILoggingTransport(transport, logger, hidelist_request, hidelist_response)

        self.server = xmlrpclib.ServerProxy(url, transport=transport)
        self.auser = user
        self.passwd = password
        self.trnsId = None

    def __getattr__(self, name):
        return BMAPI.Server(name, self)

    def _fill_request(self, request):
        if self.auser:
            request['Username'] = self.auser
        if self.auser and self.passwd:
            request['Password'] = self.passwd
        if self.trnsId is not None:
            request['TransactionID'] = self.trnsId
            request['AutoCommit'] = 'No'

    def __call__(self, server, method, *args):
        request = {'Server': server, 'Method': method, 'Params': args}
        self._fill_request(request)
        server_method = server + '::' + method
        try:
            response = self.server.Execute(request)
            results = response['Result'][0]
            self.logger.debug(_T("LOG_API_CALL"), server_method, '(...)', results)
        except xmlrpclib.Fault as e:
            self.logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            short_message = base64.decodestring(e.faultString)
            err_code_pos = short_message.find('Error Code:')
            if err_code_pos > 0:
                short_message = short_message[:err_code_pos].strip()

            raise BillingAPIError(
                message=_T("API_CALL_FAILURE") % (server_method, '(...)', base64.decodestring(e.faultString)),
                short_message=short_message,
                method_name=server_method, method_args=args, inner=e
            )
        except Exception as e:
            self.logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            self.logger.debug(_T("DEBUG_API_CALL_RAISED_EXCEPTION"), server_method, '(...)', str(e))
            raise

        if method in self.resultTypes:
            result_type = self.resultTypes[method]
            if list == type(results) and len(results) > 0 and list == type(results[0]):
                return [result_type._make(result) for result in results]
            elif len(results) > 0:
                return result_type._make(results)
            else:
                return []

        return None

    def begin_transaction(self):
        request = {'Server': 'BM', 'Method': 'GetVersion_API', 'AutoCommit': 'No'}
        self._fill_request(request)
        response = self.server.Execute(request)
        self.trnsId = response["TransactionID"]

    def commit_transaction(self):
        request = {}
        self._fill_request(request)
        self.server.CommitTransaction(request)
        self.trnsId = None

    def rollback_transaction(self):
        request = {}
        self._fill_request(request)
        self.server.RollbackTransaction(request)
        self.trnsId = None
