# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from migrator.core.utils import obj
from migrator.core.language import _T


BillingPeriod = namedtuple('BillingPeriod', (
    'type',  # one of BillingPeriodType value
    'size'
))

BillingPeriodType = obj(DAY=1, MONTH=2, YEAR=3)

BillingModel = obj(
    # before billing period
    BILLING_MODEL_BBP=10,
    # after billing period
    BILLING_MODEL_ABP=20,
    # before subscription period
    BILLING_MODEL_BSP=30
)


class Order(object):
    def __init__(self, api):
        self.api = api

    def place_order_and_authorize(self, order):
        return self.api.PlaceOrderAndAuthorize_API(
            1, 1,
            '%s=%s=%s=%s' % (order.PlanID, order.PlanPeriodID, 0, -1),
            3, 0, 1,
            "DomainID=%s" % order.SubscriptionName,
            1,
            "AccountID=%s" % order.AccountID,
            2,
            "PayToolTypeID=3",
            "PluginID=0",
        )

    def order_payments_received(self, order_id):
        return self.api.OrderPaymentsReceived_API(order_id, '')

    def get_order(self, order_id):
        return self.api.GetOrder_API(order_id)

    def get_next_order_amount(self, subscription_id, plan_id, period, period_type, next_bill_date, expiration_date):
        plan = self.api.PlanDetailsGet_API(plan_id)

        if plan.recurring_type == BillingModel.BILLING_MODEL_BSP:
            renew_amount = self.api.GetSubscriptionRenewOrderAmount_API(subscription_id, period, period_type, "")
            return renew_amount.fee

        elif plan.recurring_type == BillingModel.BILLING_MODEL_BBP:
            if next_bill_date >= expiration_date:
                renew_amount = self.api.GetSubscriptionRenewOrderAmount_API(subscription_id, period, period_type, "")
                bill_amount = self.api.GetSubscriptionBillOrderAmount_API(subscription_id)
                return renew_amount.fee + bill_amount.fee
            else:
                bill_amount = self.api.GetSubscriptionBillOrderAmount_API(subscription_id)
                return bill_amount.fee

        elif plan.recurring_type == BillingModel.BILLING_MODEL_ABP:
            if next_bill_date >= expiration_date:
                renew_amount = self.api.GetSubscriptionRenewOrderAmount_API(subscription_id, period, period_type, "")
                bill_amount = self.api.GetSubscriptionBillOrderAmount_API(subscription_id)
                return renew_amount.fee + bill_amount.fee
            else:
                bill_amount = self.api.GetSubscriptionBillOrderAmount_API(subscription_id)
                return bill_amount.fee

        else:
            raise Exception(_T("BILLING_MODEL_OF_BILLING_PLAN_NOT_SUPPORTED") % (plan.recurring_type, plan_id))
