# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.language import _T
from migrator.core.actions.base.common_action import CommonAction
from migrator.core.utils.loggers.report import Issue, Problem
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.odin_automation.platform.lib.data.subscription import PlatformSubscription

logger = logging.getLogger(__name__)


class CreatePlatformSubscriptionList(CommonAction):
    def get_description(self):
        return _T("CREATE_PLATFORM_SUBSCRIPTION_LIST")

    def get_failure_message(self, global_context):
        """
        :type global_context: parallels.core.global_context.GlobalMigrationContext
        """
        return _T("FAILED_CREATE_PLATFORM_SUBSCRIPTION_LIST")

    def run(self, global_context):
        """
        :type global_context: parallels.core.global_context.GlobalMigrationContext
        """
        parent_subscriptions = []
        child_subscriptions = []
        source_system_reader = PlatformReaderDB()
        for item in global_context.migration_list_data:
            source_subscription_info = source_system_reader.get_subscription(item.id)
            if source_subscription_info is None:
                global_context.execution_wrapper.add_subscription_issue(
                    item.id,
                    Issue(Problem(
                        'import_accounts_and_subscriptions', Problem.ERROR,
                        _T("SUBSCRIPTION_IS_NOT_PRESENT_ON_SOURCE").format(subscription=item.id)
                    ), None)
                )
                continue

            subscription = PlatformSubscription(source_subscription_info, item)
            if not item.is_oa_subscription and item.id == item.mapping_subscription_id:
                parent_subscriptions.append(subscription)
            else:
                child_subscriptions.append(subscription)

        global_context.subscriptions_to_migrate = parent_subscriptions + child_subscriptions
