# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.actions.dns_forwarding.configure_source_nameservers import ConfigureSourceNameservers
from migrator.odin_automation.platform.lib.api.aps.domain import get_parent_pa_domain
from migrator.odin_automation.platform.lib.api.openapi.resource import PoaApiError


class ConfigureSourceNameserversOA(ConfigureSourceNameservers):
    def __init__(self):
        super(ConfigureSourceNameservers, self).__init__()
        self.poa_api = None

    def _get_target_zone_nameservers(self, global_context, subscription, zone):
        try:
            oa_dns_zone_name = self._get_top_parent_domain(zone.name, subscription.account_id)
            self.poa_api = self.poa_api or global_context.conn.target.poa_api()
            return [ns['ip_address'] for ns in self.poa_api.domain.get_domain_name_servers(oa_dns_zone_name)]
        except PoaApiError as e:
            # domain with no name servers (most likely an external domain)
            if e.error_code == 3000:
                return []

            raise e

    def _get_top_parent_domain(self, domain, account_id):
        parent_domain = get_parent_pa_domain(domain, account_id=account_id)
        if parent_domain:
            return self._get_top_parent_domain(parent_domain['name'], account_id)

        return domain
