# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.actions.base.common_action import CommonAction
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.data.source.xml.billing import BillingWriterXml
from migrator.core.language import _T
from migrator.settings import \
    TargetServiceType, SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS, SOURCE_PRODUCT_OA_WSH, SOURCE_PRODUCT_OA_WHL, SOURCE_PRODUCT_OA_LSH

logger = logging.getLogger(__name__)


class ExportBillingData(CommonAction):
    """Export billing data from source system."""
    def run(self, global_context):
        self._platform_db_reader = PlatformReaderDB()
        subscription_ids = {item.id for item in global_context.migration_list_data}

        exporter = global_context.source_product.get_exporter_instance(TargetServiceType.BILLING)

        filename = global_context.session_files.get_path_to_billing_xml_file()
        with BillingWriterXml(filename) as writer:
            count_account_objects = 0
            count_subscription_objects = 0

            with writer('billing_info') as billing_info:

                with billing_info('resellers'):
                    pass

                with billing_info('accounts') as accounts:
                    for account_objects in exporter.get_next_account_page(subscription_ids):
                        count_account_objects += len(account_objects)
                        accounts.write_list(account_objects)

                with billing_info('subscriptions') as subscriptions:
                    for subscription_objects in exporter.get_next_subscription_page(subscription_ids):
                        count_subscription_objects += len(subscription_objects)
                        subscriptions.write_list(subscription_objects)

            logger.info("Got %s account objects in total; '%s' subscription objects in total"
                    % (count_account_objects, count_subscription_objects))

    def get_description(self):
        return _T("EXPORT_BILLING_DATA_ACTION")

    def get_failure_message(self, global_context):
        return _T("EXPORT_BILLING_DATA_FAILURE")

    def filter_action(self, global_context):
        product = global_context.source_type
        return product in [SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS, SOURCE_PRODUCT_OA_WSH, SOURCE_PRODUCT_OA_WHL, SOURCE_PRODUCT_OA_LSH]
