# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core import _T, MigrationError
from migrator.core.actions.base.subscription_action import SubscriptionAction
from migrator.core.data.model.subscription import has_any_of_service_types, get_service_property
from migrator.odin_automation.platform.lib.data.domain_importer import \
    DomainImporter, DomainImporterPPAB, DomainImporterPBA
from migrator.settings import SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, SourceHostingClass, SOURCE_SERVICE_DNS


class ImportDomain(SubscriptionAction):
    def run(self, global_context, subscription):
        domain_name = get_service_property(subscription.source_info, SOURCE_SERVICE_DNS, 'domain')
        if domain_name is None:
            raise MigrationError(_T("DOMAIN_PROPERTY_NOT_SET").format(subscription_name=subscription.name))

        if global_context.target_panel_obj.is_billing_module_installed():
            if global_context.target_panel_obj.is_billing_essentials():
                domain_importer = DomainImporterPBA(global_context)
            else:
                domain_importer = DomainImporterPPAB(global_context)
        else:
            domain_importer = DomainImporter(global_context)

        domain_importer.import_domain(domain_name, dns_subscription=subscription)

    @staticmethod
    def _get_subscription(global_context, subscription_id):
        for subscription in global_context.subscriptions_to_migrate:
            if subscription.source_info.external_id == subscription_id:
                return subscription

        raise MigrationError(_T("INTERNAL_ERROR_OCCURRED_WHILE_CREATING_SUBSCRIPTIONS"))

    def get_description(self):
        return _T("IMPORT_DOMAIN")

    def filter_subscription(self, global_context, subscription):
        """
        :type global_context: migrator.core.global_context.GlobalMigrationContext
        :type subscription: migrator.core.migrated_subscription.MigratedSubscription
        """
        return not subscription.is_mapped_to_oa_subscription() and \
               not subscription.is_mapped_to_other_subscription_in_migration_project() and \
               has_any_of_service_types(
                   subscription.source_info,
                   SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.DNS]
               )

    def get_failure_message(self, global_context, subscription):
        """
        :type global_context: migrator.core.global_context.GlobalMigrationContext
        :type subscription: migrator.core.migrated_subscription.MigratedSubscription
        """
        return _T("FAILED_TO_IMPORT_DOMAIN").format(
            domain_name=get_service_property(subscription.source_info, SOURCE_SERVICE_DNS, 'domain'),
            subscription_name=subscription.name
        )
