# (C)2017 Ingram Micro Inc. all Rights Reserved
from checks import Checks
from migrator.core.language import _T


class CheckGroups(object):
    CHECK_GROUP_LICENSES = 'check_group_licenses'
    CHECK_GROUP_INFRASTRUCTURE = 'check_group_infrastructure'
    CHECK_GROUP_SOURCE_DATA = 'check_group_source_data'
    CHECK_GROUP_SERVICE_PLANS = 'check_group_service_plans'
    CHECK_GROUP_SHARED_HOSTING = 'check_group_shared_hosting'
    CHECK_GROUP_OACI = 'check_group_oaci'
    CHECK_GROUP_IP = 'check_group_ip'
    CHECK_GROUP_NG_HOSTING = 'check_group_lsh_ng_migration'

    @staticmethod
    def groups_descriptions():
        return {
            CheckGroups.CHECK_GROUP_LICENSES: _T('CHECK_GROUP_DESCRIPTION_LICENSES'),
            CheckGroups.CHECK_GROUP_INFRASTRUCTURE: _T('CHECK_GROUP_DESCRIPTION_INFRASTRUCTURE'),
            CheckGroups.CHECK_GROUP_SOURCE_DATA: _T('CHECK_GROUP_DESCRIPTION_SOURCE_DATA'),
            CheckGroups.CHECK_GROUP_SERVICE_PLANS: _T('CHECK_GROUP_DESCRIPTION_SERVICE_PLANS'),
            CheckGroups.CHECK_GROUP_SHARED_HOSTING: _T('CHECK_GROUP_DESCRIPTION_SHARED_HOSTING'),
            CheckGroups.CHECK_GROUP_OACI: _T('CHECK_GROUP_DESCRIPTION_OACI'),
            CheckGroups.CHECK_GROUP_IP: _T('CHECK_GROUP_DESCRIPTION_IP'),
            CheckGroups.CHECK_GROUP_NG_HOSTING: _T('CHECK_GROUP_NG_HOSTING'),
        }

    @staticmethod
    def groups_order():
        return {
            CheckGroups.CHECK_GROUP_INFRASTRUCTURE: 1,
            CheckGroups.CHECK_GROUP_LICENSES: 2,
            CheckGroups.CHECK_GROUP_SOURCE_DATA: 3,
            CheckGroups.CHECK_GROUP_SERVICE_PLANS: 4,
            CheckGroups.CHECK_GROUP_SHARED_HOSTING: 5,
            CheckGroups.CHECK_GROUP_OACI: 6,
            CheckGroups.CHECK_GROUP_IP: 7,
            CheckGroups.CHECK_GROUP_NG_HOSTING: 8,
        }

    @staticmethod
    def check_names_to_groups_map():
        return {
            CheckGroups.CHECK_GROUP_INFRASTRUCTURE: [
                Checks.CHECK_HOSTS_AVAILABILITY,
                Checks.CHECK_HOSTS_AUTHENTICATION,
                Checks.CHECK_HOSTS_NETWORK_MODE,
                Checks.CHECK_LSH_NG_SOURCE_HOSTS,
                Checks.CHECK_LSH_NG_TASKS,
                Checks.CHECK_LSH_NG_CONNECTIONS,
                Checks.CHECK_LSH_NG_BACKUP_DISKSPACE,
                Checks.CHECK_NG_NG_CONNECTIONS,
                Checks.CHECK_NG_NG_HOSTS_AVAILABILITY,
            ],
            CheckGroups.CHECK_GROUP_IP: [
                Checks.CHECK_IP_POOLS,
                Checks.CHECK_COMMUNICATION_IP,
                Checks.CHECK_NG_NG_IP_POOLS
            ],
            CheckGroups.CHECK_GROUP_LICENSES: [
                Checks.CHECK_OA_LICENSE,
                Checks.CHECK_SOURCE_LICENSE,
            ],
            CheckGroups.CHECK_GROUP_SOURCE_DATA: [
                Checks.CHECK_SOURCE_DATA,
            ],
            CheckGroups.CHECK_GROUP_SERVICE_PLANS: [
                Checks.CHECK_SERVICE_TEMPLATES,
                Checks.CHECK_SERVICE_TEMPLATES_CONFLICTS,
                Checks.CHECK_SERVICE_PLANS,
                Checks.CHECK_SERVICE_PLANS_APPLICABILITY,
                Checks.CHECK_LSH_NG_SUBSCRIPTIONS,
                Checks.CHECK_LSH_NG_CONFIG,
            ],
            CheckGroups.CHECK_GROUP_SHARED_HOSTING: [
                Checks.CHECK_HOSTS_SHM,
                Checks.CHECK_USERS_SHM,
                Checks.CHECK_HOSTS_CAPACITY_SHM
            ],
            CheckGroups.CHECK_GROUP_OACI: [
                Checks.CHECK_HOSTS_OACI,
                Checks.CHECK_OA_HOSTS_OACI,
            ],
            CheckGroups.CHECK_GROUP_NG_HOSTING: [
                Checks.CHECK_LSH_NG_TARGET_HOST,
                Checks.CHECK_NG_NG_HOST_WEBSITES_CAPACITY,
                Checks.CHECK_NG_NG_PHP_VERSIONS,
                Checks.CHECK_NG_NG_DISK_SPACE,
                Checks.CHECK_NG_NG_FTP_USERS_UNIQUENESS,
            ],
        }
