# (C)2017 Ingram Micro Inc. all Rights Reserved
import time

from migrator.core.checks.base import CheckAbstract
from migrator.core.language import _T
from migrator.odin_automation.platform.checks.checks import Checks
from migrator.odin_automation.platform.lib.api.aps.license import get_oa_licenses
from migrator.settings import TargetServiceType, SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS, SOURCE_PRODUCT_OA_NG, \
    SOURCE_PRODUCT_OA_LSH


class CheckOALicense(CheckAbstract):
    SUPPORTED_SOURCE_PRODUCTS = [
        SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS, SOURCE_PRODUCT_OA_NG, SOURCE_PRODUCT_OA_LSH
    ]
    SUPPORTED_TARGET_SERVICE_CLASSES = [TargetServiceType.SHM, TargetServiceType.OACI]
    LIMIT_KEY_ACCOUNTS = 'customers'
    LIMIT_VALUE_UNLIMITED = '-1'

    def __init__(self):
        super(CheckOALicense, self).__init__(Checks.CHECK_OA_LICENSE, CheckAbstract.PHASE_NO_PHASE)

    def get_full_description(self):
        return _T('CHECK_FULL_DESCRIPTION_OA_LICENSE')

    def _do_check(self):
        # check availability
        oa_licenses = get_oa_licenses()
        if not oa_licenses:
            return self.STATUS_ERROR, _T('CHECK_OA_LICENSE_ERROR_LICENSE_NOT_FOUND')

        # check expiration
        # ToDo: warning if license expires in 1,2,3 days ?
        active_license = oa_licenses[0]
        license_expiration_timestamp = active_license.expires
        current_timestamp = int(time.time())
        # If expires = 0, then the license is unlimited
        if license_expiration_timestamp != 0 and current_timestamp > license_expiration_timestamp:
            return self.STATUS_ERROR, _T('CHECK_OA_LICENSE_ERROR_LICENSE_EXPIRED')

        # success
        return self.STATUS_SUCCESS, _T('CHECK_OA_LICENSE_SUCCESS')
