# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils.registry import Registry
from migrator.odin_automation.billing.lib.utils import get_service_plan_tld
from migrator.odin_automation.platform.checks.checks import Checks
from migrator.odin_automation.platform.lib.data.domain_importer import DomainImporter
from migrator.settings import HTML_EOL, TargetServiceType, SOURCE_SERVICE_DNS, SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS

logger = logging.getLogger(__name__)


class CheckServicePlansApplicability(CheckAbstract):
    SUPPORTED_TARGET_SERVICE_CLASSES = [TargetServiceType.SHM]
    SUPPORTED_SOURCE_PRODUCTS = [SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS]

    def __init__(self):
        super(CheckServicePlansApplicability, self).__init__(Checks.CHECK_SERVICE_PLANS_APPLICABILITY, CheckAbstract.PHASE_NO_PHASE)

    def is_check_supported(self):
        return super(CheckServicePlansApplicability, self).is_check_supported() and \
            Registry.get_instance().get_context().target_panel_obj.is_billing_module_installed()

    def get_full_description(self):
        return _T('CHECK_FULL_DESCRIPTION_SERVICE_PLANS_APPLICABILITY')

    def _do_check(self):
        platform_reader = PlatformReaderDB()
        migration_items = platform_reader.get_migration_project_items()

        errors = []
        migration_items_to_check = [item for item in migration_items if item.target_service_plan_id is not None]
        for migration_item in migration_items_to_check:
            subscription = platform_reader.get_subscription(primary_id=migration_item.subscription_id)
            #  from PlatformSubscription.is_mapped_to_self
            if migration_item.mapping_subscription_id == subscription.external_id and not migration_item.is_oa_subscription:
                target_service_plan_id = migration_item.target_service_plan_id
                for service in [x for x in subscription.services if x.type == SOURCE_SERVICE_DNS]:
                    domain = service.get_property('domain')
                    required_tld = DomainImporter.get_tld_billing(domain)
                    plan_details, service_plan_tld = get_service_plan_tld(target_service_plan_id)
                    if required_tld != service_plan_tld.tld:
                        errors.append(_T('TLD_OF_SERVICE_PLAN_DOES_NOT_CORRESPOND_TO_DOMAIN').format(
                            assigned_tld=service_plan_tld.tld,
                            plan=plan_details.name,
                            domain=domain,
                            required_tld=required_tld
                        ))

        if errors:
            return self.STATUS_ERROR, HTML_EOL.join(errors)
        else:
            return self.STATUS_SUCCESS, _T('CHECK_SERVICE_PLANS_GENERIC_SUCCESS')
