# (C)2017 Ingram Micro Inc. all Rights Reserved
import types

from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import APS2, get_aps_connector_with_admin_token
from migrator.core.utils import cached
from migrator.core.utils.loggers.log import trace


def get_accounts_counter():
    aps = get_aps_connector_with_admin_token()
    return aps.get_resources_count(types.OAAccount)


def get_accounts():
    """
    Returns only active accounts
    """
    aps = get_aps_connector_with_admin_token()
    return aps.get_resources(filter_str='implementing(%s),eq(locked,false)' % types.OAAccount)


def get_account(account_id):
    aps = get_aps_connector_with_admin_token()
    return aps.get_first_resource(filter_str='implementing(%s),eq(id,%s)' % (types.OAAccount, account_id))


def get_account_user_by_subscription(subscription_id):
    aps = get_aps_connector_with_admin_token()
    account_uid = get_account_by_subscription(subscription_id)['aps']['id']
    return aps.get_first_resource(resource_path='%s/users' % account_uid, filter_str='implementing(%s),eq(isAccountAdmin,true)' % types.PAAdminUser)


def get_account_by_subscription(subscription_id):
    aps = get_aps_connector_with_admin_token()
    return aps.get_first_resource(
        filter_str='implementing(%s),eq(subscriptionId,%s),select(account)' % (types.PASubscription, subscription_id)
    )['account']


@cached
def get_country_rule(country_id):
    aps_admin = APS2.get_connector()
    sm = aps_admin.get_first_resource(filter_str='implementing(%s)' % types.PABSSSMServices)
    aps = get_aps_connector_with_admin_token()
    currency_states = aps.get_resource('%s/bss/account/currency-states' % sm.aps.id)
    for country_rule in currency_states['countriesRules']:
        if country_rule['countryId'] == country_id:
            return country_rule

    return None


def get_account_vendor_id(account_id):
    aps = get_aps_connector_with_admin_token()
    result = aps.get_first_resource(
        filter_str='implementing(%s),eq(id,%s),select(parent)' % (types.OAAccount, account_id)
    )

    if 'parent' in result:
        return result['parent']['id']

    return account_id


@trace
def set_account_contacts_locale(account_uid, locale):
    resource = {
        'aps': {'id': account_uid},
        'adminContact': {'locale': locale},
        'billingContact': {'locale': locale},
        'techContact': {'locale': locale},
    }
    aps = get_aps_connector_with_admin_token()
    aps.put_resource(account_uid, '', resource)
