# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.utils import cached
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import get_aps_connector_with_admin_token

import types

HOST_ATTRIBUTE_OACI_CLOUD_ROLE = 'Licensed Virtual Environments'


def get_oa_host(aps_guid):
    """
    :param aps_guid: string Example: 'a50ea74f-ed4d-4340-b529-fa0030c05c67'
    :return: migrator.odin_automation.platform.lib.api.aps_runtime.utils.AttrDict[]|None
    """
    aps = get_aps_connector_with_admin_token()
    return aps.get_first_resource(
        filter_str='implementing(%s),eq(aps.id,%s)' % (types.PAHost, aps_guid)
    )


@cached
def get_oa_host_id(aps_guid):
    """
    :param aps_guid: string Example: 'a50ea74f-ed4d-4340-b529-fa0030c05c67'
    :return: int|None Example: 11
    """
    oa_host = get_oa_host(aps_guid)
    return None if oa_host is None else oa_host.hostId


@cached
def get_oa_host_uid(host_id):
    aps = get_aps_connector_with_admin_token()
    return aps.get_first_resource(
        filter_str='implementing(%s),eq(hostId,%s)' % (types.PAHost, host_id)
    )['aps']['id']


def get_all_active_oa_hosts():
    aps = get_aps_connector_with_admin_token()
    return aps.get_resources(filter_str='implementing(%s),eq(provisioningStatus,READY)' % types.PAHost)


def get_ip_of_first_active_oa_pcs_host():
    first_active_pcs = get_first_active_oa_pcs_host()
    return first_active_pcs.backnetIp if first_active_pcs is not None else None


def get_first_active_oa_pcs_host():
    pcs_role_capacity_id = HOST_ATTRIBUTE_OACI_CLOUD_ROLE.lower()
    aps = get_aps_connector_with_admin_token()
    for oa_host in get_all_active_oa_hosts():
        oa_host_capacities = aps.get_resources('%s/capacities' % oa_host.aps.id)
        for oa_host_capacity in oa_host_capacities:
            if oa_host_capacity.name.lower() == pcs_role_capacity_id:
                return oa_host

    return None


def get_oam_host(ip):
    """
    :param ip: string Example: '10.37.1.59'
    :return: migrator.odin_automation.platform.lib.api.aps_runtime.utils.AttrDict|None
    """
    aps = get_aps_connector_with_admin_token()
    return aps.get_first_resource(
        filter_str='implementing(%s),eq(ip,%s)' % (types.OAMHost, ip)
    )


def get_host_capacities(host_uid):
    """
    Get host capacities by host resource APS ID
    :param host_uid: string is a APS ID of host resource
    :return: migrator.odin_automation.platform.lib.api.aps_runtime.utils.AttrDict[]|None
    """
    aps = get_aps_connector_with_admin_token()
    return aps.get_resources(resource_path='%s/capacities' % host_uid)
