# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core import MigrationNoRepeatError
from migrator.core.language import _T
from migrator.core.utils.networking.ip_utils import get_object_for_ip
from migrator.odin_automation.platform.lib.api.aps.host import get_oa_host_uid
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import get_aps_connector_with_admin_token

import subscription
import types


def provision_ip_address(subscription_id, host_id, ip='0.0.0.0', pool_id=None, ip_kind='SHARED'):
    """
    Provision IP address for subscription on host with selected type and kind and from IP pool (optionally).

    Note:
        The function always returns new APS uid for every call with the same parameters because APSC creates
        new resource every time.
    """
    aps_host_uid = get_oa_host_uid(host_id)
    pa_subscription_uid = subscription.get_subscription(subscription_id)['aps']['id']

    resource = {
        'aps': {'type': types.PAIPAddress},
        'address': ip,
        'netmask': '0.0.0.0',
        'type': get_object_for_ip(ip, 'IPv4', 'IPv6'),
        'ipKind': ip_kind,
        'host': {'aps': {'id': aps_host_uid}},
        'subscription': {'aps': {'id': pa_subscription_uid}}
    }
    if pool_id:
        resource['poolId'] = pool_id

    aps = get_aps_connector_with_admin_token()
    aps.setAdditionalHeaders({'APS-Subscription-ID': pa_subscription_uid})

    return aps.post_resource('', '', resource)['aps']['id']


def get_ip_address_uid(ip, subscription_id):
    """
    Search IP address under subscription.

    Note:
        The function return None if the IP address is not allocated. If the IP address was allocated under another
        subscription, function throw exception.

    :param ip: string
    :param subscription_id: int
    :return:
    """
    aps = get_aps_connector_with_admin_token()

    ip_address_with_subscription_resource = aps.get_first_resource(
        filter_str='implementing(%s),eq(address,%s),select(subscription)' % (types.PAIPAddress, ip)
    )

    if ip_address_with_subscription_resource is None:
        return None

    pa_subscription_resource = subscription.get_subscription(subscription_id)
    if ip_address_with_subscription_resource['subscription']['aps']['id'] != pa_subscription_resource['aps']['id']:
        raise MigrationNoRepeatError(
            _T('UNABLE_TO_ALLOCATE_IP_ADDRESS').format(ip=ip, name=pa_subscription_resource['name'])
        )

    return ip_address_with_subscription_resource['aps']['id']
