# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import APS2
from migrator.odin_automation.platform.lib.api.openapi.resource import PoaApiError

import types


def get_subscription(subscription_id):
    if subscription_id is None:
        return None

    aps = APS2.get_connector()
    return aps.get_first_resource(
        filter_str='implementing(%s),eq(subscriptionId,%s),select(account)' % (types.PASubscription, subscription_id)
    )


def get_service_template_id(subscription_id):
    """
    Get a service template ID of OA subscription

    :param subscription_id: str Example: u'1000013'
    :return: None|int Example: 2
    """
    subscription = get_subscription(subscription_id)
    return None if not subscription else subscription['serviceTemplateId']


def get_subscription_aps_id(subscription_id):
    """
    :param subscription_id: int Example: 1000020
    :return: string|None Example: 'f00370ce-db90-4eb7-a653-3fb1de9c4c44'
    """
    subscription = get_subscription(subscription_id)
    return subscription['aps']['id'] if subscription is not None else None


def get_subscriptions(subscription_ids):
    filter_str = 'implementing(%s)' % types.PASubscription

    if subscription_ids:
        filter_str += ',in(subscriptionId,(%s))' % ','.join([str(x) for x in subscription_ids])

    aps = APS2.get_connector()
    return aps.get_resources(filter_str=filter_str)


def get_subscription_service(oa_subscription_aps_id, service_type):
    """
    Get service of the specified OA subscription by type
    
    :param oa_subscription_aps_id: string
    :param service_type: string
    :return: migrator.odin_automation.platform.lib.api.aps_runtime.utils.AttrDict|None
    """
    aps = APS2.get_connector()
    return aps.get_first_resource(resource_path='%s/services' % oa_subscription_aps_id,
                                  filter_str='implementing(%s)' % service_type)


def get_billing_subscription(account_id, subscription_id):
    """
    :param account_id: string|integer Examples: '1000001', 1000001
    :param subscription_id: integer Example: 1000004
    :return:
    """
    if subscription_id is None:
        return None

    aps_admin = APS2.get_connector()
    sm = aps_admin.get_first_resource(filter_str='implementing(%s)' % types.PABSSSMServices)
    try:
        aps_user = APS2.get_connector_for_account_user(account_id)
        return aps_user.get_first_resource('%s/sm/subscriptions' % sm.aps.id,
                                           filter_str='eq(subscriptionId,%s)' % subscription_id
                                           )
    except PoaApiError, e:
        if e.module_id == 'AccountManagement' and e.error_code == 1006:
            # There is no such account
            return None
        else:
            raise


def get_service_template_info(subscription_id):
    aps = APS2.get_connector()
    service_template_id = aps.get_first_resource(
        filter_str='implementing(%s),eq(subscriptionId,%s)' % (types.PASubscription, subscription_id)
    )['serviceTemplateId']
    return aps.get_first_resource(
        filter_str='implementing(%s),eq(serviceTemplateId,%s)' % (types.PAServiceTemplate, service_template_id)
    )
