# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core import InternalError
from migrator.core.language import _T


class APSResource(object):
    _cache = {}

    def __init__(self, base_type, versions=None):
        self.str = base_type
        self.versions = versions

    def __str__(self):
        self._update_cache()
        return self._cache[self.str]['id']

    def __eq__(self, other):
        if isinstance(other, APSResource):
            self._update_cache()
            return other._cache[self.str]['id'] == self._cache[self.str]['id']

        raise TypeError('invalid type comparison')

    def __hash__(self):
        self._update_cache()
        return hash(self._cache[self.str]['id'])

    def _update_cache(self):
        if self.str not in self._cache:
            from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import APS2
            aps = APS2.get_connector()
            versions = self.versions or ['']
            for v in versions:
                aps_type = self.str + ('/' + v if v else '')
                types = [x for x in aps.get_types(aps_type) if x['id'].startswith(aps_type)]
                if types:
                    self._cache[self.str] = max(types, key=lambda x: x['id'])
                    break

            if self.str not in self._cache:
                raise InternalError(_T('APS_TYPE_WAS_NOT_FOUND'))

    @property
    def post(self):
        self._update_cache()
        return self._cache[self.str]['id']

    @property
    def properties(self):
        self._update_cache()
        return self._cache[self.str]['properties']

    @staticmethod
    def from_string(s):
        if isinstance(s, APSResource):
            return s
        elif not isinstance(s, (str, unicode)):
            raise TypeError("The value of the variable must be a string.")

        return APSResource(s.rsplit('.', 1)[0])

    put = post


class APSApplication(str):
    def __init__(self, base_type):
        super(APSApplication, self).__init__(base_type)
