# (C)2017 Ingram Micro Inc. all Rights Reserved
try:
    import simplejson as json
except ImportError:
    import json


class APSException(Exception):
    def __init__(self, message='', code=0):
        self.message = message
        self.code = code


class AttrDict(dict):
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __getattr__(self, name):
        try:
            return super(AttrDict, self).__getitem__(name)
        except KeyError:
            raise AttributeError(name)

    def toJSON(self):
        return json_encode(self)


def json_decode(data):
    """
    :type data: basestring
    :rtype: migrator.odin_automation.platform.lib.api.aps_runtime.utils.AttrDict | list
    """
    def _hook(obj):
        if isinstance(obj, dict):
            return AttrDict(obj)
        return obj

    # Since the response contentType is defined as "application/json", the function must return either an empty string,
    # or a string in the JSON format.
    if data == '':
        return None

    return json.loads(data, object_hook=_hook)


def json_encode(obj, indent=None):
    """
    :type obj: object
    :type indent: int | None
    :rtype: basestring
    """
    return json.dumps(obj, ensure_ascii=False, indent=indent)


def subscribe(aps, resource_uid, event, handler, relation=None, source_uid=None, source_type=None):
    event = {
        "event": event,
        "handler": handler,
        "relation": relation,
        "source": {"id": source_uid, "type": source_type}
    }
    aps.post_resource(resource_uid, 'aps/subscriptions', event)


