# (C)2017 Ingram Micro Inc. all Rights Reserved
import migrator.core.utils.networking.dns
import migrator.core.utils.networking.ip_utils
from migrator.core.utils import if_not_none
from migrator.odin_automation.platform.lib.connections.target_server import PleskTargetServer
from migrator.core.connections.database_server import DatabaseServer
from migrator.odin_automation.platform.lib.api.openapi.host import HostNotFoundException


class AutomationDatabaseTargetServer(DatabaseServer):
    def __init__(self, conn, db_type, host, port, user, password, ip):
        """
        :type conn: migrator.odin_automation.platfrom.lib.connections.target_connections.AutomationTargetConnections
        """
        self._db_type = db_type
        self._host = host
        self._port = port
        self._user = user
        self._password = password
        self._conn = conn
        self._ip = self._detect_ip(ip)

        try:
            self.oa_host_id = if_not_none(
                self._ip,
                lambda ip: self._conn.poa_api().host.get_host_id_by_ip(ip)
            )
        except HostNotFoundException:
            self.oa_host_id = None

    def type(self):
        return self._db_type

    def host(self):
        return self._host

    def port(self):
        return self._port

    def user(self):
        return self._user

    def password(self):
        return self._password

    @property
    def panel_server(self):
        return None

    @property
    def physical_server(self):
        if self.oa_host_id is not None:
            # If we have direct access to the server by POA protocols - use it
            return PleskTargetServer(self._conn, id=self.oa_host_id)
        else:
            # Otherwise, this database server is external - return None
            return None

    def _detect_ip(self, ip):
        """Detect IP address by hostname. In fact, we pass some IP in argument, but don't you mind.

        XXX Storing MSSQL instance name in 'host' breaks things;
        TODO Create a subclass and store the msqql-specific option separately
        """
        mssql_host = self.host().split('\\')[0]
        if self.host() == 'localhost' or mssql_host == '.':
            # 'localhost' for database server in Automation means Plesk node with local database
            return ip
        else:
            # host may contain MS SQL instance name, e.g. 10.50.52.100\MSSQLSERVER2008
            return migrator.core.utils.networking.dns.resolve(mssql_host)
