# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.data.model.account import AccountModel, ContactModel, AddressModel, PhoneModel, StaffMemberModel,\
    AuthModel, PersonalInfoModel


class OpenapiConverter(object):
    def convert_account_data(self, account_data):
        """
        :param account_data: migrator.odin_automation.platform.lib.api.openapi.account.AccountInfo
        :return: migrator.core.data.model.account.AccountModel
        """
        staff_members = []
        for staff_member in account_data.staff_members:
            staff_members.append(self.convert_staff_member_data(staff_member))

        account = AccountModel(
            external_id=None,
            info=PersonalInfoModel(
                company_name=account_data.company_name,
                admin_contact=self.convert_contact_data(account_data.admin_contact),
                billing_contact=self.convert_contact_data(account_data.billing_contact),
                tech_contact=self.convert_contact_data(account_data.tech_contact),
                company_address=self.convert_address_data(account_data.address),
                language=''
            ),
            staff_members=staff_members,
            subscriptions=[],
            oa_id=account_data.account_id
        )

        account.account_type = account_data.account_type
        return account

    def convert_staff_member_data(self, staff_member_data):
        """
        :param staff_member_data: migrator.odin_automation.platform.lib.api.openapi.account.MemberInfo
        :return: migrator.core.data.model.account.StaffMemberModel
        """
        return StaffMemberModel(
            authentication=self.convert_auth_data(staff_member_data.auth),
            contact=self.convert_contact_data(staff_member_data.contact),
            address=self.convert_address_data(staff_member_data.address),
            language=''
        )

    def convert_contact_data(self, contact_data):
        """
        :param contact_data: migrator.odin_automation.platform.lib.api.openapi.account.ContactInfo
        :return: migrator.core.data.model.account.ContactModel
        """
        return ContactModel(
            title='',
            first_name=contact_data.first_name,
            middle_name=contact_data.middle_name,
            last_name=contact_data.last_name,
            phone=self.convert_phone_data(contact_data.phone),
            fax=self.convert_phone_data(contact_data.fax),
            email=contact_data.email,
        )

    @staticmethod
    def convert_address_data(address_data):
        """
        :param address_data: migrator.odin_automation.platform.lib.api.openapi.account.AddressInfo
        :return: migrator.core.data.model.account.AddressModel
        """
        return AddressModel(
            city=address_data.city,
            address1=address_data.street_name,
            address2=address_data.address2,
            zipcode=address_data.zipcode,
            country=address_data.country,
            state=address_data.state,
        )

    @staticmethod
    def convert_phone_data(phone_data):
        """
        :param phone_data: migrator.odin_automation.platform.lib.api.openapi.account.PhoneInfo
        :return: migrator.core.data.model.account.PhoneModel
        """
        return PhoneModel(
            country_code=phone_data.country_code,
            area_code=phone_data.area_code,
            phone_num=phone_data.phone_num,
            ext_num=phone_data.ext_num,
        )

    @staticmethod
    def convert_fax_data(fax_data):
        """
        :param fax_data: migrator.odin_automation.platform.lib.api.openapi.account.FaxInfo
        :return: migrator.core.data.model.account.PhoneModel
        """
        return PhoneModel(
            country_code=fax_data.country_code,
            area_code=fax_data.area_code,
            phone_num=fax_data.phone_num,
            ext_num=fax_data.ext_num,
        )

    @staticmethod
    def convert_auth_data(auth_data):
        """
        :param auth_data: migrator.odin_automation.platform.lib.api.openapi.account.AuthInfo
        :return: migrator.core.data.model.account.AuthModel
        """
        return AuthModel(
            login=auth_data.login,
            password=auth_data.password,
        )
