# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core import InternalError
from migrator.core.actions.import_objects import SourceMappingSubscriptionInfo
from migrator.core.utils import cached
from migrator.odin_automation.platform.lib.api.aps.account import get_account_by_subscription


class PlatformSubscription(SourceMappingSubscriptionInfo):
    """The object that represents subscription with migration's data and which is created in Odin Automation"""

    def __init__(self, source_subscription_info, mapping):
        """

        :param source_subscription_info: migrator.core.data.model.subscription.SubscriptionModel
        :param mapping: migrator.core.session.migration_list.MigrationListData
        :return:
        """
        super(PlatformSubscription, self).__init__(model=source_subscription_info, mapping=mapping)
        self.source_info = self.model
        self.service = None

    @property
    def name(self):
        return self.source_info.name

    @property
    def external_id(self):
        return self.source_info.external_id

    @property
    def name_idn(self):
        return self.source_info.name.encode('idna')

    @property
    def id(self):
        return self.source_info.oa_id

    @property
    @cached
    def account_id(self):
        # If parent subscription was already created, we can return OA account id of subscription which is not created, yet
        if self.source_info.account.oa_id is not None:
            return self.source_info.account.oa_id

        elif self.id:
            return get_account_by_subscription(self.id)['id']

        raise InternalError('Neither Subscription {name} (#{external_id}) no its parent were not created in OA, yet'.format(
            name=self.name, external_id=self.external_id
        ))

    def pretty_print(self):
        return u"Subscription (name=%s, external_id=%s, id=%s)" % (self.name, self.source_info.external_id, self.id)
