from migrator.odin_automation.platform.lib.db import OaServiceName, _get_offset, _get_limit, _get_sort_field, \
    _get_sort_order, _get_list_of_where_conditions, _execute_sql_and_get_paginal_response, \
    execute_sql_and_get_formatted_result, transform_list_to_in_construction

import logging

logger = logging.getLogger(__name__)


def get_wsh_accounts(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                     eq_values=None):
    return get_accounts_by_service(OaServiceName.WSH, offset, limit, sort_field, sort_order, like, in_values, eq_values)


def get_whl_accounts(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                     eq_values=None):
    return get_accounts_by_service(OaServiceName.APACHE_NG, offset, limit, sort_field, sort_order, like, in_values, eq_values)


def get_lsh_accounts(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                     eq_values=None):
    return get_accounts_by_service(OaServiceName.APACHE, offset, limit, sort_field, sort_order, like, in_values, eq_values)


def get_accounts_by_service(service, offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                     eq_values=None):

    available_condition_fields = ['accounts.account_id', 'accounts.company_name', 'acc_email.value']

    sort_order = _get_sort_order(sort_order)
    sort_field = _get_sort_field(sort_field, 'accounts.account_id')
    limit = _get_limit(limit)
    offset = _get_offset(offset)
    where_list, parameters = _get_list_of_where_conditions(like, in_values, eq_values, available_condition_fields)

    sql = 'SELECT' \
          ' DISTINCT accounts.account_id  AS oa_account_id,' \
          ' accounts.company_name AS oa_account_company_name,' \
          " acc_email.value AS oa_account_email" \
          ' FROM services' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' JOIN subscriptions ON domain_services.subscription_id = subscriptions.sub_id' \
          ' JOIN accounts ON accounts.account_id = subscriptions.owner_id' \
          ' JOIN contact_properties AS acc_email ON acc_email.contact_id = accounts.adm_contact_id' \
          " WHERE services.name IN %s AND domain_services.hd_id IS NOT NULL" % transform_list_to_in_construction(service) + \
          " AND acc_email.name = 'email' AND subscriptions.is_active = 1 AND subscriptions.st_id IS NOT NULL" \
          '%s' % ((' AND ' + ' AND '.join(where_list)) if len(where_list) > 0 else '') + \
          ' ORDER BY {0} {1}'.format(sort_field, sort_order)

    columns = ('oa_account_id', 'oa_account_company_name', 'oa_account_email')

    return _execute_sql_and_get_paginal_response(sql, columns, parameters, 'oa_account_id', limit, offset)


def get_wsh_accounts_subscriptions_list():
    return get_accounts_subscriptions_list_by_service(OaServiceName.WSH)


def get_whl_accounts_subscriptions_list():
    return get_accounts_subscriptions_list_by_service(OaServiceName.APACHE_NG)


def get_lsh_accounts_subscriptions_list():
    return get_accounts_subscriptions_list_by_service(OaServiceName.APACHE)


def get_accounts_subscriptions_list_by_service(service):
    sql = 'SELECT' \
          ' accounts.account_id AS id,' \
          " array_to_string(array_agg(DISTINCT subscriptions.sub_id),',') AS sub_ids" \
          ' FROM services' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' JOIN subscriptions ON domain_services.subscription_id = subscriptions.sub_id' \
          ' JOIN accounts ON accounts.account_id = subscriptions.owner_id' \
          " WHERE services.name IN %s AND domain_services.hd_id IS NOT NULL" % transform_list_to_in_construction(service) + \
          ' AND subscriptions.is_active = 1 AND subscriptions.st_id IS NOT NULL' \
          ' GROUP BY accounts.account_id'

    columns = ('id', 'sub_ids')

    return {
        'items': execute_sql_and_get_formatted_result(sql, columns)
    }
