from migrator.odin_automation.platform.lib.db import OaServiceName, _get_offset, _get_limit, _get_sort_field, \
    _get_sort_order, _get_list_of_where_conditions, _execute_sql_and_get_paginal_response, \
    execute_sql_and_get_formatted_result, transform_list_to_in_construction
from migrator.settings import TargetServiceType


class OaHostType(object):
    # NG Cluster virtual node, i.e. container for all cluster nodes
    NG_CLUSTER = 'e'
    # Web server of NG cluster
    NG_CLUSTER_WEB_SERVER = 'f'
    # Standalone "Web Server NG" service node
    NG_STANDALONE_WEB_SERVER = 'g'


def get_oa_host(host_id):
    """
    :param host_id: int Example: 6
    :return: None|Dict
    """
    sql = 'SELECT' \
          ' hosts.primary_name, hosts.aps_uuid' \
          ' FROM hosts' \
          " WHERE hosts.host_id = %s"
    columns = ('primary_name', 'aps_guid')
    query_result = execute_sql_and_get_formatted_result(sql, columns, [host_id])

    return None if not query_result else query_result[0]


def get_oa_hostname(host_id):
    """
    :param host_id: int Example: 6
    :return: string|None Example: 'MSSQL-D2943078.D2943078.AQA.INT.ZONE'
    """
    oa_host = get_oa_host(host_id)
    return None if oa_host is None else oa_host['primary_name']


def _get_source_hosts(oa_service_name, offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None, eq_values=None, ):
    available_condition_fields = ['hosts.host_id', 'hosts.primary_name', 'hosts.htype', 'oa_host_platform']

    sort_order = _get_sort_order(sort_order)
    sort_field = _get_sort_field(sort_field, 'hosts.host_id')
    limit = _get_limit(limit)
    offset = _get_offset(offset)
    where_list, parameters = _get_list_of_where_conditions(
        like,
        in_values,
        eq_values,
        available_condition_fields,
        {
            'like': {
                'oa_host_platform': lambda v: (_get_platform_composite_field_sql(), v)
            },
            'eq': {
                'hosts.htype': lambda v: (
                    'hosts.htype', OaHostType.NG_CLUSTER if v == 'cluster' else OaHostType.NG_STANDALONE_WEB_SERVER
                )
            }
        }
    )

    sql = 'SELECT' \
          ' DISTINCT hosts.host_id AS oa_host_id,' \
          ' hosts.primary_name AS oa_host_name,' \
          " (CASE WHEN hosts.htype = '%s' THEN '%s' ELSE '%s' END) AS host_type," % (
              OaHostType.NG_CLUSTER, 'cluster', 'standalone-web-server') + \
          " %s AS oa_host_platform," % _get_platform_composite_field_sql() + \
          ' hosts.htype AS sortable_host_type' \
          ' FROM hosts' \
          ' JOIN services ON hosts.host_id = services.host_id' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' JOIN subscriptions ON domain_services.subscription_id = subscriptions.sub_id' \
          ' JOIN platforms ON hosts.platform_id = platforms.platform_id' \
          " WHERE services.name IN %s AND domain_services.hd_id IS NOT NULL" % transform_list_to_in_construction(oa_service_name) + \
          ' AND subscriptions.is_active = 1 AND subscriptions.st_id IS NOT NULL' \
          '%s' % ((' AND ' + ' AND '.join(where_list)) if len(where_list) > 0 else '') + \
          ' ORDER BY {0} {1}'.format(sort_field, sort_order)

    columns = ('oa_host_id', 'oa_host_name', 'oa_host_type', 'oa_host_platform')

    return _execute_sql_and_get_paginal_response(sql, columns, parameters, 'oa_host_id', limit, offset)


def get_ng_source_hosts(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                        eq_values=None):
    return _get_source_hosts(OaServiceName.APACHE_NG, offset, limit, sort_field, sort_order, like, in_values, eq_values)


def get_ng_hosts_subscriptions_list():
    sql = 'SELECT' \
          ' hosts.host_id AS id,' \
          " array_to_string(array_agg(DISTINCT domain_services.hd_id),',') AS sub_ids" \
          ' FROM hosts' \
          ' JOIN services ON hosts.host_id = services.host_id' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' JOIN subscriptions ON domain_services.subscription_id = subscriptions.sub_id' \
          " WHERE services.name IN %s AND domain_services.hd_id IS NOT NULL" % transform_list_to_in_construction(OaServiceName.APACHE_NG) + \
          ' AND subscriptions.is_active = 1' \
          ' GROUP BY hosts.host_id'

    columns = ('id', 'sub_ids')

    return {
        'items': execute_sql_and_get_formatted_result(sql, columns)
    }


def get_wsh_source_hosts(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                         eq_values=None):
    return _get_source_hosts(OaServiceName.WSH, offset, limit, sort_field, sort_order, like, in_values, eq_values)


def get_lsh_source_hosts(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                         eq_values=None):
    return _get_source_hosts(OaServiceName.APACHE, offset, limit, sort_field, sort_order, like, in_values, eq_values)


def get_lsh_hosts_subscriptions_list():
    return get_hosts_subscriptions_list_by_service(OaServiceName.APACHE)


def get_wsh_hosts_subscriptions_list():
    return get_hosts_subscriptions_list_by_service(OaServiceName.WSH)


def get_whl_hosts_subscriptions_list():
    return get_hosts_subscriptions_list_by_service(OaServiceName.APACHE_NG)


def get_hosts_subscriptions_list_by_service(service):
    sql = 'SELECT' \
          ' hosts.host_id AS id,' \
          " array_to_string(array_agg(DISTINCT subscriptions.sub_id),',') AS sub_ids" \
          ' FROM hosts' \
          ' JOIN services ON hosts.host_id = services.host_id' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' JOIN subscriptions ON domain_services.subscription_id = subscriptions.sub_id' \
          " WHERE services.name IN %s AND domain_services.hd_id IS NOT NULL" % transform_list_to_in_construction(service) + \
          ' AND subscriptions.is_active = 1 AND subscriptions.st_id IS NOT NULL' \
          ' GROUP BY hosts.host_id'

    columns = ('id', 'sub_ids')

    return {
        'items': execute_sql_and_get_formatted_result(sql, columns)
    }


def get_service_nodes(account_id=None, target_service_type=None):
    # Check that host is ready to provide
    where_conditions = ['hosts.provisioning = 1']
    parameters = []

    if account_id is not None:
        where_conditions.append('hosts.owner_id = %s')
        parameters.append(account_id)

    if target_service_type == TargetServiceType.NG:
        sql = 'SELECT' \
              ' DISTINCT hosts.host_id AS id,' \
              ' hosts.primary_name AS hostname,' \
              " (CASE WHEN hosts.htype = '%s' THEN '%s' ELSE '%s' END) AS host_type," % (
                  OaHostType.NG_CLUSTER, 'cluster', 'standalone-web-server') + \
              " %s AS host_platform" % _get_platform_composite_field_sql() + \
              ' FROM hosts' \
              ' JOIN platforms ON hosts.platform_id = platforms.platform_id' \
              " WHERE hosts.htype IN ('%s', '%s')" % (
                  OaHostType.NG_CLUSTER, OaHostType.NG_STANDALONE_WEB_SERVER) + \
              '%s' % (' AND ' + ' AND '.join(where_conditions) if len(where_conditions) > 0 else '')
    elif target_service_type is None:
        sql = 'SELECT' \
              ' DISTINCT hosts.host_id AS id,' \
              ' hosts.primary_name AS hostname' \
              ' FROM hosts' \
              '%s' % (' WHERE ' + ' AND '.join(where_conditions) if len(where_conditions) > 0 else '')
    else:
        return []

    columns = ('id', 'hostname', 'host_type', 'host_platform')

    return execute_sql_and_get_formatted_result(sql, columns, parameters)


def _get_platform_composite_field_sql():
    return "CONCAT(platforms.arch, ' ', platforms.opsys, ' ', platforms.osrel)"
