from migrator.odin_automation.platform.lib.db import OaServiceName, _get_offset, _get_limit, _get_sort_field, \
    _get_sort_order, _get_list_of_where_conditions, _execute_sql_and_get_paginal_response, \
    transform_list_to_in_construction


def get_lsh_subscriptions(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                          eq_values=None):
    return get_subscriptions_by_service(
        OaServiceName.APACHE, offset, limit, sort_field, sort_order, like, in_values, eq_values
    )


def get_ng_webspaces(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                     eq_values=None):
    available_condition_fields = ['domain_services.hd_id', 'domain_services.name', 'service_templates.name',
                                  'subscriptions.sub_id', 'subscriptions.name', 'accounts.account_id',
                                  'accounts.company_name', 'hosts.primary_name', 'hosts.host_id', 'acc_email.value',
                                  'acc_first_name.value', 'acc_last_name.value']

    sort_order = _get_sort_order(sort_order)
    sort_field = _get_sort_field(sort_field, 'subscriptions.sub_id')
    where_list, parameters = _get_list_of_where_conditions(like, in_values, eq_values, available_condition_fields)
    limit = _get_limit(limit)
    offset = _get_offset(offset)

    sql = 'SELECT' \
          ' DISTINCT domain_services.hd_id AS oa_webspace_id,' \
          ' domain_services.name AS oa_webspace_name,' \
          ' service_templates.name AS oa_service_template_name,' \
          ' service_templates.st_id AS oa_service_template_id,' \
          ' subscriptions.sub_id AS oa_subscription_id,' \
          ' subscriptions.name AS oa_subscription_name,' \
          ' accounts.account_id AS oa_account_id,' \
          ' accounts.company_name AS oa_account_company_name,' \
          ' hosts.primary_name AS oa_host_name,' \
          ' hosts.host_id AS oa_host_id,' \
          ' acc_email.value AS oa_account_email,' \
          ' acc_first_name.value AS oa_account_first_name,' \
          ' acc_last_name.value AS oa_account_last_name' \
          ' FROM services' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' JOIN subscriptions ON domain_services.subscription_id = subscriptions.sub_id' \
          ' JOIN service_templates ON service_templates.st_id = subscriptions.st_id' \
          ' JOIN accounts ON accounts.account_id = subscriptions.owner_id' \
          ' JOIN contact_properties AS acc_email ON acc_email.contact_id = accounts.adm_contact_id' \
          ' JOIN contact_properties AS acc_first_name ON acc_first_name.contact_id = accounts.adm_contact_id' \
          ' JOIN contact_properties AS acc_last_name ON acc_last_name.contact_id = accounts.adm_contact_id' \
          ' JOIN hosts ON hosts.host_id = services.host_id' \
          " WHERE services.name IN %s AND subscriptions.sub_id IS NOT NULL" % transform_list_to_in_construction(OaServiceName.APACHE_NG) + \
          ' AND subscriptions.is_active = 1' \
          " AND acc_email.name = '{0}' AND acc_first_name.name = '{1}' AND acc_last_name.name = '{2}'".format(
              'email', 'first_name', 'last_name'
          ) + \
          '%s' % ((' AND ' + ' AND '.join(where_list)) if len(where_list) > 0 else '') + \
          ' ORDER BY {0} {1}'.format(sort_field, sort_order)

    columns = ('oa_webspace_id', 'oa_webspace_name', 'oa_service_template_name', 'oa_service_template_id',
               'oa_subscription_id', 'oa_subscription_name', 'oa_account_id', 'oa_account_company_name',
               'oa_host_name', 'oa_host_id', 'oa_account_email', 'oa_account_first_name', 'oa_account_last_name')

    return _execute_sql_and_get_paginal_response(sql, columns, parameters, 'oa_subscription_id', limit, offset)


def get_wsh_subscriptions(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                          eq_values=None):
    return get_subscriptions_by_service(
        OaServiceName.WSH, offset, limit, sort_field, sort_order, like, in_values, eq_values
    )


def get_whl_subscriptions(offset=None, limit=None, sort_field=None, sort_order=None, like=None, in_values=None,
                          eq_values=None):
    return get_subscriptions_by_service(
        OaServiceName.APACHE_NG, offset, limit, sort_field, sort_order, like, in_values, eq_values
    )


def get_subscriptions_by_service(service_name, offset=None, limit=None, sort_field=None, sort_order=None, like=None,
                                 in_values=None, eq_values=None):
    available_condition_fields = ['subscriptions.sub_id', 'subscriptions.name', 'accounts.account_id',
                                  'accounts.company_name', 'hosts.primary_name', 'hosts.host_id',
                                  'acc_email.value', 'acc_first_name.value', 'acc_last_name.value']

    sort_order = _get_sort_order(sort_order)
    sort_field = _get_sort_field(sort_field, 'subscriptions.sub_id')
    where_list, parameters = _get_list_of_where_conditions(like, in_values, eq_values, available_condition_fields)
    limit = _get_limit(limit)
    offset = _get_offset(offset)

    sql = 'SELECT' \
          ' DISTINCT subscriptions.sub_id AS oa_subscription_id,' \
          ' subscriptions.name AS oa_subscription_name,' \
          ' accounts.account_id AS oa_account_id,' \
          ' accounts.company_name AS oa_account_company_name,' \
          " array_to_string(array_agg(hosts.primary_name ORDER BY hosts.host_id ASC),',') AS oa_host_name," \
          " array_to_string(array_agg(hosts.host_id ORDER BY hosts.host_id ASC),',') AS oa_host_id," \
          ' acc_email.value AS oa_account_email,' \
          ' acc_first_name.value AS oa_account_first_name,' \
          ' acc_last_name.value AS oa_account_last_name' \
          ' FROM services' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' JOIN subscriptions ON domain_services.subscription_id = subscriptions.sub_id' \
          ' JOIN service_templates ON service_templates.st_id = subscriptions.st_id' \
          ' JOIN accounts ON accounts.account_id = subscriptions.owner_id' \
          ' JOIN contact_properties AS acc_email ON acc_email.contact_id = accounts.adm_contact_id' \
          ' JOIN contact_properties AS acc_first_name ON acc_first_name.contact_id = accounts.adm_contact_id' \
          ' JOIN contact_properties AS acc_last_name ON acc_last_name.contact_id = accounts.adm_contact_id' \
          ' JOIN hosts ON hosts.host_id = services.host_id' \
          " WHERE services.name IN %s AND subscriptions.sub_id IS NOT NULL" % transform_list_to_in_construction(service_name) + \
          ' AND subscriptions.is_active = 1 AND subscriptions.st_id IS NOT NULL' \
          " AND acc_email.name = '{0}' AND acc_first_name.name = '{1}' AND acc_last_name.name = '{2}'".format(
              'email', 'first_name', 'last_name'
          ) + \
          '%s' % ((' AND ' + ' AND '.join(where_list)) if len(where_list) > 0 else '') + \
          ' GROUP BY subscriptions.sub_id, accounts.account_id, acc_email.value, acc_first_name.value,' + \
          ' acc_last_name.value' \
          ' ORDER BY {0} {1}'.format(sort_field, sort_order)

    columns = ('oa_subscription_id', 'oa_subscription_name', 'oa_account_id', 'oa_account_company_name',
               'oa_host_name', 'oa_host_id', 'oa_account_email', 'oa_account_first_name', 'oa_account_last_name')

    return _execute_sql_and_get_paginal_response(sql, columns, parameters, 'oa_subscription_id', limit, offset)
