# (C)2017 Ingram Micro Inc. all Rights Reserved
import re

from migrator.core.data.model.account import AccountModel, ContactModel, AddressModel, PhoneModel, StaffMemberModel,\
    AuthModel, PersonalInfoModel
from migrator.core.language import _T

MIN_LOGIN_LENGTH = 5
MIN_PASSWORD_LENGTH = 7

MAX_COMPANY_NAME_LENGTH = 80
MAX_ZIPCODE_LENGTH = 10
MAX_COUNTRY_LENGTH = 2
MAX_STATE_LENGTH = 80
MAX_ADDRESS_LENGTH = 80
MAX_CITY_LENGTH = 40
MAX_NAME_LENGTH = 60
MAX_COUNTRY_CODE_LENGTH = 4
MAX_AREA_CODE_LENGTH = 10
MAX_PHONE_NUMBER_LENGTH = 20
MAX_EXTENSION_NUMBER_LENGTH = 10
MAX_EMAIL_LENGTH = 100
MAX_LOGIN_LENGTH = 64
MAX_PASSWORD_LENGTH = 41

LOGIN_REGEXP = '^[a-zA-Z0-9\_\.\-\@]+$'
NUMBER_FILTER_REGEXP = '[^0-9]*'
NUMBER_REGEXP = '^[0-9]+$'
EMAIL_REGEXP = '^[a-zA-Z0-9_.-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'
COUNTRIES_LIST = {
    'ac': 'Ascension Island', 'ad': 'Andorra', 'ae': 'United Arab Emirates', 'af': 'Afghanistan',
    'ag': 'Antigua and Barbuda', 'ai': 'Anguilla', 'al': 'Albania', 'am': 'Armenia', 'an': 'Netherlands Antilles',
    'ao': 'Angola', 'aq': 'Antarctica', 'ar': 'Argentina', 'as': 'American Samoa', 'at': 'Austria', 'au': 'Australia',
    'aw': 'Aruba', 'az': 'Azerbaijan', 'ba': 'Bosnia and Herzegovina', 'bb': 'Barbados', 'bd': 'Bangladesh',
    'be': 'Belgium', 'bf': 'Burkina Faso', 'bg': 'Bulgaria', 'bh': 'Bahrain', 'bi': 'Burundi', 'bj': 'Benin',
    'bm': 'Bermuda', 'bn': 'Brunei Darussalam', 'bo': 'Bolivia', 'br': 'Brazil', 'bs': 'Bahamas', 'bt': 'Bhutan',
    'bv': 'Bouvet Island', 'bw': 'Botswana', 'by': 'Belarus', 'bz': 'Belize', 'ca': 'Canada',
    'cc': 'Cocos (Keeling) Islands', 'cd': 'Congo Democratic People\'s Republic', 'cf': 'Central African Republic',
    'cg': 'Congo Republic of', 'ch': 'Switzerland', 'ci': 'Ivory Coast', 'ck': 'Cook Islands', 'cl': 'Chile',
    'cm': 'Cameroon', 'cn': 'China', 'co': 'Colombia', 'cr': 'Costa Rica', 'cu': 'Cuba', 'cv': 'Cape Verde',
    'cx': 'Christmas Island', 'cy': 'Cyprus', 'cz': 'Czech Republic', 'de': 'Germany', 'dj': 'Djibouti',
    'dk': 'Denmark', 'dm': 'Dominica', 'do': 'Dominican Republic Nevis', 'dz': 'Algeria', 'ec': 'Ecuador',
    'ee': 'Estonia', 'eg': 'Egypt', 'eh': 'Western Sahara', 'er': 'Eritrea', 'es': 'Spain', 'et': 'Ethiopia',
    'fi': 'Finland', 'fj': 'Fiji', 'fk': 'Falkland Islands (Malvinas)', 'fm': 'Micronesia (Federal State of)',
    'fo': 'Faroe Islands', 'fr': 'France', 'fx': 'Metropolitan France', 'ga': 'Gabon', 'gb': 'United Kingdom',
    'gd': 'Grenada', 'ge': 'Georgia', 'gf': 'French Guiana', 'gh': 'Ghana', 'gi': 'Gibraltar', 'gl': 'Greenland',
    'gm': 'Gambia', 'gn': 'Guinea', 'gp': 'Guadeloupe', 'gq': 'Equatorial Guinea', 'gr': 'Greece',
    'gs': 'South Georgia and South Sandwich Islands', 'gt': 'Guatemala', 'gu': 'Guam', 'gw': 'Guinea-bissau',
    'gy': 'Guyana', 'hk': 'Hong Kong', 'hm': 'Heard and McDonald Islands', 'hn': 'Honduras', 'hr': 'Croatia/Hrvatska',
    'ht': 'Haiti', 'hu': 'Hungary', 'id': 'Indonesia', 'ie': 'Ireland', 'il': 'Israel', 'in': 'India',
    'io': 'British Indian Ocean Territory', 'iq': 'Iraq', 'ir': 'Iran (Islamic Republic of)', 'is': 'Iceland',
    'it': 'Italy', 'jm': 'Jamaica', 'jo': 'Jordan', 'jp': 'Japan', 'ke': 'Kenya', 'kg': 'Kyrgyzstan (or Kirgizia)',
    'kh': 'Cambodia', 'ki': 'Kiribati', 'km': 'Comoros', 'kn': 'Saint Kitts and Nevis',
    'kp': 'Korea (Democratic People\'s Republic of) [North]', 'kr': 'Korea (Republic of) [South]', 'kw': 'Kuwait',
    'ky': 'Cayman Islands', 'kz': 'Kazakhstan', 'la': 'Laos People\'s Democratic Republic', 'lb': 'Lebanon',
    'lc': 'Saint Lucia', 'li': 'Liechtenstein', 'lk': 'Sri Lanka', 'lr': 'Liberia', 'ls': 'Lesotho', 'lt': 'Lithuania',
    'lu': 'Luxembourg', 'lv': 'Latvia', 'ly': 'Libya (Libyan Arab Jamahiriya)', 'ma': 'Morocco', 'mc': 'Monaco',
    'md': 'Moldova (Republic of)', 'mg': 'Madagascar', 'mh': 'Marshall Islands',
    'mk': 'Macedonia Former Yugoslav Republic', 'ml': 'Mali', 'mm': 'Myanmar (Burma)', 'mn': 'Mongolia', 'mo': 'Macau',
    'mp': 'Northern Mariana Islands', 'mq': 'Martinique', 'mr': 'Mauritania', 'ms': 'Montserrat', 'mt': 'Malta',
    'mu': 'Mauritius', 'mv': 'Maldives', 'mw': 'Malawi', 'mx': 'Mexico', 'my': 'Malaysia', 'mz': 'Mozambique',
    'na': 'Namibia', 'nc': 'New Caledonia', 'ne': 'Niger', 'nf': 'Norfolk Island', 'ng': 'Nigeria', 'ni': 'Nicaragua',
    'nl': 'The Netherlands', 'no': 'Norway', 'np': 'Nepal', 'nr': 'Nauru', 'nu': 'Niue', 'nz': 'New Zealand',
    'om': 'Oman', 'pa': 'Panama', 'pe': 'Peru', 'pf': 'French Polynesia', 'pg': 'Papua New Guinea', 'ph': 'Philippines',
    'pk': 'Pakistan', 'pl': 'Poland', 'pm': 'St. Pierre and Miquelon', 'pn': 'Pitcairn Island', 'pr': 'Puerto Rico',
    'ps': 'Palestinian Territories', 'pt': 'Portugal', 'pw': 'Palau', 'py': 'Paraguay', 'qa': 'Qatar',
    're': 'Reunion Island', 'ro': 'Romania', 'rs': 'Serbia', 'ru': 'Russian Federation', 'rw': 'Rwanda',
    'sa': 'Saudi Arabia', 'sb': 'Solomon Islands', 'sc': 'Seychelles', 'sd': 'Sudan', 'se': 'Sweden', 'sg': 'Singapore',
    'sh': 'St. Helena', 'si': 'Slovenia', 'sj': 'Svalbard and Jan Mayen Islands', 'sk': 'Slovakia (Slovak Republic)',
    'sl': 'Sierra Leone', 'sm': 'San Marino', 'sn': 'Senegal', 'so': 'Somalia', 'sr': 'Suriname',
    'st': 'Sao Tome and Principe', 'sv': 'El Salvador', 'sy': 'Syria (Syrian Arab Republic)', 'sz': 'Swaziland',
    'tc': 'Turks and Caicos Islands', 'td': 'Chad', 'tf': 'French Southern Territories', 'tg': 'Togo', 'th': 'Thailand',
    'tj': 'Tajikistan', 'tk': 'Tokelau', 'tm': 'Turkmenistan', 'tn': 'Tunisia', 'to': 'Tonga', 'tr': 'Turkey',
    'tt': 'Trinidad and Tobago', 'tv': 'Tuvalu', 'tw': 'Taiwan (Province of China)',
    'tz': 'Tanzania (United Republic of)', 'ua': 'Ukraine', 'ug': 'Uganda',
    'um': 'United States Minor Outlying Islands', 'us': 'United States of America', 'uy': 'Uruguay', 'uz': 'Uzbekistan',
    'va': 'Holy See (City Vatican State)', 'vc': 'Saint Vincent and the Grenadines', 've': 'Venezuela',
    'vg': 'Virgin Islands (British)', 'vi': 'Virgin Islands (USA)', 'vn': 'Vietnam', 'vu': 'Vanuatu',
    'wf': 'Wallis and Futuna Islands', 'ws': 'Western Samoa', 'ye': 'Yemen', 'yt': 'Mayotte', 'za': 'South Africa',
    'zm': 'Zambia', 'zr': 'Zaire', 'zw': 'Zimbabwe'
}


class InfoFormatter(object):
    def __init__(self):
        self.is_critical = False

    def format_account_data(self, account_data):
        """
        :param account_data: migrator.core.data.model.account.AccountModel
        :return: migrator.core.data.model.account.AccountModel
        """

        issues = []
        staff_members = []
        for staff_member in account_data.staff_members:
            staff_member_issues = []
            staff_members.append(self._format_staff_member_data(staff_member, staff_member_issues))
            if staff_member_issues:
                issues.append(_T("THERE_ARE_ISSUES_WITH_THE_STAFF_MEMBER").format(
                    login=staff_member.authentication.login,
                    issues='\n'.join(set(staff_member_issues))
                ))

        account_isssues = []
        account = AccountModel(
            external_id=account_data.external_id,
            info=PersonalInfoModel(
                company_name=self._format_company_name(account_data.info.company_name, account_isssues),
                admin_contact=self._format_contact_data(account_data.info.admin_contact, account_isssues),
                billing_contact=self._format_contact_data(account_data.info.billing_contact, account_isssues),
                tech_contact=self._format_contact_data(account_data.info.tech_contact, account_isssues),
                company_address=self._format_address_data(account_data.info.company_address, account_isssues),
                language=self._format_language_name(account_data.info.language, account_isssues)
            ),
            staff_members=staff_members,
            subscriptions=[]
        )
        if account_isssues:
            issues.append(_T("THERE_ARE_ISSUES_WITH_THE_ACCOUNT").format(
                login=account_data.staff_members[0].authentication.login,
                issues='\n'.join(set(account_isssues))
            ))

        issue_message = None
        if issues:
            issue_message = _T("THE_FOLLOWING_ISSUES_WERE_FOUND_DURING_THE_MIGRATION_OF").format(
                login=account_data.staff_members[0].authentication.login,
                issues='\n'.join(issues)
            )
            if self.is_critical:
                issue_message += '\n{message}'.format(message=_T("AFTER_RESOLVING_THE_ISSUES_RESTART_MIGRATION"))

        return account, issue_message, self.is_critical

    def _format_staff_member_data(self, staff_member_data, issues):
        """
        :param staff_member_data: migrator.core.data.model.account.StaffMemberModel
        :return: migrator.core.data.model.account.StaffMemberModel
        """
        return StaffMemberModel(
            authentication=self._format_auth_data(staff_member_data.authentication, issues),
            contact=self._format_contact_data(staff_member_data.contact, issues),
            address=self._format_address_data(staff_member_data.address, issues),
            language=self._format_language_name(staff_member_data.language, issues)
        )

    def _format_contact_data(self, contact_data, issues):
        """
        :param contact_data: migrator.core.data.model.account.ContactModel
        :return: migrator.core.data.model.account.ContactModel
        """
        return ContactModel(
            title=self._format_title(contact_data.title, issues),
            first_name=self._format_first_name(contact_data.first_name, issues),
            middle_name=self._format_middle_name(contact_data.middle_name, issues),
            last_name=self._format_last_name(contact_data.last_name, issues),
            phone=self._format_phone_data(contact_data.phone, issues),
            fax=self._format_fax_data(contact_data.fax, issues),
            email=self._format_email(contact_data.email, issues),
        )

    def _format_address_data(self, address_data, issues):
        """
        :param address_data: migrator.odin_automation.platform.lib.api.openapi.account.AddressInfo
        :return: migrator.odin_automation.platform.lib.api.openapi.account.AddressInfo
        """
        return AddressModel(
            city=self._format_city(address_data.city, issues),
            address1=self._format_address1(address_data.address1, issues),
            address2=self._format_address2(address_data.address2, issues),
            zipcode=self._format_zipcode(address_data.zipcode, issues),
            country=self._format_country(address_data.country, issues),
            state=self._format_state(address_data.state, issues),
        )

    def _format_phone_data(self, phone_data, issues):
        """
        :param phone_data: migrator.core.data.model.account.PhoneModel
        :return: migrator.core.data.model.account.PhoneModel
        """
        return PhoneModel(
            country_code=self._format_country_code(phone_data.country_code, issues),
            area_code=self._format_area_code(phone_data.area_code, issues),
            phone_num=self._format_phone_num(phone_data.phone_num, issues),
            ext_num=self._format_ext_num(phone_data.ext_num, issues),
        )

    def _format_fax_data(self, phone_data, issues):
        """
        :param phone_data: migrator.core.data.model.account.PhoneModel
        :return: migrator.core.data.model.account.PhoneModel
        """
        return PhoneModel(
            country_code=self._format_country_code(phone_data.country_code, issues),
            area_code=self._format_area_code(phone_data.area_code, issues),
            phone_num=self._format_fax_num(phone_data.phone_num, issues),
            ext_num=self._format_ext_num(phone_data.ext_num, issues),
        )

    def _format_auth_data(self, auth_data, issues):
        """
        :param auth_data: migrator.core.data.model.account.AuthModel
        :return: migrator.core.data.model.account.AuthModel
        """
        # When migrating from all panels, we use email as the login.
        # For this reason, we are not checking the validity of login, to get rid of similar messages.
        return AuthModel(
            login=auth_data.login,
            password=self._format_password(auth_data.password, issues),
        )

    def _format_company_name(self, company_name, issues):
        return self._format_string('company name', company_name, MAX_COMPANY_NAME_LENGTH, issues)

    def _format_zipcode(self, zipcode, issues):
        return self._format_string('zipcode', zipcode, MAX_ZIPCODE_LENGTH, issues, is_required=True)

    def _format_country(self, country, issues):
        return self._format_string('country', country, MAX_COUNTRY_LENGTH, issues)

    def _format_state(self, state, issues):
        return self._format_string('state', state, MAX_STATE_LENGTH, issues)

    def _format_address1(self, address, issues):
        return self._format_string('address line 1', address, MAX_ADDRESS_LENGTH, issues, is_required=True)

    def _format_address2(self, address, issues):
        return self._format_string('address line 2', address, MAX_ADDRESS_LENGTH, issues)

    def _format_city(self, city, issues):
        return self._format_string('city', city, MAX_CITY_LENGTH, issues, is_required=True)

    def _format_title(self, title, issues):
        return title

    def _format_first_name(self, name, issues):
        return self._format_string('first name', name, MAX_NAME_LENGTH, issues, is_required=True)

    def _format_middle_name(self, name, issues):
        return self._format_string('middle name', name, MAX_NAME_LENGTH, issues)

    def _format_last_name(self, name, issues):
        return self._format_string('last name', name, MAX_NAME_LENGTH, issues)

    def _format_country_code(self, country_code, issues):
        country_code = self._remove_incorrect_symbols(country_code, NUMBER_FILTER_REGEXP)
        return self._format_string('country code', country_code, MAX_COUNTRY_CODE_LENGTH, issues)

    def _format_area_code(self, area_code, issues):
        area_code = self._remove_incorrect_symbols(area_code, NUMBER_FILTER_REGEXP)
        return self._format_string('area code', area_code, MAX_AREA_CODE_LENGTH, issues)

    def _format_phone_num(self, phone_num, issues):
        phone_num = self._remove_incorrect_symbols(phone_num, NUMBER_FILTER_REGEXP)
        return self._format_string('phone number', phone_num, MAX_PHONE_NUMBER_LENGTH, issues, is_required=True)

    def _format_fax_num(self, phone_num, issues):
        phone_num = self._remove_incorrect_symbols(phone_num, NUMBER_FILTER_REGEXP)
        return self._format_string('fax number', phone_num, MAX_PHONE_NUMBER_LENGTH, issues)

    def _format_ext_num(self, ext_num, issues):
        return self._format_string('extention number', ext_num, MAX_EXTENSION_NUMBER_LENGTH, issues)

    def _format_language_name(self, language_name, issues):
        return language_name

    def _format_email(self, email, issues):
        self._validate_email('Email', email, issues)
        return self._format_string('email', email, MAX_EMAIL_LENGTH, issues, is_critical=True, is_required=True)

    def _format_login(self, login, issues):
        self._validate_email('Login', login, issues)
        return self._format_string('login', login, MAX_LOGIN_LENGTH, issues, is_critical=True, is_required=True)

    def _format_password(self, password, issues):
        return password

    def _format_string(self, field_name, value, length, issues, is_required=False, is_critical=False):
        if value is None:
            value = ''

        if value == '' and is_required:
            self.is_critical = True
            issues.append(_T("FIELD_NAME_CANNOT_BE_EMPTY").format(
                field_name=field_name,
            ))

        new_value = value[:length]
        if value != new_value:
            if is_critical:
                self.is_critical = True
                issues.append(_T("THE_LENGTH_OF_FIELD_EXCEEDS_MAXIMUM_CRITICAL").format(
                    field_name=field_name,
                    max_len=length,
                    new_value=new_value,
                ))
            else:
                issues.append(_T("THE_LENGTH_OF_FIELD_EXCEEDS_MAXIMUM").format(
                    field_name=field_name,
                    max_len=length,
                    new_value=new_value,
                ))

        return new_value

    def _validate_email(self, field_name, value, issues):
        if value is None or value == '':
            return

        match = re.match(EMAIL_REGEXP, value)
        if match is None:
            self.is_critical = True
            issues.append(_T("FIELD_NAME_CAN_CONTATIN_ONLY_LATIN").format(
                field_name=field_name, lower_filed_name=field_name.lower(), value=value
            ))

    def _remove_incorrect_symbols(self, value, regexp):
        if value is None:
            value = ''

        return re.sub(regexp, '', value)
