# (C)2017 Ingram Micro Inc. all Rights Reserved
from base64 import b64decode, b64encode
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
from poaupdater import uCrypt

from migrator.core import InternalError

AES_MARK = '$AES'
PEMBF_MARK = '{PEMBF}'
PEMBF_KEY = 'Ujcgjlb<kfujckfdbLtdtkjgthjd'
PEMBF_BLOCK_SIZE = 8
PEMBF_PADDING_SYMBOL = '\0'


class EncryptionType(object):
    PEMBF = 'pembf'
    AES = 'aes'


def decrypt_data(data):
    if data.startswith(PEMBF_MARK):
        return _decrypt_pembf_data(data)
    elif data.startswith(AES_MARK):
        # uCrypt.decryptData may return password padded with whitespace chars. Such a password may break migration.
        # So, we perform additional cleanup before returning it.
        return uCrypt.decryptData(data).rstrip()
    else:
        raise InternalError('An internal error occurred during data decryption. Unsupported encryption type.')


def encrypt_data(data, encryption_type=EncryptionType.AES):
    if encryption_type == EncryptionType.PEMBF:
        return _encrypt_pembf_data(data)
    elif encryption_type == EncryptionType.AES:
        return uCrypt.encryptData(data)
    else:
        raise InternalError('An internal error occurred during data encryption. Unsupported encryption type.')


def _decrypt_pembf_data(data):
    cipher = Cipher(algorithms.Blowfish(PEMBF_KEY), modes.ECB(), backend=default_backend())
    decryptor = cipher.decryptor()
    data_for_decryption = b64decode(data[len(PEMBF_MARK):])
    result = decryptor.update(data_for_decryption) + decryptor.finalize()
    # remove padding from result
    for i, c in enumerate(result):
        if ord(c) == ord(PEMBF_PADDING_SYMBOL):
            result = result[:i]
            break

    return result


def _encrypt_pembf_data(data):
    cipher = Cipher(algorithms.Blowfish(PEMBF_KEY), modes.ECB(), backend=default_backend())
    encryptor = cipher.encryptor()
    # add padding to data
    padding = PEMBF_PADDING_SYMBOL * (PEMBF_BLOCK_SIZE - (len(data) % PEMBF_BLOCK_SIZE))
    data_for_encryption = data + padding
    result = encryptor.update(data_for_encryption) + encryptor.finalize()
    result = PEMBF_MARK + b64encode(result)
    return result
