# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import MigrationError, _T
from migrator.core.transfer.service_import_api import ServiceImportApiBase
from migrator.core.utils.loggers.log import trace
from migrator.odin_automation.platform.actions.dns_forwarding.configure_source_nameservers import \
    ConfigureSourceNameserversOA
from migrator.odin_automation.platform.actions.import_dns_records import ImportDnsRecords
from migrator.odin_automation.platform.actions.import_domain import ImportDomain
from migrator.odin_automation.platform.lib.data.subscription_importer import OaSubscriptionParameters
from migrator.settings import MIGRATION_TYPE_FULL

logger = logging.getLogger(__name__)


class ServiceImportApiPlatform(ServiceImportApiBase):
    @trace
    def get_service_specific_activation_parameters(self, global_context, migration_item_model):
        """
        :type global_context: migrator.core.session.GlobalMigrationContext
        :type migration_item_model: migrator.core.data.model.migration_project_item.MigrationProjectItemModel
        """
        return OaSubscriptionParameters(
            service_activation_parameters=[],
            synchronize_resource_limits=True
        )

    @trace
    def import_subscription_services(self, global_context, subscription):
        service_import_actions = self._get_service_import_actions(global_context)
        self._run_actions_on_subscription(global_context, subscription, service_import_actions)

    @trace
    def switch_subscription_services_to_production(self, global_context, subscription):
        switch_to_production_actions = self._get_switch_to_production_actions(global_context)
        self._run_actions_on_subscription(global_context, subscription, switch_to_production_actions)

    @trace
    def _run_actions_on_subscription(self, global_context, subscription, actions):
        platform_subscription = self._get_subscription(global_context, subscription.external_id)
        platform_subscription.source_info = subscription
        for action in actions:
            if action.filter_subscription(global_context, platform_subscription):
                logger.info(action.get_description())
                try:
                    action.run(global_context, platform_subscription)
                except Exception as e:
                    logger.error(_T("LOG_EXCEPTION"), exc_info=True)
                    raise MigrationError("%s: %s" % (
                        action.get_failure_message(global_context, platform_subscription), unicode(e)
                    ))

    @trace
    def _get_service_import_actions(self, global_context):
        base_actions = [
            ImportDomain(),
            ImportDnsRecords()
        ]
        if self._is_in_full_migration_mode(global_context):
            return base_actions + self._get_service_import_actions_full_migration(global_context)
        else:
            return base_actions + self._get_service_import_actions_takeover(global_context)

    @trace
    def _get_switch_to_production_actions(self, global_context):
        return [
            ConfigureSourceNameserversOA()
        ]

    @trace
    def _get_service_import_actions_takeover(self, global_context):
        return []

    @trace
    def _get_service_import_actions_full_migration(self, global_context):
        return []

    @staticmethod
    @trace
    def _get_subscription(global_context, subscription_id):
        for subscription in global_context.subscriptions_to_migrate:
            if subscription.source_info.external_id == subscription_id:
                return subscription

        raise MigrationError(_T("INTERNAL_ERROR_OCCURRED_WHILE_CREATING_SUBSCRIPTIONS"))

    @staticmethod
    @trace
    def _is_in_full_migration_mode(global_context):
        return global_context.migration_type.lower() == MIGRATION_TYPE_FULL.lower()
