# (C)2017 Ingram Micro Inc. all Rights Reserved
import codecs
import json
import logging
import os

from migrator.core import MigrationError, LshNgMigratorError
from migrator.core.language import _T
from migrator.core.utils.networking.run_command import LocalUnixRunner
from migrator.core.actions.base.subscription_action import SubscriptionAction
from migrator.settings import LSH_NG_MIGRATOR_BASE_DIR

logger = logging.getLogger(__name__)


class MigrateSubscription(SubscriptionAction):
    def run(self, global_context, subscription):
        migrator_path = os.path.join(LSH_NG_MIGRATOR_BASE_DIR, 'migrate.py')
        path_to_config = global_context.migrator_server.get_session_file_path('config.ini')
        results_file = global_context.migrator_server.get_session_file_path('result_file')
        is_billing_module_installed = global_context.target_panel_obj.is_billing_module_installed()
        cmd = '{migrator_path} --target-id {target_id} --subscription-id {subscription_id} --config {path_to_config} --results-file {results_file} --batch'
        if is_billing_module_installed:
            section_name = 'service plans'
            cmd += ' --with-pba'
        else:
            section_name = 'service templates'

        cmd += ' migrate'
        plan_mapping = '%s=%s' % (subscription.source_info.service_plan_external_id, subscription.mapping.plan_id)
        plan_subscription_period = None
        for subscription_period in global_context.migration_project.settings['planSubscriptionPeriods']:
            if subscription_period['planId'] == subscription.mapping.plan_id:
                plan_subscription_period = subscription_period['subscriptionPeriodId']
                break

        if plan_subscription_period is not None:
            plan_mapping = '%s,%s' % (plan_mapping, plan_subscription_period)

        config_content = "\n".join(['[%s]' % section_name, plan_mapping])
        local_runner = LocalUnixRunner()
        local_runner.upload_file_content(path_to_config, config_content)

        try:
            local_runner.sh_unchecked(cmd.format(
                migrator_path=migrator_path,
                target_id=global_context.migration_project.settings['oaHost']['id'],
                subscription_id=subscription.source_info.external_id,
                path_to_config=path_to_config,
                results_file=results_file,
            ))

            with codecs.open(results_file, 'r', 'utf-8') as f:
                data = json.load(f)
                if data['exit_code'] != 0:
                    raise LshNgMigratorError(data['message'])
        except LshNgMigratorError as e:
            raise MigrationError(e.message)
        except Exception:
            logger.debug(_T('LOG_EXCEPTION'), exc_info=True)
            raise MigrationError(_T('UNABLE_MIGRATE_LSH_SUBSCRIPTION').format(subscription_name=subscription.name))

    def get_description(self):
        return _T('MIGRATE_LSH_SUBSCRIPTION')

    def get_failure_message(self, global_context, subscription):
        return _T('FAILED_MIGRATE_LSH_SUBSCRIPTION').format(name=subscription.name)

    def filter_subscription(self, global_context, subscription):
        return True
