# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging
import os
import pickle

from migrator.core.actions.base.subscription_action import SubscriptionAction
from migrator.core.language import _T
from migrator.core.utils.loggers.log import trace
from migrator.odin_automation.services.ng.lib.server import AutomationServerNg
from migrator.odin_automation.services.ng.lib.transfer import \
    transfer_webspace, wait_task_queue, move_content, replace_abs_paths
from migrator.odin_automation.services.ng.lib.webspace import Webspace

logger = logging.getLogger(__name__)


class MigrationState(object):
    INITIAL = 0
    GET_SOURCE_PATH = 1
    MOVE_WEBSPACE = 2
    GET_TARGET_PATH = 3
    WAIT_TASK = 4
    MOVE_CONTENT = 5
    REPLACE_ABS_PATH = 6


class MoveWebspace(SubscriptionAction):
    @trace
    def run(self, global_context, subscription):
        migrator_server = global_context.migrator_server
        path_to_checkpoint = migrator_server.common_dir.get_file_path(
            'checkpoint_%s' % str(subscription.source_info.external_id)
        )

        source_path = None
        target_path = None
        conn = global_context.target_panel_obj.get_connections(global_context)
        if os.path.exists(path_to_checkpoint):
            with open(path_to_checkpoint, 'r') as f:
                saved_data = pickle.load(f)
                webspace, migration_state, source_host_id, destination_host_id, source_path, target_path = saved_data

            self.migration_state = migration_state
            source_host = AutomationServerNg.get(conn, id=source_host_id)
            destination_host = AutomationServerNg.get(conn, id=destination_host_id)
        else:
            self.migration_state = MigrationState.INITIAL
            webspace = Webspace(subscription.id, subscription.source_info.external_id)
            source_host = AutomationServerNg.get(conn, id=webspace.host_id)
            destination_host = AutomationServerNg.get(conn, id=global_context.migration_project.settings['oaHost']['id'])

        if source_host.host_id == destination_host.host_id:
            logger.debug(
                "Webspace %s is served by host %s already, nothing to migrate.", webspace,
                destination_host.description()
            )
            return

        try:
            source_path = self._process_step(
                source_path, MigrationState.GET_SOURCE_PATH, self._get_webspacse_path, webspace
            )
            self._process_step(None, MigrationState.MOVE_WEBSPACE, transfer_webspace, webspace, destination_host)
            target_path = self._process_step(
                target_path, MigrationState.GET_TARGET_PATH, self._get_webspacse_path, webspace
            )
            self._process_step(None, MigrationState.WAIT_TASK, wait_task_queue, webspace)
            self._process_step(
                None, MigrationState.MOVE_CONTENT, move_content, source_host, source_path, destination_host,
                target_path
            )
            self._process_step(
                None, MigrationState.REPLACE_ABS_PATH, replace_abs_paths, webspace, destination_host,
                source_path, target_path
            )
            if os.path.exists(path_to_checkpoint):
                os.remove(path_to_checkpoint)
        except Exception as e:
            logger.debug(_T('LOG_EXCEPTION'), exc_info=True)
            with open(path_to_checkpoint, 'w+') as f:
                data_to_save = (
                    webspace, self.migration_state, source_host.host_id, destination_host.host_id, source_path,
                    target_path
                )
                pickle.dump(data_to_save, f)

            raise e

    @staticmethod
    def _get_webspacse_path(webspace):
        return webspace.path

    def _process_step(self, value, step, func, *arg):
        if self.migration_state < step:
            return func(*arg)
            self.migration_state = step
        else:
            return value

    def get_description(self):
        return _T('MOVE_NG_WEBSPACE')

    def get_failure_message(self, global_context, subscription):
        conn = global_context.target_panel_obj.get_connections(global_context)
        return _T('FAILED_MOVE_NG_WEBSPACE').format(
            name=subscription.name,
            host=AutomationServerNg.get(conn, id=global_context.migration_project.settings['oaHost']['id'])
        )

    def filter_subscription(self, global_context, subscription):
        return True
