# (C)2017 Ingram Micro Inc. all Rights Reserved
import codecs
import json
import logging
import os

from migrator.core import LshNgMigratorError
from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils.networking.run_command import LocalUnixRunner
from migrator.core.utils.registry import Registry
from migrator.settings import LSH_NG_MIGRATOR_BASE_DIR, TargetServiceType, SOURCE_PRODUCT_OA_LSH

logger = logging.getLogger(__name__)


class CheckLshNgBase(CheckAbstract):
    SUPPORTED_TARGET_SERVICE_CLASSES = [TargetServiceType.NG]
    SUPPORTED_SOURCE_PRODUCTS = [SOURCE_PRODUCT_OA_LSH]
    CHECK_NAME = None
    ASPECT_NAME = None
    SUCCESS_MESSAGE = None
    UNEXPECTED_ERROR = None

    def __init__(self):
        super(CheckLshNgBase, self).__init__(self.CHECK_NAME, CheckAbstract.PHASE_NO_PHASE)

    def _do_check(self):
        try:
            global_context = Registry.get_instance().get_context()
            migrator_path = os.path.join(LSH_NG_MIGRATOR_BASE_DIR, 'migrate.py')
            is_billing_module_installed = global_context.target_panel_obj.is_billing_module_installed()
            platform_reader = PlatformReaderDB()
            cmd = '{migrator_path} --target-id {target_id} --subscription-id {subscription_id} --config {path_to_config} --results-file {results_file} --batch'
            if is_billing_module_installed:
                section_name = 'service plans'
                cmd += ' --with-pba'
            else:
                section_name = 'service templates'

            cmd += ' --aspect={aspect} check'.format(aspect=self.ASPECT_NAME)
            for migration_item in platform_reader.get_migration_project_items():
                subscription = platform_reader.get_subscription(primary_id=migration_item.subscription_id)
                results_file = global_context.migrator_server.get_session_file_path(
                    'result_file_{subscription_id}'.format(subscription_id=migration_item.subscription_id)
                )
                path_to_config = global_context.migrator_server.get_session_file_path(
                    'config_{subscription_id}.ini'.format(subscription_id=migration_item.subscription_id)
                )

                plan_mapping = '%s=%s' % (
                    subscription.service_plan_external_id, migration_item.target_service_plan_id
                )
                plan_subscription_period = None
                for subscription_period in global_context.migration_project.settings['planSubscriptionPeriods']:
                    if subscription_period['planId'] == migration_item.target_service_plan_id:
                        plan_subscription_period = subscription_period['subscriptionPeriodId']
                        break

                if plan_subscription_period is not None:
                    plan_mapping = '%s,%s' % (plan_mapping, plan_subscription_period)

                config_content = "\n".join(['[%s]' % section_name, plan_mapping])
                local_runner = LocalUnixRunner()
                local_runner.upload_file_content(path_to_config, config_content)
                local_runner.sh_unchecked(cmd.format(
                    migrator_path=migrator_path,
                    target_id=global_context.migration_project.settings['oaHost']['id'],
                    subscription_id=subscription.external_id,
                    path_to_config=path_to_config,
                    results_file=results_file,
                ))
                with codecs.open(results_file, 'r', 'utf-8') as f:
                    data = json.load(f)
                    if data['exit_code'] != 0:
                        raise LshNgMigratorError(data['message'])

                return self.STATUS_SUCCESS, self.SUCCESS_MESSAGE
        except LshNgMigratorError as e:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return self.STATUS_ERROR, e.message
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return self.STATUS_ERROR, self.UNEXPECTED_ERROR
