# (C)2017 Ingram Micro Inc. all Rights Reserved
import codecs
import json
import logging
import os

from migrator.core import NgNgMigratorError
from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils.networking.run_command import LocalUnixRunner
from migrator.core.utils.registry import Registry
from migrator.settings import NG_NG_MIGRATOR_BASE_DIR, TargetServiceType, SOURCE_PRODUCT_OA_NG

logger = logging.getLogger(__name__)


class CheckNgNgBase(CheckAbstract):
    SUPPORTED_TARGET_SERVICE_CLASSES = [TargetServiceType.NG]
    SUPPORTED_SOURCE_PRODUCTS = [SOURCE_PRODUCT_OA_NG]
    CHECK_NAME = None
    ASPECT_NAME = None
    SUCCESS_MESSAGE = None
    UNEXPECTED_ERROR = None

    def __init__(self):
        super(CheckNgNgBase, self).__init__(self.CHECK_NAME, CheckAbstract.PHASE_NO_PHASE)

    def _do_check(self):
        try:
            global_context = Registry.get_instance().get_context()
            migrator_path = os.path.join(NG_NG_MIGRATOR_BASE_DIR, 'migrate.py')
            platform_reader = PlatformReaderDB()
            cmd = '{migrator_path} --webspace-ids {webspace_id} --target-host-id {target_host_id} --results-file {results_file} --aspect={aspect} check'
            for migration_item in platform_reader.get_migration_project_items():
                subscription = platform_reader.get_subscription(primary_id=migration_item.subscription_id)
                results_file = global_context.migrator_server.get_session_file_path(
                    'result_file_{subscription_id}'.format(subscription_id=migration_item.subscription_id)
                )
                local_runner = LocalUnixRunner()
                local_runner.sh_unchecked(cmd.format(
                    migrator_path=migrator_path,
                    webspace_id=subscription.external_id,
                    target_host_id=global_context.migration_project.settings['oaHost']['id'],
                    results_file=results_file,
                    aspect=self.ASPECT_NAME,
                ))
                with codecs.open(results_file, 'r', 'utf-8') as f:
                    data = json.load(f)
                    if data['exit_code'] == 2:
                        return self.STATUS_WARNING, data['message']
                    elif data['exit_code'] == 1:
                        return self.STATUS_ERROR, data['message']
                    else:
                        return self.STATUS_SUCCESS, data['message'] or self.SUCCESS_MESSAGE
        except NgNgMigratorError as e:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return self.STATUS_ERROR, e.message
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return self.STATUS_ERROR, self.UNEXPECTED_ERROR
