import logging

import migrator.odin_automation.services.ng.lib.aps.types as types

from migrator.core.data.model.web_content_shm import ContentItemModel
from migrator.core.transfer.content_migrator import get_content_migrator
from migrator.core.utils.loggers.log import trace
from migrator.core.utils.registry import Registry
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import get_aps_connector_with_admin_token
from migrator.odin_automation.platform.lib.task_manager import wait_for_all_scheduled_tasks_on_resource_completed
from migrator.odin_automation.platform.lib.utils.networking.hcl_runner.hcl_common import HclRunnerException

logger = logging.getLogger(__name__)


@trace
def transfer_webspace(webspace, destination_host):
    aps = get_aps_connector_with_admin_token()
    application_aps_id = aps.get_first_resource(filter_str='implementing(%s)' % types.WHLApplication)['aps']['id']
    args = {
        'webspaceId': webspace.id,
        'destinationHostId': destination_host.host_id
    }
    aps.post_resource(application_aps_id, types.aps_operations['moveWebspace'], args)


@trace
def wait_task_queue(webspace):
    ds_id = webspace.get_domain_service()
    wait_for_all_scheduled_tasks_on_resource_completed('domain_service%s' % ds_id, interval=2)


@trace
def move_content(source_host, source_path, destination_host, target_path):
    global_context = Registry.get_instance().get_context()
    with get_content_migrator(global_context, None, source_host, destination_host, None) as content_migrator:
        content_migrator.copy_content([ContentItemModel(source_path=source_path, target_path=target_path)])

    with source_host.runner() as runner:
        runner.remove_directory(source_path)

@trace
def replace_abs_paths(webspace, target_host, source_path, target_path):
    logger.debug("Checking content of webspace %s for absolute paths", webspace.id)

    lines = []
    try:
        with target_host.runner() as runner:
            output = runner.sh(
                "/usr/bin/find '%s' -type f ! -iname '*log' -exec grep -lI '%s' '{}' \;" %
                (target_path, source_path)
            )
            lines = output.splitlines()

        lines = output.splitlines()
    except HclRunnerException, e:
        if 1 != e.exit_code:
            raise

    if lines:
        logger.debug("Following files seem to have absolute file paths in them:")
        for line in lines:
            logger.debug("%s", line)

        with target_host.runner() as runner:
            for line in lines:
                if line.endswith(".bak"):
                    continue

                runner.sh(
                    "/bin/sed --in-place=.bak --expression='s|%s|%s|g' {path}" % (source_path, target_path),
                    {'path': line}
                )
