from migrator.core import InternalError
from migrator.core.utils.loggers.log import trace
from migrator.core.utils.registry import Registry
from migrator.odin_automation.platform.lib.db import get_platform_db_connection

APACHENG = 'ApacheNG'


class Webspace(object):
    def __init__(self, subscription_id, id):
        self.subscription_id = int(subscription_id)
        self.id = int(id)

    def __str__(self):
        return '#%s' % self.id

    @trace
    def _get_webspace(self, func):
        webspaces = filter(lambda x: x['webspace_id'] == self.id, func(self.subscription_id))
        if not webspaces:
            raise InternalError("There is not webspace #%s at subscription #%s" % (self.id, self.subscription_id))

        return webspaces[0]

    @trace
    def _get_subscription_webspace(self):
        global_context = Registry.get_instance().get_context()
        poa_api = global_context.conn.target.poa_api()
        return self._get_webspace(poa_api.subscription.get_subscription_webspaces)

    @trace
    def _get_apache_webspace(self):
        global_context = Registry.get_instance().get_context()
        poa_api = global_context.conn.target.poa_api()
        return self._get_webspace(poa_api.subscription.get_apache_webspaces)

    @property
    def path(self):
        return self._get_apache_webspace()['path']

    @property
    def host_id(self):
        return self._get_subscription_webspace()['host_id']

    @trace
    def get_domain_service(self):
        con = get_platform_db_connection()
        cur = con.cursor()
        cur.execute("""
                    SELECT ds_id
                    FROM domain_services ds JOIN services s ON ds.service_id = s.service_id
                    WHERE ds.hd_id = %s AND s.name = %s""", [self.id, APACHENG])
        return cur.fetchone()[0]
