# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import MigrationError
from migrator.core.actions.base.common_action import CommonAction
from migrator.core.data.model.migration_project import MigrationMode
from migrator.core.data.model.vps_oaci import HostingModel
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.data.source.xml.vps_parameters_oaci import OaciVpsParametersWriterXml
from migrator.core.language import _T
from migrator.settings import \
    TargetServiceType, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, SourceHostingClass, MIGRATION_TYPE_FULL

logger = logging.getLogger(__name__)


class ExportVpsParameters(CommonAction):
    """Export VPS parameters (HW & OS) from source system."""

    def get_description(self):
        """
        :return: string
        """
        return _T("EXPORT_VPS_HOSTING_PARAMETERS_ACTION")

    def get_failure_message(self, global_context):
        """
        :param global_context: global_context: migrator.core.global_context.GlobalMigrationContext
        :return: string
        """
        return _T("EXPORT_VPS_HOSTING_PARAMETERS_FAILURE")

    def filter_action(self, global_context):
        """
        :param global_context: migrator.core.global_context.GlobalMigrationContext
        :return: bool False - to skip subscription processing
        """
        return global_context.migration_type == MIGRATION_TYPE_FULL \
            and global_context.migration_mode == MigrationMode.PRODUCTION

    def run(self, global_context):
        """
        :param global_context: migrator.core.global_context.GlobalMigrationContext
        :return: void
        """
        subscriptions = {}
        exporter = global_context.source_product.get_exporter_instance(TargetServiceType.OACI)

        platform_reader = PlatformReaderDB()
        hosting_subscriptions = platform_reader.get_subscriptions_by_services(
            SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.VPS]
        )
        hosting_subscription_ids = {sub.external_id for sub in hosting_subscriptions}
        for item in global_context.migration_list_data:
            if item.id not in subscriptions and item.id in hosting_subscription_ids:
                subscriptions[item.id] = exporter.get_subscription(item.id)

        oaci_model = HostingModel(subscriptions=subscriptions.values())

        xml_writer = OaciVpsParametersWriterXml(global_context.session_files.get_path_to_oaci_vps_settings_xml_file())
        try:
            xml_writer.write(oaci_model)
        except:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            raise MigrationError(_T("FAILED_TO_EXPORT_VPS_PARAMETERS_INFORMATION"))
