# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import MigrationNoRepeatError
from migrator.core.actions.base.subscription_action import SubscriptionAction
from migrator.core.data.model.migration_project import MigrationMode
from migrator.core.data.model.subscription import has_any_of_service_types
from migrator.core.language import _T
from migrator.settings import \
    MIGRATION_TYPE_TAKEOVER, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, SourceHostingClass, SOURCE_PRODUCT_OBAS

logger = logging.getLogger(__name__)


class SuspendObasSubscription(SubscriptionAction):
    def run(self, global_context, subscription):
        """
        :param global_context: migrator.core.global_context.GlobalMigrationContext
        :param subscription: migrator.odin_automation.platform.lib.data.subscription.PlatformSubscription
        """
        try:
            panel = global_context.source_product
            if panel.get_id() == SOURCE_PRODUCT_OBAS:
                panel.suspend_subscription(subscription)
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            raise MigrationNoRepeatError(_T('UNEXPECTED_ERROR_OCCURRED_SEE_THE_LOGFILE_MSG'))

    def get_description(self):
        """
        :return: string
        """
        return _T("SUSPEND_OBAS_VPS_SUBSCRIPTION")

    def filter_subscription(self, global_context, subscription):
        """
        :param global_context: global_context: migrator.core.global_context.GlobalMigrationContext
        :param subscription: subscription: migrator.odin_automation.platform.lib.data.subscription.PlatformSubscription
        :return: bool False - to skip subscription processing
        """
        return global_context.migration_type == MIGRATION_TYPE_TAKEOVER and \
               global_context.migration_mode == MigrationMode.PRODUCTION and \
               has_any_of_service_types(subscription.source_info, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.VPS])

    def is_critical(self):
        """
        :return: bool
        """
        return True

    def get_failure_message(self, global_context, subscription):
        """
        :param global_context: global_context: migrator.core.global_context.GlobalMigrationContext
        :param subscription: migrator.odin_automation.platform.lib.data.subscription.PlatformSubscription
        :return: string
        """
        return _T("FAILED_SUSPEND_SUBSCRIPTION_IN_OBAS").format(sub_id=subscription.name)
