# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.odin_automation.platform.checks.checks import Checks
from migrator.odin_automation.platform.lib.api.aps.host import get_oam_host
from migrator.settings import TargetServiceType, PANEL_TYPE_VIRTUOZZO, SOURCE_PRODUCT_OBAS


logger = logging.getLogger(__name__)


class CheckHostsOACI(CheckAbstract):
    SUPPORTED_TARGET_SERVICE_CLASSES = [TargetServiceType.OACI]
    SUPPORTED_PCS_VERSIONS = [6]
    SUPPORTED_SOURCE_PRODUCTS = [SOURCE_PRODUCT_OBAS]

    def __init__(self):
        super(CheckHostsOACI, self).__init__(Checks.CHECK_HOSTS_OACI, CheckAbstract.PHASE_NO_PHASE)

    def get_full_description(self):
        return _T('CHECK_FULL_DESCRIPTION_HOSTS_OACI')

    def _do_check(self):
        try:
            incompatible_pcs_hosts = set()
            pcs_hosts_to_check = PlatformReaderDB().get_migration_required_source_hosts(PANEL_TYPE_VIRTUOZZO)
            for pcs_host_to_check in pcs_hosts_to_check:
                oam_pcs_host = get_oam_host(pcs_host_to_check.ip)
                if oam_pcs_host is None:
                    continue

                oam_pcs_host_version = int(oam_pcs_host['os']['version'])
                if oam_pcs_host_version not in self.SUPPORTED_PCS_VERSIONS:
                    incompatible_pcs_hosts.add(pcs_host_to_check)

            if incompatible_pcs_hosts:
                return self.STATUS_ERROR, _T('CHECK_OACI_HOSTS_ERROR_FOUND_INCOMPATIBLE_NODES').format(
                    nodes_identifiers=', '.join('%s (%s)' % (pcs.name, pcs.ip) for pcs in incompatible_pcs_hosts),
                    allowed_vz_versions=', '.join('%s.x' % str(vz_version) for vz_version in self.SUPPORTED_PCS_VERSIONS)
                )
            else:
                return self.STATUS_SUCCESS, _T('CHECK_OACI_HOSTS_SUCCESS')
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return self.STATUS_ERROR, _T('CHECK_OACI_HOSTS_UNEXPECTED')
