# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils.registry import Registry
from migrator.odin_automation.platform.checks.checks import Checks
from migrator.odin_automation.services.oaci.lib.aps.application import get_oaci_vz_hosts
from migrator.odin_automation.platform.lib.api.openapi.host import HostNotFoundException
from migrator.settings import TargetServiceType, PANEL_TYPE_VIRTUOZZO, MIGRATION_TYPE_TAKEOVER, SOURCE_PRODUCT_OBAS

logger = logging.getLogger(__name__)


class CheckOAHostsOACI(CheckAbstract):
    SUPPORTED_TARGET_SERVICE_CLASSES = [TargetServiceType.OACI]
    SUPPORTED_MIGRATION_TYPES = [MIGRATION_TYPE_TAKEOVER]
    SUPPORTED_SOURCE_PRODUCTS = [SOURCE_PRODUCT_OBAS]
    DOC_LINK = '88412.htm'

    def __init__(self):
        super(CheckOAHostsOACI, self).__init__(Checks.CHECK_OA_HOSTS_OACI, CheckAbstract.PHASE_NO_PHASE)

    def get_full_description(self):
        return _T('CHECK_FULL_DESCRIPTION_OA_HOSTS_OACI')

    def _do_check(self):
        try:
            unregistered_pcs_hosts = set()
            poa_api = Registry.get_instance().get_context().conn.target.poa_api()

            oaci_pcses = get_oaci_vz_hosts()
            pcs_hosts_to_check = PlatformReaderDB().get_migration_required_source_hosts(PANEL_TYPE_VIRTUOZZO)
            for pcs_host_to_check in pcs_hosts_to_check:
                try:
                    host_id = poa_api.host.get_host_id_by_ip(pcs_host_to_check.ip)
                except HostNotFoundException:
                    unregistered_pcs_hosts.add(pcs_host_to_check)
                    continue

                if not self._is_pcs_registered_in_oa(host_id, oaci_pcses):
                    unregistered_pcs_hosts.add(pcs_host_to_check)

            if unregistered_pcs_hosts:
                return self.STATUS_ERROR, _T('CHECK_OACI_OA_HOSTS_ERROR_FOUND_UNREGISTERED_NODES').format(
                    nodes_identifiers=', '.join('%s (%s)' % (pcs.name, pcs.ip) for pcs in unregistered_pcs_hosts),
                    precheck_doc_link=self.DOC_LINK
                )
            else:
                return self.STATUS_SUCCESS, _T('CHECK_OACI_OA_HOSTS_SUCCESS')

        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return self.STATUS_ERROR, _T('CHECK_OACI_OA_HOSTS_UNEXPECTED')

    def _is_pcs_registered_in_oa(self, host_id_to_check, oaci_hosts):
        """
        :param host_id_to_check: int
        :param oaci_hosts: list[dict]
        :return: bool
        """
        for oaci_host in oaci_hosts:
            if str(oaci_host['uuid']) == str(host_id_to_check):
                return True

        return False
