# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging
import urllib2
import zipfile
import re

from migrator.core import MigrationError
from migrator.core.actions.base.common_action import CommonAction
from migrator.core.data.model.migration_project import MigrationMode
from migrator.core.language import _T
from migrator.settings import MIGRATION_TYPE_FULL, SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS

logger = logging.getLogger(__name__)

PANEL_MIGRATOR_URL = 'https://ext.plesk.com/packages/bebc4866-d171-45fb-91a6-4b139b8c9a1b-panel-migrator/download'


class DownloadApsBackupTool(CommonAction):
    def run(self, global_context):
        try:
            url = urllib2.urlopen(PANEL_MIGRATOR_URL)
            zip_file = global_context.migrator_server.get_session_file_path('migrator.zip')
            with open(zip_file, "wb") as f:
                f.write(url.read())

            zip_archive = zipfile.ZipFile(zip_file)
            aps_backup_path = None
            for info in zip_archive.infolist():
                if re.match(r'.*/ApsBackup-12\.0\.php[\e]?', info.filename):
                    aps_backup_path = info.filename
                    break

            if not aps_backup_path:
                raise MigrationError(_T("COULD_NOT_EXTRACT_THE_APS_BACKUP_UTILITY_FROM_ARCHIVE"))

            zip_file = zip_archive.open(aps_backup_path)
            aps_backup_file = global_context.session_files.get_path_to_aps_backup_tool()
            with open(aps_backup_file, "wb") as f:
                f.write(zip_file.read())
        except MigrationError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            raise e
        except Exception:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            raise MigrationError(_T("ERROR_OCCURRED_WHILE_RETRIEVING_APS_APPLICATIONS"))

    def filter_action(self, global_context):
        product = global_context.source_type
        return product in [SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS] and \
               global_context.migration_type == MIGRATION_TYPE_FULL and \
               global_context.migration_mode != MigrationMode.TEST_MIGRATION

    def get_description(self):
        return _T("COPYING_THE_APS_BACKUP_UTILITY")

    def get_failure_message(self, global_context):
        return _T("COULD_NOT_COPY_THE_APS_BACKUP")
