# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import MigrationError
from migrator.core.actions.base.common_action import CommonAction
from migrator.core.data.model.mail_content_shm import MailContentModel
from migrator.core.data.model.migration_project import MigrationMode
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.data.source.xml.billing import BillingReaderXml, subscription_statuses_allowed_for_migration
from migrator.core.data.source.xml.mail_content_shm import ShmMailContentWriterXml, SubscriptionModel
from migrator.core.language import _T
from migrator.settings import \
    TargetServiceType, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, MIGRATION_TYPE_FULL, SERVICE_EXPORT_NOT_SUPPORTED, \
    SourceHostingClass

logger = logging.getLogger(__name__)


class ExportMailContentData(CommonAction):
    """Export SHM mail content data from source system."""

    def run(self, global_context):
        subscriptions = {}
        exporter = global_context.source_product.get_content_exporter_instance(TargetServiceType.MAIL)
        if exporter is SERVICE_EXPORT_NOT_SUPPORTED:
            return

        platform_reader = PlatformReaderDB()
        hosting_subscriptions = platform_reader.get_subscriptions_by_services(
            SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.SHARED]
        )
        billing_reader = BillingReaderXml(global_context.session_files.get_path_to_billing_xml_file())
        hosting_subscription_ids = {sub.external_id for sub in hosting_subscriptions}
        for item in global_context.migration_list_data:
            if item.id not in subscriptions and item.id in hosting_subscription_ids:
                billing_info = billing_reader.get_subscription(item.id)
                if billing_info and billing_info.status in subscription_statuses_allowed_for_migration:
                    subscriptions[item.id] = exporter.get_subscription(item.id)
                else:
                    subscriptions[item.id] = SubscriptionModel(external_id=item.id, webspaces=[])

        mail_model = MailContentModel(subscriptions=subscriptions.values())
        xml_writer = ShmMailContentWriterXml(global_context.session_files.get_path_to_shm_mail_content_xml_file())
        try:
            xml_writer.write(mail_model)
        except:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            raise MigrationError(_T("FAILED_TO_EXPORT_SHM_MAIL_CONTENT_INFORMATION"))

    def filter_action(self, global_context):
        return global_context.migration_type == MIGRATION_TYPE_FULL \
            and global_context.migration_mode != MigrationMode.TEST_MIGRATION

    def get_description(self):
        return _T("EXPORT_SHM_MAIL_CONTENT_DATA_ACTION")

    def get_failure_message(self, global_context):
        return _T("EXPORT_SHM_MAIL_CONTENT_DATA_FAILURE")
