# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.actions.base.common_action import CommonAction
from migrator.core.language import _T
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import get_app_instance_uid
from migrator.odin_automation.platform.lib.api.aps import tm
from migrator.odin_automation.services.shm.lib.aps.types import SHMApplication
from migrator.settings import SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS, SOURCE_PRODUCT_OA_WSH, SOURCE_PRODUCT_OA_WHL, SOURCE_PRODUCT_OA_LSH

logger = logging.getLogger(__name__)


class SynchronizeSHMStatistics(CommonAction):
    """
    Run SHM APS2 specific periodic task to update SHM subscriptions APS2 resources usage counters with
    data from SHM Plesk nodes statistics.
    """
    def get_description(self):
        return _T("SYNCHRONIZE_SHM_RESOURCE_USAGE")

    def get_failure_message(self, global_context):
        return _T("FAILED_SYNCHRONIZE_SHM_RESOURCE_USAGE")

    def run(self, global_context):
        try:
            shm_application_uid = get_app_instance_uid(SHMApplication)
            task = tm.get_application_periodic_task(shm_application_uid)
            if task['status'] != 'RUNNING':
                tm.run_application_periodic_task(shm_application_uid)
            else:
                logger.debug(
                    'WebHosting Plesk application periodic resource usage update task is already running. Skipping...'
                )
        except Exception as e:
            logger.debug(
                u"Failed to schedule WebHosting Plesk application periodic resource usage update task. "
                u"Resource usage will be updated on the next scheduled task execution. Message: %s", unicode(e)
            )

    def filter_action(self, global_context):
        product = global_context.source_type
        return product in [SOURCE_PRODUCT_PLESK, SOURCE_PRODUCT_OBAS, SOURCE_PRODUCT_OA_WSH, SOURCE_PRODUCT_OA_WHL, SOURCE_PRODUCT_OA_LSH]
