# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.actions.base.common_action import CommonAction
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.settings import MIGRATION_TYPE_TAKEOVER, SOURCE_PRODUCT_OBAS, TargetServiceType, PANEL_TYPE_PLESK

logger = logging.getLogger(__name__)


class ServiceNodesConfigure(CommonAction):
    def get_description(self):
        return _T("CONFIGURE_PLESKS_FOR_TAKEOVER")

    def get_failure_message(self, global_context):
        return _T("FAILED_TO_CONFIGURE_PLESK_FOR_TAKEOVER")

    def run(self, global_context):
        takeoverable_plesks = PlatformReaderDB().get_migration_required_source_hosts(PANEL_TYPE_PLESK)
        for takeoverable_plesk in takeoverable_plesks:
            # Disallow new accounts and disable DNS synchronization for Plesk domains and Plesk clients
            global_context.migrator.agent.configure_plesk_node_for_takeover(takeoverable_plesk.external_id)

        logger.debug(_T('SUCCESSFULLY_CONFIGURED_PLESKS_FOR_TAKEOVER').format(
            ids=', '.join(takeoverable_plesk.external_id for takeoverable_plesk in takeoverable_plesks)
        ))

    def filter_action(self, global_context):
        return global_context.migration_type == MIGRATION_TYPE_TAKEOVER and \
            global_context.source_type == SOURCE_PRODUCT_OBAS and \
            global_context.config.get('GLOBAL', 'target-type').lower() == TargetServiceType.SHM
