# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core import MigrationError
from migrator.core.actions.base.common_action import CommonAction
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils import FunctionResultWaiter
from migrator.core.utils.networking.ip_utils import is_ipv4
from migrator.odin_automation.platform.lib.api.openapi import RequestStatus
from migrator.odin_automation.services.shm.lib.openapi.host import register_linux_service_node,\
    register_windows_service_node
from migrator.settings import SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, SourceHostingClass


class ServiceNodesRegistration(CommonAction):
    def get_description(self):
        return _T("SERVICE_NODES_REGISTRATION")

    def get_failure_message(self, global_context):
        return _T("FAILED_SERVICE_NODES_REGISTRATION")

    def run(self, global_context):
        def check_request_status(result):
            (status, message) = result
            if status == RequestStatus.SUCCESS:
                return True
            elif status == RequestStatus.FAILED:
                raise MigrationError(
                    _T("UNABLE_TO_REGISTER_SERVICE_NODE_WITH_ERROR").format(
                        ip_address=ip_address, error_message=message
                    )
                )
            else:
                return False

        platform_reader = PlatformReaderDB()
        hosting_subscriptions = platform_reader.get_subscriptions_by_services(
            SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.SHARED]
        )
        hosting_subscription_ids = {sub.external_id for sub in hosting_subscriptions}
        registered_nodes = []
        for item in global_context.migration_list_data:
            if item.id not in hosting_subscription_ids:
                continue

            subscription_id = item.id
            web_source_host = global_context.migrator._get_source_web_node(None, subscription_id)
            ip_address = web_source_host.ip()
            if ip_address in registered_nodes:
                continue

            poa_api = global_context.conn.target.poa_api()
            if is_ipv4(ip_address):
                host_id = poa_api.host.find_host(ip_address=ip_address)
            else:
                host_id = poa_api.host.find_host(hostname=ip_address)

            if host_id is None:
                if web_source_host.is_windows():
                    request_id = register_windows_service_node(poa_api, web_source_host)
                else:
                    request_id = register_linux_service_node(poa_api, web_source_host)

                # Wait for completion of the task 2 hours
                function = FunctionResultWaiter(
                    poa_api.get_request_status,
                    MigrationError(_T("UNABLE_TO_REGISTER_SERVICE_NODE").format(ip_address=ip_address)),
                    check_function=check_request_status,
                    max_attempts=720,
                    interval_between_attempts=10,
                )
                function.run(request_id)

            registered_nodes.append(ip_address)
