# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging
import ntpath
import posixpath

from migrator.core.actions.base.subscription_action import SubscriptionAction
from migrator.core.data.model.migration_project import MigrationMode
from migrator.core.data.model.subscription import has_any_of_service_types
from migrator.core.language import _T
from migrator.core.utils.loggers.log import trace
from migrator.settings import SHARED_HOSTING_SERVICES, MIGRATION_TYPE_FULL

logger = logging.getLogger(__name__)


class RemoveDefaultIndexFiles(SubscriptionAction):
    _default_index_files = ('index.html', 'index.php')

    @trace
    def run(self, global_context, subscription):
        for webspace in subscription.service.webspaces:
            # Remove default index files from sites docroot only
            # because there will be no customer data in webspace docroot
            for site in webspace.sites:
                logger.debug(_T("PROCESSING_SITE").format(name=site.name))
                if site.www_root:
                    self._cleanup_docroot(webspace.web_target_host, site.www_root, webspace.type == 'iis')

    @trace
    def _cleanup_docroot(self, web_target_host, www_root, is_windows=False):
        with web_target_host.runner() as runner:
            for index_file in self._default_index_files:
                pathlib = ntpath if is_windows else posixpath
                filepath = pathlib.join(www_root, index_file)

                runner.remove_file(filepath)

    @trace
    def filter_subscription(self, global_context, subscription):
        if global_context.migration_type != MIGRATION_TYPE_FULL or \
                global_context.migration_mode == MigrationMode.TEST_MIGRATION:
            logger.info(
                _T("LOG_NO_NEED_TO_COPY_WEBCONTENT_BECAUSE_DISABLED_BY_USER"),
                subscription.name
            )
            return False

        return subscription.service and has_any_of_service_types(subscription.source_info, SHARED_HOSTING_SERVICES)

    def is_critical(self):
        return False

    def get_description(self):
        return _T("REMOVE_DEFAULT_INDEX_FILES")

    def get_failure_message(self, global_context, subscription):
        return _T("FAILED_REMOVE_DEFAULT_INDEX_FILES").format(name=subscription.name)
