# (C)2017 Ingram Micro Inc. all Rights Reserved
from itertools import chain

from poaupdater import uSysDB

from migrator.core.language import _T
from migrator.core.actions.base.subscription_action import SubscriptionAction
from migrator.core.data.model.subscription import has_any_of_service_types
from migrator.core.data.model.migration_project import MigrationMode
from migrator.settings import SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, SourceHostingClass


class UnlinkingDatabasesFromSubscriptions(SubscriptionAction):
    def run(self, global_context, subscription):
        con = uSysDB.connect()
        cur = con.cursor()

        subscription_id = subscription.source_info.external_id
        cur.execute(
            """
            DELETE
            FROM dbm_users
            USING dbm_db_users, dbm_databases, dbm_provide_info
            WHERE
            dbm_users.user_id = dbm_db_users.user_id
            AND dbm_db_users.db_id = dbm_databases.db_id
            AND dbm_provide_info.provide_id = dbm_databases.provide_id
            AND dbm_provide_info.sub_id = %s
            """, subscription_id
        )

        cur.execute(
            """
            DELETE
            FROM dbm_db_users
            USING dbm_databases, dbm_provide_info
            WHERE
            dbm_db_users.db_id = dbm_databases.db_id
            AND dbm_provide_info.provide_id = dbm_databases.provide_id
            AND dbm_provide_info.sub_id = %s
            """, subscription_id
        )

        cur.execute(
            """
            DELETE
            FROM dbm_db_access_hosts
            USING dbm_databases, dbm_provide_info
            WHERE
            dbm_db_access_hosts.db_id = dbm_databases.db_id
            AND dbm_provide_info.provide_id = dbm_databases.provide_id
            AND dbm_provide_info.sub_id = %s
            """, subscription_id
        )

        cur.execute(
            """
            DELETE
            FROM dbm_db_diskspace_elems
            USING dbm_databases, dbm_provide_info
            WHERE
            dbm_db_diskspace_elems.db_id = dbm_databases.db_id
            AND dbm_provide_info.provide_id = dbm_databases.provide_id
            AND dbm_provide_info.sub_id = %s
            """, subscription_id
        )

        cur.execute(
            """
            DELETE
            FROM dbm_databases
            USING dbm_provide_info
            WHERE
            dbm_provide_info.provide_id = dbm_databases.provide_id
            AND dbm_provide_info.sub_id = %s
            """, subscription_id
        )

        cur.execute(
            """
            DELETE
            FROM dbm_provide_info
            WHERE
            dbm_provide_info.sub_id = %s
            """, subscription_id
        )
        con.commit()

    def get_description(self):
        return _T("UNLINKING_DATABASES_FROM_SUBSCRIPTIONS")

    def filter_subscription(self, global_context, subscription):
        """
        :type global_context: migrator.core.global_context.GlobalMigrationContext
        :type subscription: migrator.core.migrated_subscription.MigratedSubscription
        """
        return subscription.service and \
               global_context.migration_mode == MigrationMode.PRODUCTION and \
               has_any_of_service_types(
                   subscription.source_info,
                   chain(
                       SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.WSH],
                       SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.WHL],
                       SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.LSH],
                   )
               )

    def get_failure_message(self, global_context, subscription):
        """
        :type global_context: migrator.core.global_context.GlobalMigrationContext
        :type subscription: migrator.core.migrated_subscription.MigratedSubscription
        """
        return _T("UNABLE_UNLINKING_DATABASES_FROM_SUBSCRIPTIONS")
