# (C)2017 Ingram Micro Inc. all Rights Reserved
import hashlib

from migrator.core.data.source.xml.web_hosting_shm import ShmWebHostingReaderXml
from migrator.core.transfer.context_macro_definition import ContextMacroDefinition
from migrator.core.utils.loggers.log import trace
from migrator.core.utils.registry import Registry


@trace
def get_webspaces_list(source_subscription_id):
    global_context = Registry.get_instance().get_context()
    shm_web_hosting_reader = ShmWebHostingReaderXml(
        global_context.session_files.get_path_to_shm_web_hosting_xml_file()
    )
    return shm_web_hosting_reader.get_webspaces_list(source_subscription_id)


@trace
def get_sites_list(source_subscription_id):
    global_context = Registry.get_instance().get_context()
    shm_web_hosting_reader = ShmWebHostingReaderXml(
        global_context.session_files.get_path_to_shm_web_hosting_xml_file()
    )

    sites = []
    for webspace in shm_web_hosting_reader.get_webspaces_list(source_subscription_id):
        for site in webspace.sites:
            if site.external_id.startswith('subdomain_'):
                # subdomains in Plesk 8/9 do not have dedicated DNS zones
                continue

            sites.append(site)

    return sites


@trace
def get_database_server(database_server_id):
    global_context = Registry.get_instance().get_context()
    shm_web_hosting_reader = ShmWebHostingReaderXml(
        global_context.session_files.get_path_to_shm_web_hosting_xml_file()
    )
    return shm_web_hosting_reader.get_database_server(database_server_id)


@trace
def get_hash_name_for_database(server_name, entity, max_name_len):
    h = hashlib.md5()
    for s in [server_name, entity]:
        h.update(s)
    # database name must start with alphabetical symbol
    return 'a' + h.hexdigest()[:max_name_len-1]


@trace
def get_hash_name_for_database2(max_name_len, source_server, database_server, database):
    h = hashlib.md5()
    for s in [source_server.ip(), database_server.id, database.external_id]:
        h.update(s)
    # database name must start with alphabetical symbol
    return 'a' + h.hexdigest()[:max_name_len-1]


@trace
def get_hash_name_for_database_user2(max_name_len, source_server, database_server, user):
    h = hashlib.md5()
    for s in [source_server.ip(), database_server.id, user.database_external_id, user.login]:
        h.update(s)
    # database name must start with alphabetical symbol
    return 'a' + h.hexdigest()[:max_name_len-1]


def is_windows_webspace(webspace):
    """
    Return True if webspace is a windows one

    :type webspace: migrator.odin_automation.services.shm.lib.data.subscription.SHMWebspace
    """
    return webspace.type == 'iis'


class WebspaceMacroDefinition(ContextMacroDefinition):
    WEBSPACE_ROOT = 'WEBSPACEROOT'
    SYS_USER = 'SYSUSER'

    def __init__(self, webspace):
        super(WebspaceMacroDefinition, self).__init__()
        self.macros_definition = {
            self.WEBSPACE_ROOT: webspace.www_root,
            self.SYS_USER: webspace.system_user
        }


class SiteMacroDefinition(ContextMacroDefinition):
    SITE_ROOT = 'SITEROOT'

    def __init__(self, webspace_macro_definition, site):
        """

        :type subscription_macro_definition: SubscriptionMacroDefinition
        :type site_name: str
        """
        super(SiteMacroDefinition, self).__init__()
        self._build_macro_definition(webspace_macro_definition, {
            self.SITE_ROOT: site.www_root
        })
