# (C)2017 Ingram Micro Inc. all Rights Reserved
import types

from migrator.core.utils import cached
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import \
    get_aps_connector, get_aps_connector_with_admin_token
from migrator.odin_automation.services.shm.lib.aps.types import aps_operations


@cached
def get_service_aps_id(subscription_id, service_type):
    aps = get_aps_connector(subscription_id)
    return aps.get_first_resource(filter_str='implementing(%s)' % service_type)['aps']['id']


@cached
def get_shm_application_nodes():
    aps = get_aps_connector_with_admin_token()
    application_aps_id = aps.get_first_resource(filter_str='implementing(%s)' % types.WebApplication)['aps']['id']
    return aps.get_resources(resource_path='%s/nodeList' % application_aps_id)


def get_shm_application_nodes_site_capacity(os=None):
    """
    Get total available capacity and usage for websites in WHP module

    :param os: None|str Possible values: None, 'windows', 'linux'
    :return: int, int Available capacity, consumed capacity
    """
    available_capacity = 0
    consumed_capacity = 0
    for shm_node in get_shm_application_nodes():
        if os is not None and shm_node['os'].lower() != os:
            continue

        if shm_node['status'] == 'ready':
            available_capacity += shm_node['capacityAvailableWebsites']
            consumed_capacity += round(shm_node['capacityAvailableWebsites'] * shm_node['capacityUsageRatio'])

    return available_capacity, int(consumed_capacity)


@cached
def is_shm_node_license_valid(node_guid):
    aps = get_aps_connector_with_admin_token()
    application_aps_id = aps.get_first_resource(filter_str='implementing(%s)' % types.WebApplication)['aps']['id']
    license_details = aps.post_resource(application_aps_id, aps_operations['getLicenses'], [node_guid])[0]
    return license_details['isInstalled'] and not license_details['isError']
