# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core import MigrationNoRepeatError
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import \
    get_aps_connector_with_admin_token, get_error, get_error_message
from migrator.odin_automation.platform.lib.api.aps_runtime.apsc import APSCTransportError
import types

HOST_ATTRIBUTE_OACI_CLOUD_ROLE = 'Licensed Virtual Environments'
HOST_CAPACITY_WEBSITES = 'Websites'
HOST_INCREASE_CAPACITY_CONSUMPTION = 'increaseConsumption'
HOST_DECREASE_CAPACITY_CONSUMPTION = 'decreaseConsumption'


def update_host_websites_capacity(host_capacities, method_name):
    """
    Increase/decrease host consumption by 1 according to method_name
    :param host_capacities: migrator.odin_automation.platform.lib.api.aps_runtime.utils.AttrDict[]
    :param method_name: str
    :return: None
    """
    try:
        for capacity in host_capacities:
            if capacity['name'] == HOST_CAPACITY_WEBSITES:
                aps = get_aps_connector_with_admin_token()
                aps.post_resource(capacity['aps']['id'], method_name)
                break
    except APSCTransportError as e:
        error = get_error(e)
        if error != 'Plesk.ErrorHandling.HostManager.CapacityLimitReached':
            raise MigrationNoRepeatError(get_error_message(e))


def get_shared_hosting_host_id_by_webspace(webspace_id):
    """
    Get the backnet ip address of WHP node that hosts the webspace
    """
    aps = get_aps_connector_with_admin_token()
    return aps.get_first_resource(
        filter_str='implementing(%s),eq(aps.id,%s),select(paHost.hostId)' % (types.AbstractWebspace, webspace_id)
    )['paHost']['hostId']
