from migrator.core.language import _T
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import get_app_instance_uid
from migrator.odin_automation.platform.lib.api.aps.notification import \
    Notification, NotificationStatus, send_notification_if_not_exist, get_notification_by_context, update_notification
from migrator.odin_automation.services.shm.lib.aps.types import SHMApplication
from migrator.odin_automation.platform.lib import get_oam_aps_application_certificate_path
from migrator.odin_automation.platform.lib.api.aps.domain import get_pa_domain_by_name
from migrator.odin_automation.services.shm.lib.aps.website import get_website_linked_with_pa_domain_uid


def send_notification_migration_in_progress(account_aps_id, site_name):
    send_notification_if_not_exist(Notification(
        account_id=account_aps_id,
        package_id=get_app_instance_uid(SHMApplication),
        context=_get_notification_website_context(site_name, NotificationStatus.IN_PROGRESS),
        message=_T("NOTIFICATION_MIGRATION_IN_PROGRESS_TITLE"),
        details_message=_T("NOTIFICATION_MIGRATION_IN_PROGRESS").format(site_name=site_name),
        status=NotificationStatus.IN_PROGRESS
    ), get_oam_aps_application_certificate_path())


def send_notification_migration_completed(site_name, subscription_id):
    in_progress_notification = get_notification_by_context(
        _get_notification_website_context(site_name, NotificationStatus.IN_PROGRESS)
    )
    if in_progress_notification:
        pa_domain_uid = get_pa_domain_by_name(site_name, subscription_id)['aps']['id']
        website = get_website_linked_with_pa_domain_uid(subscription_id, pa_domain_uid)
        if not website:
            return

        update_notification(Notification(
            account_id='',
            package_id=get_app_instance_uid(SHMApplication),
            context=_get_notification_website_context(site_name, NotificationStatus.READY),
            message=_T("NOTIFICATION_MIGRATION_COMPLETED_TITLE"),
            details_message=_T("NOTIFICATION_MIGRATION_COMPLETED").format(site_name=site_name),
            status=NotificationStatus.READY,
            link_more='/v/www.odin.com/products/shm/web/application/website-details/r/%s' % website['aps']['id'],
            link_more_text_message=_T("NOTIFICATION_LINK_MORE_TEXT_VIEW_WEBSITE")
        ), in_progress_notification.id, get_oam_aps_application_certificate_path())


def _get_notification_website_context(site_name, notification_status):
    return 'OAM_%s_%s' % (notification_status, site_name.encode('idna'))
