# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.utils import urlquote
from migrator.core.utils.loggers.log import trace
from migrator.odin_automation.platform.lib.api.aps_runtime.aps2 import \
    get_aps_connector_with_admin_token, get_app_instance_uid
from migrator.odin_automation.platform.lib.api.aps.subscription import get_subscription

import types


@trace
def create_alias(subscription_id, domain_id, site_id, alias_name, seo_redirect=True):
    json_data = {
        'aps': {'type': types.WebSiteAlias},
        'domain': {'aps': {'id': domain_id}},
        'site': {'aps': {'id': site_id}},
        'seoRedirect': seo_redirect,
    }

    aps = get_aps_connector_with_admin_token()
    pa_subscription_uid = get_subscription(subscription_id)['aps']['id']
    aps.setAdditionalHeaders({'APS-Subscription-ID': pa_subscription_uid})

    alias = get_alias_by_name(subscription_id, alias_name)
    if not alias:
        alias = aps.post_resource('', '', json_data)['aps']['id']

    return alias


@trace
def register_alias(subscription_id, domain_id, site_id):
    shm_application_uid = get_app_instance_uid(types.SHMApplication)

    aps = get_aps_connector_with_admin_token()
    pa_subscription_uid = get_subscription(subscription_id)['aps']['id']
    aps.setAdditionalHeaders({'APS-Subscription-ID': pa_subscription_uid})

    resource = {
        'aps': {'type': types.WebSiteAlias},
        'domain': {'aps': {'id': domain_id}},
        'site': {'aps': {'id': site_id}}
    }

    return aps.register_resource(shm_application_uid, 'WebSiteAlias', resource)['aps']['id']


@trace
def get_alias_by_name(subscription_id, pa_domain_name):
    aps = get_aps_connector_with_admin_token()
    pa_subscription_uid = get_subscription(subscription_id)['aps']['id']
    aps.setAdditionalHeaders({'APS-Subscription-ID': pa_subscription_uid})

    return aps.get_first_resource(filter_str='implementing(%s),select(domain,site),eq(domain.name,%s)' % (
        types.WebSiteAlias, urlquote(pa_domain_name.lower())
    ))
