# (C)2017 Ingram Micro Inc. all Rights Reserved
import uuid


def register_linux_service_node(poa_api, web_source_host):
    request_id = uuid.uuid4()
    ip_address = web_source_host.ip()
    txn_id = poa_api.txn_begin(request_id)
    poa_api.host.register_linux_service_node(
        ip_address=ip_address,
        login=web_source_host.node_settings.ssh_auth.username,
        password=web_source_host.node_settings.ssh_auth.password,
        role='SHM_WEB_NODE_LINUX',
        role_params=[
            {
                'name': 'shm_web_node_linux.z_allow_preinstalled_plesk',
                'value': 'true',
            },
            {
                'name': 'shm_web_node_linux.z_install_mail_components',
                'value': 'false',
            },
            {
                'name': 'shm_web_node_linux.z_install_web_components',
                'value': 'false',
            },
        ],
        txn_id=txn_id,
        # register the node using its single ip as both communication_ip and shared_ip to avoid some random node ip
        # selection as exclusive communication_ip. this may break takeover.
        network_config={
            'communication_ip': ip_address,
            'shared_ip': ip_address
        },
        register_only_shared_ip=True
    )
    poa_api.txn_commit(txn_id)
    return request_id


def register_windows_service_node(poa_api, web_source_host):
    resource_type_id = poa_api.get_resource_types_by_class('hosts')[0]['resource_type_id']
    request_id = uuid.uuid4()
    ip_address = web_source_host.ip()
    txn_id = poa_api.txn_begin(request_id)
    poa_api.host.register_windows_service_node(
        ip_address=ip_address,
        username=web_source_host.node_settings.windows_auth.username,
        password=web_source_host.node_settings.windows_auth.password,
        resource_type_id=resource_type_id,
        role='SHM_WEB_NODE_WINDOWS',
        role_params=[
            {
                'name': 'shm_web_node_windows.z_allow_preinstalled_plesk',
                'value': 'true',
            },
            {
                'name': 'shm_web_node_windows.z_install_mail_components',
                'value': 'false',
            },
            {
                'name': 'shm_web_node_windows.z_install_web_components',
                'value': 'false',
            },
        ],
        txn_id=txn_id,
        # register the node using its single ip as both communication_ip and shared_ip to avoid some random node ip
        # selection as exclusive communication_ip. this may break takeover.
        network_config={
            'communication_ip': ip_address,
            'shared_ip': ip_address
        },
        register_only_shared_ip=True
    )
    poa_api.txn_commit(txn_id)
    return request_id
