# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import InternalError
from migrator.core.data.model.web_hosting_shm import PhysicalHostingModel
from migrator.core.data.source.xml.web_content_shm import ShmWebContentReaderXml
from migrator.core.language import _T
from migrator.core.transfer.content_list_reader_base import ContentListReaderBase, ContextedContent
from migrator.core.transfer.content_migrator import get_content_migrator
from migrator.odin_automation.services.shm.lib import WebspaceMacroDefinition, SiteMacroDefinition

logger = logging.getLogger(__name__)


class WebContentListReader(ContentListReaderBase):
    def __init__(self, content_xml_filename):
        self.content_data_reader = ShmWebContentReaderXml(content_xml_filename)

    def _get_content_list_and_migrator(self, global_context, migrated_subscription):

        webspaces_list = [] if not migrated_subscription.service else migrated_subscription.service.webspaces
        for migrated_webspace in webspaces_list:
            if len(migrated_webspace.sites) > 0:
                webspace_macro_definition = WebspaceMacroDefinition(migrated_webspace)
                content_list = self._get_webspace_content_list(
                    migrated_subscription.source_info.external_id, migrated_webspace.external_id,
                    webspace_macro_definition, migrated_webspace, migrated_webspace.sites
                )
                if len(content_list) <= 0:
                    continue

                ip_address = None
                if len(migrated_webspace.ip_addresses) > 0:
                    ip_address = migrated_webspace.ip_addresses[0]

                content_migrator = get_content_migrator(
                    global_context, migrated_webspace.name, migrated_webspace.web_source_host,
                    migrated_webspace.web_target_host, ip_address
                )

                logger.debug(
                    _T("DEBUG_START_COPY_FILES_WEBSPACE").format(webspace_id=migrated_webspace.external_id)
                )

                yield [content_migrator, content_list]

            else:
                continue

    def _get_webspace_content_list(
            self, subscription_id, webspace_id, webspace_macro_definition, migrated_webspace, migrated_sites
    ):
        """

        :type webspace_macro_definition: WebspaceMacroDefinition
        :rtype: list [migrator.core.transfer.content_list_reader_base.ContextedContent]
        :type webspace_id: str
        """
        try:
            content_list = []
            webspace_content = self.content_data_reader.get_webspace(subscription_id, webspace_id).content

            content_list.append(ContextedContent(
                webspace_content,
                WebspaceMacroDefinition(migrated_webspace)
            ))

            for site in migrated_sites:
                if not isinstance(site.hosting, PhysicalHostingModel):
                    continue

                site_content = self.content_data_reader.get_site(subscription_id, webspace_id, site.external_id).content
                if len(site_content) > 0 and site.www_root is None:
                    raise InternalError("www-root for the site %s is not specified" % site.name)

                content_list.append(ContextedContent(
                    site_content,
                    SiteMacroDefinition(webspace_macro_definition, site)
                ))

            return content_list
        except Exception as e:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            raise Exception(
                _T("FAILED_TO_RETRIEVE_CONTENT_LIST_FOR_WEBSPACE").format(id=webspace_id, reason=str(e)))
