# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from migrator.odin_automation.panel import Panel as PanelOA
from migrator.odin_automation.platform.lib.connections import target_server
from migrator.odin_automation.platform.lib.connections.database_target_server import AutomationDatabaseTargetServer
from migrator.core.language import _T
from migrator.odin_automation.services.shm.service_import_api import ServiceImportApiSHM
from migrator.products.plesk.lib import get_plesk_database, get_plesk_webspace_id_by_website
from migrator.products.plesk.target.utils.plesk_utils import PleskDbCredentialsFetcher


TargetServiceInfo = namedtuple('TargetServiceInfo', (
    'service_type',
    'node'
))


class Panel(PanelOA):
    @property
    def name(self):
        return _T("OA_WEB_SN")

    def get_service_nodes(self, conn):
        service_nodes = []
        service_by_name = {
            'bind9': 'dns',
            'ppa_mysql': 'mysql',
            'ppa_pgsql': 'postgresql',
            'ppa_mssql2012': 'mssql',
            'ppa_apache': 'web',
            'ppa_iis': 'web',
            'ppa_postfix': 'mail',
            'ppa_webmail': 'mail',
            'ppa_smartermail': 'mail'
        }

        for service_info in conn.poa_api().service.listServices():
            if service_info.service_name in service_by_name.keys():
                service_nodes.append(
                    TargetServiceInfo(
                        service_type=service_by_name[service_info.service_name],
                        node=target_server.PleskTargetServer(conn, id=service_info.host_id)
                    )
                )
        return service_nodes

    def get_target_database_server(self, global_context, target_server, webspace_name, db_name, db_type):

        webspace_id = get_plesk_webspace_id_by_website(webspace_name, target_server.plesk_api())
        database = get_plesk_database(target_server.plesk_api(), webspace_id, db_name, db_type)
        if not database:
            return None

        db_server = target_server.get_db_servers().get(database.db_server_id)

        login, password = PleskDbCredentialsFetcher.get_instance().get(target_server, db_type, db_server)

        target_database_server = AutomationDatabaseTargetServer(
            global_context.conn.target, db_type, db_server.host, db_server.port, login, password,
            target_server.ip()
        )

        external_db_server = global_context.conn.get_external_database_server(
            target_database_server.panel_server, db_type, target_database_server.host(),
            target_database_server.port(), target_database_server.user(), target_database_server.password()
        )
        if external_db_server is not None:
            return external_db_server
        else:
            return target_database_server

    def get_service_import_api(self, global_context):
        return ServiceImportApiSHM()
