# (C)2019 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.odin_automation.services.shm.lib.aps.api import get_shm_application_nodes_site_capacity

logger = logging.getLogger(__name__)


class Check(object):
    @classmethod
    def get_os(cls):
        raise NotImplementedError()

    @classmethod
    def get_sites_amount(cls, subscriptions_ids):
        raise NotImplementedError()

    def run(self):
        try:
            whp_sites_capacity, whp_sites_usage = get_shm_application_nodes_site_capacity(self.get_os())
            migrated_sites_amount = self.get_migrated_sites_amount()
            not_migrated_sites_amount = self.get_not_migrated_sites_amount()

            logger.debug(_T('CHECK_SHM_HOSTS_CAPACITY_DEBUG').format(
                whp_sites_capacity=whp_sites_capacity,
                whp_sites_usage=whp_sites_usage,
                migrated_sites_amount=migrated_sites_amount,
                not_migrated_sites_amount=not_migrated_sites_amount
            ))

            if not_migrated_sites_amount > whp_sites_capacity:
                return CheckAbstract.STATUS_ERROR, _T("CHECK_SHM_HOSTS_CAPACITY_ERROR").format(
                    sites_amount=not_migrated_sites_amount,
                    avail_capacity=whp_sites_capacity
                )

            return CheckAbstract.STATUS_SUCCESS, _T("CHECK_SHM_HOSTS_CAPACITY_SUCCESS")
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return CheckAbstract.STATUS_ERROR, _T("CHECK_SHM_HOSTS_CAPACITY_UNEXPECTED")

    def get_migrated_sites_amount(self):
        """
        Get amount of migrated sites

        :return: int
        """
        return self._get_sites_amount(True)

    def get_not_migrated_sites_amount(self):
        """
        Get amount of not migrated sites

        :return: int
        """
        return self._get_sites_amount(False)

    def _get_sites_amount(self, migrated):
        """
        :param migrated: bool True - return amount of migrated sites | False - not migrated
        :return: int
        """
        subscriptions_ids = [
            sub.external_id for sub in PlatformReaderDB().get_migration_project_subscriptions(migrated)
        ]
        if not subscriptions_ids:
            return 0

        return self.get_sites_amount(subscriptions_ids)
