# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.odin_automation.platform.lib.db import execute_sql_and_get_formatted_result


def get_database_servers():
    sql = """
        SELECT DISTINCT
            services.host_id AS database_host_id,
            services.service_id AS database_service_id,
            dbm_provide_info.rc_name AS dbtype,
            dbm_services.c_username AS login,
            dbm_services.c_password AS password,
            dbm_services.comm_ip AS host,
            dbm_services.db_hostname AS hostname,
            dbm_services.service_ip as service_ip,
            dbm_services.port AS port
        FROM dbm_provide_info
            JOIN dbm_services ON dbm_provide_info.service_id = dbm_services.service_id
            JOIN services ON dbm_services.service_id = services.service_id
    """

    columns = ('database_host_id', 'database_service_id', 'dbtype', 'login', 'password', 'host', 'hostname', 'service_ip', 'port')
    return execute_sql_and_get_formatted_result(sql, columns)


def get_subscriptions_databases(subscriptions_ids):
    """
    :param subscriptions_ids: list[int] Examples: [1000008], [1000008, 1000009]
    :return: list[]
    """
    sql = """
        SELECT DISTINCT
            dbm_databases.db_id AS external_id,
            services.host_id AS database_host_id,
            services.service_id AS database_service_id,
            dbm_databases.name AS name
        FROM dbm_databases
            JOIN dbm_provide_info ON dbm_databases.provide_id = dbm_provide_info.provide_id
            JOIN dbm_services ON dbm_provide_info.service_id = dbm_services.service_id
            JOIN services ON dbm_services.service_id = services.service_id
        WHERE dbm_provide_info.sub_id IN %s
    """

    columns = ('external_id', 'database_host_id', 'database_service_id', 'name')
    return execute_sql_and_get_formatted_result(sql, columns, [tuple(subscriptions_ids)])


def get_subscription_database_users(subscription_id):
    sql = """
        SELECT
            dbm_db_users.db_id AS database_external_id,
            services.host_id AS database_host_id,
            services.service_id AS database_service_id,
            dbm_users.name AS login,
            dbm_users.password AS password
        FROM dbm_users
            JOIN dbm_db_users on dbm_db_users.user_id=dbm_users.user_id
            JOIN dbm_databases on dbm_databases.db_id = dbm_db_users.db_id
            JOIN dbm_provide_info ON dbm_provide_info.provide_id=dbm_databases.provide_id
            JOIN dbm_services ON dbm_services.service_id=dbm_provide_info.service_id
            JOIN services ON services.service_id=dbm_services.service_id
        WHERE dbm_provide_info.sub_id=%s
    """

    columns = ('database_external_id', 'database_host_id', 'database_service_id', 'login', 'password')
    return execute_sql_and_get_formatted_result(sql, columns, [subscription_id])
