# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.odin_automation.platform.lib.db import execute_sql_and_get_result, execute_sql_and_get_formatted_result


def get_site_ids(webspace_id):
    sql = """
        SELECT ss.subds_id AS site_id
        FROM apache_hostlist ah
        LEFT JOIN apache_homedir_entries ahe ON ah.homedir_id = ahe.homedir_id
        JOIN subdomain_services ss ON ahe.subdomain_id = ss.zone_id
        WHERE ahe.homedir_id = {homedir_id} AND ss.name not like '\*%' AND ahe.domain_name not like '\*%'
        ORDER BY ss.subds_id
    """
    return [x[0] for x in execute_sql_and_get_result(sql.format(homedir_id=webspace_id))]


def get_site_names(webspace_id):
    sql = """
        SELECT domains.name
        FROM apache_hostlist ah
        LEFT JOIN apache_homedir_entries ahe ON ah.homedir_id = ahe.homedir_id
        JOIN subdomain_services ss ON ahe.subdomain_id = ss.zone_id
        JOIN domains ON ahe.subdomain_id = domains.domain_id
        WHERE ahe.homedir_id = {homedir_id} AND ss.name not like '\*%' AND ahe.domain_name not like '\*%'
        ORDER BY ss.subds_id
    """
    return [x[0].decode('utf-8') for x in execute_sql_and_get_result(sql.format(homedir_id=webspace_id))]


def is_exists_wildcard_subdomain(subscription_id, service_names):
    sql = """
        SELECT ss.subds_id AS site_id
        FROM apache_hostlist ah
        LEFT JOIN apache_homedir_entries ahe ON ah.homedir_id = ahe.homedir_id
        JOIN subdomain_services ss ON ahe.subdomain_id = ss.zone_id
        JOIN domain_services ds ON ds.ds_id = ss.ds_id
        JOIN services s ON s.service_id = ds.service_id
        WHERE ss.name like '\*%' AND ahe.domain_name like '\*%' AND s.name IN ({server_names})
        AND ds.subscription_id = {subscription_id}
        ORDER BY ss.subds_id
    """
    return len(execute_sql_and_get_result(sql.format(
        subscription_id=subscription_id,
        server_names=','.join(["'{service_name}'".format(service_name=x) for x in service_names])
    ))) > 0

def get_site_name(site_id):
    return get_site(site_id)['name']


def get_site(site_id):
    sql = """
        SELECT domains.name, ss.path_postfix
        FROM subdomain_services ss
        JOIN domains ON ss.zone_id = domains.domain_id
        WHERE subds_id = %s
    """
    return execute_sql_and_get_formatted_result(sql, ('name', 'path_postfix'), (site_id,))[0]


def get_sites_amount(subscription_ids, service_name):
    sql = """
        SELECT COUNT(ss.subds_id)
        FROM subdomain_services ss
        JOIN domain_services ds ON ss.ds_id = ds.ds_id
        JOIN services s ON ds.service_id = s.service_id
        WHERE s.name IN %s AND ds.subscription_id IN %s
    """
    return int(execute_sql_and_get_result(sql, (tuple(service_name), tuple(subscription_ids)))[0][0])


def get_applications(site_id):
    sql = """
        SELECT aa.name, si.url_path, vd.path
        FROM siteapps_instances si
            JOIN apache_vdirs vd ON vd.aps_instance_id=si.instance_id
            JOIN aps_application_instances aai on aai.app_instance_id = vd.aps_instance_id
            JOIN aps_applications aa on aa.app_id = aai.app_id
        WHERE si.subds_id = %s;
    """
    return execute_sql_and_get_formatted_result(sql, ('app_name', 'app_url', 'app_path'), (site_id,))