# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.odin_automation.platform.lib.db import execute_sql_and_get_result, execute_sql_and_get_formatted_result


def get_webspace_name(webspace_id, service_name):
    sql = """
        SELECT ds.name AS webspace_name
        FROM domain_services ds
        JOIN services s ON s.service_id = ds.service_id
        WHERE s.name IN %s AND ds.hd_id = %s
        ORDER BY ds.hd_id
    """
    return execute_sql_and_get_result(sql, (tuple(service_name), webspace_id,))[0][0]


def get_webspace_ids(subscription_id, service_name):
    sql = """
        SELECT DISTINCT ds.hd_id AS webspace_id
        FROM services s
        JOIN domain_services ds ON s.service_id = ds.service_id
        WHERE s.name IN %s AND ds.subscription_id = %s
        ORDER BY ds.hd_id
    """

    return [x[0] for x in execute_sql_and_get_result(sql, (tuple(service_name), subscription_id))]


def get_webspace_ips(webspace_id):
    sql = 'SELECT host_ip as ipv4, ipv6, ip_type::integer FROM apache_hostlist WHERE homedir_id = %s'
    return execute_sql_and_get_result(sql, (webspace_id,))


def get_webspace_hosting_settings(webspace_id):
    sql = """
        SELECT ah.homedir_id AS webspace_id, ah.service_id, ah.separate_http_dirs, ah.cgi_state, ah.ssi_state,
            ah.ssl_state, ah.erd_state, ah.webstat_state, ah.path_to_php, ae.name AS php_version
        FROM apache_hostlist ah
        JOIN apache_extensions ae ON ah.service_id = ae.service_id
        WHERE ah.path_to_php <> '' AND ae.value = ah.path_to_php AND ah.homedir_id = %s
        UNION
        SELECT ah.homedir_id, ah.service_id, ah.separate_http_dirs, ah.cgi_state, ah.ssi_state, ah.ssl_state,
            ah.erd_state, ah.webstat_state, ah.path_to_php, '' AS php_version
        FROM apache_hostlist ah
        WHERE ah.path_to_php = '' AND ah.homedir_id = %s
    """
    fields = (
        'webspace_id', 'service_id', 'separate_http_dirs', 'cgi_state', 'ssi_state', 'ssl_state', 'erd_state',
        'webstat_state', 'path_to_php', 'php_version'
    )
    return execute_sql_and_get_formatted_result(sql, fields, (webspace_id, webspace_id))[0]


def is_webspace_ssh_access_enabled(webspace_id):
    sql = 'SELECT login FROM apache_ssh_access WHERE homedir_id = %s'
    return bool(execute_sql_and_get_result(sql, (webspace_id,)) is not None)


def get_webspace_ssl_certificates(webspace_id):
    sql = """
        SELECT homedir_id, cert_type, cert_data
        FROM apache_certificates
        WHERE homedir_id = %s
        ORDER BY cert_type DESC
    """
    return execute_sql_and_get_formatted_result(sql, ('homedir_id', 'cert_type', 'cert_data'), (webspace_id,))


def get_webspace_prot_dirs(webspace_id):
    sql = """
        SELECT ad.dir_id, dir_type, name, header, username, password
        FROM apache_directories ad
        LEFT JOIN apache_userlist a ON ad.dir_id = a.dir_id
        WHERE homedir_id = %s
        ORDER BY name, ad.dir_id
    """
    fields = ('dir_id', 'dir_type', 'name', 'header', 'username', 'password')
    return execute_sql_and_get_formatted_result(sql, fields, (webspace_id,))
