# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import InternalError, _T
from migrator.core.utils import cached
from migrator.core.utils.registry import Registry
from migrator.odin_automation.services.ng.lib.server import AutomationServerNg

logger = logging.getLogger(__name__)

WHL_FASTCGI_WRAPPER = '/usr/local/safe-bin/fcgi-wrapper'


@cached
def get_source_host_actual_php_fcgi_version(host_id):
    """
    WHL reports PHP 5.3.0 version for PHP in fastcgi mode despite of actual PHP interpreter version.
    We need to get PHP version on the host ourselves.

    :param host_id: int
    :return: str
    """
    conn = Registry.get_instance().get_context().conn.target
    whl_node = AutomationServerNg.get(conn=conn, id=host_id, ip=None)
    with whl_node.runner() as runner:
        # 1. Get fcgi wrapper script body
        cmd = u'cat %s' % WHL_FASTCGI_WRAPPER
        exit_code, stdout, stderr = runner.sh_unchecked(cmd)
        if exit_code != 0:
            raise InternalError(
                _T("UNEXPECTED_ERROR_OF_GETTING_PHP_FASTCGI_VERSION_FROM_OA_HOST").format(
                    host_id=host_id, fcgi_wrapper=WHL_FASTCGI_WRAPPER, error=stderr
                )
            )

        # 2. Get path to actual PHP binary
        interpreter = None
        for line in stdout.strip().splitlines():
            if line.startswith('exec'):
                _, interpreter = line.split(None, 1)
                interpreter = interpreter.strip()
                break

        if not interpreter:
            raise InternalError(
                _T("UNEXPECTED_ERROR_OF_GETTING_PHP_FASTCGI_VERSION_FROM_OA_HOST").format(
                    host_id=host_id, fcgi_wrapper=WHL_FASTCGI_WRAPPER,
                    error='Path to actual PHP binary not found in the file'
                )
            )
        else:
            logger.debug('Detected actual php fastcgi binary on host %s: %s', host_id, interpreter)

        # 3. Ask PHP interpreter version
        exit_code, stdout, stderr = runner.run_unchecked(unicode(interpreter), ['-v'])
        if exit_code != 0 or not stdout.startswith('PHP'):
            raise InternalError(
                _T("UNEXPECTED_ERROR_OF_GETTING_PHP_FASTCGI_VERSION_FROM_OA_HOST").format(
                    host_id=host_id, fcgi_wrapper=interpreter, error=stderr
                )
            )

        return stdout.split()[1]


@cached
def get_source_host_custom_php_settings(host_id, php_ini_path, php_settings):
    """
    Returns PHP settings retrieved from {php_ini_path} of host with ID {host_id}.

    :param host_id: int
    :param php_ini_path: str
    :param php_settings: tuple
    :return: dict
    """
    php_settings_result = {}

    conn = Registry.get_instance().get_context().conn.target
    whl_node = AutomationServerNg.get(conn=conn, id=host_id, ip=None)
    with whl_node.runner() as runner:
        if runner.file_exists(php_ini_path):
            php_ini_contents = runner.get_file_contents(php_ini_path)
            for line in php_ini_contents.splitlines():
                if line.startswith(php_settings):
                    setting, value = line.split('=', 1)
                    setting = setting.strip()
                    if setting in php_settings:
                        php_settings_result[setting] = value.strip()

    return php_settings_result
