# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.products.oa.checks import hosts_capacity
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.odin_automation.platform.lib.db import OaServiceName
from migrator.products.oa.wsh.lib.api.db.site import is_webspace_wwwroot_should_be_migrated_as_site, get_sites_amount
from migrator.products.oa.lib.api.db.webspace import get_webspace_ids
from migrator.settings import OS_WINDOWS

logger = logging.getLogger(__name__)


class Check(hosts_capacity.Check):

    @classmethod
    def get_os(cls):
        return OS_WINDOWS

    @classmethod
    def get_sites_amount(cls, subscriptions_ids):
        return get_sites_amount(subscriptions_ids)

    def _get_sites_amount(self, migrated):
        """
        :param migrated: bool True - return amount of migrated sites | False - not migrated
        :return: int
        """
        subscriptions_ids = [sub.external_id for sub in PlatformReaderDB().get_migration_project_subscriptions(migrated)]

        # Count additional sites for cases when wwwroot of webspace is not equal to wwwroots of all its sites
        technical_sites_amount = 0
        for subscription_id in subscriptions_ids:
            for webspace_id in get_webspace_ids(subscription_id, OaServiceName.WSH):
                if is_webspace_wwwroot_should_be_migrated_as_site(webspace_id):
                    technical_sites_amount += 1

        return super(Check, self)._get_sites_amount(migrated) + technical_sites_amount


