# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.utils import cached
from migrator.odin_automation.platform.lib.db import OaServiceName, execute_sql_and_get_formatted_result


def get_iis_host(webspace_id):
    """
    :param webspace_id: int Example: 100000
    :return: dict
    """
    sql = 'SELECT' \
          ' services.host_id' \
          ' FROM services ' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' WHERE services.name IN %s' \
          ' AND domain_services.hd_id = %s'

    columns = ('host_id',)

    return execute_sql_and_get_formatted_result(sql, columns, [tuple(OaServiceName.WSH), webspace_id])[0]


def get_iis_host_php_version(host_id):
    """
    :param host_id: int Example: 4
    :return: list[]
    """
    sql = 'SELECT packages.version' \
          ' FROM components' \
          ' JOIN packages ON components.pkg_id = packages.pkg_id' \
          " WHERE components.host_id = %s AND packages.name = 'PHP'"

    columns = ('php_version',)

    return execute_sql_and_get_formatted_result(sql, columns, [host_id])


@cached
def get_host_php_path(host_id):
    """
    :param host_id: int
    :return: str
    """
    sql = 'SELECT' \
          ' iis_services_feature_properties.property_value AS php_path' \
          ' FROM iis_services_feature_properties' \
          ' JOIN services ON iis_services_feature_properties.service_id = services.service_id' \
          " WHERE iis_services_feature_properties.property_name = 'php.install.full.path'" \
          ' AND host_id=%s'

    return execute_sql_and_get_formatted_result(sql, ('php_path',), [host_id])[0]['php_path']


def get_host_root_dir(host_id):
    """
    :param host_id: int
    :return: str
    """
    sql = "SELECT" \
          " pv.value" \
          " FROM property_values pv" \
          " JOIN component_properties cp ON cp.value_id=pv.value_id" \
          " JOIN components c ON c.component_id=cp.component_id" \
          " JOIN packages p ON p.pkg_id=c.pkg_id" \
          " JOIN properties pr ON pr.pkg_id=p.pkg_id and pr.prop_id=cp.prop_id" \
          " WHERE p.name='msiis' and pr.name='pem.data.root' and host_id=%s"

    return execute_sql_and_get_formatted_result(sql, ('value',), [host_id])[0]['value']