# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.utils import cached
from migrator.core.utils.registry import Registry
from migrator.odin_automation.platform.lib.db import execute_sql_and_get_result, execute_sql_and_get_formatted_result
from migrator.odin_automation.platform.lib.db.ip import get_ip_address
from migrator.products.oa.wsh.lib.api.db.webspace import get_customer_data_root_path


class SSLProtectionState(object):
    SSL_NONE = 0
    SSL_PROXY = 1
    SSL_CERT = 2


class VDirHostingType(object):
    HT_FWD = 0
    HT_DIR = 1

IS_ROOT = 1


def get_site_name(site_id):
    is_technical_site = False
    site = execute_sql_and_get_result('SELECT name FROM subdomain_services WHERE subds_id=%s', [site_id])

    if not site:
        site = execute_sql_and_get_result('SELECT name FROM domain_services WHERE ds_id=%s', [site_id])
        is_technical_site = True

    return site[0][0], is_technical_site


def get_technical_site_name(webspace_id):
    return execute_sql_and_get_result('SELECT name FROM domain_services WHERE hd_id=%s', [webspace_id])[0][0]


def get_sites_info_by_ip_address(webspace_id, ip_address_oa_id=None):
    sql = 'SELECT' \
          ' DISTINCT iis_addons.addon_id AS site_id,' \
          ' iis_webvsrvs_bindings.ip AS ip,' \
          ' iis_webvsrvs_bindings.ip_type AS ip_type' \
          ' FROM iis_webvsrvs_bindings' \
          ' JOIN iis_webvsrvs ON iis_webvsrvs_bindings.srv_id = iis_webvsrvs.srv_id' \
          ' JOIN iis_addons ON iis_webvsrvs.srv_id = iis_addons.srv_id' \
          ' JOIN iis_domain_services ON iis_addons.ds_id = iis_domain_services.ds_id' \
          ' WHERE iis_domain_services.hd_id=%s'

    parameters = [webspace_id]
    if ip_address_oa_id:
        sql += ' AND iis_webvsrvs_bindings.ip=%s'
        ip_address_for_sql_query = '.'.join([str(int(x)) for x in get_ip_address(ip_address_oa_id).split('.')])
        parameters.append(ip_address_for_sql_query)

    is_technical_site_in_list = False
    sites = execute_sql_and_get_formatted_result(sql, ('site_id', 'ip', 'ip_type'), parameters)

    if is_webspace_wwwroot_should_be_migrated_as_site(webspace_id):
        # Getting technical site
        sql = 'SELECT' \
              ' DISTINCT' \
              ' iis_domain_services.ds_id AS site_id,' \
              ' iis_webvsrvs_bindings.ip AS ip,' \
              ' iis_webvsrvs_bindings.ip_type AS ip_type' \
              ' FROM iis_webvsrvs_bindings' \
              ' JOIN iis_webvsrvs ON iis_webvsrvs_bindings.srv_id = iis_webvsrvs.srv_id' \
              ' JOIN iis_domain_services ON iis_webvsrvs.srv_id = iis_domain_services.basic_srv_id' \
              ' WHERE iis_domain_services.hd_id=%s'

        if ip_address_oa_id:
            sql += ' AND iis_webvsrvs_bindings.ip=%s'

        sites += execute_sql_and_get_formatted_result(sql, ('site_id', 'ip', 'ip_type'), parameters)
        is_technical_site_in_list = True

    return sites, is_technical_site_in_list


def get_site_ssl_certificate(site_id):
    """
    :param site_id: int
    :return: dict|None
    """
    sql = 'SELECT' \
          ' urd_key.data AS private_key,' \
          ' urd_cert.data AS cert,' \
          ' urd_ca_cert.data AS ca' \
          ' FROM iis_addons' \
          ' JOIN iis_webvsrvs ON iis_addons.srv_id = iis_webvsrvs.srv_id' \
          ' LEFT JOIN user_repo_data urd_key ON iis_webvsrvs.pkey_item_id = urd_key.item_id' \
          ' LEFT JOIN user_repo_data urd_cert ON iis_webvsrvs.cert_item_id = urd_cert.item_id' \
          ' LEFT JOIN user_repo_data urd_ca_cert ON iis_webvsrvs.ca_certs_item_id = urd_ca_cert.item_id' \
          ' WHERE iis_addons.addon_id=%s' \
          ' AND iis_webvsrvs.ssl_state!=%s'

    columns = ('private_key', 'cert', 'ca')
    data = execute_sql_and_get_formatted_result(sql, columns, [site_id, SSLProtectionState.SSL_NONE])
    return data[0] if data else None


def is_ssl_enabled_for_site(site_id):
    """
    :param site_id: int
    :return: boolean
    """
    sql = 'SELECT' \
          ' iis_webvsrvs.ssl_state AS ssl_state' \
          ' FROM iis_webvsrvs' \
          ' JOIN iis_addons ON iis_webvsrvs.srv_id = iis_addons.srv_id' \
          ' WHERE iis_addons.addon_id=%s'

    records = execute_sql_and_get_formatted_result(sql, ('ssl_state',), [site_id])
    if not records:
        sql = """
          SELECT iis_webvsrvs.ssl_state AS ssl_state
          FROM iis_webvsrvs
          JOIN iis_domain_services ON iis_webvsrvs.srv_id = iis_domain_services.basic_srv_id
          WHERE iis_domain_services.ds_id=%s
        """
        records = execute_sql_and_get_formatted_result(sql, ('ssl_state',), [site_id])

    ssl_state = records[0]['ssl_state']
    return ssl_state != SSLProtectionState.SSL_NONE


def get_site_details(site_id):
    """
    :param site_id: int Example: 100000
    :return: dict
    """

    sql = 'SELECT' \
          ' domain_services.hd_id,' \
          ' domain_services.ds_id,' \
          ' subdomain_services.path_postfix' \
          ' FROM subdomain_services' \
          ' JOIN domain_services ON subdomain_services.ds_id = domain_services.ds_id' \
          ' WHERE subdomain_services.subds_id=%s'

    columns = ('hd_id', 'ds_id', 'path_postfix',)
    records = execute_sql_and_get_formatted_result(sql, columns, [site_id])
    if not records:
        sql = """
            SELECT
              iis_domain_services.hd_id AS hd_id,
              iis_domain_services.ds_id AS ds_id,
              iis_webvdirs.location AS path_postfix
            FROM iis_domain_services
            JOIN iis_webvdirs ON iis_domain_services.basic_srv_id = iis_webvdirs.srv_id
            WHERE iis_domain_services.ds_id=%s
        """
        records = execute_sql_and_get_formatted_result(sql, columns, [site_id])

    return records[0]


def get_sites_postfixes(webspace_id):
    """
    :param webspace_id: int Example: 100000
    :return: list[]
    """

    sql = 'SELECT' \
          ' subdomain_services.subds_id,' \
          ' subdomain_services.path_postfix' \
          ' FROM subdomain_services' \
          ' JOIN domain_services ON subdomain_services.ds_id = domain_services.ds_id' \
          ' WHERE domain_services.hd_id=%s'

    columns = ('site_id', 'site_postfix')
    records = execute_sql_and_get_formatted_result(sql, columns, [webspace_id])

    if is_webspace_wwwroot_should_be_migrated_as_site(webspace_id):
        sql = """
            SELECT
              iis_domain_services.ds_id AS site_id,
              iis_webvdirs.location AS site_postfix
            FROM iis_domain_services
            JOIN iis_webvdirs ON iis_domain_services.basic_srv_id = iis_webvdirs.srv_id
            WHERE iis_domain_services.hd_id=%s
        """
        records += execute_sql_and_get_formatted_result(sql, columns, [webspace_id])

    return records


def get_site_path(site_id):
    """
    :param site_id: int Example: 100000
    :return: str Example: C:\CustomerData\webspaces\webspace_00100000\wwwroot
    """

    site_details = get_site_details(site_id)
    webspace_hd_id = site_details['hd_id']
    webspace_ds_id = site_details['ds_id']
    site_prefix = site_details['path_postfix']
    customer_data_root_path = get_customer_data_root_path(webspace_ds_id)
    website_path = '%s\\webspaces\\webspace_%08d\\%s' % (customer_data_root_path, webspace_hd_id, site_prefix)
    return website_path.replace('/', '\\')


def get_site_read_write_permissions(site_id):
    """
    :param site_id: int
    :return: boolean, boolean
    """
    sql = """
        SELECT
          sec_read  AS read,
          sec_write AS write
        FROM iis_webvdirs
          JOIN iis_addons ON iis_addons.srv_id = iis_webvdirs.srv_id
        WHERE iis_addons.addon_id=%s AND is_root = 1
    """

    columns = ('read', 'write')
    records = execute_sql_and_get_formatted_result(sql, columns, [site_id])
    if not records:
        sql = """
            SELECT
              sec_read  AS read,
              sec_write AS write
            FROM iis_webvdirs
              JOIN iis_domain_services ON iis_domain_services.basic_srv_id = iis_webvdirs.srv_id
            WHERE iis_domain_services.ds_id=%s AND is_root = 1
        """
        records = execute_sql_and_get_formatted_result(sql, columns, [site_id])

    permissions = records[0]
    return permissions['read'] != 0, permissions['write'] != 0


@cached
def is_webspace_wwwroot_should_be_migrated_as_site(webspace_id):
    global_context = Registry.get_instance().get_context()
    if global_context.migration_project.settings['migrateTechnicalWebspace'] == 'doNotMigrateWebspace':
        return False

    sql = """
    SELECT iis_webvdirs.location
    FROM iis_webvdirs
      JOIN iis_domain_services ON iis_webvdirs.srv_id = iis_domain_services.basic_srv_id
    WHERE iis_domain_services.hd_id=%s
    INTERSECT
    SELECT iis_webvdirs.location
    FROM iis_webvdirs
      JOIN iis_addons ON iis_webvdirs.srv_id = iis_addons.srv_id
      JOIN iis_domain_services ON iis_addons.ds_id = iis_domain_services.ds_id
    WHERE iis_domain_services.hd_id=%s
    """

    records = execute_sql_and_get_result(sql, [webspace_id, webspace_id])
    return len(records) == 0


def get_virtual_dirs(site_id):
    sql = """
    SELECT
        iis_webvdirs.dir_id, iis_webvdirs.is_root, iis_webvdirs.dir_name, iis_webvdirs.desire_dir_name,
        iis_webvdirs.description, iis_webvdirs.hosting_type, iis_webvdirs.location, iis_webvdirs.desire_location,
        iis_webvdirs.sec_script_src, iis_webvdirs.sec_read, iis_webvdirs.sec_write, iis_webvdirs.sec_browse,
        iis_webvdirs.sec_log_visits, iis_webvdirs.sec_script, sec_exec, iis_webvdirs.auth_anonymous,
        iis_webvdirs.auth_windows, iis_webvdirs.idc_status, iis_webvdirs.php_status
    FROM iis_addons
        JOIN iis_webvsrvs ON iis_addons.srv_id = iis_webvsrvs.srv_id
        JOIN iis_webvdirs ON iis_webvsrvs.srv_id=iis_webvdirs.srv_id
    WHERE NOT iis_webvdirs.is_root=%s AND iis_webvdirs.hosting_type=%s AND iis_addons.addon_id=%s;
    """

    columns = (
        'dir_id', 'is_root', 'dir_name', 'desire_dir_name', 'description', 'hosting_type', 'location', 'desire_location',
        'sec_script_src', 'sec_read', 'sec_write', 'sec_browse', 'sec_log_visits', 'sec_script', 'sec_exec',
        'auth_anonymous', 'auth_windows', 'idc_status', 'php_status'
    )

    return execute_sql_and_get_formatted_result(sql, columns, [IS_ROOT, VDirHostingType.HT_DIR, site_id])


def get_virtual_dir_default_docs(dir_id):
    sql = "SELECT location FROM iis_webvdirs_default_documents WHERE dir_id=%s ORDER BY priority"
    return [rec[0] for rec in execute_sql_and_get_result(sql, [dir_id])]


def get_sites_amount(subscriptions_ids):
    """
    :param subscriptions_ids: list[int|str] Examples: [1000008], [1000008, 1000009], ['1000008', '1000009']
    :return: int
    """
    sql = """
    SELECT
      COUNT(iis_addons.addon_id) AS sites_amount
    FROM iis_domain_services
      JOIN iis_addons ON iis_domain_services.ds_id = iis_addons.ds_id
    WHERE iis_domain_services.sub_id IN %s
    """

    return int(
        execute_sql_and_get_formatted_result(sql, ('sites_amount',), [tuple(subscriptions_ids)])[0]['sites_amount']
    )


def get_site_postfix(site_id):
    """
    :param webspace_id: int Example: 100000
    :return: str
    """
    sql = 'SELECT path_postfix FROM subdomain_services WHERE subds_id=%s'

    records = execute_sql_and_get_result(sql, [site_id])
    return records[0][0] if records else None

