# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.odin_automation.platform.lib.db import OaServiceName, execute_sql_and_get_formatted_result


customer_data_root_path_property_name = 'pem.data.root'
exclusive_application_pool_type = 0


def get_webspace_scripting_params(webspace_id):
    """
    :param webspace_id: int Example: 100000
    :return: list[]
    """

    sql = 'SELECT' \
          ' iis_webvdirs.php_status,' \
          ' iis_webvdirs.activeperl_status,' \
          ' iis_webvdirs.asp_status,' \
          ' iis_webvdirs.aspdotnet_status' \
          ' FROM iis_domain_services' \
          ' JOIN iis_webvdirs ON iis_domain_services.basic_srv_id = iis_webvdirs.srv_id' \
          ' WHERE iis_domain_services.hd_id = %s'

    columns = ('php_status', 'perl_status', 'asp_status', 'aspdotnet_status')

    return execute_sql_and_get_formatted_result(sql, columns, [webspace_id])


def get_customer_data_root_path(webspace_id):
    """
    :param webspace_id: int Example: 100000
    :return: str
    """

    sql = 'SELECT' \
          ' property_values.value' \
          ' FROM component_properties' \
          ' JOIN properties ON component_properties.prop_id = properties.prop_id' \
          ' JOIN property_values ON property_values.value_id = component_properties.value_id' \
          ' JOIN services ON services.component_id = component_properties.component_id' \
          ' JOIN domain_services ON services.service_id = domain_services.service_id' \
          ' WHERE domain_services.ds_id = %s' \
          ' AND properties.name = %s' \
          ' AND services.name IN %s'

    columns = ('value',)
    return execute_sql_and_get_formatted_result(
        sql, columns, [webspace_id, customer_data_root_path_property_name, tuple(OaServiceName.WSH)]
    )[0]['value']


def get_webspace_root(webspace_id):
    sql = """
    SELECT disk_space_paths.path
    FROM disk_space_paths
        JOIN disk_space_elems ON disk_space_elems.path_id = disk_space_paths.path_id
        JOIN iis_resources ON iis_resources.rt_instance_id = disk_space_elems.rti_id
        JOIN iis_domain_services ON iis_domain_services.ds_id = iis_resources.ds_id
    WHERE iis_domain_services.hd_id=%s
    """
    return execute_sql_and_get_formatted_result(sql, ('path',), [webspace_id])[0]


def is_webspace_application_pool_exclusive(webspace_id):
    """
    :param webspace_id: int Example: 100000
    :return: list[]
    """

    sql = 'SELECT' \
          ' iis_app_pools.type' \
          ' FROM iis_domain_services' \
          ' JOIN iis_webvdirs ON iis_domain_services.basic_srv_id = iis_webvdirs.srv_id' \
          ' JOIN iis_app_pools ON iis_webvdirs.app_pool_id = iis_app_pools.app_pool_id' \
          ' WHERE iis_domain_services.hd_id = %s'

    columns = ('iis_app_pools', )
    return execute_sql_and_get_formatted_result(sql, columns, [webspace_id])[0]['iis_app_pools'] == \
           exclusive_application_pool_type
