# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging
import ntpath
import os

from migrator.core.data.format.web_content_shm import ShmWebContentReader
from migrator.core.data.model.web_content_shm import SubscriptionModel, WebspaceModel, SiteModel, ContentItemModel
from migrator.products.oa.wsh.lib.api.db.site import get_site_path, get_sites_postfixes, get_virtual_dirs
from migrator.products.oa.wsh.lib.api.db.webspace import get_webspace_root
from migrator.products.oa.wsh.lib.export.web_base_shm import WSHBaseExporter

logger = logging.getLogger(__name__)


class WSHContentExporter(WSHBaseExporter, ShmWebContentReader):
    def get_subscription(self, subscription_id):
        """
        :param subscription_id: int
        :return: SubscriptionModel
        """
        return SubscriptionModel(
            external_id=subscription_id,
            webspaces=self.get_webspaces_list(subscription_id)
        )

    def get_webspace(self, subscription_id, webspace_id, first_webspace_in_subscription=False):
        """
        :param subscription_id: int
        :param webspace_id: int
        :param first_webspace_in_subscription: boolean
        :return: WebspaceModel
        """
        sites = self.get_sites_list(subscription_id, webspace_id)
        webspace_vdir_roots = set([])
        for site in sites:
            for vdir in get_virtual_dirs(site.external_id):
                vdir_location = vdir['desire_location']
                if vdir_location.startswith('siteapps'):
                    webspace_vdir_roots.add('siteapps')

        content = []
        real_webspace_id, ip_address_oa_id, filter_by_ip = self._read_webspace_id(webspace_id)
        webspace_root = get_webspace_root(real_webspace_id)['path']
        for vdir_root in webspace_vdir_roots:
            content.append(ContentItemModel(
                    source_path=ntpath.join(webspace_root, vdir_root),
                    target_path='${WEBSPACEROOT}/'
                ))

        return WebspaceModel(
            external_id=webspace_id,
            sites=sites,
            content=content
        )

    def get_site(self, subscription_id, webspace_id, site_id):
        real_webspace_id, ip_address_oa_id, filter_by_ip = self._read_webspace_id(webspace_id)

        site_webroot = get_site_path(site_id)
        exclude_paths = ['.user.ini']
        site_prefixes = {x['site_id']: x['site_postfix'] for x in get_sites_postfixes(real_webspace_id)}
        site_prefix = site_prefixes[site_id] + '/'
        for prefix in site_prefixes.values():
            if prefix.startswith(site_prefix):
                exclude_paths.append('/' + prefix[len(site_prefix):])

        return SiteModel(
            external_id=site_id,
            name=self._get_site_name_for_migration(site_id),
            source_webroot=site_webroot,
            content=[
                ContentItemModel(
                    source_path=os.path.join(site_webroot, ''),
                    target_path='${SITEROOT}/',
                    exclude=exclude_paths
                ),
            ]
        )
