# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core import _T, InternalError
from migrator.core.utils import cached
from migrator.core.utils.registry import Registry
from migrator.odin_automation.platform.lib.connections import target_server
from migrator.products.oa.wsh.lib.api.db.host import get_host_root_dir


class AutomationServerIIS(target_server.AutomationServer):
    @property
    @cached
    def vhosts_dir(self):
        return get_host_root_dir(self.host_id)


@cached
def get_source_host_php_settings(host_external_id, php_ini_path, php_settings, throw_exception_if_file_absent=True):
    """
    Returns PHP settings retrieved from {php_ini_path} of host with ID {host_external_id}
    If {throw_exception_if_file_absent} is False - exception will not be thrown in case of absent file

    :param host_external_id: int
    :param php_ini_path: str
    :param php_settings: tuple
    :param throw_exception_if_file_absent: bool
    :return:
    """
    settings_cmd_filters = []
    for name in php_settings:
        settings_cmd_filters.append(u'/C:"^[ ]*%s[ ]*="' % name)

    conn = Registry.get_instance().get_context().conn.target
    oa_iis_server = AutomationServerIIS.get(conn=conn, id=host_external_id, ip=None)
    with oa_iis_server.runner() as runner:
        cmd = u'findstr /R {filters} {php_ini_path}'
        if not throw_exception_if_file_absent:
            cmd = u'if exist {php_ini_path} (%s)' % cmd

        exit_code, stdout, stderr = runner.sh_unchecked(u'cmd /C ' + cmd.format(
            filters=' '.join(settings_cmd_filters), php_ini_path=php_ini_path
        ))

        if exit_code != 0:
            raise InternalError(_T("UNEXPECTED_ERROR_OF_READING_PHP_SETTINGS_FROM_OA_HOST").format(
                php_ini_path=php_ini_path, host_id=host_external_id, error=stderr))

        php_settings_result = {}
        for setting_with_value in stdout.replace('\r\n', '\n').split('\n'):
            if setting_with_value == '':
                continue

            setting_name, setting_value = setting_with_value.replace(' ', '').split(';', 1)[0].split('=')
            php_settings_result[setting_name] = setting_value

        return php_settings_result
