# (C)2017 Ingram Micro Inc. all Rights Reserved
# -*- coding: utf-8 -*-
"""Sample Agent exception classes"""


class APIversionError(Exception):

    """ Indicates invalid arguments
    """

    def __init__(self, msg):
        self.msg = msg

    def what(self):
        """ Get error message """
        return self.msg

    def __str__(self):
        return self.msg


class SAException(Exception):

    """Base class for all Sample Agent exceptions"""

    def __init__(self):
        self.message = "SAException. You shouldn't see this exception"

    def __str__(self):
        return self.message


class DuplicateID(SAException):

    """Entity with specified id already exists"""

    def __init__(self, entity, key):
        self.message = entity + " with id '" + key + "' already exists"


class NoSuchElement(SAException):

    """Requested entity does not exist"""

    def __init__(self, entity, key):
        self.message = entity + " with id '" + key + "' does not exist"


class InvalidAccountType(SAException):

    """Account of inapropriate type is passed"""

    def __init__(self, accountId, accountType):
        self.message = "Account with id '" + accountId +\
            "' have invalid type: " + accountType


class LogicError(SAException):

    """Some kind of internal logic error occurred"""

    def __init__(self, msg):
        self.message = msg

class ServiceError(APIversionError):
    pass

class EmptyCSVSource(SAException):

    """CSV file is empty"""

    def __init__(self, filename):
        self.message = "Empty CSV file '%s'." % (filename)
