# (C)2017 Ingram Micro Inc. all Rights Reserved
# -*- coding: utf-8 -*-
"""Some extensions to xml.dom.minidom library"""

from xml.dom import EMPTY_NAMESPACE
from xml.dom.minicompat import NodeList
from xml.dom.minidom import Element, _write_data


class EnhElement(Element):

    def addElement(self, element):
        newElement = EnhElement(element)
        self.appendChild(newElement)
        return newElement


class TextElement (Element):

    """When using xml.dom.minidom.Element and xml.dom.minidom.TextNode to store
       text values toprettyxml() method produces incorrect XMLs due to
       indentations and newlines:
                <element>SomeText</element>
       is represented as
                <element>
                  SomeText
                </element>
       This class intended to fix it. It's very dumb, but works nicely for our
       needs. It's just stores text value and fixes toprettyxml() method"""

    def __init__(self, nodeName, textValue):
        """Create new TextElement.

           nodeName: tag name
           textValue: node text value"""
        # NOTE: this method is ugly, because it's copy-n-paste from
        # xml.dom.minidom.Element.__init__, but 'super' keyword is not
        # supported for classic classes, and xml.dom.Node is classic type and
        # so all Node's descendants
        self.tagName = self.nodeName = nodeName
        self.prefix = None
        self.namespaceURI = EMPTY_NAMESPACE
        self.childNodes = NodeList()

        self._attrs = {}   # attributes are double-indexed:
        self._attrsNS = {}  # tagName -> Attribute
        #    URI,localName -> Attribute
        # in the future: consider lazy generation
        # of attribute objects this is too tricky
        # for now because of headaches with
        # namespaces
        self._textValue = textValue

    def writexml(self, writer, indent="", addindent="", newl=""):
        """This method fixes incorrect behaviour when storing xml.

           indent: current indentation
           addindent: indentation to add to higher levels
           newl: newline string"""
        writer.write(indent + "<" + self.tagName)

        attrs = self._get_attributes()
        a_names = attrs.keys()
        a_names.sort()

        for a_name in a_names:
            writer.write(" %s=\"" % a_name)
            _write_data(writer, attrs[a_name].value)
            writer.write("\"")
        writer.write(">")
        _write_data(writer, self._textValue)
        writer.write("</%s>%s" % (self.tagName, newl))
