#!/usr/bin/perl
# (C)2017 Ingram Micro Inc. all Rights Reserved
use strict;

use XML::Simple;
use Getopt::Long;

## parse command line arguments
my %arg;
GetOptions(\%arg, "help|h", "all|all=s", "file|f=s");
## /parse

if ($arg{'help'} || (not ($arg{'all'} || $arg{'file'})))
{
	help();

} else {
	require HSPC::MT::Billing::Constants;
	require HSPC::MT::BM::HP::Constants;
	import HSPC::MT::BM::HP::Constants qw(
		BM_HP_TYPE_DM_GENERIC
		BM_HP_TYPE_HN_GENERIC
		BM_HP_TYPE_HN_VIRTUOZZO
		BM_HP_TYPE_MISC
		BM_HP_TYPE_PLESK_CLIENT
		BM_HP_TYPE_PLESK_DOMAIN
		BM_HP_TYPE_PLESK_SERVER
		BM_HP_TYPE_VE_GENERIC
		BM_HP_TYPE_VE_PLESK_HN
		BM_HP_TYPE_ONETIMEITEM
		);
	require HSPC::MT::Core::Constants;
	import HSPC::MT::Core::Constants qw(
		SW_HSP_ID
		SW_HSP
		SW_RESELLER
		SW_CUSTOMER
	);
	require SOAP::Lite;

	require HSPC::Util;
	import HSPC::Util qw(sw_purify_data);
	main();
}

exit;

## ############################################

sub _all__xml
{
	my ($objname, $type, $data) = @_;
	return <<END;
<?xml version="1.0" encoding="utf-8"?>
<filters>
	<objects_name>$objname</objects_name>
	<filter>
		<property_name>$type</property_name>
		<where>
			<in>$data</in>
		</where>
	</filter>
</filters>

END
}

## ############################################
sub main
{
	## load file
	my $filters;

	if ( $arg{'all'} ne '' ) {
		##my $type = $arg('type') || 'type';
		##my $data = $arg('type') || 'type';
		if ( $arg{'all'} =~ /^accounts$/i ) {
			$filters = _all__xml('account','type','customer,reseller');
		} elsif ( $arg{'all'} =~ /^documents$/i ) {
			$filters = _all__xml('document','type','IN,PO,PF,CA,DA');
		} elsif ( $arg{'all'} =~ /^subscriptions$/i ) {
			$filters = _all__xml('subscription','type','VE,HN,HNVZ,MISC,DM,PLSRV,PLCLT,PLDM,VEPLHN,OTFI');
		} elsif ( $arg{'all'} =~ /^hp$/i ) {
			$filters = _all__xml('hp','type','VE,HN,HNVZ,MISC,DM,PLSRV,PLCLT,PLDM,VEPLHN,OTFI');
		} else {
			help(); exit;
		}
	} elsif ($arg{file} ne '') {
		$filters = load_file($arg{file});
	} else {
		while ( <STDIN> ) {
			$filters .= $_;
		}
	}

	if ( $filters eq '' ) {
		die 'File '.$arg{'file'}.' is empty';
	}

	$filters = XMLin($filters, forcearray => [qw(filter)]);
	$filters = sw_purify_data( $filters );

	my $objects_name = $filters->{'objects_name'};
	$objects_name =~ s/^\s+//g;
	$objects_name =~ s/\s+$//g;

	my $filter = $filters->{'filter'};

	my $expdata = {};

	print "Collecting data....\n";

	if ( $objects_name eq 'account' ) {
		_handle_accounts($expdata, $filter);
	} elsif ( $objects_name eq 'document' ) {
		_handle_documents($expdata,$filter);
	} elsif ( $objects_name eq 'subscription' ) {
		_handle_subscr($expdata,$filter);
	} elsif ( $objects_name eq 'hp' ) {
		_handle_hp($expdata,$filter);
	} else {
		help(); exit;
	}
	print "\t\t\tOK\n";

	print "Exporting data....\n";

	my $file = make_export(
		'namespace' => $expdata->{'namespace'},
		'filters'   => $expdata->{'filters'},
		'hp_type'   => $expdata->{'hp_type'},
	);
	print "\t\t\tOK\n\n";

	system 'mv '.$file.' ./';

	$file =~ m|^\/tmp\/(.*?)$|;
	print 'Your file: '.$1."\n\n";
}

sub make_export {
	my %h = (@_);

	## SOAP request timeout is set to 3600s to make it possible to export data from large OBAS installations
	my $client = SOAP::Lite->proxy('http://127.0.0.1:8080/hspc/xml-api', timeout => 3600)->on_fault(sub {
			print 'SOAP Fault: ' . $_[1]->faultcode . ' - ' . $_[1]->faultstring . "\n";
			exit(1);
		}
	);
	return $client->ns('HSPC/MT/IEM/Export')->export_data( %h )->result->{filename};
}

## ################################################

sub _handle_accounts
{
	my $expdata = shift;
	my $filters = shift;

	$expdata->{'filters'} = undef;
	$expdata->{'namespace'} = 'accounts';

	foreach my $filter ( @$filters ) {

		if ($filter->{'property_name'} eq 'type' ) {

			## FILTER by accounts type aka FIRST FILTER !!!
			$expdata->{'filters'}->{'f1'} = {
				objects_name	=> 'account',
				property_name	=> 'type',
				type		=> 'options',
			};
			my @in;
			my %_in;
			if ( $filter->{'where'}->{'in'} ) {
				foreach ( split /,/, $filter->{'where'}->{'in'}  ) {
					next unless /customer|reseller|res_customers/;
					$_in{$_} = $_;
				}
			}

			push @in, &SW_CUSTOMER	if $_in{'customer'};
			push @in, &SW_RESELLER	if $_in{'reseller'};
			if ( $_in{'res_customers'} ) {
				push @in, ( &SW_CUSTOMER, &SW_RESELLER );
				$expdata->{'filters'}->{'f1'}->{'sql'} = ' vendor != 1 ';
			}

			if ( @in ) {
				$expdata->{'filters'}->{'f1'}->{'where'} = { in => join(',',@in) };
			}

			## FILTER by vendor aka SECOND FILTER !!!!!
			unless ( $_in{'res_customers'} ) {
				$expdata->{'filters'}->{'f2'} = {
					objects_name => 'account',
					property_name => 'vendor_id',
					type => 'options',
					where => { in => &SW_HSP_ID },
				};
			};
		} elsif ( $filter->{'property_name'} eq 'id' ) {
			if ( $filter->{'where'}->{'in'} ne '' ) {
				$filter->{'where'}->{'in'} =~ s/[^0-9,-]//g;
				my @ins = split /,/, $filter->{'where'}->{'in'};
				foreach my $k ( 0..$#ins ) {
					if ( $ins[$k] =~ /(\d+)-(\d+)/ ) {
						my $m1 = $1;
						my $m2 = $2;
						my $j = '';
						for my $i ( $m1..$m2 ) {
							$j .= $i;
							$j .= ',' if $i<$m2
						}
						$ins[$k] = $j;
					}
				}
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'account',
					property_name => 'id',
					type => 'options',
					where => { in => join (',', @ins) },
				};
			} elsif ( $filter->{'where'}->{'start'}
					|| $filter->{'where'}->{'end'} )
			{
				my $res = $filter->{'where'}->{'end'} - $filter->{'where'}->{'start'};
				if ( $res < 0 ) {
					die 'Start ID is bigger then End ID';
				}
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'account',
					property_name => 'id',
					type => 'id',
					where => {
						start	=> $filter->{'where'}->{'start'},
						end 	=> $filter->{'where'}->{'end'},
					},
				};
			}
		} elsif ( $filter->{'property_name'} eq 'date' ) {
			if ( $filter->{'where'}->{'start'}
				|| $filter->{'where'}->{'end'} )
			{
				unless ( is_date( $filter->{'where'}->{'start'} ) ) {
				    die 'Start date format is wrong';
				}
				unless ( is_date( $filter->{'where'}->{'end'} ) ) {
				    die 'End date format is wrong';
				}
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'account',
					property_name => 'enroll_date',
					type => 'date',
					where => {
						start	=> $filter->{'where'}->{'start'},
						end 	=> $filter->{'where'}->{'end'},
					},
				};
			}
		}
	}
}


sub _handle_documents
{
	my $expdata = shift;
	my $filters = shift;

	$expdata->{'filters'} = undef;
	$expdata->{'namespace'} = 'documents';

	foreach my $filter ( @$filters ) {

		if ($filter->{'property_name'} eq 'type' ) {

			## FILTER by documents type aka FIRST FILTER !!!
			$expdata->{'filters'}->{'f1'} = {
				objects_name => 'ar_doc',
				property_name => 'doc_type',
				type => 'options',
			};

			my @in;
			my %_in;
			if ( $filter->{'where'}->{'in'} ) {
				$filter->{'where'}->{'in'} =~ s/\s+//g;
				foreach ( split /,/, $filter->{'where'}->{'in'}  ) {
					next unless /IN|CI|PO|PF|CA|DA/;
					$_in{$_} = $_;
				}
			}
			push @in, 'IN'	if $_in{'IN'};
			push @in, 'CI'	if $_in{'CI'};
			push @in, 'PO'	if $_in{'PO'};
			push @in, 'PF'	if $_in{'PF'};
			push @in, 'CA'	if $_in{'CA'};
			push @in, 'DA'	if $_in{'DA'};

			if ( @in ) {
				$expdata->{'filters'}->{'f1'}->{'where'} = { in => join(',', @in) };
			}
		} elsif ( $filter->{'property_name'} eq 'id' ) {
			if ( $filter->{'where'}->{'in'} ne '' ) {
				$filter->{'where'}->{'in'} =~ s/[^0-9,-]//g;
				my @ins = split /,/, $filter->{'where'}->{'in'};
				foreach my $k ( 0..$#ins ) {
					if ( $ins[$k] =~ /(\d+)-(\d+)/ ) {
						my $m1 = $1;
						my $m2 = $2;
						my $j = '';
						for my $i ( $m1..$m2 ) {
							$j .= $i;
							$j .= ',' if $i<$m2
						}
						$ins[$k] = $j;
					}
				}
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'ar_doc',
					property_name => 'id',
					type => 'options',
					where => { in => join (',', @ins) },
				};
			}
			elsif ($filter->{'where'}->{'start'} || $filter->{'where'}->{'end'}) {
				my $res = $filter->{'where'}->{'end'} - $filter->{'where'}->{'start'};
				if ( $res < 0 ) {
					die 'Start ID is bigger then End ID';
				}
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'ar_doc',
					property_name => 'id',
					type => 'options',
					where => {
						start	=> $filter->{'where'}->{'start'},
						end		=> $filter->{'where'}->{'end'},
					},
				};
			}
		} elsif ( $filter->{'property_name'} eq 'date' ) {
			if ($filter->{'where'}->{'start'} || $filter->{'where'}->{'end'}) {
				die 'Start date format is wrong'
					unless is_date($filter->{'where'}->{'start'});
			    die 'End date format is wrong'
					unless is_date($filter->{'where'}->{'end'});
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'ar_doc',
					property_name => 'doc_date',
					type => 'options',
					where => {
						start	=> $filter->{'where'}->{'start'},
						end		=> $filter->{'where'}->{'end'},
					},
				};
			}
		}
	}
}

sub _handle_subscr
{
	my $expdata = shift;
	my $filters = shift;

	$expdata->{'filters'} = undef;
	$expdata->{'namespace'} = 'subscription';

	foreach my $filter ( @$filters ) {

		if ($filter->{'property_name'} eq 'type' ) {

			## FILTER by SUBSCR type aka FIRST FILTER !!!
			$expdata->{'filters'}->{'f1'} = {
				objects_name => 'subscr_base',
				property_name => 'plan_type',
				type => 'options',
			};

			my @in;
			my %_in;
			if ($filter->{'where'}->{'in'}) {
				foreach (split /,/, $filter->{'where'}->{'in'}) {
					next unless /VE|HN|HNVZ|MISC|DM|PLSRV|PLDM|PLCLT|VEPLHN,OTFI/;
					$_in{$_} = $_;
				}
			}
			push @in, &BM_HP_TYPE_VE_GENERIC	if $_in{ 'VE' };
			push @in, &BM_HP_TYPE_HN_GENERIC	if $_in{ 'HN' };
			push @in, &BM_HP_TYPE_HN_VIRTUOZZO	if $_in{ 'HNVZ' };
			push @in, &BM_HP_TYPE_MISC		if $_in{ 'MISC' };
			push @in, &BM_HP_TYPE_DM_GENERIC	if $_in{ 'DM' };
			push @in, &BM_HP_TYPE_PLESK_SERVER	if $_in{ 'PLSRV' };
			push @in, &BM_HP_TYPE_PLESK_DOMAIN	if $_in{ 'PLDM' };
			push @in, &BM_HP_TYPE_PLESK_CLIENT	if $_in{ 'PLCLT' };
			push @in, &BM_HP_TYPE_VE_PLESK_HN	if $_in{ 'VEPLHN' };
			push @in, &BM_HP_TYPE_ONETIMEITEM if $_in{ 'OTFI' };

			if ( @in ) {
				$expdata->{'filters'}->{'f1'}->{'where'} = { in => join(',', @in) };
				$expdata->{'hp_type'} = \@in;
			}
		} elsif ( $filter->{'property_name'} eq 'id' ) {
			if ( $filter->{'where'}->{'in'} ne '' ) {
				$filter->{'where'}->{'in'} =~ s/[^0-9,-]//g;
				my @ins = split /,/, $filter->{'where'}->{'in'};
				foreach my $k ( 0..$#ins ) {
					if ( $ins[$k] =~ /(\d+)-(\d+)/ ) {
						my $m1 = $1;
						my $m2 = $2;
						my $j = '';
						for my $i ( $m1..$m2 ) {
							$j .= $i;
							$j .= ',' if $i<$m2
						}
						$ins[$k] = $j;
					}
				}
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'subscr_base',
					property_name => 'id',
					type => 'options',
					where => { in => join (',', @ins) },
				};
			}
			elsif ($filter->{'where'}->{'start'} || $filter->{'where'}->{'end'}) {
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'subscr_base',
					property_name => 'id',
					type => 'options',
					where => {
						start	=> $filter->{'where'}->{'start'},
						end		=> $filter->{'where'}->{'end'},
					},
				};
			}
		} elsif ( $filter->{'property_name'} eq 'start_date' ) {
			if ($filter->{'where'}->{'start'} || $filter->{'where'}->{'end'}) {
			    die 'Start date format is wrong'
					unless is_date($filter->{'where'}->{'start'});
			    die 'End date format is wrong'
					unless is_date($filter->{'where'}->{'end'});
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'subscr_base',
					property_name => 'start_date',
					type => 'options',
					where => {
						start	=> $filter->{'where'}->{'start'},
						end		=> $filter->{'where'}->{'end'},
					},
				};
			}
		} elsif ( $filter->{'property_name'} eq 'end_date' ) {
			if ($filter->{'where'}->{'start'} || $filter->{'where'}->{'end'}) {
			    die 'Start date format is wrong'
					unless is_date($filter->{'where'}->{'start'});
			    die 'End date format is wrong'
					unless is_date($filter->{'where'}->{'end'});
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'subscr_base',
					property_name => 'end_date',
					type => 'options',
					where => {
						start	=> $filter->{'where'}->{'start'},
						end		=> $filter->{'where'}->{'end'},
					},
				};
			}
		}
	}
}


sub _handle_hp
{
	my $expdata = shift;
	my $filters = shift;

	$expdata->{'filters'} = undef;
	$expdata->{'namespace'} = 'hp';

	foreach my $filter ( @$filters ) {

		if ($filter->{'property_name'} eq 'type' ) {

			## FILTER by HP type aka FIRST FILTER !!!
			$expdata->{'filters'}->{'f1'} = {
				objects_name => 'bm_hp',
				property_name => 'type',
				type => 'options',
			};

			my @in;
			my %_in;
			if ( $filter->{'where'}->{'in'} ) {
				foreach ( split /,/, $filter->{'where'}->{'in'}  ) {
					next unless /VE|HN|HNVZ|MISC|DM|PLSRV|PLDM|PLCLT|VEPLHN|OTFI/;
					$_in{$_} = $_;
				}
			}
			push @in, &BM_HP_TYPE_VE_GENERIC	if $_in{ 'VE' };
			push @in, &BM_HP_TYPE_HN_GENERIC	if $_in{ 'HN' };
			push @in, &BM_HP_TYPE_HN_VIRTUOZZO	if $_in{ 'HNVZ' };
			push @in, &BM_HP_TYPE_MISC		if $_in{ 'MISC' };
			push @in, &BM_HP_TYPE_DM_GENERIC	if $_in{ 'DM' };
			push @in, &BM_HP_TYPE_PLESK_SERVER	if $_in{ 'PLSRV' };
			push @in, &BM_HP_TYPE_PLESK_DOMAIN	if $_in{ 'PLDM' };
			push @in, &BM_HP_TYPE_PLESK_CLIENT	if $_in{ 'PLCLT' };
			push @in, &BM_HP_TYPE_VE_PLESK_HN	if $_in{ 'VEPLHN' };
			push @in, &BM_HP_TYPE_ONETIMEITEM if $_in{ 'OTFI' };

			if ( @in ) {
				$expdata->{'filters'}->{'f1'}->{'where'} = { in => join(',', @in) };
				$expdata->{'hp_type'} = \@in;
			}
		} elsif ($filter->{'property_name'} eq 'id' ) {
			if ($filter->{'where'}->{'in'} ne '') {
				$filter->{'where'}->{'in'} =~ s/[^0-9,-]//g;
				my @ins = split /,/, $filter->{'where'}->{'in'};
				foreach my $k ( 0..$#ins ) {
					if ( $ins[$k] =~ /(\d+)-(\d+)/ ) {
						my $m1 = $1;
						my $m2 = $2;
						my $j = '';
						for my $i ( $m1..$m2 ) {
							$j .= $i;
							$j .= ',' if $i<$m2
						}
						$ins[$k] = $j;
					}
				}
				$expdata->{'filters'}->{'f3'} = {
					objects_name => 'bm_hp',
					property_name => 'id',
					type => 'options',
					where => { in => join(',', @ins ) },
				};
			}
		}
	}
}

# > file_path
# < data
sub load_file
{
	my $file_path = shift;
	open SRC, "<$file_path"
		or print "Cannot open file $file_path: $!\n" and exit(1);

	local $/ = undef;
	my $c = <SRC>;
	close(SRC);

	return $c;
}

sub is_date
{
	my $str = shift;
	if ( $str =~ /\d{4}-\d\d-\d\d \d\d:\d\d:\d\d/ ) {
		return 1;
	}
	return 0;
}

sub help
{
	print <<END;
Usage:	hspc-export.pl -f filters.xml
	hspc-export.pl -all <param>

Description:
	Export data from Parallels Business Automation - Standard database to XML.

Options:
    -h		help (this text)
    -f		file with filters

	-all	to export all objects of type <param> (accounts|documents|subscriptions|hp)


If filters.xml not defined filters get from STDIN

Filters:

<?xml version="1.0" encoding="UTF-8"?>
<filters xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      xsi:noNamespaceSchemaLocation="filters.xsd">
	<objects_name> OBJECT </objects_name>
	<filter>
		<property_name> type </property_name>
		<where>
			<!-- WHERE -->
		</where>
	</filter>
	<filter>
		<property_name> PROPERTY </property_name>
		<where>
			<!-- WHERE -->
		</where>
	</filter>
	...
</filters>

NOTE: The XML file describing filters must contain at least one
      <filter> nodes describe the type of field to be filtered,
      i.e. contain the name of this field. Other filters describe
      the filtering coditions.
      See Integration guide for details.


where tags:
	objects_name	is main filtered object:
				account, document, subscription, hp
	property_name	is property name:
			  id, type, date,
			  start_date, end_date (only for subscription)

inside node <where> can be:
	<in>	digits for id  (ex. 123 or 1,45,110-200,643 )
		customer,reseller,res_customers  for account type
		active,on_hold  for account status
	or
	<start>		ex. <start>12</start> (for id)
	<end>		ex. <end>2005-01-01 01:02:03</end> (for date)

Date format is:
	YYYY-MM-DD hh:mm:ss

END
}
