# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils.registry import Registry
from migrator.products.plesk.checks.lib.communication_ip import check_communication_ip
from migrator.settings import PANEL_TYPE_PLESK

logger = logging.getLogger(__name__)


class Check(object):
    def run(self):
        global_context = Registry.get_instance().get_context()
        source_system_reader = PlatformReaderDB()
        plesks_to_check = source_system_reader.get_migration_required_source_hosts(PANEL_TYPE_PLESK)

        failed_hosts = []
        for plesk_to_check in plesks_to_check:
            communication_ip = check_communication_ip(global_context, plesk_to_check)
            if not communication_ip:
                failed_hosts.append(plesk_to_check.name)

        source_system_reader.session.commit()

        if failed_hosts:
            return CheckAbstract.STATUS_ERROR, _T('CHECK_COMMUNICATION_IP_ERROR').format(hostname=failed_hosts)
        else:
            return CheckAbstract.STATUS_SUCCESS, _T('CHECK_COMMUNICATION_IP_SUCCESS').format(
                hostname=[x.name for x in plesks_to_check]
            )
