# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.settings import HTML_EOL, PANEL_TYPE_PLESK, PANEL_TYPE_BIND_OBAS
from migrator.core import InternalError
from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils.filetypes.config_utils import ConfigHostSection
from migrator.core.utils.registry import Registry
from migrator.products.plesk.checks.lib.hosts_authentication import get_plesk_host_authentication_status

logger = logging.getLogger(__name__)


class Check(object):
    def __init__(self):
        self.hosts_with_wrong_credentials = []
        self.hosts_with_empty_credentials = []
        self.hosts_with_unknown_error = []

    def run(self):
        global_context = Registry.get_instance().get_context()
        obas_mn_config_section = ConfigHostSection(global_context.config, 'obas')
        hosts_to_check = PlatformReaderDB().get_migration_required_source_hosts(PANEL_TYPE_PLESK) + \
                         PlatformReaderDB().get_migration_required_source_hosts(PANEL_TYPE_BIND_OBAS)
        for host_to_check in hosts_to_check:
            self._save_host_authentication_check_result(host_to_check.ip)

        if not self.hosts_with_wrong_credentials and not self.hosts_with_empty_credentials and not self.hosts_with_unknown_error:
            return CheckAbstract.STATUS_SUCCESS, _T('CHECK_OBAS_PANEL_AUTHENTICATION_SUCCESS').format(
                hostname=obas_mn_config_section.get('ip')
            )

        error_message_parts = []
        if self.hosts_with_wrong_credentials:
            error_message_parts.append(
                _T('CHECK_OBAS_PANEL_AUTHENTICATION_ERRORS_WRONG_PARAMS').format(
                    hosts=', '.join(self.hosts_with_wrong_credentials)
                )
            )

        if self.hosts_with_empty_credentials:
            error_message_parts.append(
                _T('CHECK_OBAS_PANEL_AUTHENTICATION_ERRORS_EMPTY_PARAMS').format(
                    hosts=', '.join(self.hosts_with_empty_credentials)
                )
            )

        if self.hosts_with_unknown_error:
            error_message_parts.append(
                _T('CHECK_OBAS_PANEL_AUTHENTICATION_ERRORS_UNEXPECTED_RESULT').format(
                    hosts=', '.join(self.hosts_with_unknown_error),
                )
            )

        return CheckAbstract.STATUS_ERROR, _T('CHECK_OBAS_PANEL_AUTHENTICATION_ERRORS_SUMMARY').format(
            hostname=obas_mn_config_section.get('ip'),
            message_content=HTML_EOL + HTML_EOL.join(error_message_parts)
        )

    def _save_host_authentication_check_result(self, host_section_name):
        # At the moment, there is no difference in check logic between OBAS MN and the host
        global_context = Registry.get_instance().get_context()
        if not global_context.config.has_section(host_section_name):
            self.hosts_with_empty_credentials.append(host_section_name)
            return

        plesk_config_section = ConfigHostSection(global_context.config, host_section_name)
        host_authentication_status = get_plesk_host_authentication_status(plesk_config_section)
        if host_authentication_status == 1:
            self.hosts_with_wrong_credentials.append(host_section_name)

        if host_authentication_status == 2:
            self.hosts_with_empty_credentials.append(host_section_name)

        if host_authentication_status == 3:
            self.hosts_with_unknown_error.append(host_section_name)

        if host_authentication_status != 0:
            return InternalError("Unable to check Server's authentication: an internal error occured. Contact technical support to resolve the issue.")
