# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.checks.base import CheckAbstract
from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.language import _T
from migrator.core.utils.filetypes.config_utils import ConfigHostSection
from migrator.core.utils.registry import Registry
from migrator.products.plesk.checks.lib.hosts_network_mode import \
    is_host_routed_mode, IS_HOST_ROUTED_RESULT_TRUE, IS_HOST_ROUTED_RESULT_FALSE, IS_HOST_ROUTED_RESULT_UNKNOWN, \
    DOC_LINK_ABOUT_NET_CONFIGURATIONS
from migrator.settings import PANEL_TYPE_PLESK, PANEL_TYPE_OBAS

logger = logging.getLogger(__name__)


class Check(object):
    def run(self):
        global_context = Registry.get_instance().get_context()
        unchecked_plesks = list()
        obas_config_section = ConfigHostSection(global_context.config, PANEL_TYPE_OBAS)
        obas_ip = obas_config_section.get('ip')
        try:
            plesks_to_check = PlatformReaderDB().get_migration_required_source_hosts(PANEL_TYPE_PLESK)
            for plesk_to_check in plesks_to_check:
                plesk_ip = plesk_to_check.ip
                config_section = ConfigHostSection(global_context.config, plesk_ip)
                host_network_mode = is_host_routed_mode(config_section)
                if host_network_mode == IS_HOST_ROUTED_RESULT_TRUE:
                    return CheckAbstract.STATUS_ERROR, _T(
                        'CHECK_HOSTS_NETWORK_MODE_PLESK_ERROR_DETECTED_HOST_ROUTED'
                    ).format(hostname=plesk_ip, link=DOC_LINK_ABOUT_NET_CONFIGURATIONS)

                if host_network_mode == IS_HOST_ROUTED_RESULT_FALSE:
                    logger.debug(_T('CHECK_HOSTS_NETWORK_MODE_DEBUG_PLESK_CHECKED').format(hostname=plesk_ip))

                if host_network_mode == IS_HOST_ROUTED_RESULT_UNKNOWN:
                    unchecked_plesks.append(plesk_ip)

            if unchecked_plesks:
                return CheckAbstract.STATUS_ERROR, _T('CHECK_HOSTS_NETWORK_MODE_OBAS_ERROR_UNEXPECTED').format(
                    hostname=obas_ip,
                    hosts=','.join(unchecked_plesks),
                )

            return CheckAbstract.STATUS_SUCCESS, _T('CHECK_HOSTS_NETWORK_MODE_SUCCESS').format(hostname=obas_ip)
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return CheckAbstract.STATUS_ERROR, _T('CHECK_HOSTS_NETWORK_MODE_ERROR_UNEXPECTED').format(
                hostname=obas_ip,
            )
