# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.data.model.subscription import has_any_of_service_types
from migrator.core.data.format.mail_hosting_shm import ShmMailHostingReader
from migrator.core.data.model.web_hosting_shm import SubscriptionModel
from migrator.products.obas.lib.export.base import ObasExporterBase
from migrator.products.plesk.export import get_plesk_exporter_class
from migrator.settings import TargetServiceType, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, SourceHostingClass


class ObasMailExporter(ObasExporterBase, ShmMailHostingReader):
    def get_subscription(self, subscription_id):
        return SubscriptionModel(
            external_id=subscription_id,
            webspaces=self.get_webspaces_list(subscription_id)
        )

    def get_webspaces_list(self, subscription_id):
        subscription = self._db.get_subscription(subscription_id)
        if not has_any_of_service_types(subscription, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.SHARED]):
            return []

        hw_id = subscription.services[0].source_host_external_id
        self._init_plesk_exporter(hw_id, get_plesk_exporter_class, TargetServiceType.MAIL)
        return self._get_webspaces_list(hw_id, subscription)

    def _get_webspaces_list_for_plesk_subscription(self, hw_id, plesk_subscription_id, obas_subscription_id):
        return self._exporters[hw_id].get_webspaces_list(
            plesk_subscription_id, subscription_external_id=obas_subscription_id
        )
