# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.data.model.subscription import has_any_of_service_types
from migrator.core.data.format.web_hosting_shm import ShmWebHostingReader
from migrator.core.data.model.web_hosting_shm import SubscriptionModel
from migrator.products.obas.lib.export.base import ObasExporterBase
from migrator.products.plesk.export import get_plesk_exporter_class
from migrator.settings import TargetServiceType, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP, SourceHostingClass


class ObasHostingExporter(ObasExporterBase, ShmWebHostingReader):
    def get_subscription(self, subscription_id):
        return SubscriptionModel(
            external_id=subscription_id,
            webspaces=self.get_webspaces_list(subscription_id)
        )

    @staticmethod
    def _fix_database_server_id(array, attr, hw_id):
        for x in array:
            setattr(x, attr, hw_id + '_' + getattr(x, attr))

    def get_webspaces_list(self, subscription_id):
        subscription = self._db.get_subscription(subscription_id)
        if not has_any_of_service_types(subscription, SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.SHARED]):
            return []

        hw_id = subscription.services[0].source_host_external_id
        self._init_plesk_exporter(hw_id, get_plesk_exporter_class, TargetServiceType.SHM)

        webspaces_list = self._get_webspaces_list(hw_id, subscription)
        # The database server ID may coincide on different Plesk nodes. Therefore, it is necessary to make it unique.
        for webspace in webspaces_list:
            self._fix_database_server_id(webspace.databases, 'database_server_id', hw_id)
            self._fix_database_server_id(webspace.database_users, 'database_server_id', hw_id)

        return webspaces_list

    def get_database_servers(self):
        database_servers = []
        for hw_id, source_plesk in self._source_plesks.iteritems():
            if hw_id not in self._exporters:
                self._source_plesks[hw_id] = source_plesk
                self._exporters[hw_id] = get_plesk_exporter_class(TargetServiceType.SHM, source_plesk)(source_plesk)

            # The database server ID may coincide on different Plesk nodes.
            # Therefore, it is necessary to make it unique.
            _database_servers = self._exporters[hw_id].get_database_servers()
            self._fix_database_server_id(_database_servers, 'id', hw_id)

            database_servers.extend(_database_servers)

        return database_servers

    def _get_webspaces_list_for_plesk_subscription(self, hw_id, plesk_subscription_id, obas_subscription_id):
        return self._exporters[hw_id].get_webspaces_list(
            plesk_subscription_id, subscription_external_id=str(obas_subscription_id)
        )
