# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.checks.base import CheckAbstract
from migrator.core.language import _T
from migrator.core.utils.networking.ip_port import \
    get_formatted_host_with_unavailable_ports, get_unavailable_ports_for_host, PORTS_AVAILABILITY_STATUS_SUCCESS
from migrator.core.utils.registry import Registry
from migrator.settings import SUPPORTED_OS_TYPES, OS_LINUX, SSH_DEFAULT_PORT, SMB_DEFAULT_PORT

logger = logging.getLogger(__name__)

PLESK_COMMUNICATION_PORTS_LINUX = [SSH_DEFAULT_PORT]
PLESK_COMMUNICATION_PORTS_WINDOWS = [SMB_DEFAULT_PORT]


class Check(object):
    def run(self):
        global_context = Registry.get_instance().get_context()
        plesk_os = global_context.config.get('plesk', 'os').lower()
        plesk_ip = global_context.config.get('plesk', 'ip')
        try:
            if plesk_os not in SUPPORTED_OS_TYPES:
                raise NotImplemented()

            ports_to_check = PLESK_COMMUNICATION_PORTS_LINUX if plesk_os == OS_LINUX else PLESK_COMMUNICATION_PORTS_WINDOWS
            test_results = get_unavailable_ports_for_host(plesk_ip, ports_to_check)
            if test_results['status'] == PORTS_AVAILABILITY_STATUS_SUCCESS:
                status = CheckAbstract.STATUS_SUCCESS
                message = _T('CHECK_PLESK_PANEL_AVAILABILITY_SUCCESS').format(hostname=plesk_ip)
            else:
                status = CheckAbstract.STATUS_ERROR
                message = _T('CHECK_PLESK_HOSTS_AVAILABILITY_ERRORS').format(
                    hosts=get_formatted_host_with_unavailable_ports(plesk_ip, test_results['unavailable_ports'])
                )

            return status, message
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return CheckAbstract.STATUS_ERROR, _T("CHECK_PLESK_PANEL_AVAILABILITY_UNEXPECTED_RESULT").format(
                hostname=plesk_ip
            )
