# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging
import re

from migrator.products.plesk.lib.api.xmlrpc import operator as plesk_ops

logger = logging.getLogger(__name__)


def check_communication_ip(global_context, plesk_to_check):
    _prog_ipv4={
        False: { #is_windows
            'cmd': ('ip', ['addr']),
            'regexp': re.compile('^\s*inet\s+(\d{1,3}\.\d{1,3}.\d{1,3}.\d{1,3}).*$')
        },
        True:  {  # is_windows
            'cmd': ('netsh', ['interface', 'ipv4', 'show', 'ipaddresses']),
            'regexp': re.compile('^.*(\d{1,3}\.\d{1,3}.\d{1,3}.\d{1,3}).*$')
        }
    }

    source_server = global_context.conn.get_source_server_object(plesk_to_check.ip)
    response = source_server.plesk_api().send(plesk_ops.IpOperator.Get())
    plesk_ips = set([x.ip_address for x in response.data])
    logger.debug("Plesk's {hostname} ip list: {ips}".format(hostname=plesk_to_check.name, ips=list(plesk_ips)))

    host_ips = set()
    with source_server.runner() as runner:
        settings = _prog_ipv4[source_server.is_windows()]
        cmd = settings['cmd']
        regexp = settings['regexp']
        res = runner.run(*cmd, log_output=True)
        for s in res.split('\n'):
            m = regexp.match(s)
            if m:
                host_ips.add(m.group(1))
    host_ips.discard('127.0.0.1')

    logger.debug("System's {hostname} ip list: {ips}".format(hostname=plesk_to_check.name, ips=list(host_ips)))

    free_ips = host_ips - plesk_ips

    if plesk_to_check.communication_ip and plesk_to_check.communication_ip not in free_ips:
        plesk_to_check.communication_ip = None
        logger.warn("Communication ip {ip} was discarded as used in Plesk {hostname}".format(
            ip=plesk_to_check.communication_ip, hostname=plesk_to_check.name
        ))

    if not plesk_to_check.communication_ip and free_ips:
        plesk_to_check.communication_ip = list(free_ips)[0]
        logger.debug("Free ip {ip} was selected as communication ip for Plesk {hostname}".format(
            ip=plesk_to_check.communication_ip, hostname=plesk_to_check.name
        ))

    return plesk_to_check.communication_ip
