# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import MigrationError
from migrator.core.language import _T
from migrator.core.utils.networking.ip_port import COMMUNICATION_ERROR_CODE_WRONG_AUTHENTICATION
from migrator.core.utils.networking.runner import get_runner
from migrator.core.utils.networking.run_command import ConnectionCheckError

logger = logging.getLogger(__name__)


def get_plesk_host_authentication_status(config_section):
    """
    :param migrator.core.utils.filetypes.config_utils.ConfigHostSection config_section: Plesk server config section
    :return: int | 0 - ok | 1 - wrong credentials | 2 - empty credentials | 3 - unknown error |
    """
    try:
        if config_section.are_connection_credentials_empty():
            # Windows + Linux
            return 2

        runner = get_runner(config_section)
        runner.check()
        return 0
    except ConnectionCheckError:
        # Windows
        return 1
    except MigrationError as e:
        if e.code == COMMUNICATION_ERROR_CODE_WRONG_AUTHENTICATION:
            # Linux
            return 1
        else:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return 3
    except Exception:
        logger.error(_T("LOG_EXCEPTION"), exc_info=True)
        return 3
