# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.language import _T
from migrator.core.utils.networking.runner import get_runner
from migrator.settings import OS_LINUX, OS_WINDOWS

logger = logging.getLogger(__name__)

IS_HOST_ROUTED_RESULT_TRUE = 0
IS_HOST_ROUTED_RESULT_FALSE = 1
IS_HOST_ROUTED_RESULT_UNKNOWN = 2

HOST_ROUTED_INTERFACES_NAMES = ['lo', 'venet0']
DOC_LINK_ABOUT_NET_CONFIGURATIONS = '<a href="http://docs.virtuozzo.com/virtuozzo_7_users_guide/managing-network/configuring-virtual-machines-and-containers-in-bridged-mode.html#creating-virtual-networks" target="_blank">this guide</a>'


def is_host_routed_mode(config_section):
    """
    :param migrator.core.utils.filetypes.config_utils.ConfigHostSection config_section: Host config section
    :return: int
    """
    is_host_routed = True
    found_interfaces = list()
    try:
        host_os = config_section.get('os').lower()
        host_ip = config_section.get('ip')
        # Linux
        if host_os == OS_LINUX:
            runner = get_runner(config_section)
            ip_links_info = runner.run('ip', ['-oneline', 'link'])
            active_interfaces_info = ip_links_info.splitlines()
            for active_interface_info in active_interfaces_info:
                interface_name = active_interface_info.split()[1].replace(u':', u'')
                found_interfaces.append(interface_name)
                if interface_name not in HOST_ROUTED_INTERFACES_NAMES:
                    is_host_routed = False

            logger.debug(_T('CHECK_HOSTS_NETWORK_MODE_DEBUG_PLESK_FOUND_INTERFACES').format(
                hostname=host_ip,
                interfaces=', '.join(found_interfaces)
            ))
            if is_host_routed and len(found_interfaces) == len(HOST_ROUTED_INTERFACES_NAMES):
                return IS_HOST_ROUTED_RESULT_TRUE
            else:
                return IS_HOST_ROUTED_RESULT_FALSE

        # Windows - not implemented
        if host_os == OS_WINDOWS:
            return IS_HOST_ROUTED_RESULT_FALSE

        raise NotImplemented()
    except Exception:
        logger.error(_T("LOG_EXCEPTION"), exc_info=True)
        return IS_HOST_ROUTED_RESULT_UNKNOWN
