# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.language import _T
from migrator.products.plesk.lib.api.xmlrpc.core import PleskError
from migrator.products.plesk.panel import PanelInfo as PleskPanelInfo

logger = logging.getLogger(__name__)

PLESK_LICENSE_STATUS_VALID = 0
PLESK_LICENSE_STATUS_INVALID = 1
PLESK_LICENSE_STATUS_UNEXPECTED = 2
PLESK_LICENSE_STATUS_FORBIDDEN = 3


def get_plesk_license_status(plesk_server_object):
    """
    :param migrator.odin_automation.platform.lib.connections.target_server.PleskTargetServer|migrator.products.plesk.source.plesk.server.PleskSourceServer plesk_server_object: The Plesk server object to check license
    :return: int
    """
    try:
        if PleskPanelInfo.is_plesk_server_license_valid(plesk_server_object):
            return PLESK_LICENSE_STATUS_VALID
        else:
            return PLESK_LICENSE_STATUS_INVALID
    except PleskError as e:
        if e.code == 1030:
            return PLESK_LICENSE_STATUS_INVALID
        if e.code == 1006:
            return PLESK_LICENSE_STATUS_FORBIDDEN
        else:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return PLESK_LICENSE_STATUS_UNEXPECTED
    except Exception as e:
        # special check for case when Plesk 'secret_key' tool report that license is invalid
        error_text = unicode(e)
        if 'license key' in error_text and 'invalid' in error_text:
            return PLESK_LICENSE_STATUS_INVALID

        logger.error(_T("LOG_EXCEPTION"), exc_info=True)
        return PLESK_LICENSE_STATUS_UNEXPECTED
