# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import InternalError
from migrator.core.checks.base import CheckAbstract
from migrator.core.language import _T
from migrator.core.utils.registry import Registry
from migrator.products.plesk.checks.lib.license import \
    get_plesk_license_status, PLESK_LICENSE_STATUS_VALID, PLESK_LICENSE_STATUS_INVALID, \
    PLESK_LICENSE_STATUS_UNEXPECTED, PLESK_LICENSE_STATUS_FORBIDDEN

from migrator.settings import PANEL_TYPE_PLESK

logger = logging.getLogger(__name__)


class Check(object):
    def run(self):
        global_context = Registry.get_instance().get_context()
        plesk_ip = global_context.config.get(PANEL_TYPE_PLESK, 'ip')
        try:
            plesk_license_status = get_plesk_license_status(global_context.conn.get_source_server_object(plesk_ip))
            if plesk_license_status == PLESK_LICENSE_STATUS_VALID:
                return CheckAbstract.STATUS_SUCCESS, _T('CHECK_PLESK_PANEL_LICENSE_SUCCESS').format(hostname=plesk_ip)
            elif plesk_license_status == PLESK_LICENSE_STATUS_INVALID:
                return CheckAbstract.STATUS_ERROR, _T('FAILED_TO_CHECK_PLESK_API_EXPIRED_LICENSE').format(
                    server=plesk_ip
                )
            elif plesk_license_status == PLESK_LICENSE_STATUS_UNEXPECTED:
                return CheckAbstract.STATUS_ERROR, _T('CHECK_PLESK_PANEL_LICENSE_UNEXPECTED_RESULT').format(
                    hostname=plesk_ip,
                )
            elif plesk_license_status == PLESK_LICENSE_STATUS_FORBIDDEN:
                return CheckAbstract.STATUS_ERROR, _T('CHECK_PLESK_PANEL_LICENSE_ACCESS_DENIED').format(server=plesk_ip)
            else:
                raise InternalError("Unable to check Plesk server's license: an internal error occured. Contact technical support to resolve the issue.")
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return CheckAbstract.STATUS_ERROR, _T("CHECK_PLESK_PANEL_LICENSE_UNEXPECTED_RESULT").format(
                hostname=plesk_ip,
            )
