# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.checks.base import CheckAbstract
from migrator.core.language import _T
from migrator.core.utils.registry import Registry
from migrator.products.plesk.lib.api.xmlrpc import operator as plesk_ops
from migrator.products.plesk.lib.api.xmlrpc.core import Failure

logger = logging.getLogger(__name__)


class Check(object):
    UNSUPPORTED_CLIENT_LOGIN = 'admin'
    UNSUPPORTED_SUBSCRIPTION_STATUS = 0
    SUBSCRIPTIONS_NAMES_IN_ERROR_MESSAGE_COUNT = 3

    def run(self):
        active_admin_subscriptions_names = []
        global_context = Registry.get_instance().get_context()
        plesk_ip = global_context.config.get('plesk', 'ip')
        try:
            source_server = global_context.conn.get_panel_server(global_context, 'plesk')
            result = source_server.plesk_api().send(
                plesk_ops.SubscriptionOperator.Get(
                    filter=plesk_ops.SubscriptionOperator.FilterByOwnerLogin([self.UNSUPPORTED_CLIENT_LOGIN]),
                    dataset=[plesk_ops.SubscriptionOperator.Dataset.GEN_INFO]
                )
            )

            if not result:
                return CheckAbstract.STATUS_SUCCESS, _T('CHECK_SHM_USERS_SUCCESS')

            if isinstance(result[0], Failure):
                return CheckAbstract.STATUS_ERROR, _T("CHECK_SHM_USERS_ERROR_UNEXPECTED_API_RESPONSE").format(
                    hostname=plesk_ip,
                )

            for admin_subscription in result:
                if admin_subscription.data[1].gen_info.status == self.UNSUPPORTED_SUBSCRIPTION_STATUS:
                    active_admin_subscriptions_names.append(admin_subscription.data[1].gen_info.name)

            if not active_admin_subscriptions_names:
                return CheckAbstract.STATUS_SUCCESS, _T('CHECK_SHM_USERS_SUCCESS')

            admin_subscriptions_counter = len(active_admin_subscriptions_names)
            if admin_subscriptions_counter <= self.SUBSCRIPTIONS_NAMES_IN_ERROR_MESSAGE_COUNT:
                subscriptions_names = ', '.join(active_admin_subscriptions_names)
            else:
                subscriptions_names = _T('CHECK_SHM_USERS_ERROR_MANY_SUBSCRIPTIONS_FOUND').format(
                    subscriptions_names=', '.join(
                        active_admin_subscriptions_names[:self.SUBSCRIPTIONS_NAMES_IN_ERROR_MESSAGE_COUNT]
                    ),
                    subscriptions_amount=admin_subscriptions_counter - self.SUBSCRIPTIONS_NAMES_IN_ERROR_MESSAGE_COUNT
                )

            return CheckAbstract.STATUS_ERROR, _T('CHECK_SHM_USERS_ERROR_FOUND_ADMIN_SUBSCRIPTIONS').format(
                hostname=plesk_ip,
                subscriptions_names=subscriptions_names
            )
        except Exception:
            logger.error(_T("LOG_EXCEPTION"), exc_info=True)
            return CheckAbstract.STATUS_ERROR, _T("CHECK_SHM_USERS_UNEXPECTED").format(
                hostname=plesk_ip,
            )
