# (C)2017 Ingram Micro Inc. all Rights Reserved
import migrator.products.plesk

from migrator.core import MigrationError
from migrator.core.language import _T
from migrator.core.product.lib.agent import ProductAgentService, get_agent_file_path
from migrator.core.utils.loggers.log import trace

_servers_with_installed_agent = []


@trace
def install_agent_on_server(plesk_server):
    global _servers_with_installed_agent
    plesk_server_ip = plesk_server.ip()
    if plesk_server_ip in _servers_with_installed_agent:
        return

    try:
        agent = PleskMigrationAgent(plesk_server)
        if not agent.is_installed:
            agent.install()

        agent.start(True)
        if agent.status() != agent.STATUS_RUNNING:
            raise Exception(_T("MIGRATION_AGENT_STILL_NOT_RUNNING").format(host=plesk_server_ip))
    except Exception as e:
        raise MigrationError(e)

    _servers_with_installed_agent.append(plesk_server_ip)


class PleskMigrationAgent(ProductAgentService):
    def __init__(self, source_server):
        super(PleskMigrationAgent, self).__init__(source_server)
        self._agent_product_lib['src_dir'] = get_agent_file_path(migrator.products.plesk, '')
