# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from .. import core

AliasInfo = namedtuple('AliasInfo', ('id', 'name', 'site_id', 'seo_redirect'))


class AliasOperator(object):
    FilterAll = core.FilterAll
    FilterBySiteName = core.declare_filter('FilterBySiteName', 'site-name')
    FilterByName = core.declare_filter('FilterByName', 'name')
    FilterBySiteId = core.declare_filter('FilterBySiteId', 'site-id')
    FilterBySiteName = core.declare_filter('FilterBySiteName', 'site-name')

    class Get(core.operation_with_filter('Get')):
        operator_name = 'site-alias'
        operation_name = 'get'
        min_api_version = '1.6.3.0'
        max_api_version = None

        @classmethod
        def parse(cls, elem):
            return core.filter_owner_not_found_error(core.parse_result_set(elem, cls._parse_data, 'id'))

        @classmethod
        def _parse_data(cls, elem):
            id = int(elem.findtext('id'))
            site_id = int(elem.findtext('info/site-id'))
            name = elem.findtext('info/name')
            seo_redirect = elem.findtext('info/pref/seo-redirect')

            return id, AliasInfo(id=id, name=name, site_id=site_id, seo_redirect=seo_redirect)


class DomainAliasOperator(object):
    FilterAll = core.FilterAll
    FilterByName = core.declare_filter('FilterByName', 'name')
    FilterByDomainId = core.declare_filter('FilterByDomainId', 'domain_id')
    FilterByDomainName = core.declare_filter('FilterByDomainId', 'domain_name')

    class Get(core.operation_with_filter('Get')):
        operator_name = 'domain_alias'
        operation_name = 'get'
        min_api_version = '1.4.1.2'
        max_api_version = None

        @classmethod
        def parse(cls, elem):
            return core.parse_result_set(elem, cls._parse_data, 'id')

        @classmethod
        def _parse_data(cls, elem):
            id = int(elem.findtext('id'))
            site_id = int(elem.findtext('info/domain_id'))
            name = elem.findtext('info/name')
            seo_redirect = False

            return id, AliasInfo(id=id, name=name, site_id=site_id, seo_redirect=seo_redirect)
