# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from migrator.core.utils import if_not_none, sort_list_by_another
from migrator.core.utils.filetypes.xml import elem, text_elem, seq
from migrator.products.plesk.lib.api.xmlrpc.operator.customer import CustomerOperator
from .. import core


class ClientOperator(object):
    FilterAll = core.FilterAll
    FilterByLogin = core.declare_filter('FilterByLogin', 'login')
    FilterById = core.declare_filter('FilterById', 'id')
    FilterOwnerId = core.declare_filter('FilterOwnerId', 'owner-id')
    FilterOwnerLogin = core.declare_filter('FilterOwnerId', 'owner-login')

    ClientSetInfo = namedtuple('ClientSetInfo', ('gen_info',))
    ClientSetGenInfo = namedtuple('ClientSetGenInfo', ('passwd',))

    class Dataset(CustomerOperator.Dataset):
        pass

    class Get(CustomerOperator.Get):
        operator_name = 'client'
        operation_name = 'get'
        min_api_version = '1.4.2.0'
        max_api_version = None

        def data_xml(self):
            dataset_values = sort_list_by_another(self.dataset, ClientOperator.Dataset.values)
            return seq(elem('dataset', [elem(i) for i in dataset_values]))

    class Set(core.operation_with_filter('Get', ('values',))):
        operator_name = 'client'
        operation_name = 'set'
        min_api_version = '1.4.2.0'
        max_api_version = None

        def data_xml(self):
            return seq(
                elem('values', seq(
                    if_not_none(self.values.gen_info, lambda gi: elem('gen_info', [
                        if_not_none(gi.passwd, lambda p: text_elem('passwd', gi.passwd))
                    ]))
                ))
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'))
