# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from .. import core

from migrator.core.utils.filetypes.xml import seq, text_elem, elem


FtpuserGenInfo = namedtuple('FtpuserGenInfo', ('name', 'home', 'read', 'write', 'quota'))


class FtpuserOperator(object):
    FilterAll = core.FilterAll
    FilterByWebspaceId = core.declare_filter('FilterByWebspaceId', 'webspace-id')

    class Get(core.operation_with_filter('Get')):
        operator_name = 'ftp-user'
        operation_name = 'get'
        min_api_version = '1.4.2.0'
        max_api_version = None

        @classmethod
        def parse(cls, elem):
            results = core.filter_owner_not_found_error(core.parse_result_set(elem, cls._parse_data, 'name'))
            if len(results) == 1 and results[0].ok and results[0].data.name is None:
                results = []

            return results

        @classmethod
        def _parse_data(cls, elem):
            return FtpuserGenInfo(
                name=elem.findtext('name'),
                home=elem.findtext('home'),
                read=elem.findtext('permissions/read'),
                write=elem.findtext('permissions/write'),
                quota=elem.findtext('quota'),
            )

    class Add(core.operation('Add', ('webspace_id', 'name', 'password', 'home', 'quota'))):
        operator_name = 'ftp-user'
        operation_name = 'add'
        min_api_version = '1.5.2.0'
        max_api_version = None

        def inner_xml(self):
            return seq(
                text_elem('name', self.name),
                text_elem('password', self.password),
                text_elem('home', self.home),
                text_elem('quota', self.quota),
                text_elem('webspace-id', self.webspace_id),
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'), cls._parse_data)

        @classmethod
        def _parse_data(cls, elem):
            return int(elem.findtext('id'))

    class Set(core.operation('Set', ('name', 'read', 'write'))):
        operator_name = 'ftp-user'
        operation_name = 'set'
        min_api_version = '1.5.2.0'
        max_api_version = None

        def inner_xml(self):
            return seq(
                elem('filter', [text_elem('name', self.name)]),
                elem('values', [
                    elem('permissions', [
                        text_elem('read', self.read),
                        text_elem('write', self.write),
                    ])
                ]),
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'), cls._parse_data)

        @classmethod
        def _parse_data(cls, elem):
            return int(elem.findtext('id'))
