# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from .. import core

from migrator.core.utils.filetypes.xml import seq, elem, text_elem


ProtectedDirGenInfo = namedtuple('ProtectedDirGenInfo', ('id', 'name', 'header', 'properties'))
PDUserGenInfo = namedtuple('PDUserGenInfo', ('login',))
ProtectedDirPropertyGenInfo = namedtuple('ProtectedDirPropertyGenInfo', ('name', 'value'))


class ProtectedDirectoryOperator(object):
    FilterAll = core.FilterAll
    FilterByPDId = core.declare_filter('FilterByPDId', 'pd-id')
    FilterBySiteId = core.declare_filter('FilterBySiteId', 'site-id')
    FilterByDomainId = core.declare_filter('FilterByDomainId', 'domain-id')

    class Get(core.operation_with_filter('Get')):
        operator_name = 'protected-dir'
        operation_name = 'get'
        min_api_version = '1.5.2.0'
        max_api_version = None

        @classmethod
        def parse(cls, elem):
            results = core.filter_owner_not_found_error(core.parse_result_set(elem, cls._parse_data, 'id'))
            if len(results) == 1 and results[0].ok and results[0].data.name is None:
                results = []

            return results

        @classmethod
        def _parse_data(cls, elem):
            return ProtectedDirGenInfo(
                id=elem.findtext('id'),
                name=elem.findtext('data/name'),
                header=elem.findtext('data/header'),
                properties=[ProtectedDirPropertyGenInfo(
                    name=x.findtext('name'), value=x.findtext('value')
                ) for x in elem.findall('data/location/property')],
            )

    class Add(core.operation('Add', ('site_id', 'name', 'header', 'properties'))):
        operator_name = 'protected-dir'
        operation_name = 'add'
        min_api_version = '1.5.2.0'
        max_api_version = None

        def inner_xml(self):
            return seq(
                text_elem('site-id', self.site_id),
                text_elem('name', self.name),
                text_elem('header', self.header),
                elem('location', [
                    elem('property', [
                        text_elem('name', x.name),
                        text_elem('value', x.value),
                    ]) for x in self.properties
                ]) if self.properties else None
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'), cls._parse_data)

        @classmethod
        def _parse_data(cls, elem):
            return int(elem.findtext('id'))

    class GetUser(core.operation_with_filter('GetUser')):
        operator_name = 'protected-dir'
        operation_name = 'get-user'
        min_api_version = '1.5.2.0'
        max_api_version = None

        @classmethod
        def parse(cls, elem):
            results = [core.Result.parse(r, cls._parse_data) for r in elem.findall('result')]
            if len(results) == 1 and results[0].ok and results[0].data.login is None:
                results = []

            return results

        @classmethod
        def _parse_data(cls, elem):
            return PDUserGenInfo(
                login=elem.findtext('data/login'),
            )

    class AddUser(core.operation('AddUser', ('pd_id', 'login', 'password', 'password_type'))):
        operator_name = 'protected-dir'
        operation_name = 'add-user'
        min_api_version = '1.5.2.0'
        max_api_version = None

        def inner_xml(self):
            return seq(
                text_elem('pd-id', self.pd_id),
                text_elem('login', self.login),
                text_elem('password', self.password),
                text_elem('password-type', self.password_type)
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'), cls._parse_data)

        @classmethod
        def _parse_data(cls, elem):
            return int(elem.findtext('id'))
