# (C)2017 Ingram Micro Inc. all Rights Reserved
from .. import core

from migrator.core.utils.filetypes.xml import seq, text_elem, elem


class SslOperator(object):
    FilterAll = core.FilterAll
    FilterByDomainName = core.declare_filter('FilterByDomainName', 'domain-name')
    FilterByCertificateName = core.declare_filter('FilterByCertificateName', 'name')

    class Install(core.operation('Install', ('name', 'webspace', 'csr', 'pvt', 'cert', 'ca'))):
        operator_name = 'certificate'
        operation_name = 'install'
        min_api_version = '1.3.5.0'
        max_api_version = None

        def inner_xml(self):
            return seq(
                text_elem('name', self.name),
                text_elem('webspace', self.webspace),
                elem('content', [
                    text_elem('csr', self.csr),
                    text_elem('pvt', self.pvt),
                    text_elem('cert', self.cert),
                    text_elem('ca', self.ca),
                ])
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'))

    class Remove(core.operation_with_filter('Remove', ('webspace',))):
        operator_name = 'certificate'
        operation_name = 'remove'
        min_api_version = '1.3.5.0'
        max_api_version = None

        def data_xml(self):
            return seq(
                text_elem('webspace', self.webspace),
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'))

    class Get(core.operation_with_filter('Get')):
        operator_name = 'certificate'
        operation_name = 'get-pool'
        min_api_version = '1.3.5.0'
        max_api_version = None

        @classmethod
        def parse(cls, elem):
            return [core.Result.parse(r, cls._parse_data) for r in elem.findall('result')]

        @classmethod
        def _parse_data(cls, elem):
            return [x.text for x in elem.findall('certificates/certificate/name')]
