# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from .. import core

from migrator.core.utils.filetypes.xml import elem, text_elem


SubdomainInfo = namedtuple('SubdomainInfo', ('id', 'name', 'parent', 'hosting'))


class SubdomainHostingVirtual(namedtuple('SiteHostingVirtual', ('properties'))):
    def outer_xml(self):
        subelements = [
            elem('property', [
                text_elem('name', name),
                text_elem('value', value),
            ])
            for name, value in self.properties
        ]

        return elem('vrt_hst', subelements)


class SubdomainOperator(object):
    FilterAll = core.FilterAll
    FilterByName = core.declare_filter('FilterByName', 'name')
    FilterByParentName = core.declare_filter('FilterByParentName', 'parent-name')
    FilterById = core.declare_filter('FilterById', 'id')

    class Get(core.operation_with_filter('Get')):
        operator_name = 'subdomain'
        operation_name = 'get'
        min_api_version = '1.5.2.0'
        max_api_version = None

        @classmethod
        def parse(cls, elem):
            return core.filter_owner_not_found_error(core.parse_result_set(elem, cls._parse_data, 'id'))

        @classmethod
        def _parse_data(cls, element):
            id = element.findtext('id')
            data = element.find('data')

            name = data.findtext('name')
            parent = data.findtext('parent')

            properties = dict()
            for property_node in data.findall('property'):
                properties[property_node.findtext('name')] = property_node.findtext('value')

            return id, SubdomainInfo(id=id, name=name, parent=parent, hosting=SubdomainHostingVirtual(properties))
