# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple

from .. import core

from migrator.core.utils.filetypes.xml import seq, elem, text_elem


WebuserGenInfo = namedtuple('WebuserGenInfo', ('login', 'ftp_quota', 'services'))


class WebuserOperator(object):
    FilterAll = core.FilterAll
    FilterByDomainId = core.declare_filter('FilterByDomainId', 'domain-id')
    FilterBySiteId = core.declare_filter('FilterBySiteId', 'site-id')

    class Get(core.operation_with_filter('Get')):
        operator_name = 'webuser'
        operation_name = 'get'
        min_api_version = '1.4.2.0'
        max_api_version = None

        @classmethod
        def parse(cls, elem):
            results = core.filter_owner_not_found_error(core.parse_result_set(elem, cls._parse_data, 'login'))
            if len(results) == 1 and results[0].ok and results[0].data.login is None:
                results = []

            return results

        @classmethod
        def _parse_data(cls, elem):
            services = dict()
            for service_node in elem.findall('services/service'):
                services[service_node.findtext('name')] = service_node.findtext('value')

            return WebuserGenInfo(
                login=elem.findtext('login'),
                ftp_quota=elem.findtext('ftp-quota'),
                services=services
            )

    class Add(core.operation('Add', ('site_id', 'login', 'password'))):
        operator_name = 'webuser'
        operation_name = 'add'
        min_api_version = '1.5.2.0'
        max_api_version = None

        def inner_xml(self):
            return seq(
                text_elem('site-id', self.site_id),
                text_elem('login', self.login),
                text_elem('password', self.password),
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'), cls._parse_data)

        @classmethod
        def _parse_data(cls, elem):
            return int(elem.findtext('id'))

    class Set(core.operation('Set', (
            'login', 'asp', 'asp_dot_net', 'php', 'cgi', 'perl', 'python', 'fastcgi', 'write_modify'
    ))):
        operator_name = 'webuser'
        operation_name = 'set'
        min_api_version = '1.5.2.0'
        max_api_version = None

        def inner_xml(self):
            services = {
                'asp': self.asp,
                'asp_dot_net': self.asp_dot_net,
                'php': self.php,
                'cgi': self.cgi,
                'perl': self.perl,
                'python': self.python,
                'fastcgi': self.fastcgi,
                'write_modify': self.write_modify,
            }
            return seq(
                elem('filter', [text_elem('login', self.login)]),
                elem('values', [
                    elem('services', [
                        elem('service', [
                            text_elem('name', name), text_elem('value', value),
                        ]) for name, value in services.iteritems() if value is not None
                    ])
                ]),
            )

        @classmethod
        def parse(cls, elem):
            return core.Result.parse(elem.find('result'), cls._parse_data)

        @classmethod
        def _parse_data(cls, elem):
            return int(elem.findtext('id'))
