# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging
import ntpath
import os

from xml.etree import ElementTree

from migrator.core.language import _T
from migrator.core.utils.filetypes.xml import elem, text_elem, xml_to_string_pretty

logger = logging.getLogger(__name__)


def decrypt_account_passwords(plesk_server, encrypted_passwords):
    plain_passwords = {}
    accounts_tree = ElementTree.ElementTree(
        elem('migration-dump', [
            text_elem('password', password, {'account-id': account_id, 'type': password_type})
            for account_id, password, password_type in encrypted_passwords.values() if password is not None
        ])
    )
    with plesk_server.runner() as runner:
        accounts_xml_file = plesk_server.get_session_file_path('accounts.xml')
        runner.upload_file_content(accounts_xml_file, xml_to_string_pretty(accounts_tree))
        try:
            command, arguments = _decrypt_passwords_command(plesk_server, accounts_xml_file)
            if command is not None:
                runner.run(command, arguments, log_output=False)
                accounts_root_node = ElementTree.fromstring(runner.get_file_contents(accounts_xml_file))
                for password_node in accounts_root_node:
                    if password_node.get('type') == 'plain':
                        password = password_node.text
                    else:
                        password = None

                    plain_passwords[password_node.get('account-id')] = password
            else:
                for account_id, password, password_type in encrypted_passwords.values():
                    if password_type == 'plain':
                        plain_passwords[account_id] = password
        finally:
            try:
                runner.remove_file(accounts_xml_file)
            except:
                logger.debug(_T("LOG_EXCEPTION"), exc_info=True)

    decrypted_passwords = {}
    for item in encrypted_passwords.iterkeys():
        account_id = encrypted_passwords[item][0]
        if account_id in plain_passwords:
            decrypted_passwords[item] = plain_passwords[account_id]
        else:
            decrypted_passwords[item] = None

    return decrypted_passwords


def _decrypt_passwords_command(plesk_server, accounts_xml_file):
    """
    :type accounts_xml_file: str
    :rtype: tuple[str list[str]]
    """
    arguments = []
    with plesk_server.runner() as runner:
        if plesk_server.is_windows():
            command = ntpath.join(plesk_server.plesk_dir, 'admin', 'bin', 'php')
            filename = ntpath.join(plesk_server.plesk_dir, 'admin', 'plib', 'cu', 'backup_encrypt.php')
            if not runner.file_exists(filename):
                return None, None

            arguments += ['-dauto_prepend_file=', filename, ]
        else:
            filename = os.path.join(plesk_server.plesk_dir, 'admin', 'bin', 'backup_encrypt')
            if not runner.file_exists(filename):
                filename = os.path.join(plesk_server.plesk_dir, 'bin', 'backup_encrypt')
                if not runner.file_exists(filename):
                    filename = os.path.join(plesk_server.plesk_dir, 'admin', 'plib', 'api-cli', 'backup_encrypt.php')
                    if not runner.file_exists(filename):
                        return None, None

            command = filename

    arguments += ['--decrypt-by-plesk', '-single-file', accounts_xml_file]
    return command, arguments
