# (C)2017 Ingram Micro Inc. all Rights Reserved
from migrator.core.utils.networking.dns import convert_to_national_domain_name
from migrator.products.plesk.lib.api.xmlrpc.core import Success
from migrator.products.plesk.lib.api.xmlrpc import operator as plesk_ops

subdomain_prefix_for_plesk_9 = 'subdomain_'

def fix_plesk9_subdomains_ids(subdomains_info):
    # Plesk 8 & 9 store subdomains in separate db table thus subdomain id may coincide with domain id.
    # We need to imitate uniqueness of such ids.

    res = []
    for result in subdomains_info:
        if hasattr(result, 'code'):
            res.append(result)
            continue

        subdomain_id = result.data[0]
        subdomain_info = result.data[1]

        new_subdomain_id = subdomain_prefix_for_plesk_9 + str(subdomain_id)
        new_subdomain_info = plesk_ops.SubdomainInfo(
            id=new_subdomain_id,
            name=convert_to_national_domain_name(subdomain_info.name),
            parent=convert_to_national_domain_name(subdomain_info.parent),
            hosting=subdomain_info.hosting
        )
        res.append(Success([new_subdomain_id, new_subdomain_info]))

    return res
