# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

import migrator.products.plesk.target.config as plesk_config

from migrator.core import MigrationError
from migrator.core.connections.connections import Connections
from migrator.core.language import _T
from migrator.core.utils import cached, merge_dicts
from migrator.core.utils.filetypes.config_utils import global_section
from migrator.products.plesk.source.plesk.server import PleskSourceServer

logger = logging.getLogger(__name__)


class PleskMigratorConnections(Connections):
    def __init__(self, global_context, target_panel, migrator_server):
        super(PleskMigratorConnections, self).__init__(global_context, target_panel)

        self._migrator_server = migrator_server

        # Get instances of SourcePleskUnixConfig (and SourcePleskWinbdowsConfig ?)
        self._source_plesks = self._read_plesk_settings_from_config(
            global_context.config, 'source-servers')

        for settings in self.get_source_plesks().itervalues():
            logger.info(_T("LOG_SOURCE_HOST"), settings.id, settings.ip)

    def get_panel_server(self, global_context, panel_server_section_name):
        """Create panel server object specified in the config option 'source-type'"""
        settings_list = self._read_plesk_settings_from_config(global_context.config, 'source-type')
        if panel_server_section_name not in settings_list:
            raise MigrationError(_T('SOURCE_SYSTEM_CONFIG_SECTION_NOT_FOUND').format(
                section_name=panel_server_section_name
            ))
        settings = settings_list[panel_server_section_name]

        return PleskSourceServer(panel_server_section_name, settings, self._migrator_server)

    @cached
    def get_source_server_object(self, server_id):
        all_servers = merge_dicts(self.get_source_plesks(), self.get_external_db_servers())
        settings = all_servers[server_id]
        return PleskSourceServer(server_id, settings, self._migrator_server)

    def get_information_servers(self):
        return self._source_plesks

    def get_source_plesks(self):
        """Get information about source Plesk servers

        Returns dictionary {server_id: server_settings} with connection
        information.
        """
        return self._source_plesks

    @staticmethod
    def _read_plesk_settings_from_config(config, list_name, optional=False):
        if not optional or config.has_option('GLOBAL', list_name):
            settings = {}
            list_str = global_section(config).get(list_name, '')
            if list_str.strip() != '':
                source_sections = [s.strip() for s in list_str.split(',')]
                for section_name in source_sections:
                    settings[section_name] = plesk_config.read_source_plesk_settings(config, section_name)
                return settings
            else:
                return {}
        else:
            return {}
