# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core.data.source.db.platform import PlatformReaderDB
from migrator.core.main import Migrator as CommonMigrator
from migrator.core.utils.filetypes.config_utils import get_source_type_section
from migrator.products.plesk.lib.agent import PleskMigrationAgent
from migrator.products.plesk.source.plesk import connections
from migrator.products.plesk.source.plesk.session_files import PleskSessionFiles

logger = logging.getLogger(__name__)


class Migrator(CommonMigrator):
    def __init__(self, options):
        super(Migrator, self).__init__(options)

        section = get_source_type_section(self.global_context.config)
        source_server = self.global_context.conn.get_panel_server(self.global_context, section.section_name)

        self.agent = PleskMigrationAgent(source_server)

    def _load_connections_configuration(self):
        conn = connections.PleskMigratorConnections(
            self.global_context, self._get_target_panel(), self._get_migrator_server()
        )
        return conn

    def _create_session_files(self):
        return PleskSessionFiles(self.conn, self._get_migrator_server())

    # ======================== Utility methods ==============================

    def _get_rsync(self, source_server, target_server, source_ip=None):
        # This method does not depend on source_ip, this argument is necessary for H-Sphere/Helm
        return self.global_context.rsync_pool.get(
            source_server, target_server, source_server.vhosts_dir if hasattr(source_server, 'vhosts_dir') else None
        )

    def _get_source_service_node(self, subscription_id, service):
        """@moveto SourcePanelExporterBase"""
        source_host = PlatformReaderDB().get_source_host_by_subscription_service(subscription_id, service)

        if source_host is None:
            return None

        for plesk_id in self.global_context.conn.get_source_plesks():
            source_server = self.global_context.conn.get_source_server_object(plesk_id)
            if source_host.ip == source_server.ip():
                return source_server

        return None

    def _get_source_mail_node(self, subscription_id):
        """@moveto SourcePanelExporterBase"""
        return self._get_source_service_node(subscription_id, 'mail')

    def _get_source_web_node(self, host_external_id, subscription_id):
        """@moveto SourcePanelExporterBase"""
        return self._get_source_service_node(subscription_id, 'web')
