# (C)2017 Ingram Micro Inc. all Rights Reserved
import logging

from migrator.core import MigrationNoRepeatError
from migrator.core.language import _T
from migrator.core.verification.connection import ConnectionChecker
from migrator.products.plesk.lib.api.xmlrpc.core import PleskError
from migrator.products.plesk.lib.api.xmlrpc.operator import ServerOperator

logger = logging.getLogger(__name__)


class PleskConnectionCheckError(MigrationNoRepeatError):
    pass


class PleskServerConnectionChecker(ConnectionChecker):
    @staticmethod
    def check_plesk_api(plesk_api, plesk_server):
        """
        :type plesk_server: parallels.core.connections.plesk_server.PleskServer
        """
        logger.debug(_T("DEBUG_CHECK_API_CONNECTION") % plesk_server.description())
        try:
            plesk_api.send(ServerOperator.Get([ServerOperator.Dataset.GEN_INFO])).data
        except PleskError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            if e.code == 1001:
                raise PleskConnectionCheckError(
                    _T("FAILED_TO_CHECK_PLESK_API_INVALID_CREDENTIALS").format(
                        server=plesk_server.description())
                )
            elif e.code == 1030:
                raise PleskConnectionCheckError(
                    _T("FAILED_TO_CHECK_PLESK_API_EXPIRED_LICENSE").format(
                        server=plesk_server.description())
                )
            else:
                raise PleskConnectionCheckError(
                    _T("FAILED_TO_CONNECT_BY_PLESK_API").format(
                        server=plesk_server.description(), error=unicode(e))
                )
        except EnvironmentError as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            raise PleskConnectionCheckError(
                _T("FAILED_TO_CONNECT_BY_PLESK_API").format(
                    server=plesk_server.description(), error=unicode(e))
            )
        except Exception as e:
            logger.debug(_T("LOG_EXCEPTION"), exc_info=True)
            raise PleskConnectionCheckError(
                _T("FAILED_TO_CHECK_PLESK_API_CONNECTION").format(
                    server=plesk_server.description(), error=unicode(e))
            )
