# (C)2017 Ingram Micro Inc. all Rights Reserved
from collections import OrderedDict

from migrator.core import MigrationError
from migrator.core.language import _T
from migrator.core.product.lib.host import get_service_status_linux
from migrator.core.product.panel import PanelInfoBase
from migrator.core.utils import cached


class PanelInfo(PanelInfoBase):
    @property
    def name(self):
        return 'Virtuozzo'

    @property
    def short_name(self):
        return 'vz'

    @staticmethod
    def is_installed(runner):
        return runner.file_exists('/etc/virtuozzo-release')

    @property
    def url(self):
        return None

    @property
    def version(self):
        version_str = self._get_version()
        return version_str.split('-')[0]

    @property
    def patch(self):
        return None

    def get_service_list(self):
        return OrderedDict([
            ('Parallels Virtual Automation Agent', dict(name_list=('pvaagentd',)))
        ])

    def get_service_status(self, service_name):
        status_string, exit_code, _ = get_service_status_linux(self.runner, service_name)
        return status_string, exit_code

    @cached
    def _get_version(self):
        vz_release = self.runner.sh(u'cat /etc/virtuozzo-release')
        vz_release_parts = vz_release.split()

        if len(vz_release_parts) < 1:
            raise MigrationError(_T("UNABLE_TO_DETERMINE_VZ_VERSION"))

        return vz_release_parts[-1]
