# (C)2017 Ingram Micro Inc. all Rights Reserved
import os

from collections import namedtuple

PRINT_DB_QUERIES_TO_LOG = False

SERVICE_EXPORT_NOT_SUPPORTED = 'service_export_not_supported'

OS_UNKNOWN = 'unknown_os'
OS_LINUX = 'linux'
OS_WINDOWS = 'windows'
SUPPORTED_OS_TYPES = [OS_LINUX, OS_WINDOWS]

HCL_EXEC_MULTI_ATTEMPTS_COUNT = 2
HCL_EXEC_MULTI_ATTEMPTS_INTERVAL = 3

'''Migrator global settings.'''
# Administrator's ID for internal data models of migrator
OA_ADMIN_ID = 1
OA_TARGET_OWNER_ID = 1

# Type of Panel Migrator installation via own installer as standalone product
INSTALLATION_TYPE_STANDALONE = 'STANDALONE'

IP_EXCLUSIVE = 'exclusive'
IP_SHARED = 'shared'

XML_IP_EXCLUSIVE = 'EXCLUSIVE'
XML_IP_SHARED = 'SHARED'

PLESK_PANEL_DEFAULT_PORT = 8443
SSH_DEFAULT_PORT = 22
SMB_DEFAULT_PORT = 445
RSYNC_SERVER_PORT_CLAUSE_WINDOWS = ':10156'

OA_APS_APP_SYSUSER = 'oam'
OA_APS_APP_SYSGROUP = 'oam'

# Default path to migrator's session directory on source Windows server.
# Keep it short to avoid issues with long paths.
WINDOWS_DEFAULT_SESSION_DIR = r'C:\oam-session'

# Default path to migrator's session directory on source Unix server.
UNIX_DEFAULT_SESSION_DIR = '/opt/oam-session'

OA_MIGRATOR_BASE_DIR = '/opt/oa-migrator'
OA_APS_APP_HOME_DIR = '/var/www/oam'
OA_MIGRATOR_SHARE_DIR = os.path.join(OA_MIGRATOR_BASE_DIR, 'share')
LSH_NG_MIGRATOR_BASE_DIR = os.path.join(OA_MIGRATOR_BASE_DIR, 'lib', 'lshng_migrator')
NG_NG_MIGRATOR_BASE_DIR = os.path.join(OA_MIGRATOR_BASE_DIR, 'lib', 'ng_ng_migrator')
LOCALE_PATH_FOR_UNIT_TESTS = os.path.abspath(os.path.join(os.path.dirname(__file__), '../', 'data', 'locale'))
XSD_PATH_FOR_UNIT_TESTS = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../', 'data', 'unified_format', 'schema')
)

OA_MIGRATOR_LOCALE_PATH = LOCALE_PATH_FOR_UNIT_TESTS if os.path.exists(LOCALE_PATH_FOR_UNIT_TESTS) else \
    os.path.join(OA_MIGRATOR_SHARE_DIR, 'locale/')

OA_MIGRATOR_XSD_PATH = XSD_PATH_FOR_UNIT_TESTS if os.path.exists(XSD_PATH_FOR_UNIT_TESTS) else \
    os.path.join(OA_MIGRATOR_SHARE_DIR, 'oa-migrator/unified_format/schema/')

OA_MIGRATOR_BILLING_NAMESPACE = "http://www.odin.com/oam/unified/billing/info/1.0"
OA_MIGRATOR_BILLING_XSD = "%s/billing.xsd" % OA_MIGRATOR_XSD_PATH

OA_MIGRATOR_WEB_HOSTING_SHM_NAMESPACE = "http://www.odin.com/oam/unified/web/hosting/shm/1.0"
OA_MIGRATOR_WEB_HOSTING_SHM_XSD = "%s/web_hosting_shm.xsd" % OA_MIGRATOR_XSD_PATH

OA_MIGRATOR_WEB_CONTENT_SHM_NAMESPACE = "http://www.odin.com/oam/unified/web/content/shm/1.0"
OA_MIGRATOR_WEB_CONTENT_SHM_XSD = "%s/web_content_shm.xsd" % OA_MIGRATOR_XSD_PATH

OA_MIGRATOR_MAIL_HOSTING_SHM_NAMESPACE = "http://www.odin.com/oam/unified/mail/hosting/shm/1.0"
OA_MIGRATOR_MAIL_HOSTING_SHM_XSD = "%s/mail_hosting_shm.xsd" % OA_MIGRATOR_XSD_PATH

OA_MIGRATOR_MAIL_CONTENT_SHM_NAMESPACE = "http://www.odin.com/oam/unified/mail/content/shm/1.0"
OA_MIGRATOR_MAIL_CONTENT_SHM_XSD = "%s/mail_content_shm.xsd" % OA_MIGRATOR_XSD_PATH

OA_MIGRATOR_DNS_NAMESPACE = "http://www.odin.com/oam/unified/dns/hosting/1.0"
OA_MIGRATOR_DNS_XSD = "%s/dns_hosting.xsd" % OA_MIGRATOR_XSD_PATH

OA_MIGRATOR_VPS_HOSTING_OACI_NAMESPACE = "http://www.odin.com/oam/unified/vps/hosting/oaci/1.0"
OA_MIGRATOR_VPS_HOSTING_OACI_XSD = "%s/vps_parameters_oaci.xsd" % OA_MIGRATOR_XSD_PATH

OA_BASE_DIR = '/usr/local/pem'
# Location with extras utilities.
EXTRAS_PATH = os.path.join(OA_BASE_DIR, 'OAM/oam-3rd-party-extras/content/')

DEFAULT_PLESK_SITE_PERMISSION = 750

# The maximum number of attempts to execute the function and the interval between the attempts.
# It is used for migrator.core.utils.FunctionResultWaiter.
DEFAULT_MAX_ATTEMPTS = 360
DEFAULT_INTERVAL_BETWEEN_ATTEMPTS = 5

TAKEOVER_DEFAULT_MAX_ATTEMPTS = 1800
TAKEOVER_DEFAULT_INTERVAL_BETWEEN_ATTEMPTS = 1

# Path to the directory with configuration files
OA_MIGRATOR_CONFIG_DIR = os.path.join(OA_MIGRATOR_BASE_DIR, 'conf')
# Configuration file defining how to access the OAM database
OA_MIGRATOR_DB_CONFIG = os.path.join(OA_MIGRATOR_CONFIG_DIR, 'db.ini')
# Paths to the OAM SSL certificate parts
OA_MIGRATOR_SSL_CERT = os.path.join(OA_MIGRATOR_CONFIG_DIR, 'oam.crt')
OA_MIGRATOR_SSL_PKEY = os.path.join(OA_MIGRATOR_CONFIG_DIR, 'oam.key')
# Path to the OA platform configuration file
OA_KERNEL_CONFIG_FILE = os.path.join(OA_BASE_DIR, 'etc/Kernel.conf')
# Path to the PGHA configuration file
PGHA_CONF = os.path.join(OA_BASE_DIR, 'bin/pgha/pgha_settings.py')


class DatabaseServerType(object):
    MYSQL = 'mysql'
    POSTGRESQL = 'postgresql'
    MSSQL = 'mssql'


TDatabaseServerTypeInfo = namedtuple('TDatabaseServerTypeInfo', ('full_name', 'user_name_len', 'database_name_len'))
# full_name - Full DBMS Server name for messages
# user_name_len - maximum allowed user's name length
# database_name_len - maximum allowed database's name length

DatabaseServerTypeInfo = {
    DatabaseServerType.MYSQL:
        TDatabaseServerTypeInfo(full_name='MySQL', user_name_len=16, database_name_len=64),
    DatabaseServerType.POSTGRESQL:
        TDatabaseServerTypeInfo(full_name='PostgreSQL', user_name_len=16, database_name_len=64),
    DatabaseServerType.MSSQL:
        TDatabaseServerTypeInfo(full_name='Microsoft SQL Server', user_name_len=128, database_name_len=128),
}


# Supported OA services and modules names
class TargetServiceType(object):
    BILLING = 'billing'
    PLATFORM = 'platform'
    SHM = 'shm'
    OACI = 'oaci'
    DNS = 'dns'
    MAIL = 'mail'
    NG = 'ng'

    @staticmethod
    def get_supported_target_service_classes():
        return [TargetServiceType.NG, TargetServiceType.SHM, TargetServiceType.OACI]


class SourceHostingClass(object):
    SHARED = 'shared'
    VPS = 'vps'
    DNS = 'dns'
    DNS_NS = 'dns_ns'
    LSH = 'lsh'
    NG = 'ng'
    WHL = 'whl'
    WSH = 'wsh'


# Source Hosting type -> TargetServiceType
HOSTING_SERVICE_TYPES = {
    SourceHostingClass.SHARED: TargetServiceType.SHM,
    SourceHostingClass.VPS: TargetServiceType.OACI,
    SourceHostingClass.DNS: TargetServiceType.DNS,
    SourceHostingClass.LSH: TargetServiceType.NG,
    SourceHostingClass.NG: TargetServiceType.NG,
    SourceHostingClass.WHL: TargetServiceType.SHM,
    SourceHostingClass.WSH: TargetServiceType.SHM,
}

# Source services
SOURCE_SERVICE_FTP = 'ftp'
SOURCE_SERVICE_WEB = 'web'
SOURCE_SERVICE_DB = 'db'
SOURCE_SERVICE_MAIL = 'mail'
SOURCE_SERVICE_DNS = 'dns'
SOURCE_SERVICE_DNS_NS = 'dns_ns'
SOURCE_SERVICE_DNS_CUSTOM_NS = 'dns_cns'
SOURCE_SERVICE_DNS_NON_MANAGEABLE_NS = 'dns_nmns'
SOURCE_SERVICE_VPS = 'vps'

SOURCE_SERVICE_FTP_WSH = 'ftp_wsh'
SOURCE_SERVICE_WEB_WSH = 'web_wsh'
SOURCE_SERVICE_DB_WSH = 'db_wsh'

SOURCE_SERVICE_FTP_WHL = 'ftp_whl'
SOURCE_SERVICE_WEB_WHL = 'web_whl'
SOURCE_SERVICE_DB_WHL = 'db_whl'

SOURCE_SERVICE_FTP_LSH = 'ftp_lsh'
SOURCE_SERVICE_WEB_LSH = 'web_lsh'
SOURCE_SERVICE_DB_LSH = 'db_lsh'

# Service properties
SERVICE_PROPERTY_IP_MIGRATION_STRATEGY = 'ipMigrationStrategy'

SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP = {
    SourceHostingClass.SHARED: [SOURCE_SERVICE_FTP, SOURCE_SERVICE_WEB, SOURCE_SERVICE_DB, SOURCE_SERVICE_MAIL],
    SourceHostingClass.DNS: [SOURCE_SERVICE_DNS],
    SourceHostingClass.DNS_NS: [SOURCE_SERVICE_DNS_NS],
    SourceHostingClass.VPS: [SOURCE_SERVICE_VPS],
    SourceHostingClass.WSH: [SOURCE_SERVICE_FTP_WSH, SOURCE_SERVICE_WEB_WSH, SOURCE_SERVICE_DB_WSH],
    SourceHostingClass.WHL: [SOURCE_SERVICE_FTP_WHL, SOURCE_SERVICE_WEB_WHL, SOURCE_SERVICE_DB_WHL],
    SourceHostingClass.LSH: [SOURCE_SERVICE_FTP_LSH, SOURCE_SERVICE_WEB_LSH, SOURCE_SERVICE_DB_LSH],
}

SHARED_HOSTING_SERVICES = \
    SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.SHARED] + \
    SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.WSH] + \
    SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.WHL] + \
    SOURCE_HOSTING_CLASSES_TO_SERVICES_MAP[SourceHostingClass.LSH]

MIGRATION_TYPE_FULL = 'full'
MIGRATION_TYPE_TAKEOVER = 'takeover'
MIGRATION_TYPE = [
    MIGRATION_TYPE_FULL,
    MIGRATION_TYPE_TAKEOVER,
]

MIGRATION_MODE_TEST = 'test'
MIGRATION_MODE_TEST_WITH_CONTENT = 'test-with-content'
MIGRATION_MODE_PRODUCTION = 'production'
MIGRATION_MODE = [
    MIGRATION_MODE_TEST,
    MIGRATION_MODE_TEST_WITH_CONTENT,
    MIGRATION_MODE_PRODUCTION,
]

IP_MIGRATION_MODE_SAME_IP_ADDRESS = 'sameIp'
IP_MIGRATION_MODE_SAME_IP_TYPE = 'sameType'

IP_MIGRATION_STRATEGY_ONE = 'migrateOneExclusiveIP'
IP_MIGRATION_STRATEGY_ALL = 'migrateAllExclusiveIPs'

SOURCE_PRODUCT_PACKAGE_PREFIX = 'migrator.products'
SOURCE_PRODUCT_PACKAGE_POSTFIX = 'panel'

SOURCE_PRODUCT_OBAS = 'obas'
SOURCE_PRODUCT_PLESK = 'plesk'
SOURCE_PRODUCT_VIRTUOZZO = 'virtuozzo'
SOURCE_PRODUCT_OA_WHL = 'oa.whl'
SOURCE_PRODUCT_OA_WSH = 'oa.wsh'
SOURCE_PRODUCT_OA_LSH = 'oa.lsh'
SOURCE_PRODUCT_OA_NG = 'oa.ng'
SUPPORTED_SOURCE_PRODUCTS = [
    SOURCE_PRODUCT_OBAS,
    SOURCE_PRODUCT_PLESK,
    SOURCE_PRODUCT_VIRTUOZZO,
    SOURCE_PRODUCT_OA_WHL,
    SOURCE_PRODUCT_OA_WSH,
    SOURCE_PRODUCT_OA_LSH,
    SOURCE_PRODUCT_OA_NG,
]

OA_SERVICE_MODULES = [
    SOURCE_PRODUCT_OA_WHL,
    SOURCE_PRODUCT_OA_WSH,
    SOURCE_PRODUCT_OA_LSH,
    SOURCE_PRODUCT_OA_NG
]

TARGET_PRODUCT_OA = 'platform'

PANEL_TYPE_PLESK = 'plesk'
PANEL_TYPE_OBAS = 'obas'
PANEL_TYPE_BIND_OBAS = 'ob-bind'
PANEL_TYPE_VIRTUOZZO = 'vz'
PANEL_TYPE_IIS = 'iis'
PANEL_TYPE_APACHE = 'apache'
PANEL_TYPE_APACHE_NG = 'apacheng'

# Maximum amount of time migrator waits for testing network port, in seconds
NETWORK_PORT_CHECKING_TIMEOUT = 5

# Maximum amount of time migrator waits for winexe command to complete, in seconds
WINEXE_GET_SYSTEM_INFO_COMMAND_TIMEOUT = 15

# SSH client settings: maximum amount of attempts to connect to host and the sleep interval between attempts
SSH_CLIENT_CONNECT_MAX_ATTEMPTS = 5
SSH_CLIENT_CONNECT_INTERVAL_BETWEEN_ATTEMPTS = 10

# Constants for the migration checks messages
HTML_EOL = '<br/>'
DEBUG_LOG = '/opt/oa-migrator/logs/debug.log'

# Source system agent specific settings
AGENT_PORT = 29443

DOCUMENTATION_BASE_URL = 'https://docs.cloudblue.com/oa/services/migration-tools/Home.htm'

MAX_APS_FILTER_STRING_LENGTH = 1800

ONE_MB = 1048576

HTTP_APS_CONTROLLER_URI = 'https://127.0.0.1:6308/'

# pg constants
PG_DEFAULT_PORT = 5432
PG_POOL_PASSWD = '/etc/pgpool-II/pool_passwd'

PLESK_TAKEOVER_DNS_RECORD_IS_EDIT_ALLOWED = True
SKIP_MAILBOXES_CONTENT_MIGRATION = False
SKIP_DNS_ZONE_FORWARDING = False
SKIP_MAILBOXES_ALIASES_MIGRATION = False
SKIP_ACCOUNT_BALANCE_MIGRATION = False
SKIP_RESTORE_MAILBOX_PASSWORD = False
